
/************************************************************************
 Copyright (c) 2001 Opticon Sensors Europe B.V.
 ALL RIGHTS RESERVED
*************************************************************************/

#ifndef __STDIO_H       /* avoid multiple inclusions */
#define __STDIO_H   1

typedef unsigned long fpos_t;
typedef unsigned size_t;

#ifndef __STDARG_H
typedef char *va_list;
#endif

typedef struct _iobuf
{
    char *_ptr;
    int   _cnt;
    char *_base;
    char  _flag;
    char  _file;
}
FILE;

#define NULL            ((void *)0)

#define BUFSIZ          512
#define EOF             (-1)
#define FOPEN_MAX       20
#define FILENAME_MAX    256
#define L_tmpnam        32
#define SEEK_SET        0
#define SEEK_CUR        1
#define SEEK_END        2
#define TMP_MAX         0x7FFFFFFF
#define stdin           (&_iob[0])
#define stdout          (&_iob[1])
#define stderr          (&_iob[2])

extern FILE _iob[];

#undef getc
#undef putc
#undef getchar
#undef putchar
#undef clearerr
#undef feof
#undef ferror

extern int    remove(const char *);                     // Done
extern int    rename(const char *, const char *);       // Done
//extern FILE  *tmpfile(void);
//extern char  *tmpnam(char *);
extern int    fclose(FILE *);                           // Done
extern int    fflush(FILE *);                           // Done
extern FILE  *fopen(const char *, const char *);        // Done
extern FILE  *freopen(const char *, const char *, FILE *);  // Done, see fopen.c
//extern void   setbuf(FILE *, char *);
//extern int    setvbuf(FILE *, char *, int, size_t);

extern int    fprintf(FILE *, const char *, ...);       // Done
extern int    fscanf(FILE *, const char *, ...);        // Done
extern int    printf(const char *, ...);                // Done (as call to fprintf)
extern int    scanf(const char *, ...);                 // Done (as call to sscanf)
extern int    sprintf(char *, const char *, ...);       // Done
extern int    sscanf(const char *, const char *, ...);  // Done
extern int    vfprintf(FILE *, const char *, va_list);  // Done
extern int    vprintf(const char *, va_list);           // Done (as call to vfprintf)
extern int    vsprintf(char *, const char *, va_list);  // Done

extern int    fgetc(FILE *);                            // See getc.c
extern char  *fgets(char *, int, FILE *);               // Done
extern int    fputc(int, FILE *);                       // Done, see putc
extern int    fputs(const char *, FILE *);              // Done
extern int    getc(FILE *);                             // Done
//extern int    getchar(void);                          implemented as macro
extern char  *gets(char *);                             // Done
extern int    putc(int, FILE *);                        // Done
extern int    putchar(int);
extern int    puts(const char *);                       // Done
extern int    ungetc(int, FILE *);                      // Done
extern size_t fread(void *, size_t, size_t, FILE *);    // Done
extern size_t fwrite(const void *, size_t, size_t, FILE *); // Done
extern int    fgetpos(FILE *, fpos_t *);                // Done
extern int    fseek(FILE *, long, int);                 // done
extern int    fsetpos(FILE *, const fpos_t *);          // Done
extern long   ftell(FILE *);                            // Done
extern void   rewind(FILE *);                           // done
extern void   clearerr(FILE *);                         // Done
extern int    feof(FILE *);                             // Done
extern int    ferror(FILE *);                           // Done
//extern void   perror(const char *);

#define getchar()   getc(stdin)
#define putchar(c)  putc((c),stdout)
#define clearerr(f) ((f)->_flag &= ~(_IOERR | _IOEOF))
#define feof(f)     ((f)->_flag & _IOEOF)
#define ferror(f)   ((f)->_flag & _IOERR)

#define _IOREAD   0x01      /* readable file */
#define _IOWRT    0x02      /* writable file */
#define _IONBF    0x04      /* no buffer */
#define _IOFBF    0x00      /* fully buffered (unimplemented) */
#define _IOLBF    0x00      /* line buffered (unimplemented) */
#define _IOMYBUF  0x08      /* system supplied buffer from heap */
#define _IOEOF    0x10      /* end of file has been detected */
#define _IOERR    0x20      /* error has been detected */
#define _IOSTRG   0x40      /* this stream is actually a string */
#define _IORW     0x80      /* reserved word */

#endif /* ifndef __STDIO_H */
