/* Demo program for DWT7133		*/
/* VERSION: CFQ19401            */



/************************************************************************/
/*      Include                                                         */
/************************************************************************/
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include        "lib.h"


/************************************************************************/
/*      Define                                                          */
/************************************************************************/
#define VERSION  		"CFQ19401"
#define NOT_OK         -1
#define CLEAR          -1
#define SCAN            1
#define KEYS            2
#define ALPHA           8
#define NUM             16
#define FLOAT           32
#define NONE            1
#define EVEN            2
#define ODD             3

#define GO_UP           179
#define GO_DOWN  		180

#define BARCODE         30
#define SYMBOLOGY       16
//#define TIMESTAMP       (2+1+2+1+2)
//#define DATESTAMP       (4+1+2+1+2)

#define ID_CD39         1   // bit 0
#define ID_EAN          2   // bit 1
#define ID_UPC          4   // bit 2
#define ID_I2O5         8   // bit 3
#define ID_D2O5        16   // bit 4
#define ID_NW7         32   // bit 5
#define ID_CD93        64   // bit 6
#define ID_CD128      128   // bit 7
#define ID_MSI        256   // bit 8
#define ID_TELEPEN    512   // bit 9
#define ID_UK        1024   // bit 10
#define ID_ADDONS    2048   // bit 11
#define ID_IATA      4096   // bit 12
#define ID_SCODE     8192   // bit 13
#define ID_ALLCODES 16384   // bit 14


#define REC_SIZE        BARCODE+1+SYMBOLOGY+1+1
/* barcode,symbology type\r\n */

#define DBASE_NAME      "DATA.FIL"
#define MAX_TIMEOUT      4		/* max # of timeouts in 1 block */
#define DEFAULT_MAXL     80
#define DEFAULT_TIME     5
#define DEFAULT_NPAD     0
#define DEFAULT_PADC     '\0'
#define DEFAULT_EOL      '\r'
#define DEFAULT_QCTL     '#'
#define MAXL             94
#define DATA_PACKET      'D'
#define ACK_PACKET       'Y'
#define NAK_PACKET       'N'
#define SINIT_PACKET     'S'
#define BREAK_PACKET     'B'
#define FILE_PACKET      'F'
#define EOF_PACKET       'Z'
#define ERROR_PACKET     'E'
#define MAX_BUF          1+1+1+1+MAXL+1+1	/* SOH+LEN+SEQ+TYP+MAXL+CHK+EOL */
#define tochar(x)        ((x) + 32)
#define unchar(x)        ((x) - 32)
#define ctl(x)           ((x) ^ 64)		/* exor */
#define check(s)         tochar ( ((s) + (((s) & 192) / 64)) & 63)
#define EDATA            0xFE
#define ENQ_DOWNLOAD     64		/* PC requests data from SLT */
#define DATA_RECORD      96
#define RETRIES          3
#define MAX_DATA         253
#define SLT_MAX_BUF      MAX_DATA+10
#define TIMEOUT          3*50	/* 3 seconds timeout */

typedef struct kermit_param {
	char max_len;				/*  max len of packet                          */
	char time;					/*  time out in seconds                        */
	char npad;					/*  number of char before each record(padding) */
	char padc;					/*  char used for padding                      */
	char eol;					/*  end of line char                           */
	char qctl;					/*  control quote char                         */
} KParam;


struct menus {
	char *item;
};

struct smenus {
	char *item;
	int value;
};


/************************************************************************/
/*      Global                                                          */
/************************************************************************/
int s_code;
char record[REC_SIZE + 1];
static int port;
static int baudrate;
static int parity;
static int stopbits;
static int databits;
static int protocol;

struct smenus symblglist[] =
{
	"UPC A"         ,   UPCA    ,
	"UPC A + 2"     ,   UPCA_2  ,
	"UPC A + 5"     ,   UPCA_5  ,
	"UPC E"         ,   UPCE    ,
	"UPC E + 2"     ,   UPCE_2  ,
	"UPC E + 5"     ,   UPCE_5  ,
	"EAN 8"         ,   EAN8    ,
	"EAN 8 + 2"     ,   EAN8_2  ,
    "EAN 8 + 5"     ,   EAN8_5  ,
    "EAN 13"        ,   EAN13   ,
    "EAN 13 + 2"    ,   EAN13_2 ,
    "EAN 13 + 5"    ,   EAN13_5 ,
    "CODE 39"       ,   CODE39  ,
    "CODABAR"       ,   CODABAR ,
    "INTERL 2 OF 5" ,   I2OF5   ,
    "INDUST 2 OF 5" ,   D2OF5   ,
	"MATRIX 2 OF 5" ,   M2OF5   ,
    "IATA CODE"     ,   IATA    ,
    "CODE 93"       ,   CODE93  ,
    "CODE 128"      ,   CODE128 ,
	"MSI PLESSEY"   ,   MSI_PLESSEY,
    "UK PLESSEY"    ,   UK_PLESSEY,
    "TELEPEN"       ,   TELEPEN,
    "SCODE"         ,   SCODE,
    "LAETUS CODE"   ,   LAETUS,
    "CD 39 FULL ASCII", C39_FA,
    "ITAL. PHARMA." ,   C39_IT,
    "CODABAR ABC"   ,   NW7_ABC,
    "CODABAR CX"    ,   NW7_CX,
    "EAN 128"       ,   EAN128,
    "CHINESE POST"  ,   CHINPOST,
    "MENU CODE"     ,   MENU_CODE,
    "UNKNOWN CODE"  ,   -1
};



/************************************************************************/
/*      extern                                                          */
/************************************************************************/
/************************************************************************/
/*      Prototype                                                       */
/************************************************************************/
int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more);
int menu(char *menu_text, struct menus *tab, int max, int goback);
int read_string_timed(char *buffer, int len, char eol, int time);
int send_recv_packet(char *spacket, char *rpacket, KParam * kp_your, int *len);
int check_recv_packet(char *packet, int *plen);
int create_packet(char *packet, int seq, char type, KParam * kp, char *msg);
int kermit_send(char *filename);
int send_slt_prot(char *filename);
int send_plain(char *filename);
void set_time_and_date(void);
int read_record(char *filename, char *record, long rec_no, int rec_size);
long get_max_records(char *filename, int rec_size);
int write_record(char *filename, char *record);
int main_menu(void);
int system_menu(void);
void sel_protocol(int *protocol);
int com_menu(void);
void sel_databits(int *databits);
void sel_stopbits(int *stopbits);
void sel_parity(int *parity);
void sel_baudrate(int *baudrate);
void display_mem(void);
void set_display(void);
void barcode_menu(int *bcode);
int getstring(char *string, int *code_id, int x, int y);
int code_id_to_index(int code_id);
int scan_labels(void);
int scroll_dbase(void);
void main(void);

int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more)
{
	int index;
	int selected;
	int key;
	index = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, GO_DOWN);
		else if (index == max)
			printf("%c%s ", GO_UP, menu_text);
		else
			printf("%c%s%c", GO_UP, menu_text, GO_DOWN);

		selected = 	(more && (tab[index].value & (*value))) ||
					(!more && (tab[index].value == (*value)));
		if (tab[index].value==-1)
			selected = 0;

		gotoxy(0, 1);
		printf("%c%s", selected ? '*' : ' ', tab[index].item);

		while ((key=getchar())==EOF)
			idle();
		switch (key) {

		case UP_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ENT_KEY:
			if (tab[index].value==-1)
				return(-1);
			if (more)
				(*value) ^= tab[index].value;
			else {
				(*value) = tab[index].value;
				return (index);
			}

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}

int menu(char *menu_text, struct menus *tab, int max, int goback)
{
	int index;
	int key;
	index = 0;
	putchar('\f');				/* clear screen */

	for (;;) {
		gotoxy(0, 0);

		if (index == 0)
			printf(" %s%c", menu_text, GO_DOWN);
		else if (index == max)
			printf("%c%s ", GO_UP, menu_text);
		else
			printf("%c%s%c", GO_UP, menu_text, GO_DOWN);

		gotoxy(0, 1);
		printf("%s", tab[index].item);

		while ((key=getchar())==EOF)
			idle();
		switch (key) {

		case UP_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case ENT_KEY:
			if (((index==0)||(index==max))&&goback)
				return (-1);
			else
				return (index);

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}



int read_string_timed(char *buffer, int len, char eol, int time)
{
	int count;
	int ch;
	count = 0;
	while (count < len) {
		if ((ch = getcom(50 * time)) == -1)
			return (NOT_OK);
		if ((buffer[count++] = ch) == eol)
			break;
	}
	buffer[count] = '\0';
	return (0);
}



int send_recv_packet(char *spacket, char *rpacket, KParam * kp_your, int *len)
{
	int timeout, start;
	int plen, count;

	timeout = 0;
	do {
		while (timeout < MAX_TIMEOUT) {
			plen = strlen(spacket);
			for (count = 0; count < plen; count++)
				putcom(spacket[count]);
			if (read_string_timed(rpacket, MAX_BUF, kp_your->eol, kp_your->time) == 0)
				break;
			timeout++;
		}
		if (timeout >= MAX_TIMEOUT)
			return (-3);
		start = check_recv_packet(rpacket, len);
		switch (*(rpacket + start + 2)) {

		case NAK_PACKET:
			timeout = 0;
			start = -4;
			break;
		case ERROR_PACKET:
			start = -5;
			return (start);

		}
	} while (start < 0);
	return (start);
}

int check_recv_packet(char *packet, int *plen)
/* returns where the packet starts, -1 if no SOH, -2 if bad checksum */
{
	int len, csum, count;
	char *ptr;
	if ((ptr = strchr(packet, SOH)) == NULL)
		return (-1);
	len = unchar(*(++ptr));
	for (count = 0, csum = 0; count < len; count++)
		csum += (*(ptr + count));
	if (check(csum) != (*(ptr + len)))
		return (-2);
	*plen = len;
	return (ptr - packet);
}

int create_packet(char *packet, int seq, char type, KParam * kp, char *msg)
{
	int blen, len, count, csum, result;
	result = 0;
	switch (type) {
	case DATA_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				DATA_PACKET
			);
		count = 0;
		blen = 0;
		result = 0;
		len = strlen(packet);
		while (blen < (kp->max_len - 3)) {
			if (msg[count] == '\0')
				break;
			if ((iscntrl(msg[count])) ||
				(kp->qctl == msg[count])) {
				packet[len++] = kp->qctl;
				packet[len++] = ctl(msg[count++]);
				blen += 2;
				result++;
			} else {
				packet[len++] = msg[count++];
				blen++;
			}
		};
		packet[len] = '\0';
		if (blen < kp->max_len - 3)
			result = 0;
		break;
	case ACK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				ACK_PACKET
			);
		break;
	case NAK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				NAK_PACKET
			);
		break;
	case SINIT_PACKET:
		sprintf(packet, "%c %c%c%c%c%c%c%c%c",
				SOH,
				tochar(seq),
				SINIT_PACKET,
				tochar(kp->max_len),
				tochar(kp->time),
				tochar(kp->npad),
				ctl(kp->padc),
				tochar(kp->eol),
				kp->qctl
			);
		break;
	case BREAK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				BREAK_PACKET
			);
		break;
	case FILE_PACKET:
		sprintf(packet, "%c %c%c%s",
				SOH,
				tochar(seq),
				FILE_PACKET,
				msg
			);
		break;
	case EOF_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				EOF_PACKET
			);
		break;
	case ERROR_PACKET:
		sprintf(packet, "%c %c%c%s",
				SOH,
				tochar(seq),
				ERROR_PACKET,
				msg
			);
	}
	len = strlen(packet);
	packet[1] = tochar(len - 1);
	for (count = 1, csum = 0; count < len; count++)
		csum += packet[count];
	packet[len++] = check(csum);
	packet[len++] = kp->eol;
	packet[len] = '\0';
	return (result);
}

int kermit_send(char *filename)
{
	char packet_buf[MAX_BUF + 10];
	char recv_buf[MAX_BUF + 10];
	char file_buf[MAX_BUF + 10];
	char *ptr;
	int len, start, correction;
	char seq;
	KParam my_side, your_side;
	FILE *fp;
	long where;

	if ((fp = fopen(filename, "rb")) == NULL) {		/* open file to send */
		return (1);
	}
	if (comopen(port) != 0) {
		fclose(fp);
		return (2);
	}
	resetreceive();
	seq = 0;
	my_side.max_len = MAXL;		/* set my defaults */
	my_side.time = DEFAULT_TIME;
	my_side.npad = DEFAULT_NPAD;
	my_side.padc = DEFAULT_PADC;
	my_side.eol = DEFAULT_EOL;
	my_side.qctl = DEFAULT_QCTL;
	your_side.max_len = DEFAULT_MAXL;	/* set the others side defaults */
	your_side.time = DEFAULT_TIME;
	your_side.npad = DEFAULT_NPAD;
	your_side.padc = DEFAULT_PADC;
	your_side.eol = DEFAULT_EOL;
	your_side.qctl = DEFAULT_QCTL;
	create_packet(packet_buf, seq, SINIT_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	ptr = recv_buf;
	ptr += (start + 3);
	if (len > 3)
		my_side.max_len = unchar(*(ptr++));
	if (len > 4)
		my_side.time = unchar(*(ptr++));
	if (len > 5)
		my_side.npad = unchar(*(ptr++));
	if (len > 6)
		my_side.padc = ctl(*(ptr++));
	if (len > 7)
		my_side.eol = unchar(*(ptr++));
	if (len > 8)
		my_side.qctl = *ptr;
	if (my_side.max_len > your_side.max_len)
		my_side.max_len = your_side.max_len;
	if (my_side.time > your_side.time)
		my_side.time = your_side.time;
	seq++;
	create_packet(packet_buf, seq, FILE_PACKET, &my_side, filename);
	if (send_recv_packet(packet_buf, recv_buf, &your_side, &len) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	seq++;
	while (!feof(fp)) {
		len = fread(file_buf, 1, (my_side.max_len - 3), fp);
		file_buf[len] = '\0';
		correction = create_packet(packet_buf, seq, DATA_PACKET, &my_side, file_buf);
		if (correction) {
			where = ftell(fp);
			where += my_side.max_len - 3 - len;
			fseek(fp, where - correction, SEEK_SET);
		}
		if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
			fclose(fp);
			comclose(port);
			return (start);
		}
		seq = (++seq) % 64;
	}
	create_packet(packet_buf, seq, EOF_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	seq = (++seq) % 64;
	create_packet(packet_buf, seq, BREAK_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	fclose(fp);
	comclose(port);
	return (0);
}

int send_slt_prot(char *filename)
{
	FILE *fp;
	int ch;
	int retry;
	unsigned int lrc;
	int count, len;
	int key;
	char buf[SLT_MAX_BUF + 1];

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	resetreceive();
	if ((fp = fopen(filename, "rb")) == NULL)
		return (ERR_OPEN_FILE);
	fseek(fp, 0L, SEEK_END);
	if (ftell(fp) == 0L) {
		fclose(fp);
		comclose(port);
		return (ERR_READ_FILE);
	}
	fseek(fp, 0L, SEEK_SET);
	buf[0] = STX;
	len = 1;
	while (len < SLT_MAX_BUF) {
		while ((ch = getcom(TIMEOUT)) != STX) {
			if ((key=getchar())!=EOF)
				if (key == DOWN_KEY) {
					fclose(fp);
					comclose(port);
					return (ERR_USER_BREAK);
				}
		}
		retry = 0;
		while ((ch = getcom(TIMEOUT)) != EDATA) {
			if ((key=getchar())!=EOF)
				if (key == DOWN_KEY) {
					fclose(fp);
					comclose(port);
					return (ERR_USER_BREAK);
				}
			if (ch == EOF) {
				if (retry++ > RETRIES) {
					fclose(fp);
					comclose(port);
					return (NOT_OK);
				}
				continue;
			}
			buf[len++] = ch;
		}
		lrc = 0;
		for (count = 1; count < len - 1; count++)
			lrc ^= buf[count];
		if (lrc == buf[len - 1])
			break;
		else
			len = 0;
	}
	if (len >= SLT_MAX_BUF) {
		fclose(fp);
		comclose(port);
		return (NOT_OK);
	}
	if (!(buf[2] & ENQ_DOWNLOAD)) {
		putcom(EOT);
		fclose(fp);
		comclose(port);
		return (ERR_HOST_REQ);
	}
	while (!feof(fp)) {
		len = 0;
		buf[len++] = STX;
		buf[len++] = 1;
		buf[len++] = DATA_RECORD;
		len += fread(buf + len, 1, MAX_DATA, fp);
		lrc = 0;
		for (count = 1; count < len; count++)
			lrc ^= buf[count];
		lrc %= 247;
		buf[len++] = lrc;
		buf[len++] = EDATA;
		for (retry = RETRIES; retry > 0; retry--) {
			for (count = 0; count < len; count++)
				putcom(buf[count]);
			ch = getcom(TIMEOUT);
			if (ch == ACK)
				break;
			if (ch == EOT) {
				fclose(fp);
				comclose(port);
				return (ERR_EOT_HOST);
			}
		}
		if (retry == 0) {
			fclose(fp);
			comclose(port);
			return (NOT_OK);
		}
	}
	fclose(fp);
	for (retry = RETRIES; retry > 0; retry--) {
		putcom(EOT);
		ch = getcom(TIMEOUT);
		if (ch == EDATA)
			break;
	}
	if (retry == 0) {
		comclose(port);
		return (NOT_OK);
	} else {
		comclose(port);
		return (0);
	}
}

int send_plain(char *filename)
{
	long count, max;
	int tx_count;

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	if ((max = get_max_records(filename, REC_SIZE)) <= 0L) {
		comclose(port);
		return (ERR_OPEN_FILE);
	}
	for (count = 0L; count < max; count++) {
		if (read_record(filename, record, count, REC_SIZE) == NOT_OK) {
			comclose(port);
			return (NOT_OK);
		}
		for (tx_count = 0; tx_count < REC_SIZE; tx_count++) {
			putcom(record[tx_count]);
			printf("%c", record[tx_count]);

		}
		delay(10);
	}
	comclose(port);
	return (OK);
}

int receive_char(unsigned char c1, unsigned char c2, int *pc)
{   int timeoutcount;

	*pc = EOF;
    timeoutcount = 0;

    while ( (*pc!=c1) && (*pc!=c2) )
    {   if (timeoutcount > 200) // 20 sec total time out
        {   printf("\fTime out of\ntransmission");
            resetkey();
            comclose(port);
            while (getchar()==EOF)
				idle();
			return(NOT_OK);
		}
		idle();
		if (getchar() == DOWN_KEY)
		{   printf("\fUser break of\ntransmission");
			resetkey();
			comclose(port);
			while (getchar()==EOF)
				idle();
			return(ERR_USER_BREAK);
		}
		*pc = getcom(5);   // 100 ms time out per loop iteration
		timeoutcount++;
	}

	return(OK);
}

int send_ack_record(int length)
{
	int tx_count;
	int recv;
	int nakcount;
	int res;

	recv = NAK;
	nakcount = 0;
	delay(1);   // 20 ms delay, so that PC can process record

	while ( (recv==NAK) && (nakcount++<=3) )
	{   for (tx_count = 0; tx_count < length; tx_count++)
		{   putcom(record[tx_count]);
			printf("%c", record[tx_count]);
		}
		resetreceive();
		res = receive_char(ACK, NAK, &recv);
		if (res != OK)
			return(res);
	}

	if ( (nakcount>3) && (recv==NAK) )
	{   printf("\fToo many NAKs");
		comclose(port);
		resetkey();
		while (getchar()==EOF)
			idle();
		return(NOT_OK);
	}

	return(OK);
}

int send_ack_nak(char *filename)
{
	long count, max;
	int recv;
	int res;

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	if ((max = get_max_records(filename, REC_SIZE)) <= 0L) {
		comclose(port);
		return (ERR_OPEN_FILE);
	}

	// first receive ENQ character
	resetreceive();
	res = receive_char(ENQ, 0, &recv);
	if (res != OK)
		return(res);

	printf("\n");
	for (count = 0L; count < max; count++) {
		if (read_record(filename, record, count, REC_SIZE) == NOT_OK) {
			comclose(port);
			return (NOT_OK);
		}
		if ( (recv=send_ack_record(REC_SIZE)) != OK )
			return(recv);
	}

	printf("EOT");
	delay(2);
	putcom(EOT);
	comclose(port);
	delay(50);          // time to show EOT on screen
	return (OK);
}


int read_record(char *filename, char *record, long rec_no, int rec_size)
{
	FILE *fp;

	if ((fp = fopen(filename, "rb")) == NULL) {
		return (NOT_OK);
	}
	fseek(fp, rec_no * rec_size, SEEK_SET);
	if (fread(record, 1, rec_size, fp) != rec_size) {
		fclose(fp);
		return (NOT_OK);
	}
	fclose(fp);
	return (0);
}

long get_max_records(char *filename, int rec_size)
{
	FILE *fp;
	long max;
	if ((fp = fopen(filename, "rb")) == NULL) {
		return (0L);
	}
	fseek(fp, 0L, SEEK_END);
	max = ftell(fp);
	fclose(fp);
	return (max / rec_size);
}

int write_record(char *filename, char *record)
{
	FILE *fp;
	int rec_size;

	if ((fp = fopen(filename, "r+b")) == NULL)
		if ((fp = fopen(filename, "wb")) == NULL) {
			return (NOT_OK);
		}
	rec_size = strlen(record);
	fseek(fp, 0L, SEEK_END);
	if (fwrite(record, 1, rec_size, fp) != rec_size) {
		fclose(fp);
		return (NOT_OK);
	}
	fclose(fp);
	return (0);
}


int main_menu(void)
{
	struct menus mainm[] =
	{
		"1 Scan labels  ",
		"2 Scroll data  ",
		"3 System menu  ",
		"4 Delete data  ",
		"5 Transmit data",
		"6 Version      "
	};
	return (menu("  MAIN MENU ", mainm, 5, FALSE));
	#define MAIN_SCAN		0
	#define MAIN_SCROLL 	1
	#define MAIN_SYSTEM		2
	#define MAIN_DELETE		3
	#define MAIN_XMIT		4
	#define MAIN_VERSION	5
}


int system_menu(void)
{
	struct menus sysm[] =
	{
		"  (back)      ",
		"1 Com port    ",
		"2 Protocol    ",
		"3 Barcodes    ",
		"4 Memory      ",
		"  (back)      "
	};
	return (menu("SYSTEM MENU ", sysm, 5, TRUE));
	#define SYS_COM			1
	#define SYS_PROTOCOL	2
	#define SYS_BARCODES	3
	#define SYS_MEMORY		4
}


void sel_protocol(int *protocol)
{
	struct smenus protm[] =
	{
		"  (back)     ", -1,
		"No protocol  ", 1,
		"NetO protocol", 2,
		"RCV protocol ", 3,
		"X-Modem      ", 4,
		"Kermit       ", 5,
		"ACK/NAK      ", 6,
		"  (back)     ", -1
	};
	sel_menu(" PROTOCOL  ", protm, 7, protocol, 0);
}

int com_menu(void)
{
	struct menus comm[] =
	{
		"   (back)     ",
		"1 Baudrate    ",
		"2 Parity      ",
		"3 Databits    ",
		"4 Stopbits    ",
		"   (back)     "
	};
	return (menu("COMPORT MENU", comm, 5, TRUE));
	#define COM_BAUD	1
	#define COM_PARITY	2
	#define COM_DATA	3
	#define COM_STOP	4
}

void sel_databits(int *databits)
{
	struct smenus databm[] =
	{
		"   (back)     ", -1,
		"7 databits    ", 7,
		"8 databits    ", 8,
		"   (back)     ", -1
	};
	sel_menu(" DATABITS  ", databm, 3, databits, 0);
	switch (*databits) {
	case 7:
		systemsetting("L0");
		break;
	case 8:
		systemsetting("L1");
		break;
	}
}

void sel_stopbits(int *stopbits)
{
	struct smenus stopbm[] =
	{
		"   (back)     ", -1,
		"1 stopbit     ", 1,
		"2 stopbits    ", 2,
		"   (back)     ", -1
	};
	sel_menu(" STOPBITS  ", stopbm, 3, stopbits, 0);
	switch (*stopbits) {
	case 1:
		systemsetting("L5");
		break;
	case 2:
		systemsetting("L6");
		break;
	}
}

void sel_parity(int *parity)
{
	struct smenus paritym[] =
	{
		"   (back)     ", -1,
		"NONE         ", NONE,
		"EVEN         ", EVEN,
		"ODD          ", ODD,
		"   (back)     ", -1
	};
	sel_menu("   PARITY  ", paritym, 4, parity, 0);
	switch (*parity) {
	case NONE:
		systemsetting("L2");
		break;
	case EVEN:
		systemsetting("L3");
		break;
	case ODD:
		systemsetting("L4");
		break;
	}
}

void sel_baudrate(int *baudrate)
{
	struct smenus baudm[] =
	{
		"   (back)     ", -1,
		"38400  Bps    ", 3840,
		"19200  Bps    ", 1920,
		" 9600  Bps    ", 960,
		" 4800  Bps    ", 480,
		" 2400  Bps    ", 240,
		" 1200  Bps    ", 120,
		"   (back)     ", -1

	};
	sel_menu("  BAUDRATE  ", baudm, 7, baudrate, 0);
	switch (*baudrate) {
	case 3840:
		systemsetting("K8");
		break;
	case 1920:
		systemsetting("K7");
		break;
	case 960:
		systemsetting("K6");
		break;
	case 480:
		systemsetting("K5");
		break;
	case 240:
		systemsetting("K4");
		break;
	case 120:
		systemsetting("K3");
		break;
	}

}

void display_mem(void)
{
	long mem_size;
	mem_size = coreleft();
	printf("\fMEMORY LEFT:\n%ld Bytes", mem_size);
	delay(150);					/* 3 seconds */
}

void barcode_menu(int *bcode)
{
	char options[20 + 1];
	struct smenus barcodem[] =
	{
		"   (back)     ", -1,
		"Code39        ", ID_CD39,
		"EAN           ", ID_EAN,
		"UPC           ", ID_UPC,
		"Interl. 2of5  ", ID_I2O5,
		"Indust. 2of5  ", ID_D2O5,
		"Codabar       ", ID_NW7,
		"Code93        ", ID_CD93,
		"Code128       ", ID_CD128,
		"MSI           ", ID_MSI,
		"Telepen       ", ID_TELEPEN,
		"UK/Plessey    ", ID_UK,
		"All addons    ", ID_ADDONS,
		"IATA code     ", ID_IATA,
		"Scode         ", ID_SCODE,
		"All codes     ", ID_ALLCODES,
		"   (back)     ", -1
	};
	sel_menu("BARCODE MENU", barcodem, 16, bcode, 1);
	strcpy(options, "B0");
	if (*bcode & ID_CD39)
		strcat(options, "B2");
	if (*bcode & ID_EAN)
		strcat(options, "R4");
	if (*bcode & ID_UPC)
		strcat(options, "R1");
	if (*bcode & ID_I2O5)
		strcat(options, "R8");
	if (*bcode & ID_D2O5)
		strcat(options, "R7");
	if (*bcode & ID_NW7)
		strcat(options, "B3");
	if (*bcode & ID_CD93)
		strcat(options, "B5");
	if (*bcode & ID_CD128)
		strcat(options, "B6");
	if (*bcode & ID_MSI)
		strcat(options, "B7");
	if (*bcode & ID_TELEPEN)
		strcat(options, "B9");
	if (*bcode & ID_UK)
		strcat(options, "B1");
	if (*bcode & ID_IATA)
		strcat(options, "B4");
	if (*bcode & ID_SCODE)
		strcat(options, "R9");
	if (*bcode & ID_ADDONS)
		strcat(options, "R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX

	if (*bcode & ID_ALLCODES)
	{   strcpy(options, "B0A0");       // no codes, all codes except add-ons
		strcat(options, "R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX
		systemsetting(options);
		strcpy(options, "JSJZ");       // Chinese post, Laetus
		strcat(options, "D4D7DB");     // Full ASCII, Ital Pharma + leading A
		strcat(options, "YCOG");       // Menu labels, EAN128 if possible
		strcat(options, "IK4V");       // ISBN ISSN if possible
		systemsetting(options);
		return;
	}
	systemsetting(options);

}


int getstring(char* string, int *code_id, int x, int y)
{
	struct barcode code;
	int c;

	code.text = string;
	strcpy(string,"");
	code.min = 1;
	code.max = BARCODE;
	scannerpower(ON, 50*6);
	resetkey();

	for (;;)
	{
		if (!scanneroff())
		{	if (readbarcode(&code)==OK)
			{   scannerpower(OFF, 0);
				goodreadled(ORANGE, 10);
				sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
				gotoxy(x, y);
				if (code.length >= (16 - x))
					printf("%s", string + (code.length - (16 - x)));
				else
					printf("%s", string);
				cursor(OFF);
				*code_id = code.id;
				return (SCAN);
			}
		}
		else
			scannerpower(ON,50*6);

		if ((c=getchar())!=EOF)
		{	switch(c)
			{
			case ENT_KEY:	sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
							continue;
							break;
			case UP_KEY:	sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
							continue;
							break;
			case DOWN_KEY:	sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
							scannerpower(OFF, 0);
							return (KEYS);
							break;
			default:		continue;
			}
		}
		idle();
	}
}


int code_id_to_index(int code_id)
{   int index;
	int value;

	index = -1;
	value = 0;

	do
	{   index++;
		value = symblglist[index].value;
	}
	while( (value!=code_id) && (value!=(-1)) );
	return (index);
}


int scan_labels(void)
{
	char barcode[BARCODE + 1];
	char symbology[SYMBOLOGY + 1];
	int code_id;
	int result;
	cursor(NOWRAP);
	printf("\f");
	gotoxy(0,1);
	printf("Scan barcode...");
	for (;;) {
		gotoxy(0,0);
		result = getstring(barcode, &code_id, 0, 0);
		if (result == KEYS)
			return(OK);

		printf("\f%-16.16s", barcode);
		gotoxy(0,1);
		code_id = code_id_to_index(code_id);
		sprintf(symbology,"%-*.*s", SYMBOLOGY, SYMBOLOGY, symblglist[code_id].item);
		printf("%-16.16s", symbology);

		sprintf(record, "%-*.*s,%-*.*s\r\n",
				BARCODE,   BARCODE,   barcode,
				SYMBOLOGY, SYMBOLOGY, symbology);

		if (write_record(DBASE_NAME, record) == NOT_OK) {
			printf("\fERROR:\nwriting file");
			while (getchar()==EOF)
				idle();
			return (NOT_OK);
		}
	}
}

int scroll_dbase(void)
{
	char barcode[BARCODE + 1];
	int count, max;
	int key;

	cursor(NOWRAP);
	if ((max = get_max_records(DBASE_NAME, REC_SIZE)) == 0L)
	{   printf("\fDATABASE EMPTY\nPress any key");
		while (getchar()==EOF)
			idle();
		printf("\f");
		return (0);
	}
	if (max == NOT_OK)
	{   printf("\fERROR:\nreading file");
		while (getchar()==EOF)
			idle();
		printf("\f");
		return (NOT_OK);
	}

	count = 0;
	putchar('\f');
	for (;;)
	{	gotoxy(0, 0);
		if (count == 0 && max == 1)
			printf("  Scroll Data   ");
		else if (count == 0 && max > 1)
			printf("  Scroll Data %c", GO_DOWN);
		else if (count == (max - 1))
			printf("%c Scroll Data  ", GO_UP);
		else
			printf("%c Scroll Data %c", GO_UP, GO_DOWN);
		if (read_record(DBASE_NAME, record, count, REC_SIZE) == NOT_OK)
		{   printf("\fERROR:\nreading FILE");
			while (getchar()==EOF)
				idle();
			printf("\f");
			return (NOT_OK);
		}

		strncpy(barcode, record, 16);
		barcode[16] = '\0';

		gotoxy(0, 1);
		printf("%-16.16s", barcode);

		while ((key=getchar())==EOF)
			idle();
		if (key == ENT_KEY)
			break;
		switch (key)
		{
		case UP_KEY:
			if (count > 0)
				count--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case DOWN_KEY:
			if (count < (max - 1))
				count++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}

	printf("\f");
	return (0);
}

void main(void)
{
	int test;
	int res;
	int key;
	char osversion[9];

	port = COM0;				/* direct Cable is default */
	baudrate = 960;
	parity = NONE;
	stopbits = 1;
	databits = 8;
	protocol = 1;				/*no protocol default */
	s_code  = ID_CD39|ID_EAN|ID_UPC|ID_I2O5|ID_D2O5|ID_NW7;
	s_code |= ID_CD93|ID_CD128|ID_MSI|ID_TELEPEN|ID_UK;
	s_code |= ID_IATA|ID_SCODE;
	systemsetting("U2B0");      // default + disable all codes
	systemsetting("B2R4R1R8R7B3");  // CD39,EAN,UPC,I2O5,,D2O5,NW7 enabled
	systemsetting("B5B6B7B9B1");    // CD93,CD128,MSI,TELEPEN,UK/PLESSEY enabled
	systemsetting("B4R9");          // IATA, SCODE enabled, menu + addons not enabled
	cursor(NOWRAP);
	for (;;) {
		test = main_menu();
		if (test == MAIN_SCAN) {
			scan_labels();
			continue;
		}
		if (test == MAIN_SCROLL) {
			scroll_dbase();
			continue;
		}
		if (test == MAIN_DELETE) {
			printf("\fDelete DBASE ?\n%c = YES\n%c = NO", GO_UP, GO_DOWN);

			while ((key=getchar())==EOF)
				idle();
			if (key == UP_KEY)
				remove(DBASE_NAME);
			continue;
		}
		if (test == MAIN_XMIT) {
			printf("\fTransmit data\nPress any key");
			while (getchar()==EOF)
				idle();
			printf("\fTransmitting..");
			switch (protocol) {
			case 1:
				res = send_plain(DBASE_NAME);
				break;
			case 2:
			case 4:
				if (comopen(port) != 0) {
					res = 2;
					break;
				}
				if (protocol == 2)
					res = neto_transmit((char(*)[12+1])DBASE_NAME, 1, "123456", DOWN_KEY, 3);
				else
					res = xmodemtransmit(DBASE_NAME);
				comclose(port);
				break;
			case 3:
				res = send_slt_prot(DBASE_NAME);
				break;
			case 5:
				res = kermit_send(DBASE_NAME);
				break;
			case 6:
				res = send_ack_nak(DBASE_NAME);
				break;
			}
			if (res == OK)
				printf("\ftransmission OK\npress any key");
			else
				printf("\fError = %d\npress any key", res);
			while (getchar()==EOF)
				idle();
			continue;
		}
		if (test == MAIN_VERSION) {
			printf("\fAppl = %s", VERSION);
			version_os(osversion);
			printf("\nO/S  = %s", osversion);
			while (getchar()==EOF)
				idle();
			continue;
		}
		if (test == MAIN_SYSTEM) {
			for (;;) {
				if ((test = system_menu()) == -1)
					break;
				if (test == SYS_COM) {
					for (;;) {
						if ((test = com_menu()) == -1)
							break;
						if (test == COM_BAUD) {
							sel_baudrate(&baudrate);
							continue;
						}
						if (test == COM_PARITY) {
							sel_parity(&parity);
							continue;
						}
						if (test == COM_DATA) {
							sel_databits(&databits);
							continue;
						}
						if (test == COM_STOP) {
							sel_stopbits(&stopbits);
							continue;
						}
					}
					continue;
				}
				if (test == SYS_PROTOCOL) {
					sel_protocol(&protocol);
					continue;
				}
				if (test == SYS_MEMORY) {
					display_mem();
					continue;
				}
				if (test == SYS_BARCODES) {
					barcode_menu(&s_code);
					continue;
				}
			}
		}
	}
}
