//  #####  ######  ####        #####   ####### ##   ##  ##### 
// ##   ##  ##  ##  ##          ## ###  ##   # ### ### ##   ##
// ##   ##  ##  ##  ##          ##  ##  ##     ####### ##   ##
// ##   ##  #####   ##          ##  ##  ####   ## # ## ##   ##
// ##   ##  ##      ##          ##  ##  ##     ##   ## ##   ##
// ##   ##  ##      ##  ##      ## ###  ##   # ##   ## ##   ##
//  #####  ###     #######     #####   ####### ##   ##  ##### 
//                                                                        
// Opticon OPL972x demo                   
// Scan labels and scroll database with menus                             
// Transmit scanned data with multiple protocols, FREE, NetO     
//    


//************************************************************************
// Include                                                         
//************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lib.h"
#include "database.h"
#include "input.h"
#include "menu.h"

//************************************************************************
// Defines
//************************************************************************
#define VERSIONSUFFIX  "33621"

// Database name
#define DBASE_NAME		"DATA.FIL"

// Database record 
#define SZ_BARCODE		30
#define SZ_SYMBOLOGY	16
#define SZ_TIME			(2+1+2+1+2)
#define SZ_DATE			(4+1+2+1+2)
#define SZ_RECORD		(SZ_BARCODE+1+SZ_SYMBOLOGY+1+SZ_TIME+1+SZ_DATE+1+1)

// barcode menu defines
#define ID_CODE39		0x0001	// bit 0
#define ID_EAN 			0x0002	// bit 1
#define ID_UPC 			0x0004	// bit 2
#define ID_I2OF5		0x0008	// bit 3
#define ID_D2OF5    	0x0010	// bit 4
#define ID_CODABAR		0x0020	// bit 5
#define ID_CODE93		0x0040	// bit 6
#define ID_CODE128		0x0080	// bit 7
#define ID_MSI_PLESSEY 	0x0100	// bit 8
#define ID_TELEPEN		0x0200	// bit 9
#define ID_UK_PLESSEY	0x0400	// bit 10
#define ID_IATA			0x0800	// bit 11
#define ID_SCODE		0x1000	// bit 12
#define ID_RSS			0x2000	// bit 13
#define ID_ADDONS		0x4000	// bit 14

#define OFF_ADDON_2 	0x1E	// Index position offset
#define OFF_ADDON_2_128	0x24
#define OFF_ADDON_5 	0x25
#define OFF_ADDON_5_128	0x2B
#define OFF_MNU_CDE		0x2C

//
// Special Terminal specific defines
//
//Load only if needed
#ifndef DISP_WIDTH 
	#if PHL 
		// PHL1700-10 and PHL1700-20
		#define DISP_WIDTH  96
	    #define DISP_HEIGHT 48
	#elif PHL1000	
		// PHL1300
		#define DISP_WIDTH  96
		// Change height to 48 when indicators need to be on (see cursor() function)
	    #define DISP_HEIGHT 64 
	#elif PHL2700	
		// PHL2700
		#define DISP_WIDTH  128
	    #define DISP_HEIGHT 64
	#else 
		// OPL972X series 
		#define DISP_WIDTH  112
	    #define DISP_HEIGHT 64
	#endif
#endif

#define ID_STOPBITS_1	1
#define ID_STOPBITS_2	2

#define ID_PARITY_NONE	1
#define ID_PARITY_EVEN	2
#define ID_PARITY_ODD	3

#define ID_115200	1
#define ID_57600    2
#define ID_38400    3
#define ID_19200    4
#define ID_9600     5
#define ID_4800     6
#define ID_2400     7

#define ID_NO_PROTOCOL		1
#define ID_NETO_PROTOCOL	2


//************************************************************************
// Special types
//************************************************************************

// barcode record structure
typedef struct
{
	char barcode[ SZ_BARCODE + 1 ];
	char symbology[ SZ_SYMBOLOGY + 1 ];
	char time[ SZ_TIME + 1 ];
	char date[ SZ_DATE + 1 ];
}db_record;


//************************************************************************
// Globals
//************************************************************************
const char time_mask[] = "29:59:59";
const char date_mask[] = "39/19/2999/E";

const char back_string[]		= "(back)";

// return codes of the barcode type
const char jan13_string[] 		= "JAN 13";
const char ean13_string[] 		= "EAN 13";
const char upca_string[] 		= "UPC A";
const char jan8_string[] 		= "JAN 8";
const char ean8_string[] 		= "EAN 8";
const char upce_string[] 		= "UPC E";
const char instore13_string[] 	= "INSTORE 13";
const char instore8_string[] 	= "INSTORE 8";
const char code39_string[] 		= "CODE 39";
const char codabar_string[] 	= "CODABAR";

const char d2of5_string[] 		= "INDUST 2 OF 5";
const char i2of5_string[] 		= "INTERL 2 OF 5";
const char code93_string[] 		= "CODE 93";
const char code128_string[] 	= "CODE 128";
const char msi_string[] 		= "MSI PLESSEY";
const char iata_string[] 		= "IATA";
const char uk_string[] 			= "UK PLESSEY";
const char telepen_string[] 	= "TELEPEN";
const char m2of5_string[] 		= "MATRIX 2 OF 5";
const char chinpost_string[] 	= "CHINESE POST";
const char nw7_abc_string[] 	= "CODABAR ABC";
const char nw7_cx_string[] 		= "CODABAR CX";
const char scode_string[] 		= "SCODE";
const char trioptic_string[] 	= "TRI-OPTIC";
const char c39_fa_string[] 		= "CD 39 FULL ASCII";
const char c39_it_string[] 		= "ITALIAN PHARMA.";
const char unknown_string[] 	= "UNKNOWN CODE";
const char all_addon_string[]	= "All ADDONS";
const char ean_string[]			= "EAN";
const char upc_string[]			= "UPC";
const char rss14_string[] 		= "GS1-DATABAR";
const char rssL_string[] 		= "GS1-DATABAR LIM.";
const char rssE_string[] 		= "GS1-DATABAR EXP.";
const char jan13_2_string[]		= "JAN 13 + 2";
const char ean13_2_string[]		= "EAN 13 + 2";
const char upca_2_string[]		= "UPC A + 2";
const char jan8_2_string[]		= "JAN 8 + 2";
const char ean8_2_string[]		= "EAN 8 + 2";
const char upce_2_string[]		= "UPC E + 2";
const char code128_2_string[]	= "CODE 128 + 2";

const char jan13_5_string[]		= "JAN 13 + 5";
const char ean13_5_string[]		= "EAN 13 + 5";
const char upca_5_string[]		= "UPC A + 5";
const char jan8_5_string[]		= "JAN 8 + 5";
const char ean8_5_string[]		= "EAN 8 + 5";
const char upce_5_string[]		= "UPC E + 5";
const char code128_5_string[]	= "CODE 128 + 5";
const char ean128_string[] 		= "EAN 128";

const char menu_string[]		= "MENU CODE";



//
// use the sSelMenu menu structure
// for getting the return codes
const sSelMenu symblglist[] =
{                                 			/* index position */
	(char*)jan13_string, 	 	JAN13, 		/* 0x00 */
	(char*)ean13_string, 		EAN13,		/* 0x01 */
	(char*)upca_string,    		UPCA,		/* 0x02 */
	(char*)jan8_string, 		JAN8,		/* 0x03 */
	(char*)ean8_string, 		EAN8,		/* 0x04 */
	(char*)upce_string, 		UPCE,		/* 0x05 */
	(char*)instore13_string,   	INSTORE13,	/* 0x06 */
	(char*)instore8_string,		INSTORE8,	/* 0x07 */
	(char*)code39_string, 		CODE39,		/* 0x08 */
	(char*)codabar_string, 		CODABAR,	/* 0x09 */
	(char*)d2of5_string,		D2OF5,		/* 0x0A */
	(char*)i2of5_string,		I2OF5,		/* 0x0B */
	(char*)code93_string, 		CODE93,		/* 0x0C */
	(char*)code128_string,     	CODE128,	/* 0x0D */
	(char*)msi_string, 			MSI_PLESSEY,/* 0x0E */
	(char*)iata_string,			IATA,		/* 0x0F */
	(char*)uk_string,			UK_PLESSEY, /* 0x10 */
	(char*)telepen_string, 		TELEPEN,	/* 0x11 */
	(char*)m2of5_string,		M2OF5,		/* 0x12 */
	(char*)chinpost_string, 	CHINPOST,	/* 0x13 */
	(char*)nw7_abc_string,  	NW7_ABC,	/* 0x14 */
	(char*)nw7_cx_string,   	NW7_CX,		/* 0x15 */
	(char*)scode_string,   		SCODE,		/* 0x16 */
	(char*)trioptic_string,		TRIOPTIC,	/* 0x17 */
	(char*)c39_fa_string, 		C39_FA,		/* 0x18 */
	(char*)c39_it_string, 		C39_IT,		/* 0x19 */
	(char*)ean128_string, 		EAN128,		/* 0x1A */
	(char*)rss14_string,   		RSS_14,		/* 0x1B */
	(char*)rssL_string,   		RSS_LIMITED,/* 0x1C */
	(char*)rssE_string,   		RSS_EXPANDED,/* 0x1D */
	(char*)jan13_2_string,      JAN13_2,	/* 0 + OFF_ADDON_2 */
	(char*)ean13_2_string,		EAN13_2,	/* 1 + OFF_ADDON_2 */
	(char*)upca_2_string,		UPCA_2,     /* 2 + OFF_ADDON_2 */
	(char*)jan8_2_string,		JAN8_2,     /* 3 + OFF_ADDON_2 */
	(char*)ean8_2_string,		EAN8_2,     /* 4 + OFF_ADDON_2 */
	(char*)upce_2_string,		UPCE_2,     /* 5 + OFF_ADDON_2 */
	(char*)code128_2_string,	CODE128_2,  /* OFF_ADDON_2_128 */
	(char*)jan13_5_string,		JAN13_5,    /* 0 + OFF_ADDON_5 */
	(char*)ean13_5_string,		EAN13_5,    /* 1 + OFF_ADDON_5 */
	(char*)upca_5_string,		UPCA_5,     /* 2 + OFF_ADDON_5 */
	(char*)jan8_5_string,		JAN8_5,     /* 3 + OFF_ADDON_5 */
	(char*)ean8_5_string,		EAN8_5,     /* 4 + OFF_ADDON_5 */
	(char*)upce_5_string,		UPCE_5,     /* 5 + OFF_ADDON_5 */
	(char*)code128_5_string,	CODE128_5,  /* OFF_ADDON_5_128 */
	(char*)menu_string,			MENU_CODE,  /* OFF_MNU_CDE 	   */
	(char*)unknown_string ,   	-1
};

long lPort;		// Serial communication port
long lProtocol;	// The protocol for transmitting the data
long lBarcodes;	// The currently supported barcode types
long lBaudrate;	// The Baudrate
long lParity;	// Parity
long lDatabits;	// Databits
long lStopbits;	// Stopbits

int  nHasKeyboard;	// Does it have a full keyboard
int  nHasUsb;		// Does it have an USB connector (DCL15xx)

//************************************************************************
// Function implementation
//************************************************************************
void display_time_date( char* time_str, char* date_str )
{
//	printbar( 7, 0 , 1);
	
	setfont( MEDIUM_FONT, NULL );
	gotoxy(0, 0);
	printf("\tr  SET TIME & DATE \tr");
	gotoxy(0, 2);
	printf("%-18.18s", time_str);
	gotoxy(0, 3);
	printf("%-18.18s", date_str);
	gotoxy(0, 4);
	cursor( ON );
}

void change_character( int x, int y, char* time_str, char* date_str )
{
	char c;
	char mask_c;

	c = ( y == 2)? time_str[x]:date_str[x];
	mask_c = ( y == 2)? time_mask[x]:date_mask[x];

	if( ++c > mask_c )
		c = '0';

	if( y == 2 )
		time_str[x] = c;
	else
		date_str[x] = c;
}



void store_time_date( char* time_str, char* date_str )
{
	char temp[ 14 + 1] = {0};
	struct date dates = {0};
	struct time times = {0};

	strcpy(temp, time_str);
	temp[10]= '\0';
	times.ti_sec = (char)atoi(temp + 6);
	temp[5] = '\0';
	times.ti_min = (char)atoi(temp + 3);
	temp[2] = '\0';
	times.ti_hour = (char)atoi(temp);
	settime( &times );

	strcpy(temp, date_str);
	dates.da_year = atoi(temp + 6);
	temp[5] = '\0';
	dates.da_mon = (char)atoi(temp + 3);
	temp[2] = '\0';
	dates.da_day = (char)atoi(temp);
	setdate( &dates );
}

int is_date_string_ok( char* date_str )
{
	char temp[ 14 + 1] = {0};
	struct date dates = {0};

	strcpy(temp, date_str);
	temp[10]= '\0';
	dates.da_year = atoi(temp + 6);
	temp[5] = '\0';
	dates.da_mon = (char)atoi(temp + 3);
	temp[2] = '\0';
	dates.da_day = (char)atoi(temp);
	return( ((checkdate( &dates )==OK)?0:2) );
}

int is_time_string_ok( char* time_str )
{
	char temp[ 8 + 1] = {0};
	struct time times = {0};
	strcpy(temp, time_str);
	times.ti_sec = (char)atoi(temp + 6);
	temp[5] = '\0';
	times.ti_min = (char)atoi(temp + 3);
	temp[2] = '\0';
	times.ti_hour = (char)atoi(temp);
	return( ((checktime( &times )==OK)?0:1) );
}

int setcursorpos( int* x, int* y, int direction )
{
	if( (*x) == -1 )
	{
		(*y) = ( (*y) == 2)?3:2;
		(*x) = (((*y) == 2)? strlen( time_mask):strlen( date_mask))-1;
	}
	if( *y == 2 )
	{
		for(;;)
		{
			switch( time_mask[ (*x) ] )
			{
				case '\0':
					if( ++(*y) == 4 )
						(*y) = 2;
					(*x) = 0;
					break;
				case ':':
					(*x) += direction;
					break;
				default:
					gotoxy( (*x), (*y));
					return( 1 );
			}
		}
	}
	else if( *y == 3 )
	{
		for(;;)
		{
			switch( date_mask[ (*x) ] )
			{
				case '\0':
					if( ++(*y) == 4 )
						(*y) = 2;
					(*x) = 0;
					break;
				case '/':
					(*x) += direction;
					break;
				default:
					gotoxy( (*x), (*y));
					return( 1 );
			}
		}
	}
	return( 0 );
}


void DateTime( void )
{
	struct date dates = {0};
	struct time times = {0};
	char time_str[8 + 1] = {0};       /* HH:MM:SS    */
	char date_str[14 + 1] = {0};      /* YYYY/MM/DD */
	int x, y;
	int direction;
	gettime(&times);
	getdate(&dates);
	sprintf(time_str, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
	sprintf(date_str, "%02d/%02d/%04d End", dates.da_day, dates.da_mon, dates.da_year);

	putchar('\f');
	x = 0;
	y = 2;
	direction = 0;

	for (;;)
	{
		display_time_date( time_str, date_str );

		setcursorpos( &x, &y, direction );
		
		setfont( SMALL_FONT, NULL );
		gotoxy( 0, 7 );
		if('E' == (( y == 2)? time_str[x]:date_str[x]))
			printf("%c    BACK    %c", 0x11,  0x10);
		else
			printf("%c   SELECT   %c", 0x11,  0x10);
		setfont( MEDIUM_FONT, NULL );
		gotoxy( x, y );
		cursor( ON );
			
		switch( WaitForKey())
		{
			case Q2_KEY: /* to right*/
				x++;
				direction = 1;
				break;
			case Q1_KEY: /* to left */
				x--;
				direction = -1;
				break;
			case TRIGGER_KEY:
				if('E' == (( y == 2)? time_str[x]:date_str[x]))
				{
					cursor( OFF );
					setfont( LARGE_FONT, NULL );

					x = is_time_string_ok( time_str);
					x |= is_date_string_ok( date_str );
					if( x == 0 )
						store_time_date( time_str, date_str );
					return;
				}
				else
					change_character( x, y, time_str, date_str );
				break;
		}
	}
}


void set_stopbits( void )
{
	if( lStopbits == ID_STOPBITS_1 )
		systemsetting("L5");
	else
		systemsetting("L6");
}

void set_databits( void )
{
	if( lDatabits == 7 )
		systemsetting("L0");
	else
		systemsetting("L1");

}

void set_parity( void )
{
	switch( lParity )
	{
		case ID_PARITY_EVEN:
			systemsetting("L3");
			break;
		case ID_PARITY_ODD:
			systemsetting("L4");
			break;
		case ID_PARITY_NONE:
		default:
			systemsetting("L2");
			break;
	}
}

void set_baudrate( void )
{
	switch( lBaudrate )
	{
		case ID_115200:
			systemsetting("SZ");
			break;
		case ID_57600:
			systemsetting("K9");
			break;
		case ID_38400:
			systemsetting("K8");
			break;
		case ID_9600:
			systemsetting("K6");
			break;
		case ID_4800:
			systemsetting("K5");
			break;
		case ID_2400:
			systemsetting("K4");
			break;
		case ID_19200:
		default:
			systemsetting("K7");
			break;
	}
}

void set_barcodes( void )
{
	systemsetting("B0"); // disable all codes
	
    if( lBarcodes & ID_CODE39)
    	systemsetting("B2D7DB"); 	// CODE39, Ital. Ph. if possible, Leading A for Ital. Ph.
    if( lBarcodes & ID_EAN)
    	systemsetting("R4");		// EAN8 and 13
    if( lBarcodes & ID_UPC)
    	systemsetting("R1KP");		// UPC-A and UPC-E (disable UPC-E1)
	if( lBarcodes& ID_I2OF5)
		systemsetting("R8");		// Interleaved 2 of 5
    if( lBarcodes & ID_D2OF5)	
    	systemsetting("R7");		// Industrial 2 of 5
    if( lBarcodes & ID_CODABAR)
    	systemsetting("B3");		// Codabar
    if( lBarcodes & ID_CODE93)	
    	systemsetting("B5");		// Code93
    if( lBarcodes & ID_CODE128)
    	systemsetting( "B6OG[BL6"); 	// Code 128, Ean 128 if possible, Don't add EAN-128 AIM ID (]C1)
   	if( lBarcodes & ID_MSI_PLESSEY)
   		systemsetting("B7");		// MSI plessey
    if( lBarcodes & ID_TELEPEN)	
        systemsetting("B9JZ");		// Telepen, Tri-Optic
    if( lBarcodes & ID_UK_PLESSEY)
        systemsetting("B1");		// UK plessey
    if( lBarcodes & ID_IATA)
        systemsetting("B4");		// IATA
    if( lBarcodes & ID_SCODE)
        systemsetting("R9");		// SCode
    if( lBarcodes & ID_RSS)
        systemsetting("JXJYDR");	// GS1-Databar variants
	if( lBarcodes & ID_ADDONS)
        systemsetting("R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX

}

// Calculate the maximum amount of characters that fit on one line
static int GetMaxCharsXPos( void )
{
	unsigned int nHeight, nWidth;
	nHeight = nWidth = 0; // prevent warning toshiba compiler
	getfontsize( &nHeight, &nWidth);

	return (DISP_WIDTH / nWidth);
}

// Calculate the maximum amount of lines that fit on the display
static int GetMaxYLines( void )
{
	unsigned int nHeight, nWidth;
	nHeight = nWidth = 0; // prevent warning toshiba compiler
	getfontsize( &nHeight, &nWidth);

	return (DISP_HEIGHT / nHeight);
}

int code_id_to_index( int id )
{
	int index;

	switch( id )
	{
		case CODE128_2:
			return( OFF_ADDON_2_128 );
		case CODE128_5:
			return( OFF_ADDON_5_128 );
		case MENU_CODE:
			return( OFF_MNU_CDE );
	}

	index = (( id-1 ) & 0x3F);
	if( id & 0x80 ) 	/* addon 5 code */
		index += OFF_ADDON_5;
	else if( id & 0x40 )/* addon 2 code */
		index += OFF_ADDON_2;
	return index;
}



void diplay_input_data( db_record *db_rec )
{
	int nMaxX;
	
	nMaxX = GetMaxCharsXPos();
	
	gotoxy(0,1);
	printf("%-*.*s", nMaxX, nMaxX, db_rec->barcode );
	
	setfont( SMALL_FONT, NULL);
	gotoxy(0,4);
	printf("%-*.*s\n", nMaxX, nMaxX, db_rec->symbology);
	printf("%-*.*s\n", nMaxX, nMaxX, db_rec->date);
    printf("%-*.*s", nMaxX, nMaxX, db_rec->time);
	setfont( LARGE_FONT, NULL);
}

// Save the data into the database
void store_input_data( db_record *db_rec )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1 ];

	// Check if there is enough space available for storing the barcode data
	// We use 5000 because the OS (NetO) also need some memory
	if( coreleft() < 5000L )
	{							 
		printf("\fRam disk full\ndata not stored!\n\nPress a key");
		WaitForKey();
		return;
	}
	// Make the record to store
	sprintf(record, "%-*.*s,%-*.*s,%-*.*s,%-*.*s\r\n",
				SZ_BARCODE, SZ_BARCODE, db_rec->barcode,
				SZ_SYMBOLOGY, SZ_SYMBOLOGY, db_rec->symbology,
				SZ_TIME, SZ_TIME, db_rec->time,
				SZ_DATE, SZ_DATE, db_rec->date );

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		if( !CreateDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError create\nDatabase\nCode=%ld\nPress a key", GetDBErrorCode() );
			WaitForKey();
			return;
		}
	}
	
	if( !WriteRecord( &dbFile, record, WRITE_APPEND ))
	{
		printf("\fError write\nrecord\nCode=%ld\nPress a key", GetDBErrorCode() );
		WaitForKey();
		return;
	}
	CloseDatabase( &dbFile );

}

void ScanLabels( void )   
{
	static char 	barcode[ SZ_BARCODE + 1 ];
	static db_record db_rec;
    struct date 	dates = {0};
    struct time 	times = {0};
    int 			nCodeId = 0;
    int 			nResult = 0;

	memset( barcode, '\0', sizeof( barcode ));
	memset( &db_rec, '\0', sizeof( db_record ));

	cursor( NOWRAP);
	setfont( LARGE_FONT, NULL );
	putchar('\f');
	gotoxy( 0, 0);
	printf("Scan or type...");
	for(;;)
	{
		setfont( SMALL_FONT, NULL );
		gotoxy( 0, (GetMaxYLines() -1));
		printf("\tr%-*.*s\tr", GetMaxCharsXPos(), GetMaxCharsXPos(), "Q1=Back");
		setfont( LARGE_FONT, NULL );
		// Display some info first

		if( nHasKeyboard )
		{
			nResult = ScanOrKeyboardInputSymbol( barcode, 1, SZ_BARCODE, INPUT_ALL, 0, 1, GetMaxCharsXPos(), &nCodeId );
			resetkey();
		}
		else
		{
			nResult = ScanBarcodeSymbol( barcode, 1, GetMaxCharsXPos(), &nCodeId);

			if(nResult == ERROR)
			{
				nResult = getchar();
			}
		}

		if( nResult == CLR_KEY || nResult == Q1_KEY)
			return;

		if( nResult == Q2_KEY || strlen(barcode) == 0)
			continue;

		// fill the record structure
		sprintf( db_rec.barcode, "%-*.*s", SZ_BARCODE, SZ_BARCODE, barcode );

		if( nResult == KEYBOARD )
			sprintf( db_rec.symbology, "%-*.*s", SZ_SYMBOLOGY, SZ_SYMBOLOGY, "KEYBOARD");
		else
		{   
			nCodeId = code_id_to_index(nCodeId);
            sprintf( db_rec.symbology, "%-*.*s", SZ_SYMBOLOGY, SZ_SYMBOLOGY, symblglist[nCodeId].text);
        }
        gettime( &times );
        getdate( &dates );
        sprintf( db_rec.time, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
        sprintf( db_rec.date, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);

        // Store the input data into our database
        store_input_data( &db_rec );

        diplay_input_data( &db_rec );
        
	}
}

void display_scroll_data( db_record *db_rec, long curr, long max )
{
	int nMaxX;
	nMaxX = GetMaxCharsXPos();
	
	gotoxy(0,0);
	printf("\tr%-*.*s %03ld/%03ld\tr", nMaxX-8, nMaxX-8, "SCROLL", curr+1, max);

	diplay_input_data( db_rec );
}

void fill_record_struct( db_record *db_rec, char* record )
{
	int offset;
	memset( db_rec, '\0', sizeof( db_record ));
	offset = 0;
	memcpy( db_rec->barcode, record+offset, SZ_BARCODE );
	offset+=SZ_BARCODE+1;
	memcpy( db_rec->symbology, record+offset, SZ_SYMBOLOGY );
	offset+= SZ_SYMBOLOGY+1;
	memcpy( db_rec->time, record+offset, SZ_TIME );
	offset+= SZ_TIME+1;
	memcpy( db_rec->date, record+offset, SZ_DATE );
}

int get_record( db_record *db_rec, long *rec_nr, long *max_rec )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		printf("\fError open\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	*max_rec = GetTotalRecords( &dbFile );
	
	if( !GotoRecord( &dbFile, *rec_nr ))
	{
		printf("\fError goto\nrecord.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	if( !ReadCurrentRecord( &dbFile,  record ))
	{
		printf("\fError read\nrecord.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	CloseDatabase( &dbFile );
	
	fill_record_struct( db_rec, record );
	return TRUE;
}


void ScrollDatabase( void )
{
	long current;
	long max = 0L;
	static db_record db_rec;

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\nPress a key");
		WaitForKey();
		return;
	}

	current = 0L;
	if( !get_record( &db_rec, &current, &max )  )
		return;
	
	putchar('\f');
	for(;;)
	{
		display_scroll_data( &db_rec, current, max );
		switch( WaitForKey() )
		{
			case RIGHT_KEY:
			case Q2_KEY:
				if(++current >= max )
					current = max-1;
				break;
			case LEFT_KEY:
			case Q1_KEY:
				if( --current < 0L )
					current = 0L;
				break;
			case TRIGGER_KEY:
			case CLR_KEY:			
				return;
		}
		if( !get_record( &db_rec, &current, &max ))
			return;
	}
}

void SelectPort( void )
{
	int  nIndex;
	int  nMaxItems;
	
	sSelMenu mnuSelPort[] = {
		(char*)back_string, -1,
		"IrDA Interface",	COM1,
		"Cradle",			COM2,
		"USB",				COM0,
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " PORT MENU", mnuSelPort, (nHasUsb) ? 4 : 3, MENU_FIXED, &lPort); // only show USB option if it's available
}

void SelectBaudrate( void )
{
	int  nIndex;

	sSelMenu mnuSelBaudrate[] = {
		(char*)back_string, -1,
		"115200", 	ID_115200,
		"57600",	ID_57600,
		"38400",	ID_38400,
		"19200",	ID_19200,
		"9600",		ID_9600,
		"4800",		ID_4800,
		"2400",		ID_2400
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " BAUDRATE MENU", mnuSelBaudrate, sizeof( mnuSelBaudrate ) / sizeof( sSelMenu ), MENU_FIXED, &lBaudrate);
	set_baudrate();
}

void SelectParity( void )
{
	int  nIndex;

	sSelMenu mnuSelParity[] = {
		(char*)back_string, -1, 
		"None", 	ID_PARITY_NONE,
		"Even", 	ID_PARITY_EVEN,	
		"Odd", 		ID_PARITY_ODD	
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " PARITY MENU", mnuSelParity, sizeof( mnuSelParity ) / sizeof( sSelMenu ), MENU_FIXED, &lParity);
	set_parity();
}

void SelectDatabits( void )
{
	int  nIndex;

	sSelMenu mnuSelDatabits[] = {
		(char*)back_string, -1,
		"7 databits", 	7,
		"8 databits", 	8
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " DATABITS MENU", mnuSelDatabits, sizeof( mnuSelDatabits ) / sizeof( sSelMenu ), MENU_FIXED, &lDatabits);
	set_databits();
}

void SelectStopbits( void )
{
	int  nIndex;

	sSelMenu mnuSelStopbits[] = {
		(char*)back_string, -1,
		"1 stopbit", 	ID_STOPBITS_1,
		"2 stopbits", 	ID_STOPBITS_2
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " STOPBITS MENU", mnuSelStopbits, sizeof( mnuSelStopbits ) / sizeof( sSelMenu ), MENU_FIXED, &lStopbits);
	set_stopbits();
}

void ComPortSettings( void )
{
	int nIndex;
	stxtMenu mnuComm[] =
    {
		(char*)back_string, NULL,
        "Port",		SelectPort,
        "Baudrate", SelectBaudrate,
        "Parity",	SelectParity,
        "Databits", SelectDatabits,
        "Stopbits",	SelectStopbits
    };
    nIndex = 0;
    ShowTextMenu( &nIndex, " COMPORT MENU", mnuComm, sizeof( mnuComm )/sizeof( stxtMenu), MENU_FIXED );
}

void SelectProtocol( void )
{
	int  nIndex;

	sSelMenu mnuSelProtocol[] = {
		(char*)back_string, -1,
		"No protocol", 		ID_NO_PROTOCOL,
		"NetO protocol",	ID_NETO_PROTOCOL
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " PROTOCOL MENU", mnuSelProtocol, sizeof( mnuSelProtocol ) / sizeof( sSelMenu ), MENU_FIXED, &lProtocol);

}

void ChangeContrast( void )
{
	int nMaxX;
    int value;
    
    nMaxX = (GetMaxCharsXPos() >> 1);
    value = contrast(-1);
    putchar('\f');
    
    printf("%-*.*s\tr%-*.*s \tr\n\tr%-*.*s\tr%-*.*s \n", nMaxX-1, nMaxX-1, "CHANGE", nMaxX, nMaxX, "CONTRAST", nMaxX-1, nMaxX-1, "CHANGE", nMaxX, nMaxX, "CONTRAST");

    printf("Using: Q1 & Q2");
    for(;;) 
    {
    	switch( WaitForKeys(5, Q1_KEY, Q2_KEY, CLR_KEY, ENT_KEY, TRIGGER_KEY ))
    	{
    		case Q1_KEY:
    			if( ++value > 31 )
    				value = 31;
    			break;
    		case Q2_KEY:
    			if( --value < 0 )
    				value = 0;
    			break;
    		
    		case CLR_KEY:
    		case ENT_KEY:
    		case TRIGGER_KEY:
    			return;
    		
    	}
    	contrast( value );
    }	
}

void SetBarcodes( void )
{
	int nIndex;
	sSelMenu mnuSelBarcodes[] = {
		(char*)back_string, -1,
        "Code39"		, ID_CODE39,
        "EAN"			, ID_EAN,
        "UPC"			, ID_UPC,
        "Interl. 2of5"	, ID_I2OF5,
        "Indust. 2of5"	, ID_D2OF5,
        "Codabar"		, ID_CODABAR,
        "Code93"		, ID_CODE93,
        "Code128"		, ID_CODE128,
        "MSI"			, ID_MSI_PLESSEY,
        "Telepen"		, ID_TELEPEN,
        "UK/Plessey"	, ID_UK_PLESSEY,
        "IATA code"		, ID_IATA,
        "Scode"			, ID_SCODE,
        "GS1 Databar"   , ID_RSS,
        "All addons"	, ID_ADDONS
    };
    nIndex = 0;
    ShowTextSelectionMenu( &nIndex, " BARCODE MENU", mnuSelBarcodes, sizeof( mnuSelBarcodes ) / sizeof( sSelMenu ), MENU_FIXED | MENU_MULTIPLE, &lBarcodes);
    set_barcodes();	// Set the new selected barcodes
}

void AvailableMemory( void )
{
   	long mem_size;
	mem_size = coreleft();
	printf("\fMEMORY LEFT:\n%ld Bytes\n\nPress a key", mem_size);	
	WaitForKey();
}

void SystemMenu( void )
{
	int nIndex = 0;
	stxtMenu mnuSystem[] = {
		(char*)back_string, NULL,
		"Date/Time",	DateTime,
		"Com port", 	ComPortSettings,
		"Protocol",		SelectProtocol,
		"Display",		ChangeContrast,
		"Barcodes",		SetBarcodes,
		"Memory",		AvailableMemory
	};
	ShowTextMenu( &nIndex, " SYSTEM MENU", mnuSystem, sizeof( mnuSystem ) / sizeof( stxtMenu ), MENU_FIXED );
}

void DeleteDatabase( void )
{
	printf("\fDelete DBASE\n\n[Q1] = YES\n[Q2] = NO");
	if( WaitForKeys( 2, Q1_KEY, Q2_KEY ) == Q1_KEY )
		remove(DBASE_NAME );
}

void TransmitData( void )
{
	int nRet;
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\nPress a key");
		WaitForKey();
		return;
	}
	if( comopen( (unsigned int)lPort ) != OK )
	{
		printf("\fError open\nCOM port\n\nPress a key");
		WaitForKey();
		return;
	}
	printf("\fTransmit data\n");
	if( lProtocol == ID_NETO_PROTOCOL )
	{
		printf("NetO protocol");
		nRet = neto_transmit( (char(*)[12+1])DBASE_NAME, 1, "123456", TRIGGER_KEY, 3 );
		cursor( NOWRAP );
		if( nRet != OK )
		{
			printf("\fError NetO\nCode=%d\n\nPress a key", nRet);
			WaitForKey();
		}
	}
	else
	{
		if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError open\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
			WaitForKey();
		}
		else
		{
			if( ReadFirstRecord( &dbFile, record ))
			{
				putchar('\f');
				do
				{
					gotoxy(0,0);
					printf("Send %04ld/%04ld", dbFile.lCurrRecord+1, dbFile.lTotalRecords);
					for( nRet = 0; nRet < SZ_RECORD; nRet++ )
			            putcom( record[ nRet ]);
        			delay(10);
				
				}while( ReadNextRecord( &dbFile, record ) );
			}
			else
			{
				printf("\fError\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
				WaitForKey();
			}
			CloseDatabase( &dbFile );
		}
	}

	comclose( (unsigned int) lPort );
}

void ShowVersion( void )
{
	static char os[ 8 + 1 ];
    printf("\fSoftware Version");
    version_os(os);
    os[3] = '\0';
    os[1] = 'F';
    printf("\n%s%s\npress any key", os,VERSIONSUFFIX);
    WaitForKey();
}

int hasKeyboard( void )
{
	static char os_ver[ 8 + 1 ];
	
	version_os( os_ver );		// LBD = OP9718 ; LBO = OPL9712 ; LBW = OPL9714/5 ; LBN = DCL1531
	 
	if( strncmp( os_ver, "LBD", 3 ) == 0 || strncmp( os_ver, "LBO", 3 ) == 0 || strncmp( os_ver, "LBW", 3 ) == 0 || strncmp( os_ver, "LBN", 3 ) == 0 )
		return TRUE;
	return FALSE;
}

int hasUsb( void )
{
	static char os_ver[ 8 + 1 ];

	version_os( os_ver );		// LBK = DCL1530 ; LBN = DCL1531

	if(strncmp( os_ver, "LBK", 3 ) == 0 || strncmp( os_ver, "LBN", 3 ) == 0)
		return TRUE;

	return FALSE;	 
}

void main( void )
{
	int nIndex = 0;
	stxtMenu mnuMain[] = {
		"Scan labels",	ScanLabels,
		"Scroll DBase",	ScrollDatabase,
		"System menu", 	SystemMenu,
		"Delete DBase",	DeleteDatabase,
		"X'mit data",  	TransmitData,
		"Version",     	ShowVersion
	};

	nHasKeyboard = hasKeyboard();
	
	nHasUsb = hasUsb();

	// Set some default values
	lPort = (nHasUsb) ? COM0 : COM1; // USB port or Cradle is used as default

	lProtocol = ID_NETO_PROTOCOL;
	lBarcodes = ID_CODE39 | ID_EAN | ID_UPC | ID_I2OF5 | ID_D2OF5 | ID_CODABAR | ID_CODE93 | ID_CODE128;
	lBarcodes |= ID_MSI_PLESSEY | ID_TELEPEN | ID_UK_PLESSEY | ID_IATA | ID_SCODE;

	lBaudrate = (nHasUsb) ? ID_115200 : ID_19200;

	lParity = ID_PARITY_NONE;
	lDatabits = 8;
	lStopbits = ID_STOPBITS_1;

	nHasKeyboard = hasKeyboard();
	
	set_barcodes();		// Set the enabled barcodes 
	set_stopbits();		// set the number of stopbits
	set_databits();		// set the number of databits
	set_parity();		// set the parity
	set_baudrate();		// set the baudrate

	flipscreen( (nHasUsb) ? OFF : AUTO ); // automatically turn the display upside down if placed into the cradle (except for DCL153x)

	for(;;)
	{
		ShowTextMenu( &nIndex, "  MAIN MENU", mnuMain, sizeof( mnuMain ) / sizeof( stxtMenu ), MENU_FIXED );

		if( nIndex == -1 )	// CLR_KEY was pressed
			nIndex = 0; // reset selection to the first item
	}	
}

