#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>

#include 		"lib.h"
#include 		"utils.h"
#include 		"menu.h"

/************************************************************************/
/* Menu functions                                                       */
/* There are 2 types of menus: action menus and selection menus.        */
/* Action menus are handled by the xmenu() function.                    */
/* Selection menus are handled by the smenu() function.                 */
/* Action menus have items that are executed when selected. These items */
/* can be submenus or actions such as "Read Tags"                       */
/* Selection menus have items that present a mutually exclusive choice. */
/* The structure of the menus is defined by a header, the menu item     */
/* strings, the menu action function names (for an xmenu), and the name */
/* and possible values of a variable (for an smenu).                    */
/* These are all defined together in the upper part of the functions    */
/* so the menu structure can be easily modified.                        */
/* For example, a submenu can be added by copying an _xmenu or _smenu   */
/* function, changing its name and the menu definitions, and adding its */
/* item string and the function name to the menu table in another xmenu.*/
/************************************************************************/

/************************************************************************/
/* the execution function for a selection menu                          */
/************************************************************************/
int smenu(char *menu_text, struct smenus *tab, int max, int *value, int more)
{
	int count;
	int index;
	int selected;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;)
	{
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		rectangle(XL, Y0, XR/*-8*/, Y1, 1);
		if (index > 2)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count<3; count++)
		{
			selected = (more && (tab[count + calcu].value & (*value)) && (tab[count + calcu].value != -1)) ||
				(!more && (tab[count + calcu].value == (*value)));
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%c%-13.13s\tr", selected ? '*' : ' ', tab[count + calcu].item);
			else
				if ((count+calcu)>max)
					printf("              ");
				else
					printf("%c%-13.13s", selected ? '*' : ' ', tab[count + calcu].item);

		}

		key=waitkey();
		switch (key)
		{
		case UP_KEY:
			keybeep();
			if (index > 0)
				index--;
			else
				index=max;
			break;


		case DOWN_KEY:
			keybeep();
			if (index < max)
				index++;
			else
				index=0;
			break;

		case TRIGGER_KEY:
			keybeep();
			if (more)
			{
				if( tab[index].value == -1 )
					return (*value);
				(*value) ^= tab[index].value;
			}
			else
			{
				if( tab[index].value == -1 )
					return (*value);
				 (*value) = tab[index].value;
			}
			break;

		default:
			errkeybeep();
			break;
		}
	}
}


int menu(char *menu_text, struct menus *tab, int max)
{
	int count;
	int index;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;)
	{   gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		rectangle(XL, Y0, XR /*-8*/, Y1, 1);
		if ((index - 2) > 0)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count < 3; count++)
		{   gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%-16.16s\tr", tab[count + calcu].item);
			else
				printf("%-16.16s", tab[count + calcu].item);

		}

		key=waitkey();
		switch (key)
		{
		case UP_KEY:
			keybeep();
			if (index > 0)
				index--;
			else
				index=max;
			break;

		case DOWN_KEY:
			keybeep();
			if (index < max)
				index++;
			else
				index=0;
			break;

		case TRIGGER_KEY:
			keybeep();
			return (index);

		default:
			errkeybeep();
			break;
		}
	}
}


/************************************************************************/
/* the execution function for an action menu                            */
/************************************************************************/
void xmenu(int *index, char *menu_text, struct xmenus *tab, int max)
{
	int count;
//	int index;
	int calcu;
	int key;
//	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;)
	{   gotoxy(0, 0);
		if ((*index) == 0)
			printf(" %s%c", menu_text, 31);
		else if ((*index) == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		rectangle(XL, Y0, XR/*-8*/, Y1, 1);
		if ((*index) > 2)
			calcu = (*index) - 2;
		else
			calcu = 0;
		for (count = 0; count < 3; count++)
		{   gotoxy(0, count + 1);
			if ((*index) == (count + calcu))
				printf("\tr%-14.14s \tr", tab[count + calcu].item);
			else
				if ((count+calcu)>max)
					printf("              ");
				else
					printf("%-14.14s ", tab[count + calcu].item);
		}

		key=waitkey();
		switch (key)
		{
		case UP_KEY:
			keybeep();
			if ((*index) > 0)
				(*index)--;
			else
				(*index)=max;
			break;


		case DOWN_KEY:
			keybeep();
			if ((*index) < max)
				(*index)++;
			else
				(*index)=0;
			break;
		case TRIGGER_KEY:
			keybeep();
			return;

		default:
			errkeybeep();
			break;
		}
	}
}

