
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include 		<stdarg.h>
#include		<limits.h>

#include        "lib.h"
#include		"utils.h"


void draw_bar( int x_pos, int y_pos, int width, int height, int color )
{
	int count;
	if( width > height )
	{
		for( count = 0; count < height; count++ )
			drawline( x_pos, y_pos+count, x_pos+width, y_pos+count, color );
		return;
	}
	for( count = 0; count < width; count++ )
		drawline( x_pos+count, y_pos, x_pos+count, y_pos+height, color );
}

/* make sound to inform the user */
void info_sound( void )
{
	sound( TSHORT, VMEDIUM, SHIGH, SLOW, SHIGH, SHIGH, SLOW, 0);
}


void keybeep(void)
{
	sound(TSHORT,VMEDIUM,SHIGH,0);
}

void errkeybeep(void)
{
	sound(TLONG,VMEDIUM,SLOW,0);
}

void okbeep(void)
{
	sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
}

void errbeep(void)
{
	sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
}

int waitkey(void)
{
	int c;

	while((c=getchar())==EOF)
		idle();
	return c;
}


void rectangle(int x1,int y1,int x2,int y2,int color)
{
	drawline(x1,y1,x2,y1,color);
	drawline(x1,y2,x2,y2,color);
	drawline(x1,y1,x1,y2,color);
	drawline(x2,y1,x2,y2,color);
}


void rectangle3d(int x1,int y1,int x2,int y2,int color)
{
	rectangle(x1,y1,x2,y2,color);
	drawline(x1+2,y2+1,x2+2,y2+1,1);
	drawline(x1+2,y2+2,x2+2,y2+1,1);
	drawline(x2+1,y1+2,x2+1,y2+2,1);
	drawline(x2+2,y1+2,x2+2,y2+2,1);
}

void setcontrast(void)
{
	int key;
	int value;
	value = contrast(-1);
	printf("\fCHANGE\trCONTRAST\tr\n\trCHANGE\trCONTRAST\n");
	printf("Use: %c and %c\n", 31, 30);

	for (;;)
	{
		key=waitkey();

		switch(key)
		{
			case TRIGGER_KEY:
				keybeep();
				return;

			case UP_KEY:
				if (value < 31)
				{	keybeep();
					value++;
					contrast(value);
				}
				else
					errkeybeep();
				break;

			case DOWN_KEY:
				if (value > 2)
				{	keybeep();
					value--;
					contrast(value);
				}
				else
					errkeybeep();
				break;

			default:
				errkeybeep();
		}
	}
}


static int sound_key_input( int typ, int show_echo )
{
	int key;
	while((key = getchar()) == EOF )
	{
		/* see if shift is ON */
//		if( (shiftstate(-1) == ON ) && ( typ & ALPHA ))
//  			show_echo ? setecho(ON) : setecho( OFF );
//		else
//		{
//			if(!((shiftstate(-1) == ON ) && (typ & NEGATIVE)))
//				shiftstate( OFF );
//			setecho(OFF);
//		}
		idle();
	}
	keybeep();
	return( key );
}


static void store_char_in_string( int key, int max_len, char *string, int typ, int decimals )
{
	int i, ok;
	char *ptr;
	i = strlen( string );

	if(( i >= max_len ) && (key != '-'))
		return;

	ok = 0;
	if( typ & FLOAT )
	{
		ok = (isdigit( key ) || ((strchr( string, '.') == NULL )&& (key == '.')));
		if( ok && ( ptr = strchr( string, '.')))
		{	/* we now need to check the decimal length */
			if((strlen(string) - (ptr-string)) > decimals )
				ok = FALSE;
		}
		else if( ok && (key != '.'))
		{
			if( (strlen(string) - ((typ & NEGATIVE)? 1:0)+1) > ((max_len-decimals) -((typ & NEGATIVE)? 1:0) - 1))
				ok = FALSE;
		}
	}
	if( !ok && ( typ & NUM ))
		ok = isdigit( key );
	if( !ok && (typ & ALPHA ))
		ok = (isalpha(key) || key == ' ');
	if( !ok && ( typ & NEGATIVE ))
		ok = ( key == '-')? TRUE : FALSE;
	if( ok )
	{
		if(key == '-')
		{
			if( i )
				string[0] = (string[0] == '-')? ' ' : '-';
			else
			{
				string[i++] = (string[0] == '-')? ' ' : '-';
				string[i] = '\0';
			}
		}
		else if( typ & NEGATIVE )
		{
			if( i == 0 )
				string[i++] = (string[0] == '-')? '-' : ' ';
			string[i++] = key;
			string[i] = '\0';
		}
		else
		{
			string[i++] = key;
			string[i] = '\0';
		}
	}
}

/* check special keys */
static int check_key_input( int key, int num, va_list key_list )
{
	int count;

	for( count = 0; count < num; count++ )
	{
		if( (int) va_arg( key_list, int ) == key )
		{
			va_end( key_list ); /* needed for normal returning */
			return( OK );
		}
	}
	va_end( key_list );
	return( ERROR );
}

static void display_input( char* string, int x, int y, int max_length,int max_x )
{
	int length;
	int output;

	if( x < 0 || y < 0 )
		return;

	length = strlen( string );

	gotoxy(x,y);
	if( (x + length) < max_x )
	{
		if( length == max_length )
			output = printf("%s", string );
		else
			output = printf("%s ", string );
	}
	else
	{
		if( length == max_length )
			output = printf("%s", string + (length - max_x + x));
		else
			output = printf("%s ", string + (length - max_x + x));
	}
	gotoxy( x+output-1, y); /* set the cursor to the correct position */
}

static void remove_one_from_buffer( char* string )
{
	if( strlen( string ) > 0 )
		string[ strlen( string ) - 1 ] = '\0';
}

static void clear_part( int x, int y, int max_x )
{
	int i;
	for( i = x ; i <= max_x; i++ )
	{
		gotoxy( i, y );
		putchar(' ');
	}
}
/*
int Keyboard( char* string, int max_length, int typ, int x, int y,int line, int num, ... )
{
	va_list key_list;
	int key;
	int do_echo;

	if( num <= 0 )
		return( EOF );

	for(;;)
	{
		va_start( key_list, num );

		display_input(string, x, y, max_length, x+max_length);
		if( line )
			drawline(0,17,125,17,1);
		/* turn the echo off when the maximum input is reached */
/*		do_echo = (strlen(string) == max_length)? 0 : 1;

		key = sound_key_input( typ, do_echo );

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			return( key );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			continue;
		}

		store_char_in_string( key,  max_length, string, typ, 0 );
	}
}

int KeyboardBarcode( char* string, int min_length, int max_length, int typ, int x, int y,int num, ... )
{
	va_list key_list;
	int key;

	if( num <= 0 )
		return( EOF );

	for(;;)
	{
		va_start( key_list, num );

		if( strlen(string) < max_length )
			cursor( ON );
		else
			cursor( OFF );
										 /* no echo */
/*		if((( key = sound_key_input( typ, 1 )) == ENT_KEY ) && (strlen(string) < min_length ))
			continue;

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			cursor( OFF );
			return( key );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			display_input(string, x, y, 15, 15);
			continue;
		}

		store_char_in_string( key,  max_length, string, typ, 0 );
		display_input(string, x, y, 15, 15);
	}
}
*/

int ScanBarcode( char* string, int *code_id, int min_length, int max_length)
{
	int key;
	struct barcode code;

	code.min = min_length;
	code.max = max_length;
	code.text = string;
	scannerpower( SINGLE, 300 );

	for(;;)
	{
		if ( readbarcode( &code ) != OK )
		{
			if( kbhit() )
			{
				if(( key = getchar() ) != TRIGGER_KEY )
				{
					ungetc( key, stdin); // put the character back in the input buffer
					scannerpower( OFF, 0);
					return( ERROR );
				}
				scannerpower( SINGLE, 300 );
			}
			idle();
		}
		else
		{	if (code.id == MENU_CODE)
			{	if (systemsetting(code.text) == ERROR)
				{	goodreadled(RED,25);
				}
				else
				{	break;
				}
			}
			else
				break;
		}
	}

	*code_id = code.id;
	goodreadled(GREEN,25);
	return( OK );
}

/*
int ScanOrKeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y )
{
	int	key;
	for(;;)
	{
		string[0] = '\0';
		clear_part( x, y, 21 );
		display_input( string, x, y, 21, 21 );
		if( ScanBarcode( string, min_length, max_length ) == OK )
		{
			display_input( string, x, y, 21, 21 );
			goodreadled( GREEN, 10 );
			scannerpower( OFF, 0);
			sound( TSTANDARD, VSTANDARD, SMEDIUM, 0 );
			return( OK | SCANNED ); /* SCAN OK */
/*		}

/*		key = KeyboardBarcode( string, min_length, max_length, typ, x, y, 8, TRIGGER_KEY, ESC_KEY, ENT_KEY, Q1_KEY, Q2_KEY, F1_KEY, F3_KEY, F4_KEY );
		switch( key )
		{
			case ENT_KEY:
				return( OK | KEYBOARD );	/* KEY OK */
/*			case TRIGGER_KEY:
				break;

			case ESC_KEY:
				if( strlen( string ) != 0)
				{
					break;
				}
			default:
				return( key );
		}
	}
}
/*
int KeyboardAantald( char* string, int max_length, int nr_dec, int typ, int x, int y,int rect, int num, ... )
{
	va_list key_list;
	int key;
	int x_max;
	int do_echo;

	if( num <= 0 )
		return( EOF );

	x_max = max_length;
	if( typ & FLOAT )
		x_max++;
	if( typ & NEGATIVE )
		x_max++;

	for(;;)
	{
		va_start( key_list, num );

		display_input(string, x, y, x_max, x+x_max);
		if( rect )
			rectangle( (x*6)-2, ((y-1)*6), ((x+x_max)*6), ((y+1)*6)-1, 1);

		/* turn the echo off when the maximum input is reached */
  /*		do_echo = (strlen(string) == max_length)? 0 : 1;

		key = sound_key_input( typ, do_echo );

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			return( key );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			continue;
		}

		store_char_in_string( key,  x_max, string, typ, nr_dec );
	}
}


int KeyboardAantal( char* string, int max_length, int typ, int x, int y,int max_x_d, int rect, int num, ... )
{
	va_list key_list;
	int key;
	int do_echo;

	if( num <= 0 )
		return( EOF );

	for(;;)
	{
		va_start( key_list, num );

		display_input(string, x, y, max_length, x+max_x_d);
		if( rect )
			rectangle( (x*6)-2, ((y-1)*6), ((x+max_x_d+1)*6), ((y+1)*6)-1, 1);

		/* turn the echo off when the maximum input is reached */
/*		do_echo = (strlen(string) == max_length)? 0 : 1;

		key = sound_key_input( typ, do_echo );

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			return( key );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			continue;
		}

		store_char_in_string( key,  max_length, string, typ, 0 );
	}
}
*/
/* wait the the key(s) you like to expect */
int WaitUntilKeys( int num, ... )
{
	va_list key_list;
	int key;

	if( num <= 0 )
		return( EOF );
	va_start( key_list, num );
	resetkey();
	for(;;)
	{
		while(( key = getchar()) == EOF )
			idle();
		sound( TSTANDARD, VSTANDARD, SMEDIUM, 0 );
		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			return( key );
		}
	}
}

int WaitForAnyKeyTimer( int wait_time )
{
	int ret;
	resetkey();
	ret = 0;
	starttimer( wait_time );
	while( !endtimer() )
	{
		if( kbhit() )
		{
			ret = 1;
			starttimer( 0 ); /* cancel waiting time */
		}
		idle();
	}
	resetkey();
	return( ret );
}


char *_strtok(char *str, char *dels)
{
	static char *bufp;

	if (str)
		bufp = str;
	if (!bufp || !*bufp)
		return (NULL);
	while (*bufp && strchr(dels, *bufp) != NULL)
		++bufp;
	str = bufp;
	while (*bufp && strchr(dels, *bufp) == NULL)
		++bufp;
	if (*bufp)
		*bufp++ = '\0';
	return (str);
}

/* 0 = sunday, 1 = monday, 2 = tuesday, 3 = wensday */
/* 4 = thursday, 5 = friday, 6 = saturday */
int what_day_of_week( void )
{
	int m, y, a;
	struct date curr_date;
	getdate( &curr_date );
	a= (14 - curr_date.da_mon ) / 12;
	y= curr_date.da_year - a;
	m= curr_date.da_mon + (12*a) - 2;
	return((curr_date.da_day + y + (y/4) - (y/100) + (y/400) + ((31*m)/12))%7 );
}

