#include <stdio.h>
#include "lib.h"

#include "pdf_lib.h"				// Included header file the (Micro)PDF417 library

//Combines all allowed combinations of 2D-composite bar codes and linear bar codes
int composite_post_processing(struct barcode *code, struct barcode *prev_code, 
												unsigned long *time_out_value, short reset_time);
void main(void)
{	
	int i;
	static char options[30 + 1];	//Holds string of menu-options
	
	static char bcr_buf[2000];		//Main bar code string buffer
	struct barcode code;			//Bar code structure for main bar code
	
	static char	prev_buff[300];		//Storage for first read part of a linear/2D Composite symbol
	struct barcode prev_code;		//Bar code structure for first read part of a linear/2D Composite)

	unsigned long time_out_value;	//Holds system time before which 2nd part should be read

	init_PDF417_decoders();			//Initialize the following decoders to the Operating System
									// -> PDF417 (inlc. its composite variant CC-C)					
									// -> MICRO_PDF417 (incl. its composite variants CC-A & CC-B)

	strcpy(options, "-Y");			//Enable RSS-14

	strcpy(options, "JX");			//Enable RSS-14
	strcat(options, "JY");			//Enable RSS-Limited
	strcat(options, "DR");			//Enable RSS-Expanded
	
	strcat(options, "OG");			//Enable EAN128
		
	strcat(options, "[BCF");		//Enable PDF417 (incl. its composite variant CC-C)
	strcat(options, "[BCG");		//Enable MicroPDF (incl. its composite variants CC-A and CC-B)
	
	strcat(options,"[BHC");			//Enable transmit RSS link flag  
									//	'0' = no 2D-composite
									//	'1' = CC-A or CC-B composite attached 
	
	strcat(options,"[BI0");			//Transmit MicroPDF/CC-A/CC-B nr of data columns 
									//	'1' = MicroPDF symbol with 1 data column
									//	'2' = MicroPDF or CC-A/B symbol with 2 data columns
									//	'3' = MicroPDF or CC-A/B symbol with 3 data columns
									//	'4' = MicroPDF or CC-A/B symbol with 4 data columns
	
	strcat(options,"OT");			//Transmit EAN128 link flag 
									//	'0' = no 2D-composite attacked
									//	'1' = CC-A or CC-B composite attached
									//	'2' = CC-C composite attached 

	systemsetting( options );		//Execute all menu-options shown above

	code.min   = 1;					//Minimum barcode length of 5
	code.max   = 2700;				//Maximum barcode length of 2700 (=max of PDF417)
	code.text  = bcr_buf;			//Pointer to result string buffer

	prev_code.min   = 1;			//Minimum barcode length of temporary stored composite component
	prev_code.max   = 400;			//Maximum barcode length of temporary stored composite component
	prev_code.text  = prev_buff;	//Pointer to result string of temporary stored composite component

	printf("\f\nPress TRIGGER\nto start\nscanning!   ");
	
	while(!kbhit()) idle(); 		//Wait untill key-pressed
		
	for(;;)
	{   
		scannerpower( TRIGGER, 10000 );	//Set scanner in TRIGGER mode

		//Call readbarcode function and if try to combine a linear code to a 2D composite symbol
		while( readbarcode( &code ) != OK || composite_post_processing( &code, &prev_code, &time_out_value, 150)!=0)
		{
			if(!triggerpressed()) 		//If trigger is released
			{	
				prev_code.id = 0; 	//Clear ID of 1st part of linear/2D composite symbol
				scannerpower( TRIGGER, 10000 );
			}
			idle();
		}
		
 		//Complete Barcode has been read

		if(code.id == MENU_CODE)
		{
			if(systemsetting(code.text) == OK)
			{
				sound(TSTANDARD,VHIGH,SMEDIUM,SHIGH,0);
				goodreadled(GREEN,10);				//Beep en show Green LED on
			}
			else
			{
				sound(TLONG,VHIGH,SMEDIUM,SLOW,0);
				goodreadled(RED,10);				//Beep en show Green RED on
			}
		}
		else
		{
			goodreadled(GREEN,10);				//Beep en show Green LED on
			sound(TSTANDARD,VHIGH,SMEDIUM,SHIGH,0);
		}

		scannerpower(OFF,0);				//Turn of scanner
		
		resetkey();
		cursor( NOWRAP );
		
		printf("\f");					//Clear screen
		i=0;
		while(i<code.length)				//Print result 
		{
			printf("\n%-13.13s",&code.text[i]);
			i += 13;

			if(kbhit())				//Abort printing if key-pressed
				break;
		}
	}
}

/******************************************************************************************
Combines all allowed combinations of 2D-composite bar codes and linear bar codes

The following combinations of a linear bar codes with a 2D composite are allowed:
-	RSS-14(standard)	-	CCA-COMPOSITE (4 data columns)
-	RSS-14(standard)	-	CCB-COMPOSITE (4 data columns)

-	RSS-14(stacked)	-	CCA-COMPOSITE (2 data columns)
-	RSS-14(stacked)	-	CCB-COMPOSITE (2 data columns)

-	RSS-LIMITED		-	CCA-COMPOSITE (3 data columns)
-	RSS-LIMITED		-	CCB-COMPOSITE (3 data columns)

-	RSS_EXPANDED		-	CCA-COMPOSITE (4 data columns)
-	RSS_EXPANDED		-	CCB-COMPOSITE (4 data columns)

-	EAN128			-	CCA_COMPOSITE (4 data columns)
-	EAN128			-	CCB_COMPOSITE (4 data columns)
-	EAN128			-	CCC_COMPOSITE

Returns:
	0		Normal barcode read OR both parts of a linear/2D composite combination read
	1		First part of a linear/2D composite combination read
**********************************************************************************************/
int composite_post_processing(struct barcode *code, struct barcode *prev_code, 
												unsigned long *time_out_value, short reset_time)
{
	char format_info,format_info_prev;

	switch(code->id)
	{
		case MICRO_PDF417:	//Remove transmitted 'nr of data columns' char at first position
			memmove(code->text,&(code->text)[1],code->length);
			code->text[--code->length] = '\0';
			prev_code->id = 0;
			return 0;			//Normal bar code, return OK
		case EAN128:
		case RSS_14:
		case RSS_LIMITED:
		case RSS_EXPANDED:
			if(code->text[0] == '0') //If no link flag set remove transmitted 'link-flag' at first pos.
			{
				memmove(code->text,&(code->text)[1],code->length);
				code->text[--code->length] = '\0';
				return 0;		// Normal bar code, return OK
			}
		case CCA_COMPOSITE:
		case CCB_COMPOSITE:
			format_info = code->text[0]; //Store format info = link-flag(RSS) or nr-of-data-colums(CC-A/B)
			break;
		case CCC_COMPOSITE:					
			break;						//Note: CC-C composite symbols have no format info
		default:
			prev_code->id = 0;		
			return 0;			//Normal bar codes, return OK

	}

	if(code->length > prev_code->max)		//Check maximum length for valid combinations
		return 0;	

	
 	//If 1st half of a combination is already known, try to combine
	if(prev_code->id != 0)		
	{
		if(GetSystemTime() < *time_out_value)
		{
			format_info_prev = prev_code->text[0];

			if (code->id == prev_code->id)	//if same bar code ID
			{
				if(strcmp((char*)code->text,(char*)prev_code->text)==0) //if same data
				{
					*time_out_value = GetSystemTime() + reset_time; //time out value
					return 1;		//First part read again!
				}

				prev_code->id = 0;		//Store this part as 1st part
			}
			else if( code->id == CCA_COMPOSITE ||  code->id == CCB_COMPOSITE )
			{
				switch(prev_code->id)		//Check if combination is invalid 
				{
					case RSS_14:
					case RSS_LIMITED:
					case RSS_EXPANDED:
					case EAN128:
						if(prev_code->id == RSS_14 && (format_info == '2' || format_info == '4'))
							break;
						if(prev_code->id == RSS_LIMITED && format_info == '3')
							break;
						if(prev_code->id == RSS_EXPANDED && format_info == '4')
							break;
						if(prev_code->id == EAN128 && format_info_prev == '1' && format_info == '4')
							break;
					default:
						prev_code->id = 0;	//Store this part as 1st part
						sound(TCLICK,VSTANDARD,SMEDIUM,0); 
				}
			}
			else if( code->id == CCC_COMPOSITE )
			{
				switch(prev_code->id)		//Check if combination is invalid 
				{
					case EAN128:
						if(format_info_prev == '2') 	//Check link flag value
							break;
					default:
						prev_code->id = 0; 		//Store this part as 1st part
				}
			}
			else if (code->id >= RSS_14 &&  code->id <= RSS_EXPANDED) 
			{
				switch(prev_code->id)
				{
					case CCA_COMPOSITE: 	//Check if combination is invalid 
					case CCB_COMPOSITE:
						if(code->id == RSS_14 && (format_info_prev == '2' || format_info_prev == '4'))
							break;
						if(code->id == RSS_LIMITED && format_info_prev == '3')
							break;
						if(code->id == RSS_EXPANDED && format_info_prev == '4')
							break;
					default:
						prev_code->id = 0;	//Store this part as 1st part
						sound(TCLICK,VSTANDARD,SMEDIUM,0); 
				}
			}
			else if( code->id == EAN128 )
			{
				switch(prev_code->id)		//Check if combination is invalid
				{
					case CCA_COMPOSITE:
					case CCB_COMPOSITE:	
					case CCC_COMPOSITE:
			  	   		if((prev_code->id==CCA_COMPOSITE || prev_code->id==CCB_COMPOSITE) 
							 && format_info == '1' && format_info_prev == '4')
							break;
					   	if(prev_code->id == CCC_COMPOSITE && format_info == '2')
							break;
					default:
						prev_code->id = 0;	//Store this part as 1st part
						sound(TCLICK,VSTANDARD,SMEDIUM,0); 
				}
			}
			else return 0;
		}
 		else 	// if Time out store this part as 1st part
		{	
			prev_code->id = 0;
			sound(TCLICK,VSTANDARD,SMEDIUM,0); 
		}
	}
	else 
		sound(TCLICK,VSTANDARD,SMEDIUM,0); 
	
	if(prev_code->id != 0)	// if 2 parts read, try to combine 
	{
		if(code->length + prev_code->length - 2 > code->max)		// too large combination
			return 0;
		
		if(code->id != CCC_COMPOSITE)	 //If not CC-C remove link flag/nr of columns of 2nd part
		{
			memmove(code->text,&(code->text)[1],code->length);
			code->text[--code->length] = '\0';
		}
		
		if(prev_code->id != CCC_COMPOSITE) //If not CC-C remove link flag/nr of columns of 1st part
		{
			memmove(prev_code->text,&(prev_code->text)[1],prev_code->length);
			prev_code->text[--prev_code->length] = '\0';
		}
		
		//Put RSS and EAN128 Codes in front and 2D (Micro)PDF417 variants behind it
		if((code->id >= RSS_14 && code->id <= RSS_EXPANDED) || code->id == EAN128)
		{
			strcpy(&(code->text)[code->length+1],prev_code->text); 
			(code->text)[code->length] = GS;	//Add Primary/Secondary data seperation char.
		}
		else
		{
			memmove(&(code->text)[prev_code->length+1],code->text,code->length+1);
			memcpy(code->text,prev_code->text,prev_code->length);

			(code->text)[prev_code->length] = GS; 	//Add Primary/Secondary data seperation char.

			code->id = prev_code->id;				//Change ID to ID of linear part
		}

		code->length += prev_code->length;			//Set combined length

		prev_code->id = 0;							//Clear stored ID

		return 0;	//Combining of a linear code with a 2D composite code -> succesfull
	}

	prev_code->id = code->id;						//Store ID of 1st part 
	prev_code->length = code->length;				//Store length of 1st part
	strcpy(prev_code->text,code->text);				//Store 1st part in prev_code
	*time_out_value = GetSystemTime() + reset_time;	//Set time out value

	return 1;	// Return 'Fist part of EAN*UCC Composite Symbol stored'
}
