/* ######  ##   ## ####        #####   ####### ##   ##  #####       ##    */
/*  ##  ## ##   ##  ##          ## ###  ##   # ### ### ##   ##     ###    */
/*  ##  ## ##   ##  ##          ##  ##  ##     ####### ##   ##      ##    */
/*  #####  #######  ##          ##  ##  ####   ## # ## ##   ##      ##    */
/*  ##     ##   ##  ##          ##  ##  ##     ##   ## ##   ##      ##    */
/*  ##     ##   ##  ##  ##      ## ###  ##   # ##   ## ##   ##      ##    */
/* ###     ##   ## #######     #####   ####### ##   ##  #####     ######  */
/*                                                                        */
/* Opticon PHL1600/PHL1700/PHL2700 demo 								  */
/* Scan labels and scroll database with menus                             */
/* Transmit scanned data with multiple protocols, FREE, RCV(slt),NetO     */
/* Compiler: Microtec C compiler for Melps 7700  Version 2.2              */


/************************************************************************/
/*      Include                                                         */
/************************************************************************/
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include        "lib.h"
#include		"rfidlib.h"

/************************************************************************/
/*      Define                                                          */
/************************************************************************/
#define VERSIONSUFFIX  "20157"
#define NOT_OK         -1
#define CLEAR          -1
#define SCAN            1
#define KEYS            2
#define ALPHA           8
#define NUM             16
#define FLOAT           32
#define NONE            1
#define EVEN            2
#define ODD             3

#define BARCODE         40
#define SYMBOLOGY       24
#define TIMESTAMP       (2+1+2+1+2) //hh:mm:ss
#define DATESTAMP       (4+1+2+1+2)

#define ID_CD39         1   // bit 0
#define ID_EAN          2   // bit 1
#define ID_UPC          4   // bit 2
#define ID_I2O5         8   // bit 3
#define ID_D2O5        16   // bit 4
#define ID_NW7         32   // bit 5
#define ID_CD93        64   // bit 6
#define ID_CD128      128   // bit 7
#define ID_MSI        256   // bit 8
#define ID_TELEPEN    512   // bit 9
#define ID_UK        1024   // bit 10
#define ID_ADDONS    2048   // bit 11
#define ID_IATA      4096   // bit 12
#define ID_SCODE     8192   // bit 13
#define ID_ALLCODES 16384   // bit 14

#define ICODE			0
#define	TAGIT			1
#define ISO15693		2	

#define REC_SIZE        BARCODE+1+SYMBOLOGY+1+TIMESTAMP+1+DATESTAMP+1+1
/* barcode,symbology type, time stamp, date stamp\r\n */

#define DBASE_NAME		"DATA.TXT"

#define MAX_TIMEOUT      4		/* max # of timeouts in 1 block */
#define DEFAULT_MAXL     80
#define DEFAULT_TIME     5
#define DEFAULT_NPAD     0
#define DEFAULT_PADC     '\0'
#define DEFAULT_EOL      '\r'
#define DEFAULT_QCTL     '#'
#define MAXL             94
#define DATA_PACKET      'D'
#define ACK_PACKET       'Y'
#define NAK_PACKET       'N'
#define SINIT_PACKET     'S'
#define BREAK_PACKET     'B'
#define FILE_PACKET      'F'
#define EOF_PACKET       'Z'
#define ERROR_PACKET     'E'
#define MAX_BUF          1+1+1+1+MAXL+1+1	/* SOH+LEN+SEQ+TYP+MAXL+CHK+EOL */
#define tochar(x)        ((x) + 32)
#define unchar(x)        ((x) - 32)
#define ctl(x)           ((x) ^ 64)		/* exor */
#define check(s)         tochar ( ((s) + (((s) & 192) / 64)) & 63)
#define EDATA            0xFE
#define ENQ_DOWNLOAD     64		/* PC requests data from SLT */
#define DATA_RECORD      96
#define RETRIES          3
#define MAX_DATA         253
#define SLT_MAX_BUF      MAX_DATA+10
#define TIMEOUT          3*50	/* 3 seconds timeout */

typedef struct kermit_param {
	char max_len;				/*  max len of packet                          */
	char time;					/*  time out in seconds                        */
	char npad;					/*  number of char before each record(padding) */
	char padc;					/*  char used for padding                      */
	char eol;					/*  end of line char                           */
	char qctl;					/*  control quote char                         */
} KParam;


struct menus {
	char *item;
};

struct smenus {
	char *item;
	int value;
};

extern unsigned char port4,port4_dir,port7;

/************************************************************************/
/*      Global                                                          */
/************************************************************************/
int s_code;
char record[REC_SIZE + 1];
static int port;
static int baudrate;
static int parity;
static int stopbits;
static int databits;
static int protocol;

const struct smenus symblglist[] =
{   "UPC A"         ,   UPCA    ,
	"UPC A + 2"     ,   UPCA_2  ,
	"UPC A + 5"     ,   UPCA_5  ,
	"UPC E"         ,   UPCE    ,
	"UPC E + 2"     ,   UPCE_2  ,
	"UPC E + 5"     ,   UPCE_5  ,
	"EAN 8"         ,   EAN8    ,
	"EAN 8 + 2"     ,   EAN8_2  ,
	"EAN 8 + 5"     ,   EAN8_5  ,
	"EAN 13"        ,   EAN13   ,
	"EAN 13 + 2"    ,   EAN13_2 ,
	"EAN 13 + 5"    ,   EAN13_5 ,
	"CODE 39"       ,   CODE39  ,
	"CODABAR"       ,   CODABAR ,
	"INTERL 2 OF 5" ,   I2OF5   ,
	"INDUST 2 OF 5" ,   D2OF5   ,
	"MATRIX 2 OF 5" ,   M2OF5   ,
	"IATA CODE"     ,   IATA    ,
	"CODE 93"       ,   CODE93  ,
	"CODE 128"      ,   CODE128 ,
	"MSI PLESSEY"   ,   MSI_PLESSEY,
	"UK PLESSEY"    ,   UK_PLESSEY,
	"TELEPEN"       ,   TELEPEN,
	"SCODE"         ,   SCODE,
	"LAETUS CODE"   ,   LAETUS,
	"CD 39 FULL ASCII", C39_FA,
	"ITAL. PHARMA." ,   C39_IT,
	"CODABAR ABC"   ,   NW7_ABC,
	"CODABAR CX"    ,   NW7_CX,
	"EAN 128"       ,   EAN128,
	"CHINESE POST"  ,   CHINPOST,
	"MENU CODE"     ,   MENU_CODE,
	"UNKNOWN CODE"  ,   -1
};



/************************************************************************/
/*      extern                                                          */
/************************************************************************/
/************************************************************************/
/*      Prototype                                                       */
/************************************************************************/
int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more);
int menu(char *menu_text, struct menus *tab, int max);
int read_string_timed(char *buffer, int len, char eol, int time);
int send_recv_packet(char *spacket, char *rpacket, KParam * kp_your, int *len);
int check_recv_packet(char *packet, int *plen);
int create_packet(char *packet, int seq, char type, KParam * kp, char *msg);
int kermit_send(char *filename);
int send_slt_prot(char *filename);
int send_plain(char *filename);
void set_time_and_date(void);
int read_record(char *filename, char *record, long rec_no, int rec_size);
long get_max_records(char *filename, int rec_size);
int write_record(char *filename, char *record);
int main_menu(void);
int system_menu(void);
void sel_protocol(int *protocol);
int com_menu(void);
void sel_databits(int *databits);
void sel_stopbits(int *stopbits);
void sel_parity(int *parity);
void sel_baudrate(int *baudrate);
void sel_com_port(int *port);
void display_mem(void);
void set_display(void);
void barcode_menu(int *bcode);
int get_string(char *string, int *code_id, int typ, int smin, int smax, int kmax, int x, int y);
int code_id_to_index(int code_id);
int scan_labels(void);
int scroll_dbase(void);

int show_taginfo(int rfidtype, char *tagcode, char *write_data, char* timestamp, char* datestamp);

int read_tags(void);
int ReadIcodeAllBlocks(unsigned char *bufp);
int ReadTagitAll(char *bufp);
int ReadISO15693(char *bufp, int *startP, int nrof4blocks);

int write_tags(char* data);
int icode_write(char* data);
int tagit_write(char* data);
int iso15693_write(char* data);

int read_lock_status(void);
int icode_lockstatus(void);
int tagit_lockstatus(void);
int iso15693_lockstatus(unsigned char nr_of_blocks);

void main(void);

int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more)
{
	int count;
	int index;
	int selected;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		if ((index - 2) > 0)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count<3; count++) {
			selected = (more && (tab[count + calcu].value & (*value))) ||
				(!more && (tab[count + calcu].value == (*value)));
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%c%s\tr", selected ? '*' : ' ', tab[count + calcu].item);
			else
				printf("%c%s", selected ? '*' : ' ', tab[count + calcu].item);

		}
		while ((key=getchar())==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}

		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case LEFT_KEY:
		case F5_KEY:
		case F7_KEY:
		case Q1_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
		case RIGHT_KEY:
		case F8_KEY:
		case F6_KEY:
		case Q2_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':

			if ((key - '1') <= max) {
				if (more)
					(*value) ^= tab[(key - '1')].value;
				else {
					(*value) = tab[(key - '1')].value;
					return (key - 1);
				}
			} else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ESC:
			return (-1);
		case ENT_KEY:
		case TRIGGER_KEY:
			if (more)
				(*value) ^= tab[index].value;
			else {
				(*value) = tab[index].value;
				return (index);
			}

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}

int menu(char *menu_text, struct menus *tab, int max)
{
	int count;
	int index;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		if ((index - 2) > 0)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count < 3; count++) {
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%s\tr", tab[count + calcu].item);
			else
				printf("%s", tab[count + calcu].item);

		}

		while ((key=getchar())==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}

		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case LEFT_KEY:
		case F7_KEY:
		case F5_KEY:
		case Q1_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
		case RIGHT_KEY:
		case F8_KEY:
		case F6_KEY:
		case Q2_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			if ((key - '1') <= max)
				return (key - '1');
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ESC:
			return (-1);
		case ENT_KEY:
		case TRIGGER_KEY:
			return (index);

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}



int read_string_timed(char *buffer, int len, char eol, int time)
{
	int count;
	int ch;
	count = 0;
	while (count < len) {
		if ((ch = getcom(50 * time)) == -1)
			return (NOT_OK);
		if ((buffer[count++] = ch) == eol)
			break;
		if (powereddown())
			backlight(OFF);
	}
	buffer[count] = '\0';
	return (0);
}



int send_recv_packet(char *spacket, char *rpacket, KParam * kp_your, int *len)
{
	int timeout, start;
	int plen, count;

	timeout = 0;
	do {
		while (timeout < MAX_TIMEOUT) {
			plen = strlen(spacket);
			for (count = 0; count < plen; count++)
				putcom(spacket[count]);
			if (read_string_timed(rpacket, MAX_BUF, kp_your->eol, kp_your->time) == 0)
				break;
			timeout++;
			if (powereddown())
				backlight(OFF);
		}
		if (timeout >= MAX_TIMEOUT)
			return (-3);
		start = check_recv_packet(rpacket, len);
		switch (*(rpacket + start + 2)) {

		case NAK_PACKET:
			timeout = 0;
			start = -4;
			break;
		case ERROR_PACKET:
			start = -5;
			return (start);

		}
	} while (start < 0);
	return (start);
}

int check_recv_packet(char *packet, int *plen)
/* returns where the packet starts, -1 if no SOH, -2 if bad checksum */
{
	int len, csum, count;
	char *ptr;
	if ((ptr = strchr(packet, SOH)) == NULL)
		return (-1);
	len = unchar(*(++ptr));
	for (count = 0, csum = 0; count < len; count++)
		csum += (*(ptr + count));
	if (check(csum) != (*(ptr + len)))
		return (-2);
	*plen = len;
	return (ptr - packet);
}

int create_packet(char *packet, int seq, char type, KParam * kp, char *msg)
{
	int blen, len, count, csum, result;
	result = 0;
	switch (type) {
	case DATA_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				DATA_PACKET
			);
		count = 0;
		blen = 0;
		result = 0;
		len = strlen(packet);
		while (blen < (kp->max_len - 3)) {
			if (msg[count] == '\0')
				break;
			if (powereddown())
				backlight(OFF);
			if ((iscntrl(msg[count])) ||
				(kp->qctl == msg[count])) {
				packet[len++] = kp->qctl;
				packet[len++] = ctl(msg[count++]);
				blen += 2;
				result++;
			} else {
				packet[len++] = msg[count++];
				blen++;
			}
		};
		packet[len] = '\0';
		if (blen < kp->max_len - 3)
			result = 0;
		break;
	case ACK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				ACK_PACKET
			);
		break;
	case NAK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				NAK_PACKET
			);
		break;
	case SINIT_PACKET:
		sprintf(packet, "%c %c%c%c%c%c%c%c%c",
				SOH,
				tochar(seq),
				SINIT_PACKET,
				tochar(kp->max_len),
				tochar(kp->time),
				tochar(kp->npad),
				ctl(kp->padc),
				tochar(kp->eol),
				kp->qctl
			);
		break;
	case BREAK_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				BREAK_PACKET
			);
		break;
	case FILE_PACKET:
		sprintf(packet, "%c %c%c%s",
				SOH,
				tochar(seq),
				FILE_PACKET,
				msg
			);
		break;
	case EOF_PACKET:
		sprintf(packet, "%c %c%c",
				SOH,
				tochar(seq),
				EOF_PACKET
			);
		break;
	case ERROR_PACKET:
		sprintf(packet, "%c %c%c%s",
				SOH,
				tochar(seq),
				ERROR_PACKET,
				msg
			);
	}
	len = strlen(packet);
	packet[1] = tochar(len - 1);
	for (count = 1, csum = 0; count < len; count++)
		csum += packet[count];
	packet[len++] = check(csum);
	packet[len++] = kp->eol;
	packet[len] = '\0';
	return (result);
}

int kermit_send(char *filename)
{
	static char packet_buf[MAX_BUF + 10];
	static char recv_buf[MAX_BUF + 10];
	static char file_buf[MAX_BUF + 10];
	char *ptr;
	int len, start, correction;
	char seq;
	KParam my_side, your_side;
	FILE *fp;
	long where;

	if ((fp = fopen(filename, "rb")) == NULL) {		/* open file to send */
		return (1);
	}
	if (comopen(port) != 0) {
		fclose(fp);
		return (2);
	}
	resetreceive();
	seq = 0;
	my_side.max_len = MAXL;		/* set my defaults */
	my_side.time = DEFAULT_TIME;
	my_side.npad = DEFAULT_NPAD;
	my_side.padc = DEFAULT_PADC;
	my_side.eol = DEFAULT_EOL;
	my_side.qctl = DEFAULT_QCTL;
	your_side.max_len = DEFAULT_MAXL;	/* set the others side defaults */
	your_side.time = DEFAULT_TIME;
	your_side.npad = DEFAULT_NPAD;
	your_side.padc = DEFAULT_PADC;
	your_side.eol = DEFAULT_EOL;
	your_side.qctl = DEFAULT_QCTL;
	create_packet(packet_buf, seq, SINIT_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	ptr = recv_buf;
	ptr += (start + 3);
	if (len > 3)
		my_side.max_len = unchar(*(ptr++));
	if (len > 4)
		my_side.time = unchar(*(ptr++));
	if (len > 5)
		my_side.npad = unchar(*(ptr++));
	if (len > 6)
		my_side.padc = ctl(*(ptr++));
	if (len > 7)
		my_side.eol = unchar(*(ptr++));
	if (len > 8)
		my_side.qctl = *ptr;
	if (my_side.max_len > your_side.max_len)
		my_side.max_len = your_side.max_len;
	if (my_side.time > your_side.time)
		my_side.time = your_side.time;
	seq++;
	create_packet(packet_buf, seq, FILE_PACKET, &my_side, filename);
	if (send_recv_packet(packet_buf, recv_buf, &your_side, &len) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	seq++;
	while (!feof(fp)) {
		len = fread(file_buf, 1, (my_side.max_len - 3), fp);
		file_buf[len] = '\0';
		correction = create_packet(packet_buf, seq, DATA_PACKET, &my_side, file_buf);
		if (correction) {
			where = ftell(fp);
			where += my_side.max_len - 3 - len;
			fseek(fp, where - correction, SEEK_SET);
		}
		if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
			fclose(fp);
			comclose(port);
			return (start);
		}
		seq = (++seq) % 64;
		if (powereddown())
			backlight(OFF);
	}
	create_packet(packet_buf, seq, EOF_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	seq = (++seq) % 64;
	create_packet(packet_buf, seq, BREAK_PACKET, &my_side, NULL);
	if ((start = send_recv_packet(packet_buf, recv_buf, &your_side, &len)) < 0) {
		fclose(fp);
		comclose(port);
		return (start);
	}
	fclose(fp);
	comclose(port);
	return (0);
}

int send_slt_prot(char *filename)
{
	FILE *fp;
	int ch;
	int retry;
	unsigned int lrc;
	int count, len;
	static char buf[SLT_MAX_BUF + 1];

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	resetreceive();
	if ((fp = fopen(filename, "rb")) == NULL)
		return (ERR_OPEN_FILE);
	fseek(fp, 0L, SEEK_END);
	if (ftell(fp) == 0L) {
		fclose(fp);
		comclose(port);
		return (ERR_READ_FILE);
	}
	fseek(fp, 0L, SEEK_SET);
	buf[0] = STX;
	len = 1;
	while (len < SLT_MAX_BUF) {
		while ((ch = getcom(TIMEOUT)) != STX) {
			if (powereddown())
				backlight(OFF);
			if (kbhit())
				if (getchar() == ESC_KEY) {
					fclose(fp);
					comclose(port);
					return (ERR_USER_BREAK);
				}
		}
		retry = 0;
		while ((ch = getcom(TIMEOUT)) != EDATA) {
			if (powereddown())
				backlight(OFF);
			if (kbhit())
				if (getchar() == ESC_KEY) {
					fclose(fp);
					comclose(port);
					return (ERR_USER_BREAK);
				}
			if (ch == EOF) {
				if (retry++ > RETRIES) {
					fclose(fp);
					comclose(port);
					return (NOT_OK);
				}
				continue;
			}
			buf[len++] = ch;
		}
		lrc = 0;
		for (count = 1; count < len - 1; count++)
			lrc ^= buf[count];
		if (lrc == buf[len - 1])
			break;
		else
			len = 0;
	}
	if (len >= SLT_MAX_BUF) {
		fclose(fp);
		comclose(port);
		return (NOT_OK);
	}
	if (!(buf[2] & ENQ_DOWNLOAD)) {
		putcom(EOT);
		fclose(fp);
		comclose(port);
		return (ERR_HOST_REQ);
	}
	while (!feof(fp)) {
		if (powereddown())
			backlight(OFF);
		len = 0;
		buf[len++] = STX;
		buf[len++] = 1;
		buf[len++] = DATA_RECORD;
		len += fread(buf + len, 1, MAX_DATA, fp);
		lrc = 0;
		for (count = 1; count < len; count++)
			lrc ^= buf[count];
		lrc %= 247;
		buf[len++] = lrc;
		buf[len++] = EDATA;
		for (retry = RETRIES; retry > 0; retry--) {
			for (count = 0; count < len; count++)
				putcom(buf[count]);
			ch = getcom(TIMEOUT);
			if (ch == ACK)
				break;
			if (ch == EOT) {
				fclose(fp);
				comclose(port);
				return (ERR_EOT_HOST);
			}
		}
		if (retry == 0) {
			fclose(fp);
			comclose(port);
			return (NOT_OK);
		}
	}
	fclose(fp);
	for (retry = RETRIES; retry > 0; retry--) {
		putcom(EOT);
		ch = getcom(TIMEOUT);
		if (ch == EDATA)
			break;
	}
	if (retry == 0) {
		comclose(port);
		return (NOT_OK);
	} else {
		comclose(port);
		return (0);
	}
}

void SendPlainRecord(void)
{   int tx_count;
	char temp[3];

	if (record[0]==ICODE || record[0]==TAGIT || record[0]==ISO15693)
	{	for(tx_count=1;tx_count<64+1;tx_count++)
		{	sprintf(temp,"%02X",record[tx_count]);
			putcom(temp[0]);
			putcom(temp[1]);
			//printf("%c", record[tx_count]);
		}
		for (tx_count = 65; tx_count < REC_SIZE; tx_count++)
		{	putcom(record[tx_count]);
			//printf("%c", record[tx_count]);
		}
	}
	else
	{	for (tx_count = 0; tx_count < REC_SIZE; tx_count++)
		{	putcom(record[tx_count]);
			//printf("%c", record[tx_count]);
		}
	}
}


int send_plain(char *filename)
{	long count, max;

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	if ((max = get_max_records(filename, REC_SIZE)) <= 0L) {
		comclose(port);
		return (ERR_OPEN_FILE);
	}
	for (count = 0L; count < max; count++) {
		if (read_record(filename, record, count, REC_SIZE) == NOT_OK) {
			comclose(port);
			return (NOT_OK);
		}
		SendPlainRecord();
		delay(10);
	}
	comclose(port);
	return (OK);
}

int receive_char(unsigned char c1, unsigned char c2, int *pc)
{   int timeoutcount;

	*pc = EOF;
	timeoutcount = 0;

	while ( (*pc!=c1) && (*pc!=c2) )
	{   if (timeoutcount > 200) // 20 sec total time out
		{   printf("\fTime out of\ntransmission");
			resetkey();
			comclose(port);
			while (getchar()==EOF)
			{   if (powereddown())
					backlight(OFF);
				idle();
			}
			return(NOT_OK);
		}
		idle();
		if (getchar() == ESC_KEY)
		{   printf("\fUser break of\ntransmission");
			resetkey();
			comclose(port);
			while (getchar()==EOF)
			{   if (powereddown())
					backlight(OFF);
				idle();
			}
			return(ERR_USER_BREAK);
		}
		*pc = getcom(5);   // 100 ms time out per loop iteration
		timeoutcount++;
	}

	return(OK);
}

int send_ack_record(void)
{	int recv;
	int nakcount;
	int res;

	recv = NAK;
	nakcount = 0;
	delay(1);   // 20 ms delay, so that PC can process record

	while ( (recv==NAK) && (nakcount++<=3) )
	{   SendPlainRecord();
		resetreceive();
		res = receive_char(ACK, NAK, &recv);
		if (powereddown())
			backlight(OFF);
		if (res != OK)
			return(res);
	}

	if ( (nakcount>3) && (recv==NAK) )
	{   printf("\fToo many NAKs");
		comclose(port);
		resetkey();
		while (getchar()==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}
		return(NOT_OK);
	}

	return(OK);
}

int send_ack_nak(char *filename)
{
	long count, max;
	int recv;
	int res;

	if (comopen(port) != 0) {
		return (NOT_OK);
	}
	if ((max = get_max_records(filename, REC_SIZE)) <= 0L) {
		comclose(port);
		return (ERR_OPEN_FILE);
	}

	// first receive ENQ character
	resetreceive();
	res = receive_char(ENQ, 0, &recv);
	if (res != OK)
		return(res);

	printf("\n");
	for (count = 0L; count < max; count++) {
		if (read_record(filename, record, count, REC_SIZE) == NOT_OK) {
			comclose(port);
			return (NOT_OK);
		}
		if ( (recv=send_ack_record()) != OK )
			return(recv);
	}

	printf("EOT");
	delay(2);
	putcom(EOT);
	comclose(port);
	delay(50);          // time to show EOT on screen
	return (OK);
}



void set_time_and_date(void)
{
	struct date dates;
	struct time times;
	static char time_str[8 + 1];       /* HH:MM:SS    */
	static char date_str[10 + 1];		/* DD/MM/YYYY */
	static char temp[10 + 1];
	int x, y;
	int key;
	gettime(&times);
	getdate(&dates);
	sprintf(time_str, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
	sprintf(date_str, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);
	printf("\f\trSET TIME & DATE\tr");
	x = 0;
	y = 2;
	cursor(ON);
	for (;;) {
		gotoxy(0, 2);
		printf("%s", time_str);
		gotoxy(0, 3);
		printf("%s", date_str);
		gotoxy(x, y);
		while ((key=getchar())==EOF)
		{	if (powereddown())
				backlight(OFF);
			idle();
		}

		if ((key == ENT_KEY) || (key == TRIGGER_KEY)) {
			strcpy(temp, date_str);
			dates.da_year = atoi(temp + 6);
			temp[5] = '\0';
			dates.da_mon = atoi(temp + 3);
			temp[2] = '\0';
			dates.da_day = atoi(temp);
			if (setdate(&dates) == ERROR)
				continue;
			strcpy(temp, time_str);
			times.ti_sec = atoi(temp + 6);
			temp[5] = '\0';
			times.ti_min = atoi(temp + 3);
			temp[2] = '\0';
			times.ti_hour = atoi(temp);
			if (settime(&times) == ERROR)
				continue;
			cursor(OFF);
			return;
		}
		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case ESC:
			cursor(OFF);
			return;

		case UP_KEY:
		case F7_KEY:
			if (y == 3) {
				y = 2;
				x = 0;
			} else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case DOWN_KEY:
		case F8_KEY:
			if (y == 2) {
				y = 3;
				x = 0;
			} else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case LEFT_KEY:
		case F5_KEY:
			if ((x == 3) || (x == 6))
				x -= 2;
			else
				x--;
			if ((y == 2) && (x < 0)) {
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
				x = 0;
			}
			if ((y == 3) && (x < 0)) {
				y = 2;
				x = 7;
			}
			break;

		case RIGHT_KEY:
		case F6_KEY:
			if ((x == 1) || (x == 4))
				x += 2;
			else
				x++;
			if ((y == 2) && (x > 7)) {
				x = 0;
				y = 3;
			}
			if ((y == 3) && (x > 9))
				x = 9;
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			if (y == 2) {
				time_str[x] = (char) key;
				if ((x == 1) || (x == 4))
					x += 2;
				else
					x++;
				if (x > 7) {
					y = 3;
					x = 0;
				}
			} else {
				date_str[x] = (char) key;
				if ((x == 1) || (x == 4))
					x += 2;
				else
					x++;
				if (x > 9)
					x = 9;
			}
			break;
		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);

		}
	}
}

int read_record(char *filename, char *record, long rec_no, int rec_size)
{
	FILE *fp;

	if ((fp = fopen(filename, "rb")) == NULL) {
		return (NOT_OK);
	}
	fseek(fp, rec_no * rec_size, SEEK_SET);
	if (fread(record, 1, rec_size, fp) != rec_size) {
		fclose(fp);
		return (NOT_OK);
	}
	fclose(fp);
	return (0);
}

long get_max_records(char *filename, int rec_size)
{
	FILE *fp;
	long max;
	if ((fp = fopen(filename, "rb")) == NULL) {
		return (0L);
	}
	fseek(fp, 0L, SEEK_END);
	max = ftell(fp);
	fclose(fp);
	return (max / rec_size);
}

int write_record(char *filename, char *record)
{	FILE *fp;
	int rec_size;

	if ((fp = fopen(filename, "r+b")) == NULL)
		if ((fp = fopen(filename, "wb")) == NULL) {
			return (NOT_OK);
		}
	
	if (record[0]==ICODE || record[0]==TAGIT || record[0]==ISO15693)
		rec_size = 65+strlen(record+65);
	else
		rec_size = strlen(record);
	fseek(fp, 0L, SEEK_END);
	if (fwrite(record, 1, rec_size, fp) != rec_size)
	{	fclose(fp);
		return (NOT_OK);
	}
	fclose(fp);
	return (0);
}


int main_menu(void)
{
	const struct menus mainm[] =
	{
		"1 Scan labels ",
		"2 Read tags   ",
		"3 Write tags  ",
		"4 Get lockstat",
		"5 Scroll DBase",
		"6 System menu ",
		"7 Delete DBase",
		"8 X'mit Data  ",
		"9 Versions    "
	};
	return (menu("  MAIN MENU ", (struct menus *)mainm, 8));
}

int system_menu(void)
{
	const struct menus sysm[] =
	{
		"1 Date/Time   ",
		"2 Com port    ",
		"3 Protocol    ",
		"4 Display     ",
		"5 Barcodes    ",
		"6 Memory      "
	};
	return (menu("SYSTEM MENU ", (struct menus *)sysm, 5));
}


void sel_protocol(int *protocol)
{
	const struct smenus protm[] =
	{
		"No protocol  ", 1,
		"NetO protocol", 2,
		"RCV protocol ", 3,
		"X-Modem      ", 4,
        "Kermit       ", 5,
        "ACK/NAK      ", 6
	};
	sel_menu(" PROTOCOL  ", (struct smenus *)protm, 5, protocol, 0);
}

int com_menu(void)
{
	const struct menus comm[] =
	{
		"1 Port        ",
		"2 Baudrate    ",
		"3 Parity      ",
		"4 Databits    ",
		"5 Stopbits    "
	};
	return (menu("COMPORT MENU", (struct menus *)comm, 4));
}

void sel_databits(int *databits)
{
	const struct smenus databm[] =
	{
		"7 databits   ", 7,
		"8 databits   ", 8,
		"             ", -1
	};
	sel_menu(" DATABITS  ", (struct smenus *)databm, 1, databits, 0);
	switch (*databits) {
	case 7:
		systemsetting("L0");
		break;
	case 8:
		systemsetting("L1");
		break;
	}
}

void sel_stopbits(int *stopbits)
{
	const struct smenus stopbm[] =
	{
		"1 stopbit   ", 1,
		"2 stopbits   ", 2,
		"             ", -1
	};
	sel_menu(" STOPBITS  ", (struct smenus *)stopbm, 1, stopbits, 0);
	switch (*stopbits) {
	case 1:
		systemsetting("L5");
		break;
	case 2:
		systemsetting("L6");
		break;
	}
}

void sel_parity(int *parity)
{
	const struct smenus paritym[] =
	{
		"NONE         ", NONE,
		"EVEN         ", EVEN,
		"ODD          ", ODD
	};
	sel_menu("   PARITY  ", (struct smenus *)paritym, 2, parity, 0);
	switch (*parity) {
	case NONE:
		systemsetting("L2");
		break;
	case EVEN:
		systemsetting("L3");
		break;
	case ODD:
		systemsetting("L4");
		break;
	}
}

void sel_baudrate(int *baudrate)
{
	const struct smenus baudm[] =
	{
		"38400  Bps    ", 3840,
		"19200  Bps    ", 1920,
		" 9600  Bps    ", 960,
		" 4800  Bps    ", 480,
		" 2400  Bps    ", 240,
		" 1200  Bps    ", 120

	};
	sel_menu("  BAUDRATE  ", (struct smenus *)baudm, 5, baudrate, 0);
	switch (*baudrate) {
	case 3840:
		systemsetting("K8");
		break;
	case 1920:
		systemsetting("K7");
		break;
	case 960:
		systemsetting("K6");
		break;
	case 480:
		systemsetting("K5");
		break;
	case 240:
		systemsetting("K4");
		break;
	case 120:
		systemsetting("K3");
		break;
	}

}

void sel_com_port(int *port)
{
	const struct smenus compm[] =
	{
		"DIRECT CABLE  ", COM0,
		"IrDA INTERFACE", COM1,
		"CRADLE        ", COM2,
	};
	sel_menu("COMPORT MENU", (struct smenus *)compm, 2, port, 0);
}

void display_mem(void)
{
	long mem_size;
	mem_size = coreleft();
	printf("\fMEMORY LEFT:\n\n%ld Bytes", mem_size);
	delay(150);					/* 3 seconds */
}


void set_display(void)
{
	int key;
	int value;
	value = contrast(-1);
	printf("\fCHANGE\trCONTRAST\tr\n\trCHANGE\trCONTRAST\n");
	printf("Using: %c and %c", 31, 30);
	for (;;) {
		while ((key=getchar())==EOF)
		{	if (powereddown())
				backlight(OFF);
			idle();
		}

		if (key == ESC)
			return;
		if ((key == UP_KEY) || (key == F7_KEY)) {
			if (value < 31)
				value++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			contrast(value);
		}
		if ((key == DOWN_KEY) || (key == F8_KEY)) {
			if (value > 2)
				value--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			contrast(value);
		}

		if ((key==BL_KEY)||(key==F4_KEY))
			backlight(TOGGLE);

	}

}


void barcode_menu(int *bcode)
{
	static char options[20 + 1];

	const struct smenus barcodem[] =
	{
		"Code39        ", ID_CD39,
		"EAN           ", ID_EAN,
		"UPC           ", ID_UPC,
        "Interl. 2of5  ", ID_I2O5,
        "Indust. 2of5  ", ID_D2O5,
		"Codabar       ", ID_NW7,
		"Code93        ", ID_CD93,
		"Code128       ", ID_CD128,
        "MSI           ", ID_MSI,
        "Telepen       ", ID_TELEPEN,
        "UK/Plessey    ", ID_UK,
        "All addons    ", ID_ADDONS,
        "IATA code     ", ID_IATA,
        "Scode         ", ID_SCODE,
        "All codes     ", ID_ALLCODES
	};

    sel_menu("BARCODE MENU", (struct smenus *)barcodem, 14, bcode, 1);
	strcpy(options, "B0");
	if (*bcode & ID_CD39)
		strcat(options, "B2");
	if (*bcode & ID_EAN)
		strcat(options, "R4");
	if (*bcode & ID_UPC)
		strcat(options, "R1");
	if (*bcode & ID_I2O5)
		strcat(options, "R8");
	if (*bcode & ID_D2O5)
		strcat(options, "R7");
	if (*bcode & ID_NW7)
		strcat(options, "B3");
	if (*bcode & ID_CD93)
		strcat(options, "B5");
	if (*bcode & ID_CD128)
		strcat(options, "B6");
	if (*bcode & ID_MSI)
		strcat(options, "B7");
	if (*bcode & ID_TELEPEN)
        strcat(options, "B9");
    if (*bcode & ID_UK)
        strcat(options, "B1");
	if (*bcode & ID_IATA)
		strcat(options, "B4");
    if (*bcode & ID_SCODE)
        strcat(options, "R9");
    if (*bcode & ID_ADDONS)
        strcat(options, "R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX

    if (*bcode & ID_ALLCODES)
    {   strcpy(options, "B0A0");       // no codes, all codes except add-ons
        strcat(options, "R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX
        systemsetting(options);
        strcpy(options, "JSJZ");       // Chinese post, Laetus
        strcat(options, "D4D7DB");     // Full ASCII, Ital Pharma + leading A
        strcat(options, "YCOG");       // Menu labels, EAN128 if possible
        strcat(options, "IK4V");       // ISBN ISSN if possible
        systemsetting(options);
        return;
    }
    systemsetting(options);

}

int get_string(char *string, int *code_id, int typ, int smin, int smax, int kmax, int x, int y)
{
	static char buf[42];
	static char bcr_buf[42];
	struct barcode code;
	int result;
	int key;
	int count;
	int ok;

	strcpy(buf, string);
	cursor(NOWRAP);
	cursor(ON);
	resetkey();
    *code_id = 0;
	if (typ & SCAN) {
		code.text = bcr_buf;
		code.min = smin;
		code.max = smax;
		scannerpower(SINGLE, 300);
	}
	count = 0;
	gotoxy(x, y);
	buf[0] = '\0';
	for (;;) {
		while (((key=getchar())==EOF) && scanneroff())
		{	if (powereddown())
				backlight(OFF);
			idle();
		}
		if ((count==0) && (typ&SCAN) && ((key==TRIGGER_KEY)||(!scanneroff())) ) {
			while ((result = readbarcode(&code)) != OK) {
				if (powereddown())
					backlight(OFF);
				if (scanneroff())
					scannerpower(SINGLE, 300);
				if (kbhit())
					if ((key = getchar()) != TRIGGER_KEY)
						break;
			}
			if (result == OK) {
				scannerpower(OFF, 0);
				goodreadled(GREEN, 10);
				sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
				strcpy(buf, code.text);
				gotoxy(x, y);
				if (code.length >= (16 - x))
					printf("%s", buf + (code.length - (16 - x)));
				else
					printf("%s", buf);
				strcpy(string, buf);
				cursor(OFF);
				*code_id = code.id;
				return (SCAN);
			}
		}
		scannerpower(OFF, 0);
		sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
		if (!(typ & KEYS)) {
			if (key == ESC) {
				cursor(OFF);
				return (CLEAR);
			} else
				continue;
		}
		if ((key==BL_KEY)||(key==F4_KEY))
			backlight(TOGGLE);

		if (typ & KEYS) {
			switch (key) {
			case DOWN_KEY:
			case RIGHT_KEY:
			case UP_KEY:
			case LEFT_KEY:
			case F5_KEY:
			case F6_KEY:
			case F7_KEY:
			case F8_KEY:
				break;
			case TRIGGER_KEY:
				if (count > 0) {
					memset(buf, ' ', count);
					buf[count] = '\0';
					gotoxy(x, y);
					printf("%s", buf);
					gotoxy(x, y);
				}
				count = 0;
				buf[0]='\0';
				scannerpower(SINGLE, 300);
				break;

			case '\b':			/* backspace */
				
				if(typ != (NUM | KEYS))
				{
					if (count > 0) 
					{
						buf[--count] = '\0';
						if ((x + count) > 13) 
						{
							gotoxy(x, y);
							printf("%s", buf + (count - 14 + x));
						} 
						else 
						{
							gotoxy(x + count, y);
							printf("  ");
							gotoxy(x + count, y);
						}
					}
				}
				else
				{
					if (count > 0) 
					{
						--count;
						
						gotoxy(x, y);
						printf("%s", buf);
						gotoxy(x+count, y);
					}
				}
				break;

			case ESC:
				if(typ != (NUM | KEYS))
				{
					if (count > 0) {
						memset(buf, ' ', count);
						buf[count] = '\0';
						gotoxy(x, y);
						printf("%s", buf);
						gotoxy(x, y);
						count = 0;
					} else {
						cursor(OFF);
						*string = '\0';
						return (CLEAR);
					}
				}
				else
				{
					cursor(OFF);
					
					return (CLEAR);
				}
				break;
			case ENT_KEY:
				if(typ != (NUM | KEYS))
				{
					if (count >= (smin - 1)) {
						cursor(OFF);
						strcpy(string, buf);
						return (KEYS);
					}
				}
				else
				{
					cursor(OFF);
					strcpy(string, buf);
					return (KEYS);
				}

				break;
			default:
				ok = 0;
				if (count < kmax) {		/* max -1 */
					if (typ & FLOAT)
						ok = ((isdigit(key)) || (key == '.'));
					if ((!ok) && (typ & NUM))
						ok = isdigit(key);
					if ((!ok) && (typ & ALPHA))
						ok = ((isalpha(key)) || (key == ' '));
					if (ok) {
						buf[count++] = (char) key;
						
						if(typ != (NUM | KEYS))
							buf[count] = '\0';

						gotoxy(x, y);
						if (x + count < 14)
							printf("%s", buf);
						else
							printf("%s", buf + (count - 14 + x));

						if(typ == (NUM | KEYS))
						gotoxy(x+count, y);
					}
					break;
				}
			}
		}
	}
}

int code_id_to_index(int code_id)
{   int index;
	int value;

	index = -1;
	value = 0;

	do
	{   index++;
		value = symblglist[index].value;
	}
	while( (value!=code_id) && (value!=(-1)) );
	return (index);
}


int scan_labels(void)
{	static char barcode[BARCODE + 1];
	static char symbology[SYMBOLOGY + 1];
	static char timestamp[TIMESTAMP + 1];
	static char datestamp[DATESTAMP + 1];
	static struct date dates;
	static struct time times;
	int code_id;
	int result;
	cursor(NOWRAP);
	setfont(LARGE_FONT,NULL);
	printf("\f");
	gotoxy(0,3);
	printf("Scan or type...");
	for (;;) {
		gotoxy(0,0);
		result = get_string(barcode, &code_id, ALPHA | NUM | SCAN | KEYS, 1, BARCODE, BARCODE, 0, 0);
		if (result == CLEAR)
			return (0);
		gotoxy(0,0);
		printf("%-16.16s", " ");
		gotoxy(0,1);
		printf("%-16.16s\n", barcode);

		setfont(SMALL_FONT,NULL);
		gotoxy(0,5);
		if (result == KEYS)
			sprintf(symbology,"%-*.*s", SYMBOLOGY, SYMBOLOGY, "KEYBOARD");
		else
		{   code_id = code_id_to_index(code_id);
			sprintf(symbology,"%-*.*s", SYMBOLOGY, SYMBOLOGY, symblglist[code_id].item);
		}
		printf("%-16.16s\n", symbology);
		gettime(&times);
		getdate(&dates);
		sprintf(timestamp, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
		sprintf(datestamp, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);
		printf("%-16.16s\n", datestamp);
		printf("%-16.16s", timestamp);
		setfont(LARGE_FONT,NULL);

		sprintf(record, "%-*.*s,%-*.*s,%-*.*s,%-*.*s\r\n",
				BARCODE,   BARCODE,   barcode,
				SYMBOLOGY, SYMBOLOGY, symbology,
				DATESTAMP, DATESTAMP, datestamp,
				TIMESTAMP, TIMESTAMP, timestamp);

		if (write_record(DBASE_NAME, record) == NOT_OK) {
			printf("\fERROR:\nwriting\nfile");
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey;
			return (NOT_OK);
		}
	}
}

unsigned Hex2Bin(unsigned c)
{	if ((c>='0') && (c<='9'))
		return c-'0';
	else
		return c-'A'+10;
}

int ReadIcodeAllBlocks(unsigned char *bufp)
{
    int r;	
    
    r=rfid_ic_getblocks(bufp,0,13);   
    if (r<0)
       return (ERROR);
    
	return (OK);
}


int ReadTagitAll(char *bufp)
{
	char dummy;

    //version is already read in AutoTagTypeDetect()

	if (rfid_ti_getblocks(bufp+8, 0, 8, 4, &dummy)<0)
		return (ERROR);

	return (OK);
}

int ReadISO15693(char *bufp, int *startP, int nrof4blocks)
{
	unsigned int i;
    char dummy;
    
	//systeminfo is already read in AutoTagTypeDetect()
	
	//read blocks max. of 4 at a time using the read multiple block command
	for (i=(*startP); i<nrof4blocks; i++) {		//  max = 16x4x4 blocks of data is 256 bytes
       if (rfid_iso_getblocks_fast(bufp+13+16*i, i*4, 4, 4, &dummy)<0)
            return (ERROR);
	   else 
		   ++(*startP);
    }

	return (OK);
}

int AutoTagTypeDetect(char *bufp) {
	int res;
	if( (res=rfid_ic_getblocks(bufp,0,1))>=0)
		return 0;
	if( (res=rfid_ti_version(bufp))>=0)
		return 1;
	if( (res=rfid_iso_systeminfo(bufp))>=0)	
		return 2;

	return ERROR;
}


int tagpower(int onoff)
{
    if (onoff==ON)
    {
        if (rfidopen()<0)
            return (OFF);
        goodreadled(RED,-1);		// Red LED on
        return (ON);
    }
    rfidclose();

    goodreadled(RED,0);		    // Red LED off
}

int show_taginfo(int rfidtype, char *tagcode, char *write_data, char* timestamp, char* datestamp)
{
	int c,x,y,startpos;
	
	printf("\f");
	setfont(TINY_FONT,NULL);

	if (rfidtype==0)
    {
		printf("TYPE: I-Code        ");
		printf("\nUID: ");
		for (x=7; x>=0; --x)
			printf("%02X", tagcode[x]); 
		printf("\nFamily code: %02X     " , tagcode[16]);
		printf("\nApplication ID: %02X  \n" , tagcode[17]);
		startpos = 20;
    }
    else if(rfidtype==1)
    {
		printf("TYPE: Tag-it         ");
		printf("\nUID: %02X%02X%02X%02X        ",tagcode[0],tagcode[1],tagcode[2],tagcode[3]);
		printf("\nChip type: %02d.%02d",(tagcode[4]>>1),tagcode[5]);
		printf("\nMemory format:%2d*%-02.02d\n",(tagcode[6]+1)&0x1F,tagcode[7]+1);
		startpos = 12;
    }
	else //iso15693
	{
	    printf("TYPE: ISO15693-Tag   ");
		printf("\nUID: ");
		for (x=7; x>=0; --x)
			printf("%02X", tagcode[x]); 

		printf("\nDSFID: %02X   AFI: %02X  ", tagcode[8], tagcode[9]);
		printf("\nMemory format:%02d * %-02.02d \n",tagcode[10]+1,tagcode[11]+1);
		startpos = 13;
    }

	timestamp[TIMESTAMP] = '\0';
	datestamp[DATESTAMP] = '\0';
	printf("Date: %s     \n", datestamp);
	printf("Time: %s       \n", timestamp);
	printf("Data: ");
	
	c=OK;
	
	for(x=y=3;x<30;x++)
	{   
		if (y==10)
		{	
			putchar('\n');
			y=0;
		}
		
		if(write_data==NULL)
			printf("%02X",tagcode[(x-3)+startpos]);
		else
		{
			if(tagcode[(x-3)+startpos] == ((write_data[((x-3)%4)*2]-0x30)<<4) + (write_data[((x-3)%4)*2+1]-0x30))
				printf("%02X",tagcode[(x-3)+startpos]);
			else
			{
				printf("\tr%02X\tr",tagcode[(x-3)+startpos]);
				c=ERROR;
			}
		}
		
		y++;
	}
		
	if(c==ERROR)
		sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, SHIGH, SLOW, SMEDIUM, SHIGH, SMEDIUM, SLOW, NULL);
	else
		sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);

	setfont(LARGE_FONT,NULL);
}

int read_tags(void)
{	
	static char tagcode[64];
	static char timestamp[TIMESTAMP + 1];
	static char datestamp[DATESTAMP + 1];
	struct date dates;
	struct time times;
	int c,x,startpos;
	int rfidtype;

	resetkey();
	cursor(NOWRAP);
	setfont(LARGE_FONT,NULL);
	printf("\f");
	
	gotoxy(0,3);
	
	printf("Read Tag...");
	
	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c==ESC_KEY)
			{	tagpower(OFF);
				return 0;
			}
		}

		starttimer(250);					// wait for OK

		if (tagpower(ON)==OFF)
            return (NOT_OK);

		for(;;)
		{   if (getchar()==ESC_KEY)
			{   tagpower(OFF);
				return 0;
			}

			startpos = 0;

            rfidtype = x = AutoTagTypeDetect(tagcode);
			
			if(rfidtype<0) {
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			if (rfidtype==0)
			    x=ReadIcodeAllBlocks(tagcode);
            else if (rfidtype==1)// TAG-IT
                x=ReadTagitAll(tagcode);
			else //iso15693
				x=ReadISO15693(tagcode, &startpos,2); //read first 32 bytes

			if (x!=ERROR)
				break;
			if (endtimer())
				break;
		}
		
		tagpower(OFF);

		if (x==ERROR)
			continue;

		gettime(&times);
		getdate(&dates);
		sprintf(timestamp, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
		sprintf(datestamp, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);

		show_taginfo(rfidtype, tagcode, NULL, timestamp, datestamp);

		record[0]=rfidtype;		// To identify type of tag record;
		memcpy(record+1,tagcode,64);
		sprintf(record+65, ",%-*.*s,%-*.*s\r\n",
				DATESTAMP, DATESTAMP, datestamp,
				TIMESTAMP, TIMESTAMP, timestamp);

		if (write_record(DBASE_NAME, record) == NOT_OK)
		{	printf("\fERROR:\nwriting\nfile");
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey;
			return (NOT_OK);
		}
	}
}

int write_tags(char* write_data)
{	
	static char tagcode[64];
	static char options[5];
	static char timestamp[TIMESTAMP + 1];
	static char datestamp[DATESTAMP + 1];
	struct date dates;
	struct time times;
	int c,x,startpos;
	int rfidtype;

	int code_id, result;
	int readonly;

	resetkey();
	cursor(NOWRAP);
	
	printf("\f");
	
	setfont(SMALL_FONT,NULL);
	printf("\nPress '.' to set\nthe write data\n  (%s)",write_data); 

	setfont(LARGE_FONT,NULL);
	gotoxy(0,3);
	
	printf("Write Tag...");
		
	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c=='.')
			{
				resetkey();

				printf("\fEnter new write\ndata: %s", write_data);
								
				result = get_string(write_data, &code_id, NUM | KEYS, 1, 8, 8, 6, 1);

				resetkey();
				cursor(NOWRAP);
	
				setfont(SMALL_FONT,NULL);
				printf("\fPress '.' to set\nthe write data\n  (%s)",write_data); 

				setfont(LARGE_FONT,NULL);
				gotoxy(0,3);
				printf("Write Tag...");
				
				continue;
			}
			else if (c==ESC_KEY)
			{	resetkey();
				tagpower(OFF);
				return 0;
			}
		}

		starttimer(250);					// wait for OK

		if (tagpower(ON)==OFF)
            return (NOT_OK);

		readonly = -1; 

		for(;;)
		{   if (getchar()==ESC_KEY)
			{   
				tagpower(OFF);
				return 0;
			}

			startpos = 0;

            rfidtype = x = AutoTagTypeDetect(tagcode);
			
			if(rfidtype<0) {
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			if(readonly == -1)
			{
				if (rfidtype==0)
				{
					x = icode_write(write_data);

					if(x!=ERROR)
						readonly = rfidtype;
				}
			    else if (rfidtype==1)// TAG-IT
				{
					x = tagit_write(write_data);

					if(x!=ERROR)
						readonly = rfidtype;
				}
				else //iso15693
				{
					if( tagcode[6] == 0x04 )	//Set option bit for writing of ISO15693 Philips OMRON tags
					{
						options[0] = 0x01;
						rfidsetoptions(options);
					}

					x = iso15693_write(write_data);
					
					if( tagcode[6] == 0x04 )	//Reset option bit after writing of ISO15693 Philips OMRON tags
					{
						options[0] = 0x00;
						rfidsetoptions(options);
					}

					if(x!=ERROR)
						readonly = rfidtype;
				}
				if(x==ERROR)
					readonly = rfidtype;
			}

			x = ERROR;

			if (readonly==0)
			{
				x=ReadIcodeAllBlocks(tagcode);
			}
            else if (readonly==1)					// read tagit
			{
				x=ReadTagitAll(tagcode);
			}
			else if(readonly!=-1) //iso15693
			{
				x=ReadISO15693(tagcode, &startpos,2); //read first 32 bytes
			}

			if (x!=ERROR)
				break;
			if (endtimer())
				break;
		}
		
		tagpower(OFF);

		if (x==ERROR)
			continue;

		gettime(&times);
		getdate(&dates);
		sprintf(timestamp, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
		sprintf(datestamp, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);

		show_taginfo(rfidtype, tagcode, write_data, timestamp, datestamp);
		
	}
}

// ************************************************************************
// icode_write: demonstrates the writing of blocks 5 to 15
// ************************************************************************
int icode_write(char* data)
{
    int i;
    int err=0;
    static unsigned char buf[5];
	int start, nrofblcks;
	
	start = 5;
	nrofblcks = 8;
    
    printf("\fI-Code detected\nStart writing\n");
	
	for (i=0; i<4; i++)
        buf[i] = ((data[i*2]-0x30)<<4) + (data[i*2+1]-0x30);

	i = 0;
	
	starttimer(250);					// wait for OK
	resetkey();
	
	for(;;)
    {
        if (getchar()==ESC_KEY)
		{   
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}
				
		if(err=rfid_ic_putblock(buf, i+start)==0) 
		{
			gotoxy(0,3);
			++i;
			printf("%02d bytes written", i*4);
			starttimer(100);					// wait for OK
		}
			
		if(i>=nrofblcks) 
			break;

		if (endtimer()) 
			return ERROR;
    }
   
	starttimer(150);					// wait for OK
	return OK;
}

// ************************************************************************
// tagit_write: demonstrates the writing of all data blocks of tagit
// this test assumes 8 blocks of 4 bytes
// ************************************************************************
int tagit_write(char* data)
{
    unsigned char error;
    int i, err;
    static unsigned char buf[5];
	int start, nrofblcks;

	start = 0;
	nrofblcks = 8;

	for (i=0; i<4; i++)
        buf[i] = ((data[i*2]-0x30)<<4) + (data[i*2+1]-0x30);

	i=0;

	starttimer(250);					// wait for OK

    printf("\fTag-it detected\nStart writing\n");

    for(;;)
	{   
		if (getchar()==ESC_KEY)
		{   
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}

		if((err=rfid_ti_putblock(buf, i+start, 4, &error))>=0) 
		{
			gotoxy(0,3);
			++i;
			printf("%02d bytes written", i*4);
			starttimer(100);					// wait for OK
		}
			
		if(i>=nrofblcks) 
			break;

		if (endtimer()) 
			return ERROR;
	}
    
    starttimer(150);					// wait for OK
	return OK;

}

// ************************************************************************
// iso15693_write: demonstrates the writing of first 32 data blocks.
// ************************************************************************
int iso15693_write(char* data)
{
    unsigned char error;
    int i, err;
    static unsigned char buf[5];
	int start, nrofblcks;
	
	start = 0;
	nrofblcks = 8;

    printf("\fISO15693 detected\nStart writing\n");

    starttimer(250);					// wait for OK
	
	for (i=0; i<4; i++)
        buf[i] = ((data[i*2]-0x30)<<4) + (data[i*2+1]-0x30);

	i=0;

	for(;;)
	{   
		if (getchar()==ESC_KEY)
		{   
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}

        if((err=rfid_iso_putblock (buf, i+start, 4, &error))==0) 
		{
			gotoxy(0,3);
			++i;
			printf("%02d bytes written", i*4);
			starttimer(100);					// wait for OK
		}
		//break if block is locked or not present so skip this block
		else if(err==-3 && error>=0x10 && error<=0x12) 
		{
			gotoxy(0,3);
			++i;
			printf("%02d bytes written", i*4);
			starttimer(100);					// wait for OK
		}
	
		if(i>=nrofblcks) 
			break;

		if (endtimer()) 
			return ERROR;
	}
	
	starttimer(150);					// wait for OK
	return OK;
}

int read_lock_status(void)
{
	static char tagcode[16];
	int c,x,startpos;
	int rfidtype;

	resetkey();
	cursor(NOWRAP);
	
	printf("\f");
	gotoxy(0,3);
	
	printf("Read Lock status");
		
	for (;;) {
		gotoxy(0,0);						// Wait for trigger key
		for(;;)
		{	c=getchar();
			if (c==TRIGGER_KEY)
				break;
			else if (c==ESC_KEY)
			{	resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return 0;
			}
		}

		starttimer(250);					// wait for OK

		if (tagpower(ON)==OFF)
            return (NOT_OK);

		for(;;)
		{   if (getchar()==ESC_KEY)
			{   
				resetkey();
				tagpower(OFF);
				setfont(LARGE_FONT,NULL);
				return 0;
			}

			startpos = 0;

            rfidtype = x = AutoTagTypeDetect(tagcode);
			
			if(rfidtype<0) 
			{
				if (x!=ERROR)
					break;
				if (endtimer())
					break;
				continue;
			}

			if (rfidtype==0) //I-code
				x = icode_lockstatus();
			else if (rfidtype==1)// TAG-IT
				x = tagit_lockstatus();
			else //iso15693
				x = iso15693_lockstatus(tagcode[10]+1);
						
			if (x!=ERROR)
				break;

			if (endtimer())
				break;
		}
		
		tagpower(OFF);

		if (x==ERROR)
			continue;

		sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
	}
}

// ************************************************************************
// icode_lockstatus: shows the lock status of the data blocks. 
// ************************************************************************
int icode_lockstatus(void)
{
    int r;	
	static unsigned char bufp[4];

	starttimer(50);					// wait for OK
	resetkey();

	setfont(TINY_FONT,NULL);

	printf ("\fI-Code lock status:\n(0 = Unlocked)");
    
	for(;;)
	{   
		if (getchar()==ESC_KEY)
		{   
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}

		r=rfid_ic_getblocks(bufp,2,1);   
    
		if(r>=0) 
		{
			//show all statusses
			for(r=0;r<4*8;++r) 
			{
				if(r%16==0) 
					printf("\n\nBlk %02d - %02d:\n",r,r+15);
				
				if( bufp[r/8] & (1 << (r%8))== 0)
					printf("1");
				else
					printf("0");
			}
			
			return OK;
		}

		if (endtimer()) 
			return (ERROR);	
	}
}

// ************************************************************************
// tagit_lockstatus: shows the lock status of the data blocks. 
// ************************************************************************
int tagit_lockstatus(void)
{
    unsigned char error;
    int i, j, err;
    static unsigned char buf[32];

	setfont(TINY_FONT,NULL);

	printf ("\fTag-it lock status\n(0 = Unlocked)\n");

	 //read all 64 lock status, 8 x 8
	starttimer(100);					// wait for OK
	j=0;
	
	for(;;)
	{   
		if (getchar()==ESC_KEY)
		{   
			resetkey();
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}

		for(i=j;i<8;++i) 
		{
			if((err=rfid_ti_lockstat(buf, i, 4, &error))<0) 
				break;
			else 
			{
				if(i==0)
					printf("\nBlk 00 - 08:\n\n");
				
				printf("%c ", '0'+(buf[0]&3));
				++j;
			}
        }

		if(i==8) 
			break;

		if (endtimer()) 
		{
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}
	}

	setfont(LARGE_FONT,NULL);

	return (OK);
}

// ************************************************************************
// iso15693_lockstatus: shows the lock statusses of the data blocks. 
// ************************************************************************
int iso15693_lockstatus(unsigned char nr_of_blocks)
{
    unsigned char error;
    int x, i, j, k;
    static unsigned char buf[8];

	setfont(TINY_FONT,NULL);

    printf("\fISO15693 lockstatus:\n");

     //read all lock status
	starttimer(150);					// wait for OK
	j=0;
	
	for(;;)
	{   
		if (getchar()==ESC_KEY)
		{   
			resetkey();
			setfont(LARGE_FONT,NULL);
		    return ERROR;
		}
		       
		for(i=j;i<((nr_of_blocks-1)/8)+1;++i) 
		{
			if((x=rfid_iso_lockstats(buf, i*8, ((((i+1)*8)<=nr_of_blocks)?(8):((nr_of_blocks)-(i*8))), &error))<0) 
				break;
			else 
			{
				++j;

				if(i%2==0) 
					printf("Blk %02d - %02d:\n",i*8,(((i*8)+15 <= nr_of_blocks-1) ? ((i*8)+15) : nr_of_blocks-1));
				
				for(k=0;k< (((i*8)+7 <= nr_of_blocks-1) ? (8) : (nr_of_blocks - (i*8))) ;++k)
					printf("%c", '0'+(buf[k]&3));
		
				if(i%2==1) 
					printf("\n");
			}
        }

		if(x>=0) 
			break;

		if (endtimer()) 
		{
			x = -4;
			break;
		}
	}

	if (x<0) 
	{
		setfont(LARGE_FONT,NULL);
	    return ERROR;        
	}
	
	setfont(LARGE_FONT,NULL);

    return(OK);
}

int scroll_dbase(void)
{	static char barcode[BARCODE + 1];
	static char symbology[SYMBOLOGY + 1];
	static char timestamp[TIMESTAMP + 1];
	static char datestamp[DATESTAMP + 1];
	int count, max;
	int key,x,y;
	int startpos;

	setfont(SMALL_FONT,NULL);
	cursor(NOWRAP);
	if ((max = get_max_records(DBASE_NAME, REC_SIZE)) == 0L) {
		printf("\fDATABASE  EMPTY\n\nPress any key");
		while (getchar()==EOF)
		{	if (powereddown())
				backlight(OFF);
			idle();
		}
		resetkey();
		setfont(LARGE_FONT,NULL);
		printf("\f");
		return (0);
	}
	if (max == NOT_OK) {
		printf("\fERROR:\nreading\nfile");
		while (getchar()==EOF)
		{	if (powereddown())
				backlight(OFF);
			idle();
		}
		resetkey();
		setfont(LARGE_FONT,NULL);
		printf("\f");
		return (NOT_OK);
	}
	count = 0;
	putchar('\f');
	for (;;) {
		gotoxy(0, 0);
		if (count == 0)
			printf("\tr  Scroll Data %c\tr", 31);
		else if (count == (max - 1))
			printf("\tr%c Scroll Data  \tr", 30);
		else
			printf("\tr%c Scroll Data %c\tr", 30, 31);
		if (read_record(DBASE_NAME, record, count, REC_SIZE) == NOT_OK) {
			printf("\fERROR:\nreading\nFILE");
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey();
			setfont(LARGE_FONT,NULL);
			printf("\f");
			return (NOT_OK);
		}
		if (record[0]==ICODE || record[0]==TAGIT || record[0]==ISO15693)
		{   setfont(TINY_FONT,0);
			gotoxy(0,2);
         
			if (record[0]==ICODE || record[0]==TAGIT || record[0]==ISO15693)  
			{
				if (record[0]==ICODE)  
				{
					printf("TYPE: I-Code        ");
					printf("\nUID: ");
					for (x=7; x>=0; --x)
						printf("%02X", record[x+1]); 
					printf("\nFamily code: %02X     " , record[16+1]);
					printf("\nApplication ID: %02X  \n" , record[17+1]);

					startpos = 20;
				}
				else if (record[0]==TAGIT) 
				{	//32	00018B1A	Transponder serial number is 00018B1A
					//7		0000001		Chip manufacturer code (1=Texas Instruments)
					//9		000000101	Chip version 5
					//3		000			Reserved
					//5		00011		Block size (in bytes) minus one (3+1=4 bytes=32 bits)
					//8		07			Number of blocks minus one (7+1=8 blocks)
					//32    31313131    Lock stats?
					//256	data
					printf("TYPE: Tag-it         ");
					printf("\nUID: ");
					for(x=0; x<4; ++x)
						printf("%02X",record[x+1]);
					printf("\nChip type: %02d.%02d",(record[4+1]>>1),record[5+1]);
					printf("\nMemory format:%2d*%-02.02d\n",(record[6+1]+1)&0x1F,record[7+1]+1);
					startpos = 12;
				}
				else if (record[0]==ISO15693)
				{	//show ISO15693 tag info
					printf("TYPE: ISO15693-Tag   ");
					printf("\nUID: ");
					for (x=7; x>=0; --x)
						printf("%02X", record[x+1]); 

					printf("\nDSFID: %02X   AFI: %02X  ", record[8+1], record[9+1]);
					printf("\nMemory format:%02d * %-02.02d \n",record[10+1]+1,record[11+1]+1);
					startpos = 13;
				}
				
				strncpy(timestamp, record+BARCODE+1+SYMBOLOGY+1+DATESTAMP+1, TIMESTAMP);
				timestamp[8] = '\0';
				printf("Time: %s       \n", timestamp);
				printf("Data: ");
				
				for(x=y=3;x<30;x++)
				{   
					if (y==10)
					{	putchar('\n');
						y=0;
					}
					printf("%02X",record[(x-3)+startpos+1]);
					y++;
				}
			}
			
			setfont(SMALL_FONT,0);
		}
		else
		{   setfont(SMALL_FONT,0);
			gotoxy(0,1);
			printf("                    \n\n\n");
			printf("                    \n");
			printf("                    ");

			strncpy(barcode  , record,                                   BARCODE);
			strncpy(symbology, record+BARCODE+1,                         SYMBOLOGY);
			strncpy(datestamp, record+BARCODE+1+SYMBOLOGY+1,             DATESTAMP);
			strncpy(timestamp, record+BARCODE+1+SYMBOLOGY+1+DATESTAMP+1, TIMESTAMP);
			barcode[BARCODE]     = '\0';
			symbology[SYMBOLOGY] = '\0';
			datestamp[DATESTAMP] = '\0';
			timestamp[TIMESTAMP] = '\0';

			gotoxy(0, 2);
			printf("%-16.16s\n", barcode);
			printf("%-*.*s\n\n",   (BARCODE-16),(BARCODE-16),barcode+16);

			setfont(SMALL_FONT,0);
			gotoxy(0,4);
			printf("%-*.*s\n%-*.*s\n%-*.*s               \n                ",
				SYMBOLOGY, SYMBOLOGY, symbology,
				DATESTAMP, DATESTAMP, datestamp,
				TIMESTAMP, TIMESTAMP, timestamp );
		}
		while ((key=getchar())==EOF)
		{	if (powereddown())
				backlight(OFF);
			idle();
		}
		if (key == ESC)
			break;
		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case F7_KEY:
		case Q1_KEY:
			if (count > 0)
				count--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		case DOWN_KEY:
		case F8_KEY:
		case Q2_KEY:
			if (count < (max - 1))
				count++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
	setfont(LARGE_FONT,NULL);
	printf("\f");
	return (0);
}



void main(void)
{
	int c,test;
	int res;
	int key;
	static char os[8+1];
	static char firmware[8+1];
	static char write_data[8];
	char *lib;
		
	port = COM0;				/* direct Cable is default */
	baudrate = 960;
	parity = NONE;
	stopbits = 1;
	databits = 8;
	protocol = 1;				/*no protocol default */
	s_code  = ID_CD39|ID_EAN|ID_UPC|ID_I2O5|ID_D2O5|ID_NW7;
	s_code |= ID_CD93|ID_CD128|ID_MSI|ID_TELEPEN|ID_UK;
	s_code |= ID_IATA|ID_SCODE;
	systemsetting("U2B0");      // default + disable all codes
	systemsetting("B2R4R1R8R7B3");  // CD39,EAN,UPC,I2O5,,D2O5,NW7 enabled
	systemsetting("B5B6B7B9B1");    // CD93,CD128,MSI,TELEPEN,UK/PLESSEY enabled
	systemsetting("B4R9");          // IATA, SCODE enabled ; Menu + addons not enabled
	resetkey();

	for(c=0;c<8;++c)
		write_data[c] = '0';
	write_data[c] = '\0';
	
	for (;;) {
		test = main_menu();
		if (test == 0) 
		{
			scan_labels();
			continue;
		}
		if (test == 1) 
		{
			read_tags();
			continue;
		}
		if (test == 2) 
		{
			write_tags(write_data);
			continue;
		}
		if (test == 3) 
		{
			read_lock_status();
			continue;
		}
		if (test == 4) 
		{
			scroll_dbase();
			continue;
		}
		if (test == 5) 
		{
			for (;;) 
			{
				if ((test = system_menu()) == -1)
					break;
				if (test == 0) 
				{
					set_time_and_date();
					continue;
				}
				if (test == 1) {
					for (;;) {
						if ((test = com_menu()) == -1)
							break;
						if (test == 0) {
							sel_com_port(&port);
							continue;
						}
						if (test == 1) {
							sel_baudrate(&baudrate);
							continue;
						}
						if (test == 2) {
							sel_parity(&parity);
							continue;
						}
						if (test == 3) {
							sel_databits(&databits);
							continue;
						}
						if (test == 4) {
							sel_stopbits(&stopbits);
							continue;
						}
					}
					continue;
				}
				if (test == 2) {
					sel_protocol(&protocol);
					continue;
				}
				if (test == 6) {
					display_mem();
					continue;
				}
				if (test == 4) {
					barcode_menu(&s_code);
					continue;
				}
				
				if (test == 3) {
					set_display();
					continue;
				}
			}
		}
		if (test == 6) {
			printf("\fDelete DBASE\n\n[ENT] = YES\n[CLR] = NO");
			while ((key=getchar())==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			if (key == ENT_KEY)
				remove(DBASE_NAME);
			continue;
		}
		if (test == 7) {
			printf("\fTransmit data\nPress any key");
			while ((key=getchar())==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			if (key == ESC)
				continue;
			printf("\fTransmitting..");
			switch (protocol) {
			case 1:
				res = send_plain(DBASE_NAME);
				break;
			case 2:
			case 4:
				if (comopen(port) != 0) {
					res = 2;
					break;
				}
				if (protocol == 2)
				{	//strcpy(database_array[1],DBASE_NAME);
					//res = neto_transmit(database_array, 1, "123456", ESC_KEY, 3);
					res = neto_transmit( (char(*)[12+1])DBASE_NAME, 1, "123456", CLR_KEY, 3 );
				}
				else
					res = xmodemtransmit(DBASE_NAME);
				comclose(port);
				break;
			case 3:
				res = send_slt_prot(DBASE_NAME);
				break;
			case 5:
				res = kermit_send(DBASE_NAME);
				break;
			case 6:
				res = send_ack_nak(DBASE_NAME);
				break;
			}
			if (res == OK)
				printf("\ftransmission OK\npress any key");
			else
				printf("\ferror\n= %d\npress any key", res);
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey();
			continue;
		}
		if (test == 8) {
			printf("\fSoftware Version");
			version_os(os);
			os[3] = '\0';
			os[1] = 'F';
			printf("\n   %s%s\n press any key", os,VERSIONSUFFIX);
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey();
					  
			printf("\fLibrary Version ");
			
			lib = rfid_lib_version();
			
			printf("\n   %s\n press any key",lib);
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey();
					  
			printf("\fFirmware Version");

		    if (tagpower(ON)==OFF)
				printf("\nFirmware ERROR\n");
			else
				if (rfidversion (firmware)<0)
					printf("\nFirmware ERROR\n");
			else
				printf("\n   %s\n", firmware);
			tagpower(OFF);
			
			printf(" press any key");
			while (getchar()==EOF)
			{	if (powereddown())
					backlight(OFF);
				idle();
			}
			resetkey();
			continue;
		}
		
	}
}
