/************************************************************************/
/* Opticon PHL-2700 TIRIS RFID test program           				    */
/* Program for testing all the functions of the TIRIS RFID library      */
/* Compiler: Microtec C compiler for Melps 7700  Version 2.2            */
/************************************************************************/


/************************************************************************/
/*      Include                                                         */
/************************************************************************/
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include        "lib.h"
#include        "tiris.h"

/************************************************************************/
/*      Define                                                          */
/************************************************************************/
#define VERSIONSUFFIX  "23651"
#define NOT_OK         -1
#define CLEAR          -1
#define SCAN            1
#define KEYS            2
#define ALPHA           8
#define NUM             16
#define FLOAT           32
#define NONE            1
#define EVEN            2
#define ODD             3


struct xmenus {
	char *item;                 // menu item string
    void(* itemfunc)(void);     // function to be executed on selection of item
};

struct smenus {
	char *item;                 // menu item string
	int value;                  // value to be set on selection of item
};

/************************************************************************/
/*      Global                                                          */
/************************************************************************/
static int port;
static int baudrate;
static int parity;
static int stopbits;
static int databits;
static int protocol;

int current_page = 1;
int displayHex = FALSE;
unsigned char data[8] = {'0','0','0','0','0','0','0',0};

/************************************************************************/
/*      extern                                                          */
/************************************************************************/
/************************************************************************/
/*      Prototype                                                       */
/************************************************************************/
int xmenu(char *menu_text, struct xmenus *tab, int max); 
int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more);

int main_menu(void);
int system_menu(void);

void main(void);

int module_power(int onoff);
void wait_key(void);

void test_TIRIS(void);

void Read_ID(void);
void Read_Page(void);
void Program_Page(void); 
void Lock_Page(void);

void tiris_read(int pagenr, int selective);
void tiris_program(int pagenr, int selective);
void tiris_lock(int pagenr, int selective);

void selectPage(void);

void processError(int error);

void library_version(void);

void scan_barcode(void);
void read_barc(char *conf_bar);

/*********************************4***************************************/
/* the execution function for a selection menu                          */
/************************************************************************/
int sel_menu(char *menu_text, struct smenus *tab, int max, int *value, int more)
{
	int count;
	int index;
	int selected;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		if (index > 2)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count<3; count++) {
			selected = (more && (tab[count + calcu].value & (*value))) ||
				(!more && (tab[count + calcu].value == (*value)));
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%c%s\tr", selected ? '*' : ' ', tab[count + calcu].item);
			else
			    if ((count+calcu)>max)
                    printf("              ");
                else
                	printf("%c%s", selected ? '*' : ' ', tab[count + calcu].item);

		}
		while ((key=getchar())==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}

		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case LEFT_KEY:
		case F5_KEY:
		case F7_KEY:
		case Q1_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
		case RIGHT_KEY:
		case F8_KEY:
		case F6_KEY:
		case Q2_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':

			if ((key - '1') <= max) {
				if (more)
					(*value) ^= tab[(key - '1')].value;
				else {
					(*value) = tab[(key - '1')].value;
					return (key - 1);
				}
			} else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ESC:
			return (-1);
		case ENT_KEY:
		case TRIGGER_KEY:
			if (more)
				(*value) ^= tab[index].value;
			else {
				(*value) = tab[index].value;
				return (index);
			}

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}

/************************************************************************/
/* the execution function for an action menu                            */
/************************************************************************/
int xmenu(char *menu_text, struct xmenus *tab, int max)
{
	int count;
	int index;
	int calcu;
	int key;
	index = 0;
	count = 0;
	putchar('\f');				/* clear screen */
	for (;;) {
		gotoxy(0, 0);
		if (index == 0)
			printf(" %s%c", menu_text, 31);
		else if (index == max)
			printf("%c%s ", 30, menu_text);
		else
			printf("%c%s%c", 30, menu_text, 31);
		if (index > 2)
			calcu = index - 2;
		else
			calcu = 0;
		for (count = 0; count < 3; count++) {
			gotoxy(0, count + 1);
			if (index == (count + calcu))
				printf("\tr%s\tr", tab[count + calcu].item);
			else
                if ((count+calcu)>max)
                    printf("              ");
                else            
				    printf("%s", tab[count + calcu].item);
		}

		while ((key=getchar())==EOF)
		{   if (powereddown())
				backlight(OFF);
			idle();
		}

		switch (key) {

		case BL_KEY:
		case F4_KEY:
			backlight(TOGGLE);
			break;

		case UP_KEY:
		case LEFT_KEY:
		case F7_KEY:
		case F5_KEY:
		case Q1_KEY:
			if (index > 0)
				index--;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case DOWN_KEY:
		case RIGHT_KEY:
		case F8_KEY:
		case F6_KEY:
		case Q2_KEY:
			if (index < max)
				index++;
			else
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;


		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			if ((key - '1') <= max)
				return (key - '1');
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;

		case ESC:
			return (-1);
		case ENT_KEY:
		case TRIGGER_KEY:
			return (index);

		default:
			sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			break;
		}
	}
}


/************************************************************************/
/* prototypes for menu item functions                                   */
/************************************************************************/
void general_xmenu(void);
void tiris_xmenu(void);
int tiris_transponder_xmenu(void);

/************************************************************************/
/* Menu functions                                                       */
/* There are 2 types of menus: action menus and selection menus.        */
/* The names of action menus end on _xmenu, selection menus on _smenu.  */
/* Action menus have items that are executed when selected. These items */
/* can be submenus or actions such as "Read Tags"                       */
/* Selection menus have items that present a mutually exclusive choice. */
/* The structure of the menus is defined by a header, the menu item     */
/* strings, the menu action function names (for an xmenu), and the name */
/* and possible values of a variable (for an smenu).                    */
/* These are all defined together in the upper part of the functions    */
/* so the menu structure can be easily modified.                        */
/* For example, a submenu can be added by copying an _xmenu or _smenu   */
/* function, changing its name and the menu definitions, and adding its */
/* item string and the function name to the menu table in another xmenu.*/
/************************************************************************/
void main_xmenu(void)
{
    static char myheader[]="  MAIN MENU ";
	static struct xmenus mymenu[] =
	{
		"1 TIRIS      ", tiris_xmenu	,
		"2 General    ", general_xmenu
	};
    // ### edit the above lines to define the menu ###
    
    int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;
        mymenu[i].itemfunc();
    }
}


void general_xmenu(void)
{
    static char myheader[]=" GENERAL     ";
	static struct xmenus mymenu[] =
	{
		"1 Scan barcode ", scan_barcode	,
		"2 Test hardware", test_TIRIS	,
		"3 Lib. version ", library_version
	};
    // ### edit the above lines to define the menu ###

    int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;    
        mymenu[i].itemfunc();
    }
}

void tiris_xmenu(void)
{
    static char myheader[]="   TIRIS     ";
	static struct xmenus mymenu[] =
	{
		"1 Read ID     ",	Read_ID					,
		"2 Read Page   ",	Read_Page				,
		"3 Program Page",	Program_Page			,
		"4 Lock Page   ",	Lock_Page
	};

	// ### edit the above lines to define the menu ###
	int i;
    for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
        if (i<0)
            return;    
        mymenu[i].itemfunc();
    }
}

int tiris_transponder_xmenu(void)
{
    static char myheader[]=" SELECT TYPE ";
	static struct xmenus mymenu[] =
	{
		"1 Read only     ",	idle,
		"2 Read/Write    ",	idle,
		"3 Multiple.page ",	idle,
		"4 SAMPT         ",	idle,
		"5 SAMPTS        ",	idle
	};

	// ### edit the above lines to define the menu ###
	int i=0;
	for (;;)
    {
        i=xmenu(myheader, mymenu, sizeof (mymenu)/sizeof(struct xmenus)-1);
                
		if(i<0) 
			return i;
		//mymenu[i].itemfunc();

		return i;

    }
}

void scan_barcode(void)
{
	printf("\fCurrent data:\n");
	
	printf("%08s",data);

	printf("\nScan barcode:\n");
	read_barc(data);

	wait_key();
}

// general wait-key function
void wait_key(void)
{
    printf("press any key...");
    while (getchar()==EOF)
	{   if (powereddown())
			backlight(OFF);
		idle();
	}
}


/************************************************************************/
/* Actual test functions that are called from the menus                 */
/************************************************************************/

// ************************************************************************
// firmware version: gets and prints the firmware version of the RFID module
// ************************************************************************
void library_version(void)
{
    char* vrs;

    printf("\flibrary version");
    
    vrs  = tiris_lib_version();
       
    printf("\n%s\n", vrs);
    
    wait_key();
}

void test_TIRIS(void)
{
	int result;

	setfont( SMALL_FONT, NULL );

	module_power(ON);
	
	if((result=tiris_init())<0) {
		processError(result);
		module_power(OFF);
		setfont( LARGE_FONT, NULL );
		return;
	}

	printf("\fAntenna ON...\n");

	tiris_antenna_power(1);
		
	resetkey();
	while(!kbhit()) idle();
	resetkey();

	tiris_antenna_power(0);

	printf("\nAntenna OFF...\n");

	tiris_close();

	module_power(OFF);

	printf("\nTest successful\n\n");

	wait_key();

	setfont( LARGE_FONT, NULL );

	return;
}

void Read_ID(void) 
{
	int tag_type;
	
	tag_type = tiris_transponder_xmenu();

	if(tag_type==-1)
		return;
	
	setfont( SMALL_FONT, NULL );
	
	printf("\fReading ID...\n");
	
	switch(tag_type) 
	{
	case READ_ONLY_TRANSPONDER:
	case READ_WRITE_TRANSPONDER:
	case MPT_TRANSPONDER:
		tiris_read(0, FALSE);
		break;
	case SAMPT_TRANSPONDER:
	case SAMPTS_TRANSPONDER:
		tiris_read(1, FALSE);
	}
		
	setfont( LARGE_FONT, NULL );
}

void Read_Page(void) 
{
	int selective = FALSE;
	int tag_type = tiris_transponder_xmenu();

	if(tag_type==-1)
		return;
	
	setfont( SMALL_FONT, NULL );

	switch(tag_type) 
	{
	case SAMPT_TRANSPONDER:
	case SAMPTS_TRANSPONDER:
		selective = TRUE;
	case MPT_TRANSPONDER:
		printf("\fSelect page\nto read\n\nCurrent page: %d?",current_page);
		selectPage();
		printf("\fReading page %d..\n",current_page);
		tiris_read(current_page, selective);
		break;
	case READ_ONLY_TRANSPONDER:
	case READ_WRITE_TRANSPONDER:
	default:
		resetkey();
		printf("\f\nThis operation\nis not possible\nwith this type\nof transponder\n\n");
		wait_key();
	}
		
	setfont( LARGE_FONT, NULL );
}


void Program_Page(void) 
{
	int tag_type = tiris_transponder_xmenu();

	if(tag_type==-1)
		return;
	
	setfont( SMALL_FONT, NULL );

	switch(tag_type) 
	{
	case SAMPT_TRANSPONDER:
	case SAMPTS_TRANSPONDER:
		printf("\fSelect page\nto program\n\nCurrent page: %d?",current_page);
		selectPage();
		printf("\fProgramming\npage %d...\n",current_page);
		tiris_program(current_page, SAMPT_TRANSPONDER);
		break;
	case MPT_TRANSPONDER:
		printf("\fSelect page\nto program\n\nCurrent page: %d?",current_page);
		selectPage();
		printf("\fProgramming\npage %d...\n",current_page);
		tiris_program(current_page, MPT_TRANSPONDER);
		break;
	case READ_WRITE_TRANSPONDER:
		printf("\fProgramming R/W.\n");
		tiris_program(current_page, READ_WRITE_TRANSPONDER);
		break;
	case READ_ONLY_TRANSPONDER:
	default:
		resetkey();
		printf("\f\nThis operation\nis not possible\nwith this type\nof transponder\n\n");
		wait_key();
	}

	setfont( LARGE_FONT, NULL );
}

void Lock_Page(void) 
{
	unsigned char key;
	int selective = FALSE;
	int tag_type = tiris_transponder_xmenu();

	if(tag_type==-1)
		return;
	
	setfont( SMALL_FONT, NULL );

	switch(tag_type) 
	{
	case SAMPT_TRANSPONDER:
	case SAMPTS_TRANSPONDER:
		selective = TRUE;
	case MPT_TRANSPONDER:
		printf("\fSelect page\nto lock\n\nCurrent page: %d?",current_page);
		selectPage();

		printf("\fPage %d will be\npermanently\nlocked",current_page);
		printf("\n\nPress trigger\nto continue");
		while(!kbhit()) idle;

		if((key=getchar())!=TRIGGER_KEY && key!=ENT_KEY) {
			setfont( LARGE_FONT, NULL );
			return;
		}

		printf("\fLocking page %d..\n",current_page);
		tiris_lock(current_page, selective);
		break;
	case READ_WRITE_TRANSPONDER:
	case READ_ONLY_TRANSPONDER:
	default:
		resetkey();
		printf("\f\nThis operation\nis not possible\nwith this type\nof transponder\n\n");
		wait_key();
	}
	
	setfont( LARGE_FONT, NULL );
}

void tiris_read(int pagenr, int selective)
{
    int i, result, error;
	char selective_addr[3];

	module_power(ON);
	
	if((result=tiris_init())<0) {
		processError(result);
		tiris_close();
		module_power(OFF);
		return;
	}

    resetkey();

	result = error = -1;

	while(result<0) {
		if(!selective) {
			if(pagenr==0)
				result = tiris_charge_only_read();
			else 
				result = tiris_read_page(pagenr);
		}
		else 
		{
			if( (result=tiris_read_page(1))>=0 ) {
				selective_addr[0] = recv_data[0];
				selective_addr[1] = recv_data[1];
				selective_addr[2] = recv_data[2];
				result = tiris_selective_read_page(pagenr, selective_addr);
			}
		}

		//the error return values are numbered from -1 (least fatal error) to -10 (most fatal error) 
		//only the most fatal error that occurs will be stored in error;
		if(result<0 && result<error)
			error = result;

		//fatal errors
		if(result==LOCKED_PAGE_ADDRESS_ERROR ||	result==UNLOCKED_PAGE_ADDRESS_ERROR	||
			result==HARDWARE_FAILURE) {
			sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
			break;
		}

		if(kbhit() && getchar()==CLR_KEY) {
			result = error;
			break;
		}
	}
		
	if(result<0)
		processError(result);
	else {
		printf("\n");
		if(result==READ_ONLY_OK) {
			printf("Read only, Page %d:\n\n",pagenr);
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);
			
			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}		
		else if(result==READ_WRITE_OK) {
			printf("Read/Write, ID:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else if(result==LOCKED_PAGE_READ_OK) {
			printf("Locked Page\nread, DATA:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else if(result==UNLOCKED_PAGE_READ_OK) {
			printf("Unlocked Page\nread, DATA:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else 
			printf("Unexpected reply: %d",result);

		printf("\n");

	}
	
	tiris_close();
	
	module_power(OFF);

	wait_key();

	setfont( LARGE_FONT, NULL );
}


void tiris_program(int pagenr, int transponder_type) 
{
	int i, result, error;
	char selective_addr[3];

	setfont( SMALL_FONT, NULL );
	
	module_power(ON);
	
	if((result=tiris_init())<0) {
		processError(result);
		tiris_close();
		module_power(OFF);
		return;
	}
	
    resetkey();
	
	result = error = -1;

	while(result<0) 
	{
		if(transponder_type == MPT_TRANSPONDER)
			result = tiris_program_page(pagenr, data);
		else if(transponder_type == READ_WRITE_TRANSPONDER)
			result = tiris_program_RW(data);			
		else if(transponder_type == SAMPT_TRANSPONDER || transponder_type == SAMPTS_TRANSPONDER)
		{
			if( (result=tiris_read_page(1))>=0 ) {
				selective_addr[0] = recv_data[0];
				selective_addr[1] = recv_data[1];
				selective_addr[2] = recv_data[2];
				result = tiris_selective_program_page(pagenr, selective_addr, data);
			}
			
		}
		else 
			return;

		if(result==PAGE_PROGRAMMED_OK || result==READ_WRITE_OK)
		{
			for(i=0;i<8;++i)	{
				if(data[i] != recv_data[i]) {
					result = RESULT_NOT_RELIABLE;
					break;
				}
			}
		}
		else if(result==READ_ONLY_OK)
			result = READ_ONLY_ERROR;
		
		
		//the error return values are numbered from -1 (least fatal error) to -10 (most fatal error) 
		//only the most fatal error that occurs will be stored in error;
		if(result<0 && result<error)
			error = result;

		//fatal errors
		if(result==LOCKED_PAGE_ADDRESS_ERROR	||	result==UNLOCKED_PAGE_ADDRESS_ERROR	||
			result==PAGE_WAS_LOCKED				||  result==HARDWARE_FAILURE			|| 
			result==READ_ONLY_ERROR) {
			sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
			break;
		}

		if(kbhit() && getchar()==CLR_KEY) {
			result = error;
			break;
		}
	}
		
	if(result<0)
		processError(result);
	else {
		printf("\n");
		if(result==READ_WRITE_OK) {
			printf("Page programmed,Data:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else if(result==PAGE_PROGRAMMED_OK) {
			printf("Page programmed,Data:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else 
			printf("Unexpected reply: %d",result);

		printf("\n");

	}
	
	tiris_close();
	
	module_power(OFF);

	wait_key();

	setfont( LARGE_FONT, NULL );
}

void tiris_lock(int pagenr, int selective)  
{
	int i, result, error;
	char selective_addr[3];

	setfont( SMALL_FONT, NULL );
	
	module_power(ON);
	
	if((result=tiris_init())<0) {
		processError(result);
		tiris_close();
		module_power(OFF);
		return;
	}
	
    resetkey();

	result = error= -1;
	while(result<0) 
	{
		if(!selective)
			result = tiris_lock_page(pagenr);
		else {
			if( (result=tiris_read_page(1))>=0 ) {
				selective_addr[0] = recv_data[0];
				selective_addr[1] = recv_data[1];
				selective_addr[2] = recv_data[2];
				result = tiris_selective_lock_page(pagenr, selective_addr);
			}
		}

		if(result==READ_ONLY_OK)
			result = READ_ONLY_ERROR;
		else if(result==READ_WRITE_OK) 
			result = READ_WRITE_ONLY_ERROR;
		

		//the error return values are numbered from -1 (least fatal error) to -10 (most fatal error) 
		//only the most fatal error that occurs will be stored in error;
		if(result<0 && result<error)
			error = result;

		//fatal errors
		if(result==LOCKED_PAGE_ADDRESS_ERROR	||	result==UNLOCKED_PAGE_ADDRESS_ERROR	||
			result==PAGE_NOT_LOCKED				||  result==PAGE_WAS_LOCKED				||
		    result==HARDWARE_FAILURE			||  result==READ_ONLY_ERROR			||
		    result==READ_WRITE_ONLY_ERROR) {
			sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
			break;
		}

		if(kbhit() && getchar()==CLR_KEY) {
			result = error;
			break;
		}
	}
		
	if(result<0)
		processError(result);
	else {
		printf("\n");
		if(result==PAGE_LOCKED_OK) {
			printf("Page locked,Data:\n");
			
			for(i=0;i<8;++i)
				printf("%02x",recv_data[i]);

			sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
		}
		else 
			printf("Unexpected reply: %d",result);

		printf("\n");

	}
		
	tiris_close();
	
	module_power(OFF);

	wait_key();

	setfont( LARGE_FONT, NULL );
}


void selectPage(void) 
{
	unsigned char key;

	resetkey();
	printf("\n\nPress trigger\nto accept");

	while(!kbhit()) idle;

	key = getchar();

	if(key!=TRIGGER_KEY && key!=ENT_KEY)
	{
		printf("\nEnter page\nnumber (1..8).");

		while(key<='0' || key>'9') {
			if(kbhit())
				key = getchar();
			else 
				idle();
		}
		current_page = key - '0';
	}
	resetkey();
}


void processError(int error) 
{

	switch (error) {
	case NO_RESPONSE:
		printf("\nNo responce\n");
		break;
	case HARDWARE_FAILURE:
		printf("\nHardware failure\n");
		break;
	case NO_ID_DATA_IN_PAGE:
		printf("\nNo data in page\n");
		break;
	case INSUFFICIENT_FIELD_STRENGTH:
		printf("\nInsufficient\nfield strength\n");
		break;
	case PAGE_WAS_LOCKED:	
		printf("\nPage already\nlocked\n");
		break;
	case LOCKED_PAGE_ADDRESS_ERROR:
		printf("\nLocked page\naddress error\n");
		break;
	case UNLOCKED_PAGE_ADDRESS_ERROR:
		printf("\nUnlocked page\naddress error\n");
		break;
	case PAGE_NOT_LOCKED:
		printf("\nPage could not\n be locked\n");
		break;
	case RESULT_NOT_RELIABLE:
		printf("\nResult is\nnot reliable\n");
		break;
	case READ_ONLY_ERROR:
		printf("\nRead only data\n");
		break;
	case READ_WRITE_ONLY_ERROR:
		printf("\nRead/Write only\n");
		break;
	default:
		printf("\nUnknown error: %d\n",error);
		break;
	}
}

int module_power(int onoff)
{
    if (onoff==ON)
    {
        goodreadled(RED,-1);		// Red LED on
        return (ON);
    }
    goodreadled(RED,0);		    // Red LED off
}

/****************************************************************
	ROUTINE FOR SCANNING NEEDED CONFIGURE DATA
****************************************************************/
void read_barc(char *conf_bar)
{
	struct 	barcode code;
	int key, result;
	systemsetting("B0");		// Disable all barcodes
	systemsetting("B2B6D4");	// Code 39, Code 128, Extended Code 39
	code.min   = 1;
    code.max   = 8;
    code.text  = conf_bar;
	
	while((key=getchar())!=TRIGGER_KEY) 
	{
		if(key == CLR_KEY)	
			return;
		idle();
	}

	scannerpower(SINGLE,250);

	resetkey();

	scanbar:
	while((result=readbarcode(&code)!=OK)){

		if (kbhit() && getchar() == CLR_KEY)
		{
			scannerpower(OFF, 0);
			return;
		}
		
		if (scanneroff())
			break;

		idle();
	}
	if(result==OK)
	{
		scannerpower(OFF, 0);
		goodreadled(GREEN, 10);
		sound(TSTANDARD,VHIGH,SMEDIUM,SHIGH,0);
		printf("%s\n",code.text);
	}
	else goto scanbar;
	return;
}
/****************************************************************/


void main(void)
{
	current_page = 1;
    
	cursor (NOWRAP);
	resetkey();
	for (;;) 
        main_xmenu();
}
