// ######  ##   ## ####        #####   ####### ##   ##  ##### 
//  ##  ## ##   ##  ##          ## ###  ##   # ### ### ##   ##
//  ##  ## ##   ##  ##          ##  ##  ##     ####### ##   ##
//  #####  #######  ##          ##  ##  ####   ## # ## ##   ##
//  ##     ##   ##  ##          ##  ##  ##     ##   ## ##   ##
//  ##     ##   ##  ##  ##      ## ###  ##   # ##   ## ##   ##
// ###     ##   ## #######     #####   ####### ##   ##  ##### 
//                                                                        
// Opticon PHL1300/PHL1600/PHL1700/PHL2700 demo                   
// Scan labels and scroll database with menus                             
// Transmit scanned data with multiple protocols, FREE, RCV(slt),NetO     
// Compiler: Microtec C compiler for Melps 7700  Version 2.2              


//************************************************************************
// Include                                                         
//************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lib.h"
#include "database.h"
#include "input.h"
#include "menu.h"

//************************************************************************
// Defines
//************************************************************************
#define VERSIONSUFFIX  "xXxXx"

// Database name
#define DBASE_NAME		"DATA.FIL"

// Database record 
#define SZ_BARCODE		30
#define SZ_SYMBOLOGY	16
#define SZ_TIME			(2+1+2+1+2)
#define SZ_DATE			(4+1+2+1+2)
#define SZ_RECORD		(SZ_BARCODE+1+SZ_SYMBOLOGY+1+SZ_TIME+1+SZ_DATE+1+1)

// barcode menu defines
#define ID_CD39         0x0001   // bit 0
#define ID_EAN          0x0002   // bit 1
#define ID_UPC          0x0004   // bit 2
#define ID_I2O5         0x0008   // bit 3
#define ID_D2O5        	0x0010   // bit 4
#define ID_NW7         	0x0020   // bit 5
#define ID_CD93        	0x0040   // bit 6
#define ID_CD128      	0x0080   // bit 7
#define ID_MSI        	0x0100   // bit 8
#define ID_TELEPEN    	0x0200   // bit 9
#define ID_UK        	0x0400   // bit 10
#define ID_ADDONS    	0x0800   // bit 11
#define ID_IATA      	0x1000   // bit 12
#define ID_SCODE     	0x2000   // bit 13

//
// Special Terminal specific defines
//
//Load only if needed
#ifndef DISP_WIDTH 
	#if PHL 
		// PHL1700-10 and PHL1700-20
		#define DISP_WIDTH  96
	    #define DISP_HEIGHT 48
	#elif PHL1000	
		// PHL1300
		#define DISP_WIDTH  96
		// Change height to 48 when indicators need to be on (see cursor() function)
	    #define DISP_HEIGHT 64 
	#elif PHL2700	
		// PHL2700
		#define DISP_WIDTH  128
	    #define DISP_HEIGHT 64
	#else 
		// OPL972X series 
		#define DISP_WIDTH  112
	    #define DISP_HEIGHT 64
	#endif
#endif

#define ID_STOPBITS_1	1
#define ID_STOPBITS_2	2

#define ID_PARITY_NONE	1
#define ID_PARITY_EVEN	2
#define ID_PARITY_ODD	3

#define ID_115200	1
#define ID_57600    2
#define ID_38400    3
#define ID_19200    4
#define ID_9600     5
#define ID_4800     6
#define ID_2400     7

#define ID_NO_PROTOCOL		1
#define ID_NETO_PROTOCOL	2


//************************************************************************
// Special types
//************************************************************************

// barcode record structure
typedef struct
{
	char barcode[ SZ_BARCODE + 1 ];
	char symbology[ SZ_SYMBOLOGY + 1 ];
	char time[ SZ_TIME + 1 ];
	char date[ SZ_DATE + 1 ];
}db_record;


//************************************************************************
// Globals
//************************************************************************

// return codes of the barcode type
const char upca_string[] 		= "UPC A";
const char upca2_string[] 		= "UPC A + 2";
const char upca5_string[] 		= "UPC A + 5";
const char upce_string[] 		= "UPC E";
const char upce2_string[] 		= "UPC E + 2";
const char upce5_string[] 		= "UPC E + 5";
const char ean8_string[] 		= "EAN 8";
const char ean82_string[] 		= "EAN 8 + 2";
const char ean85_string[] 		= "EAN 8 + 5";
const char ean13_string[] 		= "EAN 13";
const char ean132_string[] 		= "EAN 13 + 2";
const char ean135_string[] 		= "EAN 13 + 5";
const char code39_string[] 		= "CODE 39";
const char codabar_string[] 	= "CODABAR";
const char i2of5_string[] 		= "INTERL 2 OF 5";
const char d2of5_string[] 		= "INDUST 2 OF 5";
const char m2of5_string[] 		= "MATRIX 2 OF 5";
const char iata_string[] 		= "IATA CODE";
const char code93_string[] 		= "CODE 93";
const char code128_string[] 	= "CODE 128";
const char msi_string[] 		= "MSI PLESSEY";
const char uk_string[] 			= "UK PLESSEY";
const char telepen_string[] 	= "TELEPEN";
const char scode_string[] 		= "SCODE";
const char laetus_string[] 		= "TRI-OPTIC CODE";
const char c39_fa_string[] 		= "CD 39 FULL ASCII";
const char c39_it_string[] 		= "ITAL. PHARMA.";
const char nw7_abc_string[] 	= "CODABAR ABC";
const char nw7_cx_string[] 		= "CODABAR CX";
const char ean128_string[] 		= "EAN 128";
const char chinpost_string[] 	= "CHINESE POST";
const char menu_code_string[] 	= "MENU CODE";
const char unknown_string[] 	= "UNKNOWN CODE";

//
// use the sSelMenu menu structure
// for getting the return codes
const sSelMenu symblglist[] =
{
	upca_string		,   UPCA    ,
	upca2_string  	,   UPCA_2  ,
	upca5_string   	,   UPCA_5  ,
	upce_string     ,   UPCE    ,
	upce2_string  	,   UPCE_2  ,
	upce5_string 	,   UPCE_5  ,
	ean8_string    	,   EAN8    ,
	ean82_string    ,   EAN8_2  ,
	ean85_string 	,   EAN8_5  ,
	ean13_string    ,   EAN13   ,
	ean132_string   ,   EAN13_2 ,
	ean135_string 	,   EAN13_5 ,
	code39_string  	,   CODE39  ,
	codabar_string  ,   CODABAR ,
	i2of5_string	,   I2OF5   ,
	d2of5_string	,   D2OF5   ,
	m2of5_string	,   M2OF5   ,
	iata_string  	,   IATA    ,
	code93_string  	,   CODE93  ,
	code128_string  ,   CODE128 ,
	msi_string 		,   MSI_PLESSEY,
	uk_string 		,   UK_PLESSEY,
	telepen_string  ,   TELEPEN,
	scode_string  	,   SCODE,
	laetus_string  	,   LAETUS,
	c39_fa_string	, 	C39_FA,
	c39_it_string 	,   C39_IT,
	nw7_abc_string  ,   NW7_ABC,
	nw7_cx_string   ,   NW7_CX,
	ean128_string 	,   EAN128,
	chinpost_string ,   CHINPOST,
	menu_code_string,   MENU_CODE,
	unknown_string  ,   -1
};

long lPort;		// Serial communication port
long lProtocol;	// The protocol for transmitting the data
long lBarcodes;	// The currently supported barcode types
long lBaudrate;	// The Baudrate
long lParity;	// Parity
long lDatabits;	// Databits
long lStopbits;	// Stopbits

//************************************************************************
// Function implementation
//************************************************************************
void set_stopbits( void )
{
	if( lStopbits == ID_STOPBITS_1 )
		systemsetting("L5");
	else
		systemsetting("L6");
}

void set_databits( void )
{
	if( lDatabits == 7 )
		systemsetting("L0");
	else
		systemsetting("L1");

}

void set_parity( void )
{
	switch( lParity )
	{
		case ID_PARITY_EVEN:
			systemsetting("L3");
			break;
		case ID_PARITY_ODD:
			systemsetting("L4");
			break;
		case ID_PARITY_NONE:
		default:
			systemsetting("L2");
			break;
	}
}

void set_baudrate( void )
{
	switch( lBaudrate )
	{
		case ID_115200:
			systemsetting("SZ");
			break;
		case ID_57600:
			systemsetting("K9");
			break;
		case ID_38400:
			systemsetting("K8");
			break;
		case ID_9600:
			systemsetting("K6");
			break;
		case ID_4800:
			systemsetting("K5");
			break;
		case ID_2400:
			systemsetting("K4");
			break;
		case ID_19200:
		default:
			systemsetting("K7");
			break;
	}
}

void set_barcodes( void )
{
	systemsetting("B0"); // disable all codes
	
    if( lBarcodes & ID_CD39)
    	systemsetting("B2D7DB"); 	// CODE39, Ital. Ph. if possible, Leading A for Ital. Ph.
    if( lBarcodes & ID_EAN)
    	systemsetting("R4");		// EAN8 and 13
    if( lBarcodes & ID_UPC)
    	systemsetting("R1");		// UPC
	if( lBarcodes& ID_I2O5)
		systemsetting("R8");		// Interleaved 2 of 5
    if( lBarcodes & ID_D2O5)	
    	systemsetting("R7");		// Industrial 2 of 5
    if( lBarcodes & ID_NW7)
    	systemsetting("B3");		// Codabar
    if( lBarcodes & ID_CD93)	
    	systemsetting("B5");		// Code93
    if( lBarcodes & ID_CD128)
    	systemsetting( "B6OG"); 	// Code 128, Ean 128 if possible
   	if( lBarcodes & ID_MSI)
   		systemsetting("B7");		// MSI plessey
    if( lBarcodes & ID_TELEPEN)	
        systemsetting("B9JZ");		// Telepen, Tri-Optic
    if( lBarcodes & ID_UK)
        systemsetting("B1");		// UK plessey
    if( lBarcodes & ID_IATA)
        systemsetting("B4");		// IATA
    if( lBarcodes & ID_SCODE)
        systemsetting("R9");		// SCode
    if( lBarcodes & ID_ADDONS)
        systemsetting("R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX

}

// Calculate the maximum amount of characters that fit on one line
static int GetMaxCharsXPos( void )
{
	unsigned int nHeight, nWidth;
	nHeight = nWidth = 0; // prevent warning toshiba compiler
	getfontsize( &nHeight, &nWidth);

	return (DISP_WIDTH / nWidth);
}

// Calculate the maximum amount of lines that fit on the display
static int GetMaxYLines( void )
{
	unsigned int nHeight, nWidth;
	nHeight = nWidth = 0; // prevent warning toshiba compiler
	getfontsize( &nHeight, &nWidth);

	return (DISP_HEIGHT / nHeight);
}

int code_id_to_index(int code_id)
{   int index;
    int value;

    index = -1;
    value = 0;

    do
    {   index++;
        value = symblglist[index].value;
    }
    while( (value!=code_id) && (value!=(-1)) );
    return (index);
}

void diplay_input_data( db_record *db_rec )
{
	int nMaxX;
	
	nMaxX = GetMaxCharsXPos();
	
	gotoxy(0,1);
	printf("%-*.*s", nMaxX, nMaxX, db_rec->barcode );
	
	setfont( SMALL_FONT, NULL);
	gotoxy(0,5);
	printf("%-*.*s\n", nMaxX, nMaxX, db_rec->symbology);
	printf("%-*.*s\n", nMaxX, nMaxX, db_rec->date);
    printf("%-*.*s", nMaxX, nMaxX, db_rec->time);
	setfont( LARGE_FONT, NULL);
}

// Save the data into the database
void store_input_data( db_record *db_rec )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1 ];

	// Check if there is enough space available for storing the barcode data
	// We use 5000 because the OS (NetO) also need some memory
	if( coreleft() < 5000L )
	{							 
		printf("\fRam disk full\ndata not stored!\n\nPress a key");
		WaitForKey();
		return;
	}
	// Make the record to store
	sprintf(record, "%-*.*s,%-*.*s,%-*.*s,%-*.*s\r\n",
				SZ_BARCODE, SZ_BARCODE, db_rec->barcode,
				SZ_SYMBOLOGY, SZ_SYMBOLOGY, db_rec->symbology,
				SZ_TIME, SZ_TIME, db_rec->time,
				SZ_DATE, SZ_DATE, db_rec->date );

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		if( !CreateDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError create\nDatabase\nCode=%ld\nPress a key", GetDBErrorCode() );
			WaitForKey();
			return;
		}
	}
	
	if( !WriteRecord( &dbFile, record, WRITE_APPEND ))
	{
		printf("\fError write\nrecord\nCode=%ld\nPress a key", GetDBErrorCode() );
		WaitForKey();
		return;
	}
	CloseDatabase( &dbFile );

}

void ScanLabels( void )   
{
	static char 	barcode[ SZ_BARCODE + 1 ];
	static db_record db_rec;
    struct date 	dates;
    struct time 	times;
    int 			nCodeId = 0;
    int 			nResult;

	memset( barcode, '\0', sizeof( barcode ));
	memset( &db_rec, '\0', sizeof( db_record ));

	cursor( NOWRAP);
	setfont( LARGE_FONT, NULL );
	putchar('\f');
	gotoxy( 0, (GetMaxYLines() -1));
	printf("Scan or type...");
	for(;;)
	{
		// Display some info first
		nResult = ScanOrKeyboardInputSymbol( barcode, 1, SZ_BARCODE, INPUT_ALL, 0, 0, GetMaxCharsXPos(), &nCodeId );
		if( nResult == CLR_KEY )
			return;
		
		// fill the record structure
		sprintf( db_rec.barcode, "%-*.*s", SZ_BARCODE, SZ_BARCODE, barcode );
		if( nResult == KEYBOARD )
			sprintf( db_rec.symbology, "%-*.*s", SZ_SYMBOLOGY, SZ_SYMBOLOGY, "KEYBOARD");
		else
		{   
			nCodeId = code_id_to_index(nCodeId);
            sprintf( db_rec.symbology, "%-*.*s", SZ_SYMBOLOGY, SZ_SYMBOLOGY, symblglist[nCodeId].text);
        }
        gettime( &times );
        getdate( &dates );
        sprintf( db_rec.time, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
        sprintf( db_rec.date, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);

        // Store the input data into our database
        store_input_data( &db_rec );

        diplay_input_data( &db_rec );
        
	}
}

void display_scroll_data( db_record *db_rec, long curr, long max )
{
	int nMaxX;
	nMaxX = GetMaxCharsXPos();
	
	gotoxy(0,0);
	printf("\tr%-*.*s %04ld/%04ld\tr", nMaxX-10, nMaxX-10, "SCROLL", curr+1, max);

	diplay_input_data( db_rec );
}

void fill_record_struct( db_record *db_rec, char* record )
{
	int offset;
	memset( db_rec, '\0', sizeof( db_record ));
	offset = 0;
	memcpy( db_rec->barcode, record+offset, SZ_BARCODE );
	offset+=SZ_BARCODE+1;
	memcpy( db_rec->symbology, record+offset, SZ_SYMBOLOGY );
	offset+= SZ_SYMBOLOGY+1;
	memcpy( db_rec->time, record+offset, SZ_TIME );
	offset+= SZ_TIME+1;
	memcpy( db_rec->date, record+offset, SZ_DATE );
}

int get_record( db_record *db_rec, long *rec_nr, long *max_rec )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		printf("\fError open\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	*max_rec = GetTotalRecords( &dbFile );
	
	if( !GotoRecord( &dbFile, *rec_nr ))
	{
		printf("\fError goto\nrecord.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	if( !ReadCurrentRecord( &dbFile,  record ))
	{
		printf("\fError read\nrecord.\nCode=%ld\nPress a key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	
	CloseDatabase( &dbFile );
	
	fill_record_struct( db_rec, record );
	return TRUE;
}


void ScrollDatabase( void )
{
	long current;
	long max = 0L;
	static db_record db_rec;

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\nPress a key");
		WaitForKey();
		return;
	}

	current = 0L;
	if( !get_record( &db_rec, &current, &max )  )
		return;
	
	putchar('\f');
	for(;;)
	{
		display_scroll_data( &db_rec, current, max );
		switch( WaitForKey() )
		{
			case DOWN_KEY:
			case RIGHT_KEY:
			case Q2_KEY:
			case F6_KEY:
			case F8_KEY:
				if(++current >= max )
					current = max-1;
				break;
			case UP_KEY:
			case LEFT_KEY:
			case Q1_KEY:
			case F5_KEY:
			case F7_KEY:				
				if( --current < 0 )
					current = 0;
				break;
			case TRIGGER_KEY:
			case CLR_KEY:			
				return;
		}
		if( !get_record( &db_rec, &current, &max ))
			return;
	}
}

void DateTime( void )
{
   	struct date dates;
    struct time times;
    static char time_str[8 + 1];       /* HH:MM:SS    */
    static char date_str[10 + 1];      /* DD/MM/YYYY */
    static char temp[10 + 1];
    int x, y;
    int key;
    gettime(&times);
    getdate(&dates);
    sprintf(time_str, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
    sprintf(date_str, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);
    putchar('\f');
    x = GetMaxCharsXPos();
    printf("\tr%-*.*s\tr", x, x, "SET TIME & DATE");
    x = 0;
    y = 2;
    cursor(ON);
    for(;;) 
    {
        gotoxy(0, 2);
        printf("%s", time_str);
        gotoxy(0, 3);
        printf("%s", date_str);
        gotoxy(x, y);
        key = WaitForKey();

        if ((key == ENT_KEY) || (key == TRIGGER_KEY)) 
        {
            strcpy(temp, date_str);
            dates.da_year = atoi(temp + 6);
            temp[5] = '\0';
            dates.da_mon = atoi(temp + 3);
            temp[2] = '\0';
            dates.da_day = atoi(temp);
            if (setdate(&dates) == ERROR)
                continue;
            strcpy(temp, time_str);
            times.ti_sec = atoi(temp + 6);
            temp[5] = '\0';
            times.ti_min = atoi(temp + 3);
			temp[2] = '\0';
            times.ti_hour = atoi(temp);
            if (settime(&times) == ERROR)
                continue;
            cursor(OFF);
            return;
        }
        switch( key ) 
        {
	    case CLR_KEY:
            cursor(OFF);
            return;

        case UP_KEY:
        case F7_KEY:
            if( y == 3) 
                y = 2;
            else
                y = 3;
            x = 0;
            break;

        case DOWN_KEY:
        case F8_KEY:
        
            if( y == 2)
                y = 3;
           	else
           		y = 2;
            x = 0;
            break;

        case Q1_KEY:
        case LEFT_KEY:
        case F5_KEY:

            if((x == 3) || (x == 6))
				x -= 2;
            else
                x--;
            if((y == 2) && (x < 0)) 
            {
            	y = 3;
                x = 9;
            }
            if((y == 3) && (x < 0)) 
            {
                y = 2;
                x = 7;
            }
            break;

        case Q2_KEY:
        case RIGHT_KEY:
        case F6_KEY:
            if((x == 1) || (x == 4))
                x += 2;
            else
                x++;
            if((y == 2) && (x > 7)) 
            {
                x = 0;
                y = 3;
            }
            if((y == 3) && (x > 9))
            {
            	y = 2;
                x = 0;
            }
            break;
            
        case '0':	case '1':	case '2':
        case '3':   case '4':	case '5':
        case '6':	case '7':   case '8':
        case '9':
            if( y == 2) 
            {
                time_str[x] = (char) key;
                if((x == 1) || (x == 4))
                    x += 2;
                else
                    x++;
				if( x > 7) 
				{
                    y = 3;
                    x = 0;
                }
            } 
            else 
            {
                date_str[x] = (char) key;
                if((x == 1) || (x == 4))
                    x += 2;
                else
                    x++;
                if( x > 9 )
                {
                    x = 0;
                    y = 2;
                }
            }
            break;
        default:
        	break;

        }
    }	
}

void SelectPort( void )
{
	int  nIndex;
	int  nMaxItems;
	
	sSelMenu mnuSelPort[] = {
		"IrDA interface", 	COM1,
		"Cradle",			COM2,
		"Direct cable",		COM0
	};
	nMaxItems = sizeof( mnuSelPort ) / sizeof( sSelMenu );
#if PHL1000
	nMaxItems--;	// Do not show direct cable not available for PHL1300
#endif	
	nIndex = 0;

	ShowTextSelectionMenu( &nIndex, " PORT MENU", mnuSelPort, nMaxItems, MENU_FIXED, &lPort);
}

void SelectBaudrate( void )
{
	int  nIndex;

	sSelMenu mnuSelBaudrate[] = {
		"115200", 	ID_115200,
		"57600",	ID_57600,
		"38400",	ID_38400,
		"19200",	ID_19200,
		"9600",		ID_9600,
		"4800",		ID_4800,
		"2400",		ID_2400
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " BAUDRATE MENU", mnuSelBaudrate, sizeof( mnuSelBaudrate ) / sizeof( sSelMenu ), MENU_FIXED, &lBaudrate);
	set_baudrate();
}

void SelectParity( void )
{
	int  nIndex;

	sSelMenu mnuSelParity[] = {
		"None", 	ID_PARITY_NONE,
		"Even", 	ID_PARITY_EVEN,	
		"Odd", 		ID_PARITY_ODD	
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " PARITY MENU", mnuSelParity, sizeof( mnuSelParity ) / sizeof( sSelMenu ), MENU_FIXED, &lParity);
	set_parity();
}

void SelectDatabits( void )
{
	int  nIndex;

	sSelMenu mnuSelDatabits[] = {
		"7 databits", 	7,
		"8 databits", 	8
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " DATABITS MENU", mnuSelDatabits, sizeof( mnuSelDatabits ) / sizeof( sSelMenu ), MENU_FIXED, &lDatabits);
	set_databits();
}

void SelectStopbits( void )
{
	int  nIndex;

	sSelMenu mnuSelStopbits[] = {
		"1 stopbit", 	ID_STOPBITS_1,
		"2 stopbits", 	ID_STOPBITS_2
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " STOPBITS MENU", mnuSelStopbits, sizeof( mnuSelStopbits ) / sizeof( sSelMenu ), MENU_FIXED, &lStopbits);
	set_stopbits();
}

void ComPortSettings( void )
{
	int nIndex;
	stxtMenu mnuComm[] =
    {
        "Port",		SelectPort,
        "Baudrate", SelectBaudrate,
        "Parity",	SelectParity,
        "Databits", SelectDatabits,
        "Stopbits",	SelectStopbits
    };
    nIndex = 0;
    ShowTextMenu( &nIndex, " COMPORT MENU", mnuComm, sizeof( mnuComm )/sizeof( stxtMenu), MENU_FIXED );
}

void SelectProtocol( void )
{
	int  nIndex;

	sSelMenu mnuSelProtocol[] = {
		"No protocol", 		ID_NO_PROTOCOL,
		"NetO protocol",	ID_NETO_PROTOCOL
	};
	nIndex = 0;
	ShowTextSelectionMenu( &nIndex, " PROTOCOL MENU", mnuSelProtocol, sizeof( mnuSelProtocol ) / sizeof( sSelMenu ), MENU_FIXED, &lProtocol);

}

void ChangeContrast( void )
{
	int nMaxX;
    int value;
    
    nMaxX = GetMaxCharsXPos() >> 1;
    value = contrast(-1);
    putchar('\f');
    
    printf("%-*.*s\tr%-*.*s \tr\n\tr%-*.*s\tr%-*.*s \n", nMaxX, nMaxX, "CHANGE", nMaxX, nMaxX, "CONTRAST", nMaxX, nMaxX, "CHANGE", nMaxX, nMaxX, "CONTRAST");

    printf("Using: Q1 and Q2");
    for(;;) 
    {
    	switch( WaitForKeys(5, Q1_KEY, Q2_KEY, CLR_KEY, ENT_KEY, TRIGGER_KEY ))
    	{
    		case Q1_KEY:
#if PHL2700    		
    			if( ++value > 40 )
    				value = 40;
#else
    			if( ++value > 31 )
    				value = 31;
#endif
    			break;
    		case Q2_KEY:
    			if( --value < 0 )
    				value = 0;
    			break;
    		
    		case CLR_KEY:
    		case ENT_KEY:
    		case TRIGGER_KEY:
    			return;
    		
    	}
    	contrast( value );
    }	
}

void SetBarcodes( void )
{
	int nIndex;
	sSelMenu mnuSelBarcodes[] = {
        "Code39        ", ID_CD39,
        "EAN           ", ID_EAN,
        "UPC           ", ID_UPC,
        "Interl. 2of5  ", ID_I2O5,
        "Indust. 2of5  ", ID_D2O5,
        "Codabar       ", ID_NW7,
        "Code93        ", ID_CD93,
        "Code128       ", ID_CD128,
        "MSI           ", ID_MSI,
        "Telepen       ", ID_TELEPEN,
        "UK/Plessey    ", ID_UK,
        "IATA code     ", ID_IATA,
        "Scode         ", ID_SCODE,
        "All addons    ", ID_ADDONS
    };
    nIndex = 0;
    ShowTextSelectionMenu( &nIndex, " BARCODE MENU", mnuSelBarcodes, sizeof( mnuSelBarcodes ) / sizeof( sSelMenu ), MENU_FIXED | MENU_MULTIPLE, &lBarcodes);
    set_barcodes();	// Set the new selected barcodes
}

void AvailableMemory( void )
{
   	long mem_size;
	mem_size = coreleft();
	printf("\fMEMORY LEFT:\n%ld Bytes\n\nPress a key", mem_size);	
	WaitForKey();
}

void SystemMenu( void )
{
	int nIndex = 0;
	stxtMenu mnuSystem[] = {
		"Date/Time",	DateTime,
		"Com port", 	ComPortSettings,
		"Protocol",		SelectProtocol,
		"Display",		ChangeContrast,
		"Barcodes",		SetBarcodes,
		"Memory",		AvailableMemory
	};
	ShowTextMenu( &nIndex, " SYSTEM MENU", mnuSystem, sizeof( mnuSystem ) / sizeof( stxtMenu ), MENU_FIXED );
}

void DeleteDatabase( void )
{
	printf("\fDelete DBASE\n\n[ENT] = YES\n[CLR] = NO");
	if( WaitForKeys( 2, ENT_KEY, CLR_KEY ) == ENT_KEY )
		remove(DBASE_NAME );
}

void TransmitData( void )
{
	int nRet;
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\nPress a key");
		WaitForKey();
		return;
	}
	if( comopen( (unsigned int)lPort ) != OK )
	{
		printf("\fError open\nCOM port\n\nPress a key");
		WaitForKey();
		return;
	}
	printf("\fTransmit data\n");
	if( lProtocol == ID_NETO_PROTOCOL )
	{
		printf("NetO protocol");
		nRet = neto_transmit( (char(*)[12+1])DBASE_NAME, 1, "123456", CLR_KEY, 3 );
		cursor( NOWRAP );
		if( nRet != OK )
		{
			printf("\fError NetO\nCode=%d\n\nPress a key", nRet);
			WaitForKey();
		}
	}
	else
	{
		if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError open\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
			WaitForKey();
		}
		else
		{
			if( ReadFirstRecord( &dbFile, record ))
			{
				putchar('\f');
				do
				{
					gotoxy(0,0);
					printf("Send %04ld/%04ld", dbFile.lCurrRecord+1, dbFile.lTotalRecords);
					for( nRet = 0; nRet < SZ_RECORD; nRet++ )
			            putcom( record[ nRet ]);
        			delay(10);
				
				}while( ReadNextRecord( &dbFile, record ) );
			}
			else
			{
				printf("\fError\ndatabase.\nCode=%ld\nPress a key", GetDBErrorCode());
				WaitForKey();
			}
			CloseDatabase( &dbFile );
		}
	}

	comclose( (unsigned int) lPort );
}

void ShowVersion( void )
{
	static char os[ 8 + 1 ];
    printf("\fSoftware Version");
    version_os(os);
    os[3] = '\0';
    os[1] = 'F';
    printf("\n%s%s\npress any key", os,VERSIONSUFFIX);
    WaitForKey();
}

void main( void )
{
	int nIndex = 0;
	stxtMenu mnuMain[] = {
		"Scan labels",	ScanLabels,
		"Scroll DBase",	ScrollDatabase,
		"System menu", 	SystemMenu,
		"Delete DBase",	DeleteDatabase,
		"X'mit data",  	TransmitData,
		"Version",     	ShowVersion
	};

	// Set some default values
	lPort = COM2; // Cradle port is used as default	
	lProtocol = ID_NETO_PROTOCOL;
	lBarcodes = ID_CD39 | ID_EAN | ID_UPC | ID_I2O5 | ID_D2O5 | ID_NW7 | ID_CD93 | ID_CD128;
	lBarcodes |= ID_MSI | ID_TELEPEN | ID_UK | ID_IATA | ID_SCODE;
	lBaudrate = ID_19200;
	lParity = ID_PARITY_NONE;
	lDatabits = 8;
	lStopbits = ID_STOPBITS_1;
	
	set_barcodes();		// Set the enabled barcodes 
	set_stopbits();		// set the number of stopbits
	set_databits();		// set the number of databits
	set_parity();		// set the parity
	set_baudrate();		// set the baudrate

	for(;;)
	{
		ShowTextMenu( &nIndex, "  MAIN MENU", mnuMain, sizeof( mnuMain ) / sizeof( stxtMenu ), MENU_FIXED );

		if( nIndex == -1 )	// CLR_KEY was pressed
			nIndex = 0; // reset selection to the first item
	}	
}

