//
// Small application that transmits a scanned barcode
// directly after it is scanned by Bluetooth.
// Menubook programming is (partially) supported.
//
#include <stdio.h>
#include <string.h>

#include "lib.h"
#include "msgscr.h"

#define VERSION "XFFxxxxx"
VERSION_INFO unsigned char Version[16] = VERSION;

#define BCR_MAX  			 	50
#define COM_USED 				COM5

#define MAX_BARCODE_LENGTH      1000
#define MAX_PREFIX_LENGTH       10
#define MAX_SUFFIX_LENGTH       10
#define DEFAULT_SUFFIX        "\r\n"

#define SYMBOLOGY_UPCA	        0x01
#define SYMBOLOGY_UPCA_2        0x02
#define SYMBOLOGY_UPCA_5        0x03
#define SYMBOLOGY_UPCE	        0x04
#define SYMBOLOGY_UPCE_2	    0x05
#define SYMBOLOGY_UPCE_5	    0x06
#define SYMBOLOGY_EAN13	        0x07
#define SYMBOLOGY_EAN13_2 	    0x08
#define SYMBOLOGY_EAN13_5     	0x09
#define SYMBOLOGY_EAN8	        0x0A
#define SYMBOLOGY_EAN8_2	    0x0B
#define SYMBOLOGY_EAN8_5	    0x0C
#define SYMBOLOGY_CODE39        0x0D
#define SYMBOLOGY_CODABAR       0x0E
#define SYMBOLOGY_I2OF5	        0x0F
#define SYMBOLOGY_D2OF5         0x10
#define SYMBOLOGY_M2OF5	        0x11
#define SYMBOLOGY_DTL2OF5       0x12
#define SYMBOLOGY_IATA          0x13
#define SYMBOLOGY_CODE93        0x14
#define SYMBOLOGY_CODE128       0x15
#define SYMBOLOGY_MSI_PLESSEY   0x16
#define SYMBOLOGY_UK_PLESSEY    0x17
#define SYMBOLOGY_TELEPEN       0x18
#define SYMBOLOGY_SCODE         0x1A
#define SYMBOLOGY_LAETUS	    0x1B
#define SYMBOLOGY_C39_FA	    0x1C
#define SYMBOLOGY_C39_IT	    0x1D
#define SYMBOLOGY_NW7_ABC		0x1E
#define SYMBOLOGY_NW7_CX		0x1F
#define SYMBOLOGY_CHINPOST		0x20
#define SYMBOLOGY_RSS_14          0x21
#define SYMBOLOGY_RSS_LIMITED     0x22
#define SYMBOLOGY_RSS_EXPANDED    0x23

#define SYMBOLOGY_PDF417          0x24
#define SYMBOLOGY_CCC_COMPOSITE   0x25

#define SYMBOLOGY_MICRO_PDF417    0x26
#define SYMBOLOGY_CCA_COMPOSITE   0x27
#define SYMBOLOGY_CCB_COMPOSITE   0x28

#define MAX_SYMBOLOGIES       SYMBOLOGY_CCB_COMPOSITE
#define MENU_SET_LABEL        "ZZ"
#define NOT_IN_MENU_MODE       0
#define IN_MENU_MODE           1

typedef struct
{
	char prefix[MAX_PREFIX_LENGTH + 1];
	char suffix[MAX_SUFFIX_LENGTH + 1];
}Param;


#define PWR_SAVE_TIME_STEPSIZE 5
#define MAX_PWR_SAVE_TIME 60/PWR_SAVE_TIME_STEPSIZE
#define MAX_PWR_SAVE_PARAM 10
#define MAX_PWR_SAVE_MODE 3
#define MAX_ANTENNA_PWR_SETTING 10

#define PWR_SAVE_SNIFF 0
#define PWR_SAVE_PARK 1
#define PWR_SAVE_HOLD 2
#define PWR_SAVE_DISCONNECT 3

int pwr_save_time;
int pwr_save_mode;
int pwr_save_param;
int antenna_pwr_setting;

int menu_state;
Param param_tab[MAX_SYMBOLOGIES];
int prefix;
int suffix;

int conv_symbology(int symbo);
/*
int composite_post_processing(struct barcode *code, struct barcode *prev_code,
												unsigned long *time_out_value, short reset_time);
*/
/*
 * Constatnts for the com_idle function
 */
#define PARK					1
#define EXIT_PARK				2
#define SNIFF					3
#define EXIT_SNIFF				4
#define HOLD					5
#define	EXIT_HOLD_CHECK			6

/* PX-25 dummie function */
int com_idle(int mode, unsigned int arg)
{
	mode=mode;
	arg=arg;
//	idle();
	return OK;
}


void info_sound( void )
{
	sound( TSHORT, VMEDIUM, SHIGH, SLOW, SHIGH, SHIGH, SLOW, 0);
}

void keybeep(void)
{
	sound(TSHORT,VMEDIUM,SHIGH,0);
}

void okbeep(void)
{
	sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
}

void errbeep(void)
{
	sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
}

int waitkey(void)
{
	int c;

	while((c=getchar())==EOF)
	{
		idle();
	}
	keybeep(); // make the beeping sound
	return c;
}

int check_connection( void )
{
	int result;
	//
	// Check if the bluetooth device is still connected
	//
	for(;;)
	{
		result = blt_is_connected();
		if( result )
			goodreadled( GREEN, 100 );

		switch( result )
		{
			case NG:
				//
				// Connection can not be determined at
				// the moment we have to wait a little while.
				//
				delay(5);
				break;
			default:
				return result;

		}
	}
}


int setvalue(int current, int max)
{
	int c;

	while ((c=getchar())==EOF)
		idle();
	if (c==TRIGGER_KEY)
		return (-1);
	if (c==Q1_KEY)
		return (current>0?current-1:current);
	if (c==Q2_KEY)
		return (current<max?current+1:current);
	return(-1);
}


void submenu(int item)
{
	int i, done=0;

	for(;;)
	{

		switch (item)
		{
			case 1:
				if (pwr_save_time==0)
					printf("\fPWR SAVE TIME\nSet time:\n  MIN");
				else
					printf("\fPWR SAVE TIME\nSet time:\n  %3d", pwr_save_time*PWR_SAVE_TIME_STEPSIZE);
				i=setvalue(pwr_save_time, MAX_PWR_SAVE_TIME);
				if (i<0)
					done++;
				else
					pwr_save_time=i;

				break;

			case 2:
				switch (pwr_save_mode)
				{
					case 0:
						printf("\fPWR SAVE MODE\nSelect mode:\n[Sniff]");
						break;
					case 1:
						printf("\fPWR SAVE MODE\nSelect mode:\n[Park]");
						break;
					case 2:
						printf("\fPWR SAVE MODE\nSelect mode:\n[Hold]");
						break;
					case 3:
						printf("\fPWR SAVE MODE\nSelect mode:\n[Disconnect]");
						break;
					default:
						printf("\fillegal pwr\n save mode\n value");
						break;

				}
				i=setvalue(pwr_save_mode, MAX_PWR_SAVE_MODE);
				if (i<0)
					done++;
				else
					pwr_save_mode=i;

				break;

			case 3:
				printf("\fPWR SAVE PARAM\nSet value:\n  %3d", pwr_save_param);
				i=setvalue(pwr_save_param, MAX_PWR_SAVE_PARAM);
				if (i<0)
					done++;
				else
					pwr_save_param=i;

				break;

			default:
				break;
		}
		if (done)
			break;
	}

	printf ("\npress a key...");
	while ((getchar())==EOF)
		;
}

void printmenu(int item)
{
	switch(item)
	{
		case 0:
			printf("\fSETTINGS MENU\n\trExit         \tr\nPwr save time\nPwr save mode");
			break;
		case 1:
			printf("\fSETTINGS MENU\nExit         \n\trPwr save time\tr\nPwr save mode");
			break;
		case 2:
			printf("\fSETTINGS MENU\nPwr save time\n\trPwr save mode\tr\nPwr save parm");
			break;
		case 3:
			printf("\fSETTINGS MENU\nPwr save time\nPwr save mode\tr\nPwr save parm\tr");
			break;
	}
}

void settingsmenu( void )
{
	int c;
	int selection;

	selection=0;


	for (;;)
	{
		printmenu(selection);
		while ((c=getchar())==EOF)
			idle();
		if (c==TRIGGER_KEY)
		{
			if (selection==0)
				break;

			submenu(selection);
			break;
		}
		if (c==Q1_KEY)
		{
			if (selection>0)
				selection--;
			continue;
		}
		if (c==Q2_KEY)
		{
			if (selection<3)
				selection++;
			continue;
		}
	}
	printf("\f");
	gotoxy(0,1);
	printf("Scan Barcode.");
}



int ScanBarcode(char* string, int min_length, int max_length, int *code_id, int connected)
{
	static int fActive=0;
	static int pwr_save_state=PWR_SAVE_SNIFF;

	struct barcode code = {0};
	int counter, ch;

	struct barcode prev_code = {0};
	static char prev_buff[400];

	unsigned long time_out_value;

	prev_code.min = min_length;
	prev_code.max = 400;
	prev_code.id = 0;
	prev_code.text = prev_buff;
	time_out_value = 0;

	code.min = min_length;
	code.max = max_length;
	code.text = string;

	code.min = min_length;
	code.max = max_length;

	if( connected && !fActive )
	{
		scannerpower( OFF, 0 );

		switch (pwr_save_state)
		{
			case PWR_SAVE_SNIFF:
				com_idle( EXIT_SNIFF, 0 );
				break;
			case PWR_SAVE_PARK:
				com_idle( EXIT_PARK, 0 );
				break;
			default:
				break;
		}
		fActive = TRUE;
		gotoxy(0,3);
		printf("Active...");
	}

	scannerpower( SINGLE, 10000 );

	counter = pwr_save_time*PWR_SAVE_TIME_STEPSIZE;
	starttimer( 50 ); // wait 12 seconds before powersaving mode
	while( readbarcode( &code ) != OK  /* || composite_post_processing( &code, &prev_code, &time_out_value, 150)!=0 */ )
	{
		if(!triggerpressed())
			prev_code.id = 0;

		if( kbhit() )
		{
			if(!connected)
			{
				if( (ch = getchar()) != TRIGGER_KEY)
				{
					menu_state = NOT_IN_MENU_MODE;
					ungetc(ch, stdin);
					return ERROR;
				}
			}
			else
			{
				if (getchar()==Q2_KEY)
				{
//					settingsmenu();
				}

				resetkey();
				counter = pwr_save_time*PWR_SAVE_TIME_STEPSIZE;
				starttimer( 50 ); // wait 12 seconds before powersaving mode
				if( !fActive )
				{
					scannerpower( OFF, 0 );
					switch (pwr_save_state)
					{
						case PWR_SAVE_SNIFF:
							com_idle( EXIT_SNIFF, 0 );
							break;
						case PWR_SAVE_PARK:
							com_idle( EXIT_PARK, 0 );
							break;
						default:
							break;
					}
				}

				fActive = TRUE;
				gotoxy(0,3);
				printf("Active...");
			}
			scannerpower( SINGLE, 10000 );
		}

		if(connected)
		{
			//
			// Check if the OPL is still connected to the PC
			//
			if( check_connection() == FALSE )
				return ERROR;

			if(!fActive && pwr_save_mode==PWR_SAVE_HOLD)
			{
				if (com_idle(EXIT_HOLD_CHECK, 0)!=-1)
				{
					com_idle( HOLD, 4+pwr_save_param*900 );
					pwr_save_state=PWR_SAVE_HOLD;
				}
			}
		}

		if (endtimer())
		{
			if (menu_state == IN_MENU_MODE)
				sound(TSTANDARD, VHIGH, SMEDIUM, NULL);
			else if (counter>0)
				counter--;

			starttimer(50);
		}

		if( connected && menu_state == NOT_IN_MENU_MODE && scanneroff() && fActive && endtimer() && (counter==0))
		{
			fActive = FALSE;
			gotoxy(0,3);
			switch (pwr_save_mode)
			{
				case 0:
					com_idle( SNIFF, 66+pwr_save_param*900 );
					printf("Sniff...     ");
					pwr_save_state=PWR_SAVE_SNIFF;
					break;
				case 1:
					com_idle( PARK, 200+pwr_save_param*900 );
					printf("Park...      ");
					pwr_save_state=PWR_SAVE_PARK;
					break;
				case 2:
					com_idle( HOLD, 4+pwr_save_param*900 );
					pwr_save_state=PWR_SAVE_HOLD;
					printf("Hold...      ");
					break;
				case 3:
					comclose( COM_USED );
					pwr_save_state=PWR_SAVE_HOLD;
					printf("Disconnect...");
					break;
			}

		}
		idle();
	}

	scannerpower( OFF, 0);

	*code_id = conv_symbology(code.id);

	return( OK );
}

int transmit_barcode( char* barcode )
{
	char* ptr;

	ptr = barcode;
/*
	while( *ptr  )
	{
		if( putcom( *ptr ) == ERROR )
			return ERROR;
		ptr++;
	}
*/
	putstring( barcode );
	vibrate( 20 );
	return OK;
}

int conv_symbology(int symbo)
{
	switch (symbo) {

	case MENU_CODE:
		return (MENU_CODE);

	case EAN13:
		return (SYMBOLOGY_EAN13);

	case UPCA:
		return (SYMBOLOGY_UPCA);

	case EAN8:
		return (SYMBOLOGY_EAN8);

	case UPCE:
		return (SYMBOLOGY_EAN8);

	case CODE39:
		return (SYMBOLOGY_CODE39);

	case CODABAR:
		return (SYMBOLOGY_CODABAR);

	case D2OF5:
		return (SYMBOLOGY_D2OF5);

	case I2OF5:
		return (SYMBOLOGY_I2OF5);

	case CODE93:
		return (SYMBOLOGY_CODE93);

	case CODE128:
	case CODE128_2:
	case CODE128_5:
		return (SYMBOLOGY_CODE128);

	case MSI_PLESSEY:
		return (SYMBOLOGY_MSI_PLESSEY);

	case IATA:
		return (SYMBOLOGY_IATA);

	case UK_PLESSEY:
		return (SYMBOLOGY_UK_PLESSEY);

	case TELEPEN:
		return (SYMBOLOGY_TELEPEN);

	case M2OF5:
		return (SYMBOLOGY_M2OF5);

	case CHINPOST:
		return (SYMBOLOGY_CHINPOST);

	case NW7_ABC:
		return (SYMBOLOGY_NW7_ABC);

	case NW7_CX:
		return (SYMBOLOGY_NW7_CX);

	case SCODE:
		return (SYMBOLOGY_SCODE);

	case LAETUS:
		return (SYMBOLOGY_LAETUS);

	case C39_FA:
		return (SYMBOLOGY_C39_FA);
	case C39_IT:
		return (SYMBOLOGY_C39_IT);

	case EAN13_2:
		return (SYMBOLOGY_EAN13_2);

	case UPCA_2:
		return (SYMBOLOGY_UPCA_2);

	case EAN8_2:
		return (SYMBOLOGY_EAN8_2);

	case UPCE_2:
		return (SYMBOLOGY_UPCE_2);

	case EAN13_5:
		return (SYMBOLOGY_EAN13_5);

	case UPCA_5:
		return (SYMBOLOGY_UPCA_5);

	case EAN8_5:
		return (SYMBOLOGY_EAN8_5);

	case UPCE_5:
		return (SYMBOLOGY_UPCE_5);

	case RSS_14:
		return (SYMBOLOGY_RSS_14);

	case RSS_LIMITED:
		return (SYMBOLOGY_RSS_LIMITED);

	case RSS_EXPANDED:
		return (SYMBOLOGY_RSS_EXPANDED);

	case PDF417:
		return (SYMBOLOGY_PDF417);

	case CCC_COMPOSITE:
		return (SYMBOLOGY_CCC_COMPOSITE);

	case MICRO_PDF417:
		return (SYMBOLOGY_MICRO_PDF417);

	case CCA_COMPOSITE:
		return (SYMBOLOGY_CCA_COMPOSITE);

	case CCB_COMPOSITE:
		return (SYMBOLOGY_CCB_COMPOSITE);

	}
	return (SYMBOLOGY_CODE39);	/* keep compiler happy */
}

void SetDefault(void)
{
	int x;

	prefix = -1;
	suffix = -1;

	for (x=0; x < MAX_SYMBOLOGIES; x++)
	{
		memset(param_tab[x].prefix, '\0', MAX_PREFIX_LENGTH);
		memset(param_tab[x].suffix, '\0', MAX_SUFFIX_LENGTH);
		strcpy(param_tab[x].suffix, DEFAULT_SUFFIX);
	}
}

char direct_character_input(unsigned char byte1, unsigned char byte2)
{
	unsigned char ret_char;

	ret_char = 0xFF;

	switch(byte1)
	{
		case '0':
			if((byte2 >= 'A') && (byte2 <= 'Z'))
				ret_char = byte2;
			break;
		case '1':
			if((byte2 >= 'A') && (byte2 <= 'Z'))
				ret_char = byte2 - 'A' + 0x01;
			break;
		case '5':
			if((byte2 >= 'A') && (byte2 <= 'P'))
				ret_char = byte2 - 'A' + 0x20;
			break;
		case '6':
			if((byte2 >= 'A') && (byte2 <= 'G'))
				ret_char = byte2 - 'A' + 0x3A;
			break;
		case '7':
			switch(byte2)
			{
				case 'A':
				case 'B':
				case 'C':
				case 'D':
				case 'E':
				case 'F':
					ret_char = byte2 - 'A' + 0x5B;
					break;
			}
			break;
		case '9':
			switch(byte2)
			{
				case 'A':
				case 'B':
				case 'C':
				case 'D':
				case 'E':
					ret_char = byte2 - 'A' + 0x1B;
					break;
				case 'F':
					ret_char = 0x7F;
					break;
//				case 'G':										 0x00 is the string terminator, and therfore it cannot be used.
//					ret_char = 0x00;
//					break;
				case 'T':
				case 'U':
				case 'V':
				case 'W':
					ret_char = byte2 - 'T' + 0x7B;
					break;
			}
			break;
		case 'Q':
			if((byte2 >= '0') && (byte2 <= '9'))
				ret_char = byte2;
			break;
		case '$':
			if((byte2 >= 'A') && (byte2 <= 'Z'))
				ret_char = byte2 - 'A' + 'a';
			else if(byte2 == '2')
				ret_char = 0xAF;
			else if(byte2 == '3')
				ret_char = 0xAE;
			break;
		default:
			break;
	}

	return ret_char;
}


char barcode[ 1000 + 2+1 ];
char msg[1000+25];


void main( void )
{
	static int scanned_barcode;
	int code_id = 0;
	int error = 0;
	unsigned int command;
	int symbology = 0;
	int len;
	int ch;
	int ret;
	int com_port;
	int connected;
	static char options[54 + 1];	//Holds string of menu-options

	com_port = 0;
	pwr_save_time=10/PWR_SAVE_TIME_STEPSIZE;
	pwr_save_mode=0;
	pwr_save_param=0;
	antenna_pwr_setting=0;

	connected = FALSE;

	strcpy(options, "A0");
	strcat(options, "JX");			//Enable RSS-14
	strcat(options, "JY");			//Enable RSS-Limited
	strcat(options, "DR");			//Enable RSS-Expanded

	strcat(options, "OG");			//Enable EAN128

	strcat(options, "[BCF");		//Enable PDF417 (incl. its composite variant CC-C)
	strcat(options, "[BCG");		//Enable MicroPDF (incl. its composite variants CC-A and CC-B)


	strcat(options,"[BHE");			//Enable Composite on RSS

	strcat(options,"[BL2");			//Enable Composite on RSS

	strcat(options,"SZ");			// 115200 baud
	strcat(options,"7G");			// show charging indication LED

	systemsetting( options );		//Execute all menu-options shown above

	autopowerdown( ON, 1000 ); 		// 20 seconds

	menu_state = NOT_IN_MENU_MODE;
	InitMessageScreen();
	scanned_barcode = FALSE;

	SetDefault();

	sprintf(barcode,"This %s application tries to connect to a bluetooth SPP or accepts an incoming connection. After scanning a barcode the barcode is transmitted to the PC.", VERSION);
	ShowMessage(barcode, LANG_MSG_NL, MB_ALL | MB_ICONINFORMATION,0);

	autopowerdown(ON,120*20);		// 2 minutes

	scannerpower( SINGLE, 10000 );
	//
	// The main application loop.
	//

	for(;;)
	{
		if ((ret = ShowMessage("Do you want to connect to a bluetooth SPP", LANG_MSG_NL, MB_ALL | MB_YESNO, 0 )) == NO)
		{
			com_port = COM5;
			printf("\fWaiting for an\nincoming\nconnection...");
			comopen(COM5);
			while (check_connection() == FALSE)
				idle();

			connected = TRUE;
		}
		else if(ret == YES)
		{
			setfont( LARGE_FONT, NULL );
			printf("\fConnecting...");

			com_port = COM3;

			if( comopen( COM3 ) != 0 )
			{
				ShowMessage("Error connecting OPL9724 to bluetooth serial port.", LANG_MSG_NL, MB_ALL | MB_ICONERROR,0 );
				continue;
			}

			connected = TRUE;
		}
		else{
			connected = FALSE;
		}

		if(connected)
		{
			printf("\fConnected... ");
			sound(TVLONG, VHIGH, SMEDIUM, NULL);
		}

		for(;;)
		{
			if( connected && ( check_connection() == FALSE) )
			{
				ret = ShowMessage("Connection is lost, please get closer to the bluetooth base station.", LANG_MSG_NL, MB_YESNO | MB_ALL | MB_ICONINFORMATION ,1);
				if (ret == CONNECT)
				{
					printf("\fConnected... ");
					connected = TRUE;
					sound(TVLONG, VHIGH, SMEDIUM, NULL);
				}
				else
				{
					connected = FALSE;

					if (com_port == COM3)
					{
						printf("\fConnecting...");

						scannerpower( OFF, 0 );
						if( comopen( COM3 ) != 0 )
						{
							ShowMessage("Error connecting OPL9724 to bluetooth serial port.", LANG_MSG_NL, MB_ALL | MB_ICONERROR,0 );
							break;
						}
						setfont( LARGE_FONT, NULL );
						scannerpower( SINGLE, 10000 );
						printf("\fConnected...");
						connected = TRUE;
					}
					continue;
				}
			}

			if( connected && scanned_barcode )
			{
				//
				// A barcode was previously scanned but could not be transmitted
				//
				if( ShowMessage("Do you want to resend the previous failed transmission", LANG_MSG_NL, MB_YESNO , 0 ) == YES )
				{
					if( transmit_barcode( barcode ) == OK )
					 	scanned_barcode = FALSE;
					else
					{
						if (com_port == COM3)
							break; // connection failed
						else
							continue;
					}
				}
			}
			//
			// Set the bluetooth module in a 1 second sniff mode
			//
			com_idle( SNIFF, 100 );

			memset( barcode, '\0', sizeof( barcode ));

			if(connected)
			{
				gotoxy(0,1);
				printf("Scan barcode.");
			}
			//
			// Scan barcode
			//
			if( ScanBarcode(barcode, 1, 1000, &code_id, connected ) == ERROR)
			{
				if(!connected)
					break;

				continue;
			}


			if (code_id == MENU_CODE)
			{
				if (strcmp(barcode, "ZZ") == 0)
				{
					sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, NULL);
					goodreadled(GREEN, 10);
					menu_state = !menu_state;
					if (menu_state == NOT_IN_MENU_MODE)
					{
						prefix = -1;
						suffix = -1;
					}
					continue;
				}

				if (connected && strcmp(barcode, "+-DISC-+") == 0)
				{
					sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, NULL);
					goodreadled(GREEN, 10);

					comclose( COM_USED );
					sound(TVLONG, VHIGH, SMEDIUM, NULL);
					connected = FALSE;
					break;
				}

				if (connected && strcmp(barcode, "+-UNPR-+") == 0)
				{
					sound(TSTANDARD, VHIGH, SLOW, SMEDIUM, NULL);
					goodreadled(GREEN, 10);

					blt_unpair();

					comclose( COM_USED );
					sound(TVLONG, VHIGH, SMEDIUM, NULL);
					connected = FALSE;
					break;
				}

				if (menu_state == NOT_IN_MENU_MODE) 		// Menu label while not in menu mode is considered an error
				{
					goodreadled(RED, 10);
					sound(TLONG, VHIGH, SERROR, NULL);
					continue;
				}

				if(strlen(barcode) == 3 || strlen(barcode) == 4)
				{
					barcode[5] = '\0';
					barcode[4] = barcode[3];
					barcode[3] = barcode[2];
					barcode[2] = barcode[1];
					barcode[1] = barcode[0];
					barcode[0] = (strlen(barcode) == 3) ? '[' : ']';
				}

				error = 0;
				if (systemsetting(barcode) == OK)
				{
					if ((barcode[0] == 'U') && (barcode[1] == '2'))
						SetDefault();

					systemsetting("YC");	// enable menu label setting
					command = 0;
				}
				else switch ((command = (((unsigned int)barcode[0]) << 8) + barcode[1]))
				{
					case ('R' << 8) + 'Y':		// set prefix all codes
						for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
							memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						symbology = MAX_SYMBOLOGIES;
						prefix = 0;
						break;

					case ('N' << 8) + '1':		// set prefix UPC-A
						symbology = SYMBOLOGY_UPCA;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '0':		// set prefix UPC-A + addon
						symbology = SYMBOLOGY_UPCA_2;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_UPCA_5].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('N' << 8) + '2':		// set prefix UPC-E
						symbology = SYMBOLOGY_UPCE;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '1':		// set prefix UPC-E + addon
						symbology = SYMBOLOGY_UPCE_2;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_UPCE_5].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('N' << 8) + '3':		// set prefix EAN_13
						symbology = SYMBOLOGY_EAN13;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '2':		// set prefix EAN-13 + addon
						symbology = SYMBOLOGY_EAN13_2;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_EAN13_5].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('N' << 8) + '4':		// set prefix EAN-8
						symbology = SYMBOLOGY_EAN8;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '3':		// set prefix EAN-8 + addon
						symbology = SYMBOLOGY_EAN8_2;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_EAN8_5].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '4':		// set prefix code 39
						symbology = SYMBOLOGY_CODE39;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '5':		// set prefix codabar
						symbology = SYMBOLOGY_CODABAR;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '6':		// set prefix industrial 2 of 5
						symbology = SYMBOLOGY_D2OF5;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '7':		// set prefix interleaved 2 of 5
						symbology = SYMBOLOGY_I2OF5;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + 'B':		// set prefix S code
						symbology = SYMBOLOGY_SCODE;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('G' << 8) + 'L':		// set prefix Matrix 2of5 code
						symbology = SYMBOLOGY_M2OF5;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('I' << 8) + '8':		// set prefix IATA code
						symbology = SYMBOLOGY_IATA;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '8':		// set prefix Code 93 code
						symbology = SYMBOLOGY_CODE93;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + '9':		// set prefix Code 128 code
						symbology = SYMBOLOGY_CODE128;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('N' << 8) + '0':		// set prefix MSI/Plessey code
						symbology = SYMBOLOGY_CODE128;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('L' << 8) + '8':		// set prefix telepen code
						symbology = SYMBOLOGY_TELEPEN;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + 'A':		// set prefix UK/Plessey code
						symbology = SYMBOLOGY_UK_PLESSEY;
						memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = 0;
						break;

					case ('M' << 8) + 'G':		// clear all prefixes
						for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
							memset(param_tab[symbology].prefix, '\0', MAX_PREFIX_LENGTH);
						prefix = -1;
						break;

					case ('R' << 8) + 'Z':		// set suffix all codes
						for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
							memset(param_tab[symbology].suffix, '\0', MAX_PREFIX_LENGTH);
						symbology = MAX_SYMBOLOGIES;
						suffix = 0;
						break;

					case ('N' << 8) + '6':		// set suffix UPC-A
						symbology = SYMBOLOGY_UPCA;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '0':		// set suffix UPC-A + addon
						symbology = SYMBOLOGY_UPCA_2;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_UPCA_5].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('N' << 8) + '7':		// set suffix UPC-E
						symbology = SYMBOLOGY_UPCE;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '1':		// set suffix UPC-E + addon
						symbology = SYMBOLOGY_UPCE_2;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_UPCE_5].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('N' << 8) + '8':		// set suffix EAN_13
						symbology = SYMBOLOGY_EAN13;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '2':		// set suffix EAN-13 + addon
						symbology = SYMBOLOGY_EAN13_2;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_EAN13_5].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('N' << 8) + '9':		// set suffix EAN-8
						symbology = SYMBOLOGY_EAN8;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '3':		// set suffix EAN-8 + addon
						symbology = SYMBOLOGY_EAN8_2;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						memset(param_tab[SYMBOLOGY_EAN8_5].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '4':		// set suffix code 39
						symbology = SYMBOLOGY_CODE39;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '5':		// set suffix codabar
						symbology = SYMBOLOGY_CODABAR;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '6':		// set suffix industrial 2 of 5
						symbology = SYMBOLOGY_D2OF5;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '7':		// set suffix interleaved 2 of 5
						symbology = SYMBOLOGY_I2OF5;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + 'B':		// set suffix S code
						symbology = SYMBOLOGY_SCODE;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('G' << 8) + 'M':		// set suffix Matrix 2of8 code
						symbology = SYMBOLOGY_M2OF5;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('I' << 8) + '9':		// set suffix IATA code
						symbology = SYMBOLOGY_IATA;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '8':		// set suffix Code 93 code
						symbology = SYMBOLOGY_CODE93;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + '9':		// set suffix Code 128 code
						symbology = SYMBOLOGY_CODE128;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('N' << 8) + '5':		// set suffix MSI/Plessey code
						symbology = SYMBOLOGY_CODE128;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('L' << 8) + '9':		// set suffix telepen code
						symbology = SYMBOLOGY_TELEPEN;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('O' << 8) + 'A':		// set suffix UK/Plessey code
						symbology = SYMBOLOGY_UK_PLESSEY;
						memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = 0;
						break;

					case ('P' << 8) + 'R':		// clear all suffixes
						for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
							memset(param_tab[symbology].suffix, '\0', MAX_SUFFIX_LENGTH);
						suffix = -1;
						break;

					default:
						error = 0;
						if ((ch = direct_character_input(barcode[0],barcode[1])) !=0xFF)
						{
							if ( ((prefix != -1) && (prefix >= MAX_PREFIX_LENGTH)) ||
								 ((suffix != -1) && (suffix >= MAX_SUFFIX_LENGTH)) ||
								 ((prefix == -1) && (suffix == -1)))
							{
								error = 1;
								break;
							}

							if (prefix != -1)
							{
								if (symbology == MAX_SYMBOLOGIES)
									for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
										param_tab[symbology].prefix[prefix] = (char)ch;
								else
								{
									param_tab[symbology].prefix[prefix] = (char)ch;
									switch (symbology)
									{
										case SYMBOLOGY_UPCA_2:
											param_tab[SYMBOLOGY_UPCA_5].prefix[prefix] = (char)ch;
											break;

										case SYMBOLOGY_UPCE_2:
											param_tab[SYMBOLOGY_UPCE_5].prefix[prefix] = (char)ch;
											break;

										case SYMBOLOGY_EAN13_2:
											param_tab[SYMBOLOGY_EAN13_5].prefix[prefix] = (char)ch;
											break;

										case SYMBOLOGY_EAN8_2:
											param_tab[SYMBOLOGY_EAN8_5].prefix[prefix] = (char)ch;
											break;

									}
								}
								prefix++;
							}
							else
							{
								if (symbology == MAX_SYMBOLOGIES)
									for (symbology = 0; symbology < MAX_SYMBOLOGIES; symbology++)
										param_tab[symbology].suffix[suffix] = (char)ch;
								else
								{
									param_tab[symbology].suffix[suffix] = (char)ch;
									switch (symbology)
									{
										case SYMBOLOGY_UPCA_2:
											param_tab[SYMBOLOGY_UPCA_5].suffix[suffix] = (char)ch;
											break;

										case SYMBOLOGY_UPCE_2:
											param_tab[SYMBOLOGY_UPCE_5].suffix[suffix] = (char)ch;
											break;

										case SYMBOLOGY_EAN13_2:
											param_tab[SYMBOLOGY_EAN13_5].suffix[suffix] = (char)ch;
											break;

										case SYMBOLOGY_EAN8_2:
											param_tab[SYMBOLOGY_EAN8_5].suffix[suffix] = (char)ch;
											break;

									}
								}
								suffix++;
							}
						}
						else
							error = 1;
						break;
				}	// End switch
				if (error)
				{
					goodreadled(RED, 10);
					sound(TLONG, VHIGH, SERROR, NULL);
					continue;
				}
				goodreadled(GREEN, 10);
				sound(TSTANDARD, VHIGH, SMEDIUM, SHIGH, NULL);
				continue;
			} // end if MENU_CODE


			len = strlen(barcode);

			if (menu_state == IN_MENU_MODE || !connected)
			{
				goodreadled(RED, 10);
				sound(TLONG, VHIGH, SERROR, NULL);
				continue;
			}

			okbeep();
			gotoxy(0,2);
			printf("%-14.14s",barcode );
			scanned_barcode = TRUE;

			// SB!! 	scannerpower( TRIGGER, 10000 );

			//
			// Transmit the barcode, including prefixes and suffixes
			//
			sprintf(msg,"%s%s%s",param_tab[code_id].prefix, barcode, param_tab[code_id].suffix);
			if( scanned_barcode && transmit_barcode( msg ) == OK )
				scanned_barcode = FALSE;
		}

		//
		// Close the SPP port so that reconnection is possible
		//
		comclose( COM_USED );
	}
}
