

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lib.h"
#include "msgscr.h"

//
// Special font from oscar for BOLD display text
//
#include "boldo.fnt"

//
#include "UK_CONT.fnt"
#include "UK_YESNO.fnt"

#define OS_OPL972X		0x01
#define OS_PHL1300		0x02
#define OS_PX25			0x03
#define OS_H13			0x04

#define OPL972X_ROM "LAB"
#define DISP_X_OPL972X	112
#define DISP_Y_OPL972X	64

#define PHL1300_ROM "CAX"
#define DISP_X_PHL1300	96
#define DISP_Y_PHL1300	64

#define H13_ROM "XAC"
#define DISP_X_H13	128
#define DISP_Y_H13	128

#define PX25_ROM "XAD"
#define DISP_X_PX25	112
#define DISP_Y_PX25	64



#define INFO_SZ			12

typedef struct
{
	short x_pixels;
	short y_pixels;
	short max_chars;
	short max_lines;
	short OS;
}screen_info;

screen_info g_scr_info;
char g_msg_buffer[ 512 ]; // buffer for a parsed message


void InitMessageScreen( void )
{
	static char rom[9];
	memset( &g_scr_info, 0, sizeof( screen_info ));
	memset( &g_msg_buffer, '\0', sizeof( g_msg_buffer ));

	version_rom( rom );

	//
	// set init paramters
	//
	if( strstr( rom, OPL972X_ROM ) )
	{
		g_scr_info.x_pixels = DISP_X_OPL972X;
		g_scr_info.y_pixels = DISP_Y_OPL972X;
		g_scr_info.OS = OS_OPL972X;
	}
	else if( strstr( rom, PHL1300_ROM ) )
	{
		g_scr_info.x_pixels = DISP_X_PHL1300;
		g_scr_info.y_pixels = DISP_Y_PHL1300;
		g_scr_info.OS = OS_PHL1300;
	}
	else if(  strstr( rom, PX25_ROM  ) )
	{
		g_scr_info.x_pixels = DISP_X_PX25;
		g_scr_info.y_pixels = DISP_Y_PX25;
		g_scr_info.OS = OS_PX25;

	}
	else if( strstr( rom, H13_ROM ) )
	{
		g_scr_info.x_pixels = DISP_X_H13;
		g_scr_info.y_pixels = DISP_Y_H13;
		g_scr_info.OS = OS_H13;
	}

	//
	// Calculate the maximaum amount of characters that
	// will fit on a single line and the amount of available lines
	//
	g_scr_info.max_chars = g_scr_info.x_pixels / _boldo[1];
	g_scr_info.max_lines = (g_scr_info.y_pixels - INFO_SZ ) / _boldo[0];

}

static int StartParsing( char* message, int max_length )
{
	char* token;
	int lines;
	int curr_linesz;
	int i;
	int ii;

	lines = 0;
	curr_linesz = 0;

	//
	// Parse the psmg on every space
	//
	token = strtok( message, " " );
	while( token != NULL )
	{
		if( (strlen( token )+ curr_linesz) <= max_length )
		{
			strcat( g_msg_buffer, token );
			if( (strlen( token )+ curr_linesz) < max_length )
				strcat( g_msg_buffer, " ");

			if( (strlen( g_msg_buffer) / max_length) > lines )
				lines++;
		}
		else
		{
			lines++;
			ii = lines * max_length;

			//
			// Calculate the place to start filling spaces
			//
			for( i = strlen( g_msg_buffer ); i < ii; i++ )
				g_msg_buffer[ i ] = ' ';

			if( strlen( token ) > max_length )
				lines++;
			strcat( g_msg_buffer, token );
			if( (strlen( g_msg_buffer ) % max_length) != 0 )
				strcat( g_msg_buffer, " " );

		}

		//
		// Calculate the size of string
		//
		curr_linesz = strlen( g_msg_buffer ) % max_length;

		token = strtok( NULL, " " );
	}
	if( curr_linesz )
		lines++;
	return( lines );
}

//
// Parse the message and put result in the g_msg_buffer
// return the number of lines or ERROR on ERROR
//
static int ParseUpMessage( const char* message )
{
	char* pmsg;
	int lines;

	//
	// Cleanup the g_msg_buffer
	//
	memset( &g_msg_buffer, '\0', sizeof( g_msg_buffer ));

	pmsg = (char*) malloc( strlen( message ) + 1);
	if( pmsg == NULL )
		return ERROR; // Error allocating memory

	strcpy( pmsg, message );
	if( (lines = StartParsing( pmsg, g_scr_info.max_chars )) > g_scr_info.max_lines )
	{
		free( pmsg );
		memset( &g_msg_buffer, '\0', sizeof( g_msg_buffer ));

		pmsg = (char*) malloc( strlen( message ) + 1);
		if( pmsg == NULL )
			return ERROR; // Error allocating memory
		strcpy( pmsg, message );
		lines = StartParsing( pmsg,  g_scr_info.max_chars -1 );
	}

	free( pmsg );
	return( lines );
}

static void DisplayHeader( const short language, const short msg_type, int showupdown )
{
	int i;
	int endpos;
	unsigned int fw = 0, fh = 0;
	unsigned int x_ofs, y_ofs;
	putchar('\f');

//	if( g_scr_info.OS == OS_OPL972X  )
	{
		if( (msg_type & MB_ALL) && !(msg_type & MB_YESNO) )
			setfont( USER_FONT, (unsigned char*) _UK_CONT );
		else
			setfont( USER_FONT, (unsigned char*) _UK_YESNO );
	}

	if( msg_type & MB_YESNO )
		showupdown = TRUE;
	//
	// Get the fontsize to calculate the amount of characters fit on a line
	//
	getfontsize( &fh, &fw );
	endpos = 7 - (showupdown?0:1);

	//place key labels in the middle of the screen.
	y_ofs = g_scr_info.y_pixels % fh;
	x_ofs = (g_scr_info.x_pixels - (7 * fw))/2;
	
	set_offset( y_ofs | ( x_ofs << 16 ) ); 

	for( i = (showupdown?0:1) ; i < endpos/*(g_scr_info.x_pixels / fw)*/; i++ )
	{
		gotoxy( i, ( g_scr_info.y_pixels / fh )-1 );
		printsymbol( i );
	}
	set_offset( 0 );
}

static void DisplayText( int start, int line_length )
{
	int i;
	for( i = 0; i < g_scr_info.max_lines; i++ )
	{
		gotoxy(0, i);
		printf("%-*.*s", line_length, line_length, g_msg_buffer+((start+i)*line_length) );
	}
}

static void DisplayScrollbar( int draw, int current, int max )
{
	int stages;
	static int gap;
	unsigned int fw = 0, fh = 0;

	if( !draw )
		return; // No need to draw a progressbar

	getfontsize( &fh, &fw );
	stages = ( g_scr_info.y_pixels - fh - (g_scr_info.y_pixels%fh) ) / max;
	//
	// Remove previous gapline
	//
	drawline( g_scr_info.x_pixels -2, gap+1, g_scr_info.x_pixels-2, gap+5, 0 );
	setpixel( g_scr_info.x_pixels -3, gap, 0 );
	setpixel( g_scr_info.x_pixels -3, gap+6, 0 );

	//
	// Draw new progressbar
	//
	drawline( g_scr_info.x_pixels -4, 0/*fh*/, g_scr_info.x_pixels-4, g_scr_info.y_pixels -1 - fh - (g_scr_info.x_pixels%fh), 1 );

	gap = ( current != max)?(current*stages):(g_scr_info.y_pixels-fh-7-(g_scr_info.x_pixels%fh));
	drawline( g_scr_info.x_pixels -4, gap+1, g_scr_info.x_pixels -4, gap+5, 0 ); // small gap in line
	drawline( g_scr_info.x_pixels -2, gap+1, g_scr_info.x_pixels -2, gap+5, 1 ); // small gap in line
	setpixel( g_scr_info.x_pixels -3, gap, 1 );
	setpixel( g_scr_info.x_pixels -3, gap+6, 1 );

}


int ShowMessage( const char* message, const short language, const short msg_type, int exit_on_connect )
{
	int start;
	int lines;
	int line_length;
	int key;
	extern void keybeep(void);
	extern void errbeep( void);
	extern void info_sound( void);
	extern int check_connection(void);
	cursor( NOWRAP );
	resetkey();
	//
	// Parse the message and put the result in the g_msg_buffer
	//
	if( (lines = ParseUpMessage( message )) == ERROR )
		return ERROR;

	if( lines > g_scr_info.max_lines )
		line_length = g_scr_info.max_chars -1;
	else
		line_length = g_scr_info.max_chars;

	switch( msg_type & MB_ICONMASK )
	{
		case MB_ICONWARNING:
			break;
		case MB_ICONERROR:
			errbeep();
			break;
		case MB_ICONQUESTION:
			break;
		case MB_ICONINFORMATION:
			info_sound();
			break;

	}
	//
	// Set the kind of diplay header and beep
	//
	DisplayHeader( language, msg_type, (lines > g_scr_info.max_lines) );

	start = 0;
	setfont( USER_FONT, (unsigned char*)_boldo );
	for(;;)
	{
		idle();
		if (exit_on_connect)
		{
			if( check_connection() != FALSE)
			{
				return CONNECT;
			}
		}
		DisplayText( start, line_length );
		DisplayScrollbar( (lines > g_scr_info.max_lines), start, (lines - g_scr_info.max_lines ));

		if ((key=getchar())==EOF)
			continue;
		keybeep(); 					// make the beeping sound

		switch( key )
		{
			case Q1_KEY:
				if( msg_type & MB_YESNO )
					return YES;

				if( start > 0 )
					start--;
				break;
			case Q2_KEY:
				if( msg_type & MB_YESNO )
					return NO;

				start++;
				if( start+g_scr_info.max_lines > lines )
					start--;
				break;
			default:

				if( msg_type & MB_ALL )
				{
					if( msg_type & MB_YESNO )
					{
						ungetc(key, stdin);
						return ERROR;
					}

					return TRUE;
				}
				break;
		}
	}
}
