
#include		<stdio.h>
#include		<stdlib.h>
#include		<string.h>
#include        <ctype.h>
#include 		<stdarg.h>
#include		<limits.h>
#include 		<setjmp.h>

#include        "lib.h"
#include		"utils.h"

#include "battery.fnt" // the battery font file 7 images and batt empty and almost empty 
#include "numrev.fnt"  // the reversed numeric font file shown with battery charging icons

#define SCREEN_DUMP	"~src.tmp"

// the jump buffer when battery power is empty 
extern jmp_buf batt_power;

// this variable controls the icon to show when battery is low or empty 
extern int 	battery_msg_status;

void restore_font( int width, int height );
void display_icon( int x, int y, int state );
void check_batt_empty( void );

void empty_batt_beep( void )
{
	sound(TLONG,VHIGH,SERROR,SPAUSE,SERROR,SPAUSE, SERROR,0);	
}

void display_battery_txt_msg( int empty )
{
	putchar('\f');
	setfont( MEDIUM_FONT, NULL ); 
	
	if( empty )
	{
		gotoxy(6, 0);
		printf("Battery is");
		gotoxy(6, 1);
		printf("empty.");
		gotoxy(0, 3);
		printf("Put PX-25 in");
		gotoxy(0, 4);
		printf("cradle to charge.");
	}
	else
	{
		gotoxy(6, 0);
		printf("Battery is");
		gotoxy(6, 1);
		printf("near empty.");
		gotoxy(6, 2);
		printf("Press a key");
		gotoxy(0, 3);
		printf("or put PX-25 in");
		gotoxy(0, 4);
		printf("cradle to charge.");
	}
}

void show_empty_batt_msg( int empty )
{
	int state;
	unsigned int width, height;	// for font size 
	int curs_x, curs_y; 		// for cursor postion 

	// store the user settings 
	savedisplay((char*) SCREEN_DUMP );
	getfontsize( &height, &width ); // get the font size 
	curs_x = wherex();	// retrieve cursor x pos 
	curs_y = wherey(); 	// retrieve cursor y_pos 

	empty_batt_beep();
	display_battery_txt_msg( empty );
	if( empty )
		setfont( USER_FONT, (unsigned char*) _empty );	// the empty batter font 
	else
		setfont( USER_FONT, (unsigned char*) _almost ); // the almost empty battery font 
		
	state = 0;
	resetkey();
	do{
		display_icon( 0, 0, state );
		state ^= 1;

		delay( 15 ); // wait 300ms some time to display image 
		if( !empty )
			check_batt_empty();
	}while((ischarging() == 0) && (( getchar() == EOF ) || empty ));
	
	// restore the saved settings 
	loaddisplay((char*) SCREEN_DUMP ); // restore screen 
	remove( (char*) SCREEN_DUMP );		
	restore_font( width, height ); 	// restore font 
	gotoxy( curs_x, curs_y ); 		// restore cursor position 
}

void check_batt_empty( void )
{
	int voltage = GetBatteryVoltage_mV();
		
	if( voltage <= BATT_EMPTY )
	{
		scannerpower( OFF, 0 );		// turn scan engine off 
		
		// to start of main application 
		longjmp( batt_power, 5 );	
	}
		
	if( voltage <= BATT_LOW && battery_msg_status == 0 )
	{
		battery_msg_status++;
		show_empty_batt_msg( FALSE ); // near empty battery message 
	}
}


void restore_font( int width, int height )
{
	// use the font size to restore the font type 
	if( width == 8 && height == 16 )
		setfont( LARGE_FONT, NULL );
	else if( width == 6 && height == 12 )
		setfont( MEDIUM_FONT, NULL );
	else if( width == 8 && height == 8 )
		setfont( SMALL_FONT, NULL );
	else if( width == 6 && height == 6 )
		setfont( TINY_FONT, NULL );
	else
		setfont( MEDIUM_FONT, NULL );
}

void display_icon( int x, int y, int state )
{
	int startpos;
	startpos = state * 16;
	
	gotoxy( x, y );
	printsymbol( startpos );
	gotoxy( x+1, y );
	printsymbol( startpos+1 );
	gotoxy( x+2, y );
	printsymbol( startpos+2 );
	gotoxy( x+3, y );
	printsymbol( startpos+3 );

	gotoxy( x, y+1 );
	printsymbol( startpos+4 );
	gotoxy( x+1, y+1 );
	printsymbol( startpos+5 );
	gotoxy( x+2, y+1 );
	printsymbol( startpos+6 );
	gotoxy( x+3, y+1 );
	printsymbol( startpos+7 );

	gotoxy( x, y+2 );
	printsymbol( startpos+8 );
	gotoxy( x+1, y+2 );
	printsymbol( startpos+9 );
	gotoxy( x+2, y+2 );
	printsymbol( startpos+10 );
	gotoxy( x+3, y+2 );
	printsymbol( startpos+11 );
	
	gotoxy( x, y+3 );
	printsymbol( startpos+12 );
	gotoxy( x+1, y+3 );
	printsymbol( startpos+13 );
	gotoxy( x+2, y+3 );
	printsymbol( startpos+14 );
	gotoxy( x+3, y+3 );
	printsymbol( startpos+15 );
}

void display_rev_voltage( int x, int y, unsigned int voltage )
{
	gotoxy( x, y );
	printsymbol( 11 ); // V 
	gotoxy( x+1, y );
	voltage /= 10;
	printsymbol( voltage % 10 );
	voltage /= 10;
	gotoxy( x+2, y );
	printsymbol( voltage % 10 );
	voltage /= 10;
	gotoxy( x+3, y );
	printsymbol( 10 ); // . 
	gotoxy( x+4, y );
	printsymbol( voltage % 10 );
	
	rectangle3drev( (x*6)-1, (y*12), ((x+5)*6)+1, (y+1)*12, 1); // draw 3D box in reversed fashion
}

void is_on_cradle( void )
{
	int state;
	int charge_state;
	unsigned int width, height; // for font size 
	int curs_x, curs_y; 		// for cursor postion 
	unsigned int voltage;
	if( ischarging() )
	{
		// quickly store an image of the screen 
		savedisplay((char*) SCREEN_DUMP );
		getfontsize( &height, &width ); // get the font size 
		curs_x = wherex();	// retrieve cursor x pos 
		curs_y = wherey(); 	// retrieve cursor y_pos 
		putchar('\f'); 		// clear the screen to show battery 
		state = 0;
		while( (charge_state = ischarging()) )
		{	
			// the font with the battery images 
			setfont( USER_FONT, (unsigned char*)_battery ); 
			if( charge_state == 2 )
				state = 6;
			display_icon( 8, 2, state );
			
			if(++state > 6 )
				state = 0;
			
			setfont( USER_FONT, (unsigned char*)_num_rev ); // the reversed font 
			voltage = GetBatteryVoltage_mV();				
			display_rev_voltage( 3, 2, voltage );
			
			delay( 15 ); // wait 300ms some time to display image 
			
			if( voltage > BATT_CHARGE_LOW )
				battery_msg_status = 0; // set icon to almost empty 
		}
		
		loaddisplay((char*) SCREEN_DUMP ); 	// restore screen 
		remove( (char*) SCREEN_DUMP );		
		restore_font( width, height ); 		// restore font 

		gotoxy( curs_x, curs_y ); 			// retore cursor position 
	}
}



void draw_bar( int x_pos, int y_pos, int width, int height, int color )
{
	int count;
	if( width > height )
	{
		for( count = 0; count < height; count++ )
			drawline( x_pos, y_pos+count, x_pos+width, y_pos+count, color );
		return;
	}
	for( count = 0; count < width; count++ )
		drawline( x_pos+count, y_pos, x_pos+count, y_pos+height, color );
}

// make sound to inform the user 
void info_sound( void )
{
	sound( TSHORT, VMEDIUM, SHIGH, SLOW, SHIGH, SHIGH, SLOW, 0);
}

void keybeeplow(void)
{
	sound(TSHORT,VLOW,SHIGH,0);
}

void keybeep(void)
{
	sound(TSHORT,VMEDIUM,SHIGH,0);
}

void errkeybeep(void)
{
	sound(TLONG,VMEDIUM,SLOW,0);
}

void okbeep(void)
{
	sound(TSTANDARD,VMEDIUM,SHIGH,SMEDIUM,SHIGH,0);
}

void errbeep(void)
{
	sound(TLONG,VMEDIUM,SLOW,SMEDIUM,SHIGH,0);
}

int waitkey(void)
{
	int c;

	while((c=getchar())==EOF)
	{
		check_batt_empty(); // check the battery 
		is_on_cradle();		// check to see if OPL972x is placed on the cradle 
		idle();
	}
	keybeep(); // make the beeping sound 
	return c;
}

int waitkey_no_charge( void )
{
	int c;

	while((c=getchar())==EOF)
	{
		idle();
	}
	keybeep();
	return c;
}

void rectangle(int x1,int y1,int x2,int y2,int color)
{
	drawline(x1,y1,x2,y1,color);
	drawline(x1,y2,x2,y2,color);
	drawline(x1,y1,x1,y2,color);
	drawline(x2,y1,x2,y2,color);
}


void rectangle3d(int x1,int y1,int x2,int y2,int color)
{
	rectangle(x1,y1,x2,y2,color);
	drawline(x1+2,y2+1,x2+2,y2+1,1);
	drawline(x1+2,y2+2,x2+2,y2+2,1);
	drawline(x2+1,y1+2,x2+1,y2+2,1);
	drawline(x2+2,y1+2,x2+2,y2+2,1);
}

void rectangle3drev(int x1,int y1,int x2,int y2,int color)
{
	rectangle(x1,y1,x2,y2,color);
	
	drawline(x1-2, y1-1, x2-2, y1-1, 1);
	drawline(x1-2, y1-2, x2-2, y1-2, 1);
	drawline(x1-1, y2-2, x1-1, y1-2, 1);
	drawline(x1-2, y2-2, x1-2, y1-2, 1);
}

int ScanBarcode( char* string, int min_length, int max_length)
{
	int key;
	struct barcode code;

	unsigned long time_out_value;

	time_out_value = 0;

	code.min = min_length;
	code.max = max_length;
	code.text = string;
	scannerpower( TRIGGER, 10000 );

	while( readbarcode( &code ) != OK )
	{
		if( kbhit() )
		{
			if(( key = getchar() ) != TRIGGER_KEY )
			{
				// put the character back in the input buffer 
				// so it can be handled by another function
				ungetc( key, stdin); 
				scannerpower( OFF, 0);
				return( ERROR );
			}
			scannerpower( TRIGGER/*SINGLE*/, 10000 );

		}
		// on cradle
		check_batt_empty();
		is_on_cradle();
		idle();
	}

	return( OK );
}

// 0 = sunday, 1 = monday, 2 = tuesday, 3 = wensday 
// 4 = thursday, 5 = friday, 6 = saturday 
int what_day_of_week( void )
{
	int m, y, a;
	struct date curr_date;
	getdate( &curr_date );
	a= (14 - curr_date.da_mon ) / 12;
	y= curr_date.da_year - a;
	m= curr_date.da_mon + (12*a) - 2;
	return((curr_date.da_day + y + (y/4) - (y/100) + (y/400) + ((31*m)/12))%7 );
}

static void store_char_in_string( int key, int max_len, char *string, int typ, int decimals )
{
	int i, ok;
	char *ptr;
	i = strlen( string );

	if(( i >= max_len ) && (key != '-'))
		return;

	ok = 0;
	if( typ & FLOAT )
	{
		ok = (isdigit( key ) || ((strchr( string, '.') == NULL )&& (key == '.')));
		if( ok && ( ptr = strchr( string, '.')))
		{	// we now need to check the decimal length 
			if((strlen(string) - (ptr-string)) > decimals )
				ok = FALSE;
		}
		else if( ok && (key != '.'))
		{
			if( (strlen(string) - ((typ & NEGATIVE)? 1:0)+1) > ((max_len-decimals) -((typ & NEGATIVE)? 1:0) - 1))
				ok = FALSE;
		}
	}
	if( !ok && ( typ & NUM ))
		ok = isdigit( key );
	if( !ok && (typ & ALPHA ))
		ok = (isalpha(key) || key == ' ');
	if( !ok && ( typ & NEGATIVE ))
		ok = ( key == '-')? TRUE : FALSE;
	if( ok )
	{
		if(key == '-')
		{
			if( i )
				string[0] = (string[0] == '-')? ' ' : '-';
			else
			{
				string[i++] = (string[0] == '-')? ' ' : '-';
				string[i] = '\0';
			}
		}
		else if( typ & NEGATIVE )
		{
			if( i == 0 )
				string[i++] = (string[0] == '-')? '-' : ' ';
			string[i++] = key;
			string[i] = '\0';
		}
		else
		{
			string[i++] = key;
			string[i] = '\0';
		}
	}
}

// check special keys 
static int check_key_input( int key, int num, va_list key_list )
{
	int count;

	for( count = 0; count < num; count++ )
	{
		if( (int) va_arg( key_list, int ) == key )
		{
			va_end( key_list ); // needed for normal returning 
			return( OK );
		}
	}
	va_end( key_list );
	return( ERROR );
}

static void display_input( char* string, int x, int y, int max_length,int max_x )
{
	int length;
	int output;

	if( x < 0 || y < 0 )
		return;

	length = strlen( string );

	gotoxy(x,y);
	if( (x + length) < max_x )
	{
		if( length == max_length )
			output = printf("%s", string );
		else
			output = printf("%s ", string );
	}
	else
	{
		if( length == max_length )
			output = printf("%s", string + (length - max_x + x));
		else
			output = printf("%s ", string + (length - max_x + x));
	}
	gotoxy( x+output-1, y); // set the cursor to the correct position 
}

static void remove_one_from_buffer( char* string )
{
	if( strlen( string ) > 0 )
		string[ strlen( string ) - 1 ] = '\0';
}

static void clear_part( int x, int y, int max_x )
{
	int i;
	for( i = x ; i <= max_x; i++ )
	{
		gotoxy( i, y );
		putchar(' ');
	}
}


int KeyboardBarcode( char* string, int min_length, int max_length, int typ, int x, int y,int num, ... )
{
	va_list key_list;
	int key;

	if( num <= 0 )
		return( EOF );

	for(;;)
	{
		va_start( key_list, num );

//		if( strlen(string) < max_length )
//			cursor( ON );
//		else
//			cursor( OFF );
							
		if((( key = waitkey()) == ENT_KEY ) && (strlen(string) < min_length ))
			continue;

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			cursor( OFF );
			return( key );
		}
		va_end(key_list);

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			display_input(string, x, y, 15, 15);
			continue;
		}

		store_char_in_string( key,  max_length, string, typ, 0 );
		display_input(string, x, y, 15, 15);
	}
}

int ScanOrKeyboardInput( char* string, int min_length, int max_length, int typ, int x, int y, int offset )
{
	int	key;
	for(;;)
	{
		string[0] = '\0';
		set_offset( offset );
		clear_part( x, y, 21 );
		display_input( string, x, y, 21, 21 );
		if( ScanBarcode( string, min_length, max_length ) == OK )
		{
			display_input( string, x, y, 21, 21 );
			goodreadled( GREEN, 10 );
			scannerpower( OFF, 0);
			sound( TSTANDARD, VSTANDARD, SMEDIUM, 0 );
			set_offset( 0 );
			return( SCANNED ); // Input by scanning
		}

		key = KeyboardBarcode( string, min_length, max_length, typ, x, y, 5, TRIGGER_KEY, CLR_KEY, ENT_KEY, Q1_KEY, Q2_KEY );
		set_offset( 0 );
		switch( key )
		{
			case ENT_KEY:
				return( KEYBOARD );	// Keyboard input
			case TRIGGER_KEY:
				break;

			case CLR_KEY:
				if( strlen( string ) != 0)
				{
					break;
				}
			default:
				return( key );
		}
	}
}

static void store_char_in_string_quant( int key, int max_len, char *string, int typ )
{
	int i, ok;
		
	max_len += ((typ & NEGATIVE)&&(string[0]=='-'))?1:0;
	i = strlen( string );

	if(( i >= max_len ) && (key != '-'))
	{
		return;
	}

	ok = 0;

	if( typ & NUM )
		ok = isdigit( key );
	if( !ok && ( typ & NEGATIVE ))
		ok = ( key == '-')? TRUE : FALSE;
	if( ok )
	{
		if( key == '-')
		{
			if( string[0] == '-' )
			{
				memmove( string, string+1, i);
			}
			else
			{
				memmove( string+1, string, i);
				string[0] = '-';
			}
		}
		else
		{
			string[i++] = key;
			string[i] = '\0';	
		}
	}
}

static void display_input_quantity( char* string, int x, int y, int max_length,int max_x, int start, long def_val )
{
	int length;
	int output;

	if( x < 0 || y < 0 )
		return;

	length = strlen( string );

	gotoxy(x,y);
	if( start )
		output = printf("%*ld", max_x, def_val );
	else
		output = printf("%*.*s", max_x, max_x, string );
	
	gotoxy( x+output-1, y); // set the cursor to the correct position 
}

//
// max_lenght is the maximum length without the minus sign (-)
int KeyboardQuantity( char* string, int max_length, int typ, int x, int y, int offset, long def_val, int max_x_d, int rect, int num, ... )
{
	va_list key_list;
	int key;
	int defaul;

	defaul = TRUE;
	if( num <= 0 )
		return( EOF );

	for(;;)
	{
		va_start( key_list, num );

		set_offset( offset );

		display_input_quantity(string, x, y, max_length, max_x_d, defaul, def_val);
		if( rect )
			rectangle( (x*8)-2, (y*16)+offset, ((x+max_x_d)*8), ((y+1)*16)-3+offset, 1);

		key =waitkey(); 

		if( check_key_input( key, num, key_list ) == OK )
		{
			va_end( key_list );
			set_offset( 0 );

			if( defaul )
				return( key | DEFAULT );
			else
				return( key  );
		}
		va_end(key_list);
		defaul = FALSE;

		if( key == BS_KEY )
		{
			remove_one_from_buffer( string );
			continue;
		}

		store_char_in_string_quant( key,  max_length, string, typ );
	}
}
