/***********************************************************************
 *
 *  Copyright (C) Opticon Sensors Europe B.V.
 *
 ***********************************************************************/

#ifndef __LIB_H__
#define __LIB_H__

 /*
 *--------------------------------------
 *  General definitions
 *--------------------------------------
 */
#define OK                  0
#define ERROR               -1
#define ON                  1
#define OFF                 0
#define YES                 1
#define NO                  0
#define TRUE                1
#define FALSE               0
#define DISABLE             0
#define ENABLE				1

#define NG					100
#define STANDBY				2

/*
 *--------------------------------------
 *  ASCII definitions
 *--------------------------------------
 */
#define SOH             0x01
#define STX             0x02
#define ETX             0x03
#define EOT             0x04
#define ENQ             0x05
#define ACK             0x06
#define BEL             0x07
#define BS              0x08
#define HT              0x09
#define LF              0x0A
#define VT              0x0B
#define FF              0x0C
#define CR              0x0D
#define SO              0x0E
#define SI              0x0F
#define DLE             0x10
#define DC1             0x11
#define DC2             0x12
#define DC3             0x13
#define DC4             0x14
#define NAK             0x15
#define SYN             0x16
#define ETB             0x17
#define CAN_CHAR        0x18
#define EM              0x19
#define SUB             0x1A
#define ESC             0x1B
#define FS              0x1C
#define GS              0x1D
#define RS              0x1E
#define US              0x1F
#define SP              0x20
#define DEL             0x7F

/*
 *--------------------------------------
 *  Keyboard functions
 *--------------------------------------
 */
#define TRIGGER_KEY     0x80
#define UP_KEY          0x81
#define DOWN_KEY        0x82
#define Q1_KEY          UP_KEY
#define Q2_KEY          DOWN_KEY
#define	ENT_KEY			0x83
#define CLR_KEY			0x84
#define BS_KEY			0x85
#define LEFT_KEY		0x86
#define RIGHT_KEY		0x87
#define SHIFT_KEY		0x88

#define F1_KEY          0x90
#define F2_KEY          0x91
#define F3_KEY          0x92
#define F4_KEY          0x93
#define SHIFT_F1_KEY    0x94
#define SHIFT_F2_KEY    0x95
#define SHIFT_F3_KEY    0x96
#define SHIFT_F4_KEY    0x97
#define PW_KEY          0xFF

/*
 *--------------------------------------
 *  scnanner functions
 *--------------------------------------
 */
/* bar code ID member */
/*
 * Constants for id member
 */
#define EAN13           0x01
#define UPCA            0x02
#define EAN8            0x03
#define UPCE            0x04
#define UPCE1			0x05
#define ISBN			0x06
#define ISSN			0x07
#define ISMN			0x08
#define CODE39          0x09
#define CODABAR         0x0A
#define NW7             CODABAR
#define D2OF5           0x0B                    // industrial 2 of 5
#define I2OF5           0x0C                    // interleaved 2 of 5
#define CODE93          0x0D
#define CODE128         0x0E
#define MSI_PLESSEY     0x0F
#define IATA            0x10
#define UK_PLESSEY      0x11
#define TELEPEN         0x12
#define M2OF5           0x13
#define CHINPOST        0x14
#define NW7_ABC         0x15
#define NW7_CX          0x16
#define SCODE           0x17
#define LAETUS          0x18
#define TRIOPTIC        LAETUS
#define C39_FA          0x19                    // Full ascii
#define C39_IT          0x1A                    // Italian pharma code
#define EAN128          0x1B

#define RSS_14          0x1C
#define RSS_LIMITED     0x1D
#define RSS_EXPANDED    0x1E

#define PDF417          0x1F
#define CCC_COMPOSITE   0x20

#define MICRO_PDF417    0x21
#define CCA_COMPOSITE   0x22
#define CCB_COMPOSITE   0x23

#define CODE11          0x24

#define EAN128_CCC      0x25                    // EAN-128 with CC-C composite
#define CODE3OF5        0x26                    // Code 3 of 5 A.K.A. Korean Postal Code

/*** ADD ON +2 ***/
#define ADD_ON_2        0x40
#define EAN13_2         0x41
#define UPCA_2          0x42
#define EAN8_2          0x43
#define UPCE_2          0x44
#define UPCE1_2         0x45
#define CODE128_2       0x4E                    // the same group CODE-128

/*** ADD ON +5 ***/
#define ADD_ON_5        0x80
#define EAN13_5         0x81
#define UPCA_5          0x82
#define EAN8_5          0x83
#define UPCE_5          0x84
#define UPCE1_5         0x85
#define CODE128_5       0x8E                    // the same group CODE-128

#define PARTIAL_READ	0x40
#define ADD_ON_MASK     0xC0
#define MENU_CODE       0xF0                    // OSE Code-39 Menu label
#define MENU_CODE_PDF   0xF1                    // OSE (u)PDF417 Menu label
#define MENU_CODE_C128  0xF2                    // OSE Code-128 Menu label

/* ScannerPower */
#define TRIGGER             2
#define SINGLE              3
#define MULTIPLE            4
#define CONTINUOUS          5
#define SUSPEND				6
#define RESUME				7

/*
 *--------------------------------------
 *  Display / Power management functions
 *--------------------------------------
 */
#define DISP_WIDTH  	112
#define DISP_HEIGHT 	64

/* setpixel / drawline */
#define INVERT              2

/* setfont */
#define TINY_FONT           0x01
#define SMALL_FONT          0x02
#define MEDIUM_FONT         0x03
#define LARGE_FONT          0x04
#define HUGE_FONT    		0x05
#define USER_FONT           0x06
#define FILE_FONT           0x07

/* cursor */
#define BLINKING            2
#define AUTOWRAP            3
#define NOWRAP              4
#define DOICONS				5
#define NOICONS				6


/* DisplayDirection */
#define DISP_NORMAL         0
#define DISP_REVERSE        1

/* backlight / autopowerdown */
#define OFF                 0
#define ON                  1
#define TOGGLE              2
#define RESET               3
#define AUTO                4
#define NOW                 5
#define SUSPEND             6
#define RESUME              7

#define LOW					1
#define MEDIUM				2
#define HIGH				3

/*
 *--------------------------------------
 *  Led/sound functions
 *--------------------------------------
 */
/* GoodRead */
#define RED					0x01
#define GREEN				0x02
#define BLUE                0x04
#define VMOTOR				0x08
#define ORANGE				(RED|GREEN)
#define WHITE               (RED|GREEN|BLUE)
#define RED_FLASH			0x10
#define GREEN_FLASH			0x20
#define BLUE_FLASH          0x40
#define	VMOTOR_FLASH		0x80
#define ORANGE_FLASH		(RED_FLASH|GREEN_FLASH)
#define WHITE_FLASH         (RED_FLASH|GREEN_FLASH|BLUE_FLASH)

#define FOREVER             -1

/*
 * definitions for sound volume level
 */
#define VOFF                0
#define VLOW                1
#define VSTANDARD           8
#define VMEDIUM             32
#define VHIGH               127

/*
 * definitions for sound tone duration
 */
#define TCLICK              1
#define TSHORT              3
#define TSTANDARD           5
#define TLONG               10
#define TVLONG              20


/*
 * definitions for sound tone
 */
#define SPAUSE              1
#define SERROR              100
#define SLOW                1000
#define SMEDIUM             2500
#define SHIGH               3500

/* ischarging */
#define OFF_CRADLE          0
#define CHARGING            1
#define ON_CRADLE           2           /* in cradle and charging is complete */
#define TEMP_OUT_OF_RANGE   3
#define CHARGING_TO_ERR     4

/*
 *--------------------------------------
 *  Memory/file functions
 *--------------------------------------
 */
/*  file system */
#define D_NAME              8
#define D_EXT               3

#define ERR_NOT_SUPPORTED   (-2)
#define ERR_PARAMETER       (-3)
#define ERR_NOT_INSERTED    (-4)
#define ERR_NOT_FORMAT      (-5)

#ifndef O_BINARY
	#define O_BINARY        0x00    /*   binary file                 */
#endif

/*
 *--------------------------------------
 * Serial communication functions
 *--------------------------------------
 */
#define COM0                    0   /*  UART                 		*/
#define COM1                    1   /*  IrDA                 		*/
#define COM2                    2   /*  IrDA Cradle          		*/
#define COM3                    3   /*  Bluetooth SPP/Master 		*/
#define COM4                    4   /*  Reserved        			*/
#define COM5                    5   /*  Bluetooth SPP/Slave  		*/
#define COM6                    6   /*  Bluetooth SPP/Master(no connection) */
#define COM7                    7   /*  IEEE802.15.4         		*/
#define COM8                    8   /*  Reserved           			*/
#define COM9                    9   /*  Reserved     				*/
#define COM10                  10   /*  Reserved     				*/
#define COM11                  11   /*  Reserved      				*/
#define COM12                  12   /*  Bluetooth HID				*/
#define COM13                  13   /*  Bluetooth HID master		*/

/*
 * definitions for xmodem receive
 */
#define XMR_OK              0
#define XMR_CREATE_ERR      1
#define XMR_POLL_ERR        2
#define XMR_ABORT_ERR       3
#define XMR_BLOCK_ERR       4
#define XMR_CRC_ERR         5
#define XMR_CHECKSUM_ERR    6
#define XMR_WRITE_ERR       7
#define XMR_RESPONSE_ERR    8
#define XMR_MEM_ERR         9

/*
 * definitions for xmodem transmit
 */
#define XMT_OK              0
#define XMT_OPEN_ERR        1
#define XMT_RESPONSE_ERR    2
#define XMT_POLL_ERR        3
#define XMT_READ_ERR        4
#define XMT_ACK_ERR1        5
#define XMT_ACK_ERR2        6
#define XMT_MEM_ERR         7



#define ERR_USER_BREAK      -3
#define ERR_EOT_HOST        -5
#define ERR_HOST_REQ        -6
#define ERR_OPEN_FILE       -8
#define ERR_WRITE_FILE      -9
#define ERR_READ_FILE       -10
#define ERR_OUT_SYNC        -12
#define ERR_ALOC_MEM        -13
#define ERR_BCC_FAIL        -2          // checksum error (internal)
#define ERR_TIME_OUT        -4          // time out error (internal)
#define ERR_HOST_LOST       -7          // time out on first char (internal)
#define ERR_NAK_RECV        -11         // NAK received from host (internal)

#define VERSION_INFO  __attribute__ (( section (".app_version")))				// Use to add version info to  the application header ( 16 byte maximum )


/*
 ***********************************************************************
 *  Type Define
 ***********************************************************************
 */
struct barcode
{
    char *text;
    int length;
    int id;
    int min;
    int max;
};

struct time
{
    unsigned char ti_hour;
    unsigned char ti_min;
    unsigned char ti_sec;
};

struct date
{
    unsigned int  da_year;
    unsigned char da_day;
    unsigned char da_mon;
};

struct ffblk
{
    char name[D_NAME+1];
    char ext[D_EXT+1];
    int filesize;
    char search_name[D_NAME+1];
    char search_ext[D_EXT+1];
    int slot;
    int drive;
};

//-------------------------------------------------------------------
//---------------------- Prototypes ---------------------------------
//-------------------------------------------------------------------

// Operating system functions
void autopowerdown(unsigned short what, unsigned short time);
void backlight(short what, short time);
unsigned short batteryvoltage(void);
short checktime(struct time *timep);
short checkdate(struct date *datep);
int comopen(unsigned short port);
void comclose(unsigned short port);
int contrast(int value);
unsigned long coreleft(void);			// Returns memory left on the file system
unsigned long Heap_CoreLeft(void);		// Returns memory left on Heap
void cursor(int mode);
void delay(unsigned short time);
void download(void);
void downloadapplication(int port);
unsigned int downpressed(void);
void drawline(int x1,int y1,int x2,int y2,int mode);
unsigned int endtimer(void);
int findnext(struct ffblk *ffblk);
int findfirst(const char *fname,struct ffblk *ffblk);
int flipscreen(int mode);
int format(void);
long fsize(const char *filename);
short getcom(int timeout);
short getdate(struct date *dp);
void getfontsize(unsigned int *height, unsigned int *width);
unsigned long GetSystemTime(void);
unsigned int getterminalid(void);
unsigned int GetTickCount(void);
short gettime(struct time *tp);
void goodreadled(int on_off, int time);
void gotoxy(int x, int y);
void idle(void);
int isBuzzerOn(void);
unsigned short ischarging(void);
unsigned int kbhit(void);
int loaddisplay(char *filename);
int neto_receive(char (*filename)[12 + 1], const char *term_id, int tkey, int ypos);
int neto_receive2(char (*filename)[12 + 1], const char *term_id, unsigned int timeout, int tkey, int ypos);
int neto_transmit(char (*filename)[12 + 1], int file_count, const char *term_id, int tkey, int ypos);
int neto_transmit2(char (*filename)[12 + 1], int file_count, const char *term_id, unsigned int timeout, int tkey, int ypos);
unsigned short powereddown(void);
void printsymbol(unsigned data);
int putcom(int data);
int readbarcode(struct barcode *barcodep);
void resetkey(void);
void resetreceive(void);
int runapplfromfile(char *filename);
int savedisplay(char *filename);
unsigned short scanneroff(void);
void scannerpower(int mode, int time);
void set_offset(int offset);
void SetAlarm(struct time *timep, unsigned short onoff);
short setdate(struct date *dp);
void setecho(int echomode);
int setfont(unsigned int type, unsigned char *font_ptr);
void setpixel(int x, int y, int state);
void SetSystemTime(unsigned long);
void setterminalid(unsigned int id);
void SetTickCount(unsigned int value);
short settime(struct time *timep);
void showpicture(int x_offs, int y_offs, const unsigned char *ptr);
void sound(int time, int vol, ...);
void starttimer(unsigned int timevalue);
int systemsetting(const char *string);
unsigned int timervalue(void);
unsigned int triggerpressed(void);
unsigned int uppressed(void);
void vibrate( int time );
int wherex(void);
int wherey(void);
int xmodemreceive(const char *filename);
int xmodemtransmit(const char *filename);


// Special functions that are not mentioned in the manual

// Versions
void version_os(char os[10]);
void version_rom(char rom[10]);

char *GetOsVersion(void);
char *GetBootVersion(void);

// Flash
int GetFlashSize(void);
void eraseblock(int block_nr);
void erasesector(int sector_nr);
void programbyte(unsigned char *address, unsigned char d);
void programword(unsigned short data, unsigned short *address);
void programbuffer(unsigned char *src, unsigned char *dest, unsigned int len);

// Interrupts
void DisableInts(void);
void EnableInts(void);

// Heartbeat
void InstallHeartbeatHandler(unsigned long handler);
void RemoveHeartbeatHandler(unsigned long handler);

// Key
void WaitKey(void);

// RTC
short setdatetime(struct date *datep, struct time *timep);
short getdatetime(struct date *dp, struct time *tp);

// Filesystem
long lseek(int handle, long offset, int orgin);
long tell(int handle);
int chsize(int handle, long size);


int chsize(int handle, long size);
#define far_malloc malloc						// void *far_malloc (unsigned long nbytes);

// Comms
int	 putstring( const char* string);
int	 putnstring(const char *data, unsigned short len);

// Power.c
void SoftReset(void);

//ADC
unsigned short GetTemperature(void);
unsigned short GetSubBatteryVoltage(void);
unsigned short GetLcdVoltage(void);
unsigned short GetCalibVoltage(void);
unsigned int GetBatteryVoltage(void);

#define GetBatteryVoltage_mV	GetBatteryVoltage


// BlueTooth
unsigned char *GetBltAddress(void);
unsigned char *GetBltPin(void);
void SetBltAddress(const char *address);
void SetBltPin(const char *pincode);
unsigned char *GetBltLocalName( void );
void SetBltLocalName(const char *localname );

int	blt_is_connected( void );

typedef struct
{
	char			bd_address[13];
	char			name[20];
	unsigned long 	address_l;
	unsigned long 	address_h;
	unsigned char 	page_scan_rpt_mode;
	unsigned char 	page_scan_period_mode;
	unsigned short 	clock_offset;
}device_info;

int blt_discovery( device_info *dev_inf, int abortkey );
void blt_unpair(void);

typedef int (*pin_callback) (char* pin, int* len );
void SetBltPinCallback(pin_callback cb);
pin_callback GetBltPinCallback(void);

#endif		// __LIB_H__
