// 
// Scan Quantity demo
// With the Graphical menu items
//
// This is the main application file

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#include <unistd.h>
#include <fcntl.h>

#include "lib.h"
#include "fancy.h"
#include "utils.h"

// image header system menu 
#include "datetime.fnt"	// date time icon
#include "barcode.fnt"	// barcode icon
#include "brain.fnt"   	// brain icon
#include "cards.fnt"	// card boc icon
#include "nodata.fnt"	// empty carbox icon
#include "tools.fnt"	// tools icon
#include "trashfl.fnt"	// filled trashbin icon
#include "trashemp.fnt"	// empty trashbin icon
#include "version.fnt"	// question mark on sheet icon
#include "battsm.fnt"	// battery icon
#include "lr2.fnt"		// left right arrow icon
#include "flashi.fnt"	// inversed lightning beam
#include "comp.fnt"		// searial computer connector icon
#include "standby.fnt"	// computer screen icon with ZZZ
#include "battvolt.fnt" // 
#include "opl9723.fnt"
#include "opldata.fnt"
#include "oplsend.fnt"
#include "any_key.fnt"	// press any key bar and yes no bar
#include "msgicons.fnt" // message icons
#include "light.fnt"	// light bulb icon
#include "optiobar.fnt" // contains a bar that holds back quantity delete
#include "scrolbar.fnt" // caintains the bar that is displayed in the scroll data menu

#define ERR_QUANTITY_MIN_MAX	-1	// exceeding min max value
#define ERR_QUANTITY_ILLEGAL 	-2	// illegal input (-) only a minus is an illegal input vale

#define OPTION_BAR	4
#define SCROLL_BAR	8


// defines for the messages 
#define MB_ALL		1
#define MB_YESNO	2

#define MB_ICONWARNING			0x10
#define MB_ICONERROR			0x20
#define MB_ICONQUESTION			0x40
#define MB_ICONINFORMATION		0x80
#define SZ_INPUT_QUANT	4 // max 9999 or neg max -9999

#define SZ_BARCODE	200
#define SZ_SIGN		1
#define SZ_QUANTITY	6
#define SZ_DATE		10
#define SZ_TIME		8
#define SZ_RECORD	( SZ_BARCODE +1+ SZ_SIGN+ SZ_QUANTITY+1+SZ_DATE+1+SZ_TIME+2 )

typedef struct
{
	char barcode[ SZ_BARCODE +1 ];
	char quantity[ SZ_SIGN+ SZ_QUANTITY+1 ];
	char time[ SZ_TIME+1 ];
	char date[ SZ_DATE+1 ];
}db_record;


int menu_layers[5];
long enabled_barcodes;
long baudrate_enabled;
long protocol_enabled;
long standby_enabled;
long backlight_enabled;

const char time_mask[] = "29:59:59";
const char date_mask[] = "39/19/2999/E";

// _signs hold special sign font 
extern char _signs[];

jmp_buf batt_power;
int 	battery_msg_status;

#define DATA_BASE	"DATA.TXT"
#define VERSION		"XFF3560"

struct smenus {
	char *item;
	int value;
};

typedef struct {
	char *item;
}days;

#define ID_CODE39		   0x00000001
#define ID_EAN 			   0x00000002
#define ID_UPC 			   0x00000004
#define ID_I2OF5		   0x00000008
#define ID_D2OF5    	   0x00000010
#define ID_CODABAR		   0x00000020
#define ID_CODE93		   0x00000040
#define ID_CODE128		   0x00000080
#define ID_MSI_PLESSEY 	   0x00000100
#define ID_TELEPEN		   0x00000200
#define ID_UK_PLESSEY	   0x00000400
#define ID_IATA			   0x00000800
#define ID_SCODE		   0x00001000
#define ID_M2OF5		   0x00002000
#define ID_RSS_14          0x00004000
#define ID_RSS_LIMITED     0x00008000
#define ID_RSS_EXPANDED    0x00010000
#define ID_PDF417		   0x00020000
#define ID_MICRO_PDF417    0x00040000
#define ID_DATAMATRIX      0x00080000
#define ID_AZTEC           0x00100000
#define ID_AZTEC_RUNES     0x00200000
#define ID_QR_CODE         0x00400000
#define ID_MICRO_QR_CODE   0x00800000
#define ID_MAXICODE        0x01000000

#define ID_ADDONS		   0x02000000

const char back_string[] = "    (back)";

const fancyselmenus barcodem[] =
{
	{ (char *)back_string, 		-1 },
	{ "CODE 39", 		ID_CODE39 },
	{ "EAN", 			ID_EAN },
	{ "UPC",		    ID_UPC },
	{ "INTERL 2 OF 5",	ID_I2OF5 },
	{ "INDUST 2 OF 5",	ID_D2OF5 },
	{ "CODABAR",	 	ID_CODABAR },
	{ "CODE 93", 		ID_CODE93 },
	{ "CODE 128",  		ID_CODE128 },
	{ "MSI PLESSEY", 	ID_MSI_PLESSEY },
	{ "TELEPEN", 		ID_TELEPEN },
	{ "UK PLESSEY", 	ID_UK_PLESSEY },
	{ "IATA",			ID_IATA },
	{ "SCODE", 			ID_SCODE },
	{ "MATRIX 2 OF 5",	ID_M2OF5 },
	{ "RSS-14",			ID_RSS_14 },
	{ "RSS-Limited", 	ID_RSS_LIMITED },
	{ "RSS-Expanded",	ID_RSS_EXPANDED },
	{ "PDF417",			ID_PDF417 },
	{ "MicroPDF417",	ID_MICRO_PDF417 },
	{ "DataMatrix",		ID_DATAMATRIX },
	{ "Aztec",			ID_AZTEC },
	{ "Aztec Runes",	ID_AZTEC_RUNES },
	{ "QR Code",		ID_QR_CODE },
	{ "Micro QR Code",	ID_MICRO_QR_CODE },
	{ "Maxicode",		ID_MAXICODE },
	{ "All ADDONS",		ID_ADDONS }
};

const days day_gb[] = {	
	{ "  Sunday  " },
	{ "  Monday  " },
	{ "  Tuesday " },
	{ " Wednesday" },
	{ " Thursday " },
	{ "  Friday  " },
	{ " Saturday " } 
};

#define ID_115200	1
#define ID_57600    2
#define ID_38400    3
#define ID_19200    4
#define ID_9600     5
#define ID_4800     6
#define ID_2400     7

const fancyselmenus baudm[] = 
{
	{ (char*)back_string,	-1 },
	{ "115200 Baud",		ID_115200 },
	{ " 57600 Baud", 		ID_57600 },
	{ " 38400 Baud", 		ID_38400 },
	{ " 19200 Baud", 		ID_19200 },
	{ "  9600 Baud", 		ID_9600 },
	{ "  4800 Baud", 		ID_4800 },
	{ "  2400 Baud",  		ID_2400 }
};

#define ST_05_SEC	1
#define ST_10_SEC	2
#define ST_15_SEC	3
#define ST_30_SEC	4
#define ST_45_SEC	5
#define ST_60_SEC	6
#define ST_120_SEC	7
#define ST_180_SEC	8

const fancyselmenus standbym[] =
{
	{ (char*)back_string, 		-1 },
	{ " 5 seconds",		ST_05_SEC },
	{ "10 seconds", 	ST_10_SEC },
	{ "15 seconds", 	ST_15_SEC },
	{ "30 seconds", 	ST_30_SEC },
	{ "45 seconds", 	ST_45_SEC },
	{ "60 seconds", 	ST_60_SEC },
	{ "120 seconds",	ST_120_SEC },
	{ "180 seconds",	ST_180_SEC }
}; 

const fancyselmenus backlightm[] =
{
	{ (char*)back_string, 	-1 },
	{ "     OFF", 			OFF },
	{ "     ON",			ON },
	{ "    AUTO", 			AUTO }
};


#define ID_NO_PROTOCOL		1
#define ID_NETO_PROTOCOL	2

const fancyselmenus protocolm[] = 
{
	{ (char*)back_string,	-1 },
	{ "No Protocol", 		ID_NO_PROTOCOL },
	{ "NetO Protocol",		ID_NETO_PROTOCOL }
};


void update_quantity_rec_struct( db_record *db_rec, long quantity );
void save_data( db_record *db_rec, long pos );
int check_input_quant( char* input_quant, int use_default, long def_value, long fnd_quantity, long *new_quantity );


void printsign( int x_start, int y, int signid );

void main_delete( void );
void printbar( int y, int type, int LeftRight );
void set_font_input( void );
int show_message( const char* msg, int icons );
void print_message_bar( int bartype  );
void print_msg_icons( int x, int y, int icons );
void insertion_sort( int *fd, long items );
long bin_search(char *fname, char *record, char *sstring, int sstr_len, int start, int rec_size);
void fill_record_struct( db_record *db_rec, char* record );
long string_quantity_to_long( char* quantity, int* illegal );

void remove_record( long  record_no )
{
	static char readed_rec[ SZ_RECORD +1 ];
	long rc;
	long records;
	int fd;
	
	// not a valid record number
	if( record_no == -1L )
		return;
		
	// remove one record from database
	if( (records = fsize( DATA_BASE )) == -1L)
		return; // database does not exist
	records /= SZ_RECORD; // 
	if( records == 1L )
	{
		remove( (char*) DATA_BASE );	// only 1 record in DB
		return;
	}
	
	fd = open( (char*) DATA_BASE, O_RDWR |O_CREAT |O_BINARY,0x0 ); 
	
	if( record_no == (records-1L))
	{
		// last record in database needs to be removed
		// the file can be trunced easy way to make file
		// smaller
		chsize( fd, (records-1L)*SZ_RECORD );
		close(fd);
		return;		
	}

	for( rc = record_no+1L; rc < records; rc++ )
	{
		lseek( fd, rc*SZ_RECORD, SEEK_SET );	
		read( fd, readed_rec, SZ_RECORD );
		
		// one record back;
		lseek( fd, (rc-1L)*SZ_RECORD, SEEK_SET );	
		write( fd, readed_rec, SZ_RECORD );
	}
	chsize( fd, (records-1L)*SZ_RECORD );
	close( fd );
}


void display_change_quantity( db_record *db_rec )
{
	set_font_input();
	gotoxy(0,0);
	printf("\tr  CHANGE QUANTITY  \tr");
	setfont( LARGE_FONT, NULL );
	set_offset( -4 );
	gotoxy(0,1);
	printf("%-14.14s", db_rec->barcode ); 
	gotoxy(0,2);
	printf("Quant %-*.*s ", SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity ); 	
	set_offset( -7 );
	gotoxy(0,3);
	printf("New:             " ); 
	set_offset( 0 );

	setfont( SMALL_FONT, NULL );
	gotoxy(3,7);
	printf("               ");             
	// print the special keys bar 
	printsign(0, 7, 6 ); // back 
}

void display_barcode_options( db_record *db_rec )
{
	set_font_input();
	gotoxy(0,0);
	printf("\tr  BARCODE OPTIONS  \tr");
	setfont( LARGE_FONT, NULL );
	set_offset( -4 );
	gotoxy(0,1);
	printf("%-14.14s", db_rec->barcode ); 
	gotoxy(0,2);
	printf("Quant %-*.*s ", SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity ); 	
	set_offset( -7 );
	gotoxy(0,3);
	printf("%-14.14s", " "); // clear the line

	set_offset( 0 );
	print_message_bar( OPTION_BAR );
}

void change_quantity( db_record* db_rec )
{
	static char quant[ SZ_QUANTITY + 1 ];
	static char readed_rec[ SZ_RECORD + 1 ];
	int qu_ret;
	long org_quant;
	long new_quantity;
	long found_record;
	int illegal;
	
	org_quant = string_quantity_to_long( db_rec->quantity, &illegal );
	for(;;)	// start of input for loop
	{
		display_change_quantity( db_rec );			

		setfont( LARGE_FONT, NULL );		
		memset( quant, '\0', sizeof( quant ));
		qu_ret = KeyboardQuantity( quant, 	// string that holds the typed quantity
						SZ_QUANTITY,   		// the max quantity size
						NEGATIVE | NUM,   	// negative and numeric value possible
						6, 				  	// x starting possition
						3, 					// y position
						-7, 				// offset position
						org_quant,			// default value
						SZ_QUANTITY+SZ_SIGN,// the max display size
						TRUE, 				// show an input rectangle
						4, 					// number of special keys
						CLR_KEY, TRIGGER_KEY, Q1_KEY, ENT_KEY ); // the special keys to return from function
		
		// cut off the default flag if it exists
		if( (qu_ret & 0xFF) == CLR_KEY || (qu_ret & 0xFF) == Q1_KEY)
		{
			return;
		}

		if( check_input_quant( quant, (qu_ret & DEFAULT), org_quant, 0L, &new_quantity ) != OK )
		{	
			continue;
		}
		if( new_quantity != org_quant )
		{
			// some message if the bozo is sure
			// that he changed the quantity
			if( show_message("Are you sureyou want to store the new     quantity?", MB_YESNO | MB_ICONWARNING ) == YES )				
			{
				// find the matchin record
				found_record =  bin_search( (char*)DATA_BASE, readed_rec, db_rec->barcode, SZ_BARCODE, 0, SZ_RECORD);	
				
				if( new_quantity == 0L )
				{
					remove_record( found_record );
					memset( db_rec, '\0', sizeof( db_record ));
				}
				else
				{
					// store it
					update_quantity_rec_struct( db_rec, new_quantity );
					save_data( db_rec, found_record );
				}
			}
		}
		return;
	} // end of input for loop
}

void show_options_menu( db_record *db_rec )
{
	static char readed_rec[ SZ_RECORD +1 ];
	long found_rec;		
	for(;;)
	{
		display_barcode_options( db_rec );
		switch( waitkey() )
		{
			case TRIGGER_KEY: // change the quantity value
				change_quantity( db_rec );
				return;
	
			case Q1_KEY:
			case CLR_KEY:	// return to previous menu
				return;
				
						
			case Q2_KEY: // delete the current record from database
				info_sound();
				if( show_message("Are you sureyou want to remove the record from the database?", MB_YESNO | MB_ICONWARNING ) == YES )				
				{
					found_rec =  bin_search( (char*)DATA_BASE, readed_rec, db_rec->barcode, SZ_BARCODE, 0, SZ_RECORD);
					// remove the record
					remove_record( found_rec );
					
					// clear the db_record structure
					memset( db_rec, '\0', sizeof( db_record ));
					return;
				}
				break;
			default:
				break;
		}
	}
}


int check_input_quant( char* input_quant, int use_default, long def_value, long fnd_quantity, long *new_quantity )
{
	static char max_val[ SZ_QUANTITY + 1];
	static char min_val[ SZ_SIGN+ SZ_QUANTITY + 1];
	long max;
	long min;
	int illegal;
	
	if( use_default )
	{
		*new_quantity = def_value;
	}
	else
	{
		// quantity input over here
		*new_quantity = string_quantity_to_long( input_quant, &illegal );
		if( *new_quantity == 0L && illegal )
		{
			// zero input or no input at all do not 
			// change anything in the database
			// show a message on screen that input is not valid 
			info_sound();
			show_message("Quantity    is not a    valid value!", MB_ALL | MB_ICONINFORMATION );				
			return ERR_QUANTITY_ILLEGAL; 	
		}
	}
	
	// test for underflow and overflow
	memset(max_val, '9', sizeof( max_val ));
	max_val[ SZ_QUANTITY ] = '\0';
	max = atol( max_val );
	
	memset( min_val, '9', sizeof( min_val ));
	min_val[0] = '-';
	min_val[ SZ_SIGN + SZ_QUANTITY ] = '\0';
	min = atol( min_val );
		
	if( (fnd_quantity + *new_quantity ) > max )
	{
		// message total new quantity more then space available
		info_sound();
		show_message("Quantity    exceeding   max input value,  value not stored!", MB_ALL | MB_ICONINFORMATION );				
		return ERR_QUANTITY_MIN_MAX;
	}
	if( (fnd_quantity + *new_quantity ) < min )
	{
		// message total new quantity more then space available
		info_sound();
		show_message("Quantity    exceeding   min input value,  value not stored!", MB_ALL | MB_ICONINFORMATION );				
		return ERR_QUANTITY_MIN_MAX;
	}
	return OK;
}

long string_quantity_to_long( char* quantity, int* illegal )
{
	static char tmp[SZ_SIGN+SZ_QUANTITY+1];	
	char* ptr;
	int pos;
	
	*illegal = FALSE;
	
	memset( tmp, '\0', sizeof( tmp ));
	ptr = tmp;
	for( pos = 0; pos < (SZ_SIGN+SZ_QUANTITY); pos++ )
	{
		if( quantity[pos] == '\0' )
			break;
			
		if( quantity[pos] == ' ') // cut off all spaces
		 	continue;
		 	
		 *ptr++ = quantity[pos];
	}
	if( atol( tmp ) == 0L && tmp[0] == '-' )
		*illegal = TRUE; // illegal value a '-' or -0 are not legal values
	return( atol( tmp ));
}


long bin_search(char *fname, char *record, char *sstring, int sstr_len, int start, int rec_size)
{
	int fd;
	int test;
	long min, max, current;

	if((max = fsize((char*)DATA_BASE)) == -1L )
	{
		// file does not exist
		return -1L ;
	}

	max /= (long) rec_size;
	if ((fd = open(fname, O_RDONLY | O_BINARY,0x0)) == -1)
	{
		// cannot open the file the file exist
		// this should never happen
		return (-1L);
	}
	
	min = 0L;
	for (;;) 
	{
		current = ((max - min) >> 1) + min;
		lseek(fd, current * (long) rec_size, SEEK_SET);
		if( read(fd, record, rec_size) != rec_size) 
		{
			close(fd);
			return (-1L);
		}

		if ((test = memcmp(sstring, record + start, sstr_len)) == 0) 
		{	// found the barcode in the database 
			close(fd);
			return (current);
		}
		if (test < 0)
			max = current - 1L;
		else
			min = current + 1L;
		if (min > max) 
		{
			close(fd);
			// item is not found in the file
			return (-1L);
		}
	}
}


void print_msg_icons( int x, int y, int icons )
{
	int start;
	setfont( USER_FONT, (unsigned char*) _msgicons );
	
	if( icons & MB_ICONWARNING )
		start = 0;
	if( icons & MB_ICONQUESTION )
		start = 1;
	if( icons & MB_ICONERROR )
		start = 2;
	if( icons & MB_ICONINFORMATION )
		start = 3;

	display_icon( x, y, start );
}

// show a message with an icon max message size is 60 chars 
int show_message( const char* msg, int icons )
{
	int key;
	
	putchar('\f');
	print_message_bar( icons );

	print_msg_icons(0, 0, icons );	// show the matching icon 
	set_font_input();
	set_offset( 8 );
	if( strlen( msg ) > 0 )
	{
		gotoxy( 6, 0 );
		printf("%-12.12s", msg );
	}
	if( strlen( msg ) > 12 )
	{
		gotoxy( 6, 1 );
		printf("%-12.12s", msg+12 );
	}
	if( strlen( msg ) > 24 )
	{
		gotoxy( 0, 2 );
		printf("%-18.18s", msg+24 );
	}
	if( strlen( msg ) > 42 )
	{
		gotoxy( 0, 3 );
		printf("%-18.18s", msg+42 );
	}
	set_offset( 0 );
	resetkey();

	for(;;)
	{
		key = waitkey();
		
		if( icons & MB_ALL )
			return( OK );

		switch( key )
		{
			case Q1_KEY:	
				return( YES );

			case Q2_KEY:
				return( NO );
		}
	}
}

void printbattery( void )
{
	setfont( USER_FONT, (unsigned char*) _battvolt ); 
	
	display_icon( 9, 2, 0 );
	set_font_input();
}

void set_standby( void )
{
	switch( standby_enabled )
	{
		case ST_05_SEC:
			autopowerdown( ON, 250 );
			break;
		case ST_10_SEC:
			autopowerdown( ON, 500 );
			break;
		case ST_15_SEC:
			autopowerdown( ON, 750 );
			break;
		case ST_30_SEC:
			autopowerdown( ON, 1500 );
			break;
		case ST_45_SEC:
			autopowerdown( ON, 2250 );
			break;
		case ST_60_SEC:
			autopowerdown( ON, 3000 );
			break;
		case ST_120_SEC:
			autopowerdown( ON, 6000 );
			break;
		case ST_180_SEC:
			autopowerdown( ON, 9000 );
			break;
		default:
			standby_enabled = ST_30_SEC;
			autopowerdown( ON, 1500 );
			break;
	}
}


void print_message_bar( int bartype  )
{
	if( bartype & MB_ALL )
		setfont( USER_FONT, (unsigned char*) _any_key );	
	else if( bartype & MB_YESNO )
		setfont( USER_FONT, (unsigned char*) _yesno );	
	else if( bartype & OPTION_BAR )
		setfont( USER_FONT, (unsigned char*) _optiobar );
	else if( bartype & SCROLL_BAR )
		setfont( USER_FONT, (unsigned char*) _scrolbar );
	else	
		return;

//	setfont( USER_FONT, (unsigned char*) _any_key );	
	gotoxy(0,7);
	printsymbol( 0 );
	gotoxy(1,7);
	printsymbol( 1 );
	gotoxy(2,7);
	printsymbol( 2 );
	gotoxy(3,7);
	printsymbol( 3 );
	gotoxy(4,7);
	printsymbol( 4 );
	gotoxy(5,7);
	printsymbol( 5 );
	gotoxy(6,7);
	printsymbol( 6 );
	gotoxy(7,7);
	printsymbol( 7 );
	gotoxy(8,7);
	printsymbol( 8 );
	gotoxy(9,7);
	printsymbol( 9 );
	gotoxy(10,7);
	printsymbol( 10 );
	gotoxy(11,7);
	printsymbol( 11 );
	gotoxy(12,7);
	printsymbol( 12 );
	gotoxy(13,7);
	printsymbol( 13 );
	
	set_font_input();	
}

void display_time_date_error( int err_code )
{
	static char temp[60+1];
	static char errm[18+1];
	
	memset( temp, '\0', sizeof( temp ));
	memset( errm, '\0', sizeof( errm ));
	if( err_code & 1 )
		strcat( errm, "Time ");
	if( (err_code & 1 ) && (err_code & 2))
		strcat( errm, "and ");
	if( err_code & 2 )
		strcat( errm, "Date");
	
	sprintf(temp, "Error in    setting:    %s", errm );
	errbeep();
	show_message( temp, MB_ALL | MB_ICONERROR );
}


int display_do_setting( void )
{
	int status;
	char text[ 6 + 1];
	status = 0;
	
	printbar( 7, 0, 0 );
	
	set_font_input();	
	gotoxy(0,2);
	printf("%18.18s", " ");
	gotoxy(0,3);
	printf("%18.18s", " ");
	for(;;)
	{
		if( status == 0 )
			strcpy( text, "SAVE  ");
		else
			strcpy( text, "CANCEL");

		gotoxy(6,2);
		printf("%-6.6s", text);
		rectangle3d( (6*6)-2, ((5-1)*6)-1, ((6+6)*6), ((5+1)*6)-1, 1);

		if( waitkey() == TRIGGER_KEY )
			return status;
		status ^= 1;
	}
}

void display_time_date( char* time_str, char* date_str )
{
	printbar( 7, 0 , 1);
	
	set_font_input();
	gotoxy(0, 0);
	printf("\tr  SET TIME & DATE \tr");
	gotoxy(0, 2);
	printf("%-18.18s", time_str);
	gotoxy(0, 3);
	printf("%-18.18s", date_str);
	gotoxy(0, 4);
	cursor( ON );
}

void change_character( int x, int y, char* time_str, char* date_str )
{
	char c;
	char mask_c;

	c = ( y == 2)? time_str[x]:date_str[x];
	mask_c = ( y == 2)? time_mask[x]:date_mask[x];

	if( ++c > mask_c )
		c = '0';

	if( y == 2 )
		time_str[x] = c;
	else
		date_str[x] = c;
}



void store_time_date( char* time_str, char* date_str )
{
	char temp[ 14 + 1];
	struct date dates;
	struct time times;

	strcpy(temp, time_str);
	temp[10]= '\0';
	times.ti_sec = atoi(temp + 6);
	temp[5] = '\0';
	times.ti_min = atoi(temp + 3);
	temp[2] = '\0';
	times.ti_hour = atoi(temp);
	settime( &times );

	strcpy(temp, date_str);
	dates.da_year = atoi(temp + 6);
	temp[5] = '\0';
	dates.da_mon = atoi(temp + 3);
	temp[2] = '\0';
	dates.da_day = atoi(temp);
	setdate( &dates );
}

int is_date_string_ok( char* date_str )
{
	char temp[ 14 + 1];
	struct date dates;

	strcpy(temp, date_str);
	temp[10]= '\0';
	dates.da_year = atoi(temp + 6);
	temp[5] = '\0';
	dates.da_mon = atoi(temp + 3);
	temp[2] = '\0';
	dates.da_day = atoi(temp);
	return( ((checkdate( &dates )==OK)?0:2) );
}

int is_time_string_ok( char* time_str )
{
	char temp[ 8 + 1];
	struct time times;
	strcpy(temp, time_str);
	times.ti_sec = atoi(temp + 6);
	temp[5] = '\0';
	times.ti_min = atoi(temp + 3);
	temp[2] = '\0';
	times.ti_hour = atoi(temp);
	return( ((checktime( &times )==OK)?0:1) );
}

void setstartpos( int *x, int *y )
{
	*x = 0;
	*y = 2;
}


int setcursorpos( int* x, int* y, int direction )
{
	if( (*x) == -1 )
	{
		(*y) = ( (*y) == 2)?3:2;
		(*x) = (((*y) == 2)? strlen( time_mask):strlen( date_mask))-1;
	}
	if( *y == 2 )
	{
		for(;;)
		{
			switch( time_mask[ (*x) ] )
			{
				case '\0':
					if( ++(*y) == 4 )
						(*y) = 2;
					(*x) = 0;
					break;
				case ':':
					(*x) += direction;
					break;
				default:
					gotoxy( (*x), (*y));
					return( 1 );
			}
		}
	}
	else if( *y == 3 )
	{
		for(;;)
		{
			switch( date_mask[ (*x) ] )
			{
				case '\0':
					if( ++(*y) == 4 )
						(*y) = 2;
					(*x) = 0;
					break;
				case '/':
					(*x) += direction;
					break;
				default:
					gotoxy( (*x), (*y));
					return( 1 );
			}
		}
	}
	return( 0 );

}


void set_time_and_date(void)
{
	struct date dates;
	struct time times;
	char time_str[8 + 1];       // HH:MM:SS    
	char date_str[14 + 1];      // YYYY/MM/DD 
	int x, y;
	int direction;
	gettime(&times);
	getdate(&dates);
	sprintf(time_str, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
	sprintf(date_str, "%02d/%02d/%04d End", dates.da_day, dates.da_mon, dates.da_year);

	putchar('\f');
	setstartpos( &x, &y );
	direction = 0;

	
	for (;;)
	{
		display_time_date( time_str, date_str );

		setcursorpos( &x, &y, direction );
		if('E' == (( y == 2)? time_str[x]:date_str[x]))
			printbar( 7, 1 , 1);
		else
			printbar( 7, 0 , 1);
		set_font_input();
		gotoxy( x, y );
		cursor( ON );
			
		switch( waitkey())
		{
			case Q2_KEY: // to right
				x++;
				direction = 1;
				break;
			case Q1_KEY: // to left 
				x--;
				direction = -1;
				break;
			case TRIGGER_KEY:
				if('E' == (( y == 2)? time_str[x]:date_str[x]))
				{
					cursor( OFF );
					switch( display_do_setting() )
					{
						case 0:		// store settings 
							x = is_time_string_ok( time_str);
							x |= is_date_string_ok( date_str );
							if( x == 0 )
							{
								store_time_date( time_str, date_str );
								return;
							}
							display_time_date_error( x );
							putchar('\f');
							setstartpos( &x, &y );
							cursor( ON );
							break;
						case 1:    	// cancel 
							return;
					}
				}
				else
					change_character( x, y, time_str, date_str );

				break;
		}
	}
}


void set_font_menu( void )
{
	setfont( LARGE_FONT, NULL );
	cursor( NOWRAP );
	cursor( OFF );
}

void set_font_input( void )
{
	setfont( MEDIUM_FONT, NULL ); 
	
	cursor( NOWRAP );
	cursor( OFF );
}

void disp_no_database( void )
{
	set_font_menu(); 
	errbeep();  
	show_message("Database notpresent on  the RAM disk!!!", MB_ALL | MB_ICONWARNING );
}

void setbaudrate( long baudrate )
{
	switch( baudrate )
	{
		case ID_115200:
			systemsetting("SZ");
			break;
		case ID_57600:
			systemsetting("K9");
			break;
		case ID_38400:
			systemsetting("K8");
			break;
		case ID_19200:
			systemsetting("K7");
			break;
		case ID_9600:
			systemsetting("K6");
			break;
		case ID_4800:
			systemsetting("K5");
			break;
		case ID_2400:
			systemsetting("K4");
			break;
		default:
			baudrate_enabled = ID_115200;
			systemsetting("SZ");
			break;
	}
}

int send_neto( void )
{
	int x;

	setbaudrate( baudrate_enabled );
	if( comopen( COM1 ) != 0 ) // Open IrDA port
	{
		errbeep();
		show_message("Error while opening COM port!", MB_ALL | MB_ICONERROR );				
		return( NOT_OK );
	}
	set_font_input();
	x = neto_transmit( (char(*)[12+1])DATA_BASE, 1, "123456", TRIGGER_KEY, 3 );
	comclose( COM1 );
	set_font_input();
	if( x != OK )
	{
		errbeep();	  			 	
		show_message("Transmissionerror duringtransfer protocol!", MB_ALL | MB_ICONERROR );				
		return( NOT_OK );
	}
	return( OK );
}

int send_plain( void )
{
	static char record[ SZ_RECORD + 1 ];
	long count, max;
	int tx_count;
	int fd;

	setbaudrate( baudrate_enabled );
	if( comopen(COM1) != 0) // Open IrDA port
	{
		errbeep();
		show_message("Error while opening COM port!", MB_ALL | MB_ICONERROR );				
		return( NOT_OK );
	}

	max = fsize( (char*)DATA_BASE ) / SZ_RECORD;

	if ((fd = open((char*)DATA_BASE, O_RDONLY | O_BINARY,0x0)) == -1)
	{
		comclose( COM1 );
		errbeep();  
		show_message("Error while opening file", MB_ALL | MB_ICONERROR );		
		return( NOT_OK );
	}

	for( count = 0L; count < max; count++ )
	{
		lseek( fd, (long)((count) * SZ_RECORD), SEEK_SET);
		if( read(fd, record, SZ_RECORD) != SZ_RECORD )
		{
			comclose( COM1 );
			close(fd);                                
			show_message("Error while reading datafrom the database!", MB_ALL | MB_ICONERROR );		
			return( NOT_OK );
			
		}
		for (tx_count = 0; tx_count < SZ_RECORD; tx_count++)
		{
			putcom( record[tx_count] );
			printf("%c", record[tx_count]);

		}
		delay(10);
	}
	close( fd );
	comclose(COM1);
	return(OK);
}




void set_barcodes( void )
{
	static char options[84 + 1];
	
    strcpy(options, "B0");
	if( enabled_barcodes & ID_CODE39 )
        strcat(options, "B2");
	if( enabled_barcodes & ID_EAN )
        strcat(options, "R4");
	if( enabled_barcodes & ID_UPC )		
        strcat(options, "R1");
	if( enabled_barcodes & ID_I2OF5 )		
        strcat(options, "R8");
	if( enabled_barcodes & ID_D2OF5 )  	
        strcat(options, "R7");
	if( enabled_barcodes & ID_CODABAR )		
        strcat(options, "B3");
	if( enabled_barcodes & ID_CODE93 )	
        strcat(options, "B5");
	if( enabled_barcodes & ID_CODE128 )	
        strcat(options, "B6");
	if( enabled_barcodes & ID_MSI_PLESSEY )
        strcat(options, "B7");
	if( enabled_barcodes & ID_TELEPEN )
        strcat(options, "B9");
	if( enabled_barcodes & ID_UK_PLESSEY )
        strcat(options, "B1");
	if( enabled_barcodes & ID_IATA	)
        strcat(options, "B4");
	if( enabled_barcodes & ID_SCODE )		
        strcat(options, "R9");
	if( enabled_barcodes & ID_M2OF5 )		
        strcat(options, "BB");
	if( enabled_barcodes & ID_RSS_14 )
		strcat(options, "JX");
	if( enabled_barcodes & ID_RSS_LIMITED )
		strcat(options, "JY");
	if( enabled_barcodes & ID_RSS_EXPANDED )
		strcat(options, "DR");
	if( enabled_barcodes & ID_PDF417 )		
        strcat(options, "[BCF");
	if( enabled_barcodes & ID_MICRO_PDF417 )		
        strcat(options, "[BCG");
	if( enabled_barcodes & ID_DATAMATRIX )		
        strcat(options, "[BCC");
	if( enabled_barcodes & ID_AZTEC )		
        strcat(options, "[BCH");
	if( enabled_barcodes & ID_AZTEC_RUNES )		
        strcat(options, "[BF2");
	if( enabled_barcodes & ID_QR_CODE )		
        strcat(options, "[BCD");
	if( enabled_barcodes & ID_MICRO_QR_CODE )		
        strcat(options, "[D2U");
	if( enabled_barcodes & ID_MAXICODE )		
        strcat(options, "[BCE");

	if( enabled_barcodes & ID_ADDONS )		
        strcat(options, "R2R3R5R6H3"); // UPC +2 +5, EAN +2 +5, NW7 ABC CX

	systemsetting( options );
	
}

void display_mem(void)
{
    long mem_size;
    mem_size = coreleft();

	putchar('\f');
	set_font_input();
	gotoxy(0,0);
	printf("\tr    FREE MEMORY    \tr");
	
	gotoxy(3,2);
	printf("%6ld Bytes", mem_size );
	rectangle3d( (3*6)-2, ((5-1)*6)-1, ((3+12)*6), ((5+1)*6)-1, 1);
	
	print_message_bar( MB_ALL );

	resetkey();
	waitkey();
}

void sys_barcodes( void )
{
	menu_layers[2] = 1;
	fancyselmenu( (fancyselmenus*)barcodem, menu_layers, sizeof(barcodem) / sizeof(fancyselmenus), &enabled_barcodes, 1 );
	
	set_barcodes();
}


void sys_memory( void )
{
	display_mem();
}

void sys_datetime( void )
{
	int key;
	struct time t;
	struct date d;
	
	putchar('\f');
	printsign(0, 7, 6 ); 	// back 
	printsign(11, 7, 10 ); 	// set 
	
	set_font_input();
	gotoxy(0,0);
	printf("\tr    DATE / TIME    \tr");

	resetkey();
	while(1)
	{
		idle();
		gettime( &t );
		getdate( &d );
		gotoxy(4,1);
		printf("%-10.10s", day_gb[what_day_of_week()].item);
		gotoxy(4,2);
		printf("%02d-%02d-%04d", d.da_day, d.da_mon, d.da_year );
		gotoxy(5,3);
		printf("%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec );
		
		if( !kbhit())
		{
			check_batt_empty(); // check the battery 
			is_on_cradle();

			idle();
			delay(5); // 100 ms delay 
			continue;
		}

		key = waitkey();
		if( key == Q1_KEY )
			return;
		else if( key ==  Q2_KEY )
		{
			set_time_and_date();
			putchar('\f');
			printsign(0, 7, 6 ); 	// back 
			printsign(11, 7, 10 ); 	// set 
			
			set_font_input();
			gotoxy(0,0);
			printf("\tr    DATE / TIME    \tr");
		}
	}
}

void sys_standby( void )
{
	menu_layers[2] = 1;
	fancyselmenu( (fancyselmenus*)standbym, menu_layers, sizeof(standbym) / sizeof(fancyselmenus), &standby_enabled, 0 );
	
	set_standby();
}

void sys_backlight( void )
{
	menu_layers[2] = 1;
	fancyselmenu( (fancyselmenus*)backlightm, menu_layers, sizeof(backlightm) / sizeof(fancyselmenus), &backlight_enabled, 0 );
	
	backlight( (unsigned short) backlight_enabled, 500 );
}

void update_quantity_rec_struct( db_record *db_rec, long quantity )
{
	struct date d;
	struct time t;

	getdate( &d );
	gettime( &t );

	sprintf( db_rec->quantity, "%*ld", SZ_SIGN + SZ_QUANTITY, quantity );
	sprintf( db_rec->time, "%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec );
	sprintf( db_rec->date, "%02d/%02d/%04d", d.da_day, d.da_mon, d.da_year );
	
}

void create_db_rec_struct( db_record *db_rec, char* barcode, long quantity )
{
	struct date d;
	struct time t;

	getdate( &d );
	gettime( &t );

	sprintf( db_rec->barcode, "%-*.*s",	SZ_BARCODE, SZ_BARCODE, barcode );
	sprintf( db_rec->quantity, "%*ld", SZ_SIGN + SZ_QUANTITY, quantity );
	sprintf( db_rec->time, "%02d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec );
	sprintf( db_rec->date, "%02d/%02d/%04d", d.da_day, d.da_mon, d.da_year );
}

// insertion sort is a very fast 
// sorting function for very small changes
// even faster for small changes then the 
// heap sort algorithem
void insertion_sort( int *fd, long items )
{
	long i, j;
	static char buffer[SZ_RECORD + 1];
	static char temp[SZ_RECORD + 1];

	for (i = 1; i < items; i++) 
	{
		lseek(*fd, i*(long)SZ_RECORD, SEEK_SET);
		read(*fd,temp, SZ_RECORD );
		j = i;
		while (j > 0) 
		{
			lseek( *fd, (j-1L)*SZ_RECORD, SEEK_SET);
			read( *fd, buffer, SZ_RECORD);
			if( memcmp( buffer, temp, SZ_BARCODE) <= 0)
			   break;
			   
			lseek(*fd, j*(long)SZ_RECORD,SEEK_SET);
			write(*fd, buffer, SZ_RECORD);
			j--;
		}
		lseek(*fd, j*(long)SZ_RECORD, SEEK_SET );
		write(*fd, temp, SZ_RECORD);
	}
}

void save_data( db_record *db_rec, long pos )
{
	int fd;
	static char record[ SZ_RECORD + 1 ];

	if( coreleft() < 5000L )
	{
		errbeep();  
		show_message("RAM disk    full!       Data is NOT storedin database.", MB_ALL | MB_ICONWARNING );
		return;		
	}
	if( coreleft() < 7000L )
	{
		errbeep();  
		show_message("RAM disk    almost full!Data is stored    in database.", MB_ALL | MB_ICONINFORMATION );
	}

	sprintf(record, "%-*.*s,%-*.*s,%-*.*s,%-*.*s\r\n",
				SZ_BARCODE, SZ_BARCODE, db_rec->barcode,
				SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity,
				SZ_TIME, SZ_TIME, db_rec->time,
				SZ_DATE, SZ_DATE, db_rec->date );

	if ((fd = open( (char*) DATA_BASE, O_RDWR |O_CREAT |O_BINARY,0x0)) == -1)
	{
		errbeep();  
		show_message("Error while opening file", MB_ALL | MB_ICONERROR );		
		return;
	}
	if( pos != -1L )
	{
		// go to the right overwrite pos;	
		lseek( fd, (pos*SZ_RECORD), SEEK_SET );
	}
	else
	{
		// append the file	
		lseek( fd, 0L, SEEK_END );
	}
	
	if( write( fd, record, SZ_RECORD ) != SZ_RECORD )
	{
		errbeep();  
		show_message("Error while writing datato the database!", MB_ALL | MB_ICONERROR );		
	}
	if( pos == -1L )
	{
		// sort the database
		insertion_sort( &fd, (fsize( DATA_BASE ) / SZ_RECORD) );			
	}
	close( fd );
}

void display_scan_quantity( db_record *db_rec, long total )
{
	set_font_input();
	gotoxy(0,0);
	printf("\tr   SCAN BARCODE   \tr");
	setfont( LARGE_FONT, NULL );
	set_offset( -4 );
	gotoxy(0,1);
	printf("%-18.18s", db_rec->barcode);
	gotoxy(0,2);
	printf("Total %*ld ", SZ_SIGN+SZ_QUANTITY, total );

	set_offset( -7 );
	gotoxy(0,3);
	printf("Add           " ); 
	set_offset(0);
	setfont( SMALL_FONT, NULL );
	gotoxy(3,7);
	printf("               ");             

	// print the special keys bar 
	printsign(0, 7, 6 ); // back 
	set_font_input();	
}


void display_scan_barcode( db_record *db_rec )
{
	set_font_input();
	gotoxy(0,0);
	printf("\tr   SCAN BARCODE   \tr");
	setfont( LARGE_FONT, NULL );
	set_offset( -4 );
	gotoxy(0,1);
	printf("%-14.14s", " "); // clear the fist line
	gotoxy(0,2);
	printf("%-14.14s", db_rec->barcode );
	set_offset( -7 );
	gotoxy(0,3);
	if( db_rec->barcode[0] == '\0' )
		printf("%-14.14s", " "); // clear the fist line
	else
		printf("Quant: %-*.*s", SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity ); 

	// print the special keys bar 
	set_offset( 0 );
	setfont( SMALL_FONT, NULL );
	gotoxy(3,7);
	printf("               ");             
	
	printsign(0, 7, 6 ); // back 
	
//	printsign(4, 7, 9 ); // empty 
//	printsign(5, 7, 9 ); // empty 
//	printsign(6, 7, 9 ); // empty 
//	printsign(7, 7, 9 ); // empty 
//	printsign(8, 7, 9 ); // empty 
//	printsign(9, 7, 9 ); // empty 
	
	if( db_rec->barcode[0] != '\0' )
	{
		printsign( 10, 7, 11 ); // options
//		printsign(10, 7, 9 ); // empty 
//		printsign(11, 7, 9 ); // empty 
//		printsign(12, 7, 9 ); // empty 
//		printsign(13, 7, 9 ); // empty 
	}
//	else
//	{
//		printsign( 10, 7, 11); // options
//	}

	set_font_input();
}



void main_scan( void )
{
	static char barc[ BCR_MAX + 1 ];
	static char quant[ SZ_SIGN+SZ_INPUT_QUANT];
	static db_record db_rec;
	static db_record db_found;
	static char readed_rec[ SZ_RECORD + 1];
	long found_record;
	long fnd_quantity;
	long new_quantity;
	int ret_code;
	int qu_ret;
	int check_ret;
	int illegal;

	memset( barc, '\0', sizeof( barc ));
	memset( &db_rec, '\0', sizeof( db_record ));

	putchar('\f');
	for(;;)
	{
		display_scan_barcode( &db_rec );

		setfont( LARGE_FONT, NULL );
		ret_code = ScanOrKeyboardInput( barc, 1, BCR_MAX, ALPHA | NUM , 0, 1, -4 );
		if( ret_code == SCANNED || ret_code == KEYBOARD )
		{
			okbeep();
			goodreadled( GREEN, 25 );
			
			fnd_quantity = 0L;
			create_db_rec_struct( &db_rec, barc, 0L );
			// add the quantity
			// but first search if the previous barcode already exists
			found_record =  bin_search( (char*)DATA_BASE, readed_rec, db_rec.barcode, SZ_BARCODE, 0, SZ_RECORD);
			if( found_record != -1L )
			{
				fill_record_struct( &db_found, readed_rec );
				fnd_quantity = string_quantity_to_long( db_found.quantity, &illegal );
			}
			for(;;)	// start of input for loop
			{
				display_scan_quantity( &db_rec, fnd_quantity );			
		
				setfont( LARGE_FONT, NULL );		
				memset( quant, '\0', sizeof( quant ));
				qu_ret = KeyboardQuantity( quant, 	// string that holds the typed quantity
								SZ_INPUT_QUANT,   	// the max input quantity size
								NEGATIVE | NUM,   	// negative and numeric value possible
								8, 				  	// x starting possition
								3, 					// y position
								-7, 				// offset position
								1L,					// default value
								SZ_INPUT_QUANT+SZ_SIGN,	// the max display size
								TRUE, 				// show an input rectangle
								4, 					// number of special keys
								CLR_KEY, TRIGGER_KEY, Q1_KEY, ENT_KEY ); // the special keys to return from function
				
				// cut off the default flag if it exists
				if( (qu_ret & 0xFF) == CLR_KEY || (qu_ret & 0xFF) == Q1_KEY)
				{
					return;
				}

				if( (check_ret = check_input_quant( quant, (qu_ret & DEFAULT), 1L, fnd_quantity, &new_quantity )) == ERR_QUANTITY_ILLEGAL )
				{	
					continue;
				}
				break;
			} // end of input for loop
			
			if( check_ret != ERR_QUANTITY_MIN_MAX )
			{
				if( (fnd_quantity + new_quantity) == 0L )
				{
					remove_record( found_record );
					memset( &db_rec, '\0', sizeof( db_record ));
				}
				else
				{
					// store in database
					create_db_rec_struct( &db_rec, barc, (fnd_quantity + new_quantity)  );
					save_data( &db_rec, found_record );
				}
			}
			else
			{
				update_quantity_rec_struct( &db_rec, fnd_quantity );
			}

		}
		if( ret_code == CLR_KEY || ret_code == Q1_KEY )
			return;
		else if( ret_code == Q2_KEY && db_rec.barcode[0] != '\0' )
		{
			// change quantity an delete a record from
			// the dtabase are found in the options
			// menu
			show_options_menu( &db_rec );
	
		}
	}
}

void printsign( int x_start, int y, int signid )
{
	setfont( USER_FONT, (unsigned char*) _signs );
	gotoxy(x_start,y);
	switch( signid )
	{
		case 0: // arrow left 
			printsymbol(0); // arrow left 
			break;	
		case 1: // arrow right 
			printsymbol(1); // arrow right 
			break;
		case 2: // arrow down 
			printsymbol(2); // arrow down 
			break;
		case 3: // arrow up 
			printsymbol(3); // arrow up 
			break;
		case 4: // select 
			printsymbol(4); // select 1/4 
			gotoxy(x_start+1,y);
			printsymbol(5); // select 2/4 
			gotoxy(x_start+2,y);
			printsymbol(6); // select 3/4 
			gotoxy(x_start+3,y);
			printsymbol(7); // select 4/4 
			break;
		case 5: // clear 
			printsymbol(8);  // clear 1/4 
			gotoxy(x_start+1,y);
			printsymbol(9);  // clear 2/4 
			gotoxy(x_start+2,y);
			printsymbol(10); // clear 3/4 
			gotoxy(x_start+3,y);
			printsymbol(11); // clear 4/4 
			break;
		case 6:	// back 
			printsymbol(12); // back 1/4 
			gotoxy(x_start+1,y);
			printsymbol(13); // back 2/4 
			gotoxy(x_start+2,y);
			printsymbol(14); // back 3/4 
			gotoxy(x_start+3,y);
			printsymbol(15); // back 4/4 
			break;
		case 7:	// enter 
			printsymbol(16); // enter 1/4 
			gotoxy(x_start+1,y);
			printsymbol(17); // enter 2/4 
			gotoxy(x_start+2,y);
			printsymbol(18); // enter 3/4 
			gotoxy(x_start+3,y);
			printsymbol(19); // enter 4/4 
			break;
		case 8: // keyboard 
			printsymbol(20); // keyboard 1/6 
			gotoxy(x_start+1,y);
			printsymbol(21); // keyboard 2/6 
			gotoxy(x_start+2,y);
			printsymbol(22); // keyboard 3/6 
			gotoxy(x_start+3,y);
			printsymbol(23); // keyboard 4/6 
			gotoxy(x_start+4,y);
			printsymbol(24); // keyboard 5/6 
			gotoxy(x_start+5,y);
			printsymbol(25); // keyboard 6/6 
			break;		
		case 9: // empty 
			printsymbol(26); // empty 
			break;
		case 10: // set 
			printsymbol( 27 ); // set 1/3 
			gotoxy( x_start+1, y);
			printsymbol( 28 ); // set 2/3 
			gotoxy( x_start+2, y);
			printsymbol( 29 ); // set 3/3 
			break;
		case 11: // options
			printsymbol(30); // options 1/4 
			gotoxy(x_start+1,y);
			printsymbol(31); // options 2/4 
			gotoxy(x_start+2,y);
			printsymbol(32); // options 3/4 
			gotoxy(x_start+3,y);
			printsymbol(33); // options 4/4 
			break;
		
	}
}

void printbar( int y, int type, int LeftRight )
{
	if( LeftRight )
		printsign( 0, y, 0 ); // arrow left 
	else
		printsign( 0, y, 3 ); // arrow up 
	
	switch( type )
	{
		case 0:		// select 
			printsign( 5, y, 4); // select 
			break;
		case 1:		// back 
			printsign( 5, y, 6 ); // back 
			break;
		case 2:		// clear 
			printsign( 5, y, 5 ); // clear 
			break;
		case 3:		// enter 
			printsign( 5, y, 7 ); // enter 
			break;
	}		
	if( LeftRight )
		printsign( 13, y, 1 ); // arrow right 
	else
		printsign( 13, y, 2 ); // arrow down 
}


void main_send( void )
{
	int x;
	if( fsize((char*)DATA_BASE) == -1L )
	{
		disp_no_database();
		return;
	}
	info_sound();
	printf("\fTransmit data");
	print_message_bar( MB_ALL );

	waitkey_no_charge();

	if( protocol_enabled == ID_NO_PROTOCOL )
		x = send_plain();
	else
		x = send_neto();
	if( x == NOT_OK )
		return;

	main_delete();	
	
}

void display_scroll_data( db_record *db_rec, long curr, long max )
{
	set_font_input();
	gotoxy(0,0);
	printf("\trSCROLL %04ld / %04ld \tr", curr+1, max+1);
	setfont( LARGE_FONT, NULL );
	set_offset( -4 );
	gotoxy(0,1);
	printf("%-14.14s", db_rec->barcode ); 
	gotoxy(0,2);
	printf("Quant %-*.*s ", SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity ); 	
	set_offset( -7 );
	gotoxy(0,3);
	printf("%-14.14s", " "); // clear the line

	set_font_input();

	set_offset( -5 );
	gotoxy(0,4);
	printf("%-8.8s %-10.10s", db_rec->time, db_rec->date );
	set_offset( 0 );

	print_message_bar( SCROLL_BAR );
}


void fill_record_struct( db_record *db_rec, char* record )
{
	int offset;
	memset( db_rec, '\0', sizeof( db_record ));
	offset = 0;
	memcpy( db_rec->barcode, record+offset, SZ_BARCODE );
	offset+=SZ_BARCODE+1;
	memcpy( db_rec->quantity, record+offset, SZ_SIGN+SZ_QUANTITY );
	offset+= SZ_SIGN+SZ_QUANTITY+1;
	memcpy( db_rec->time, record+offset, SZ_TIME );
	offset+= SZ_TIME+1;
	memcpy( db_rec->date, record+offset, SZ_DATE );
}


int get_record( db_record *db_rec, long *rec_nr, long *max_rec )
{
	int fd;
	static char record[ SZ_RECORD + 1];

	if(((*max_rec) = fsize( DATA_BASE )) == -1L )
	{
		errbeep();
		return -1;
	}
	(*max_rec) /= SZ_RECORD;
	(*max_rec)--;

	if( (*rec_nr) > (*max_rec) )
	{
		errkeybeep();
		(*rec_nr) = (*max_rec);
	}
	else if( (*rec_nr) < 0L )
	{
		errkeybeep();
		(*rec_nr) = 0L;
	}

	if ((fd = open((char*)DATA_BASE, O_RDONLY | O_BINARY,0x0)) == -1)
	{
		errbeep();
		return -1;
	}
	lseek( fd, (long)((*rec_nr) * SZ_RECORD), SEEK_SET);
	if( read(fd, record, SZ_RECORD) != SZ_RECORD )
	{
		errbeep();
		close(fd);
		return -10;
	}
	close( fd );

	fill_record_struct( db_rec, record );
	return 1;
}


void main_scroll( void )
{
	long current;
	long max;
	static db_record db_rec;

	if( fsize((char*)DATA_BASE) == -1L )
	{
		disp_no_database();
		return;
	}

	current = 0L;
	if( get_record( &db_rec, &current, &max ) < 0 )
	{
		errbeep();
		return;
	}
	set_font_input();
	putchar('\f');
	for(;;)
	{
		display_scroll_data( &db_rec, current, max );
		switch( waitkey() )
		{
			case RIGHT_KEY:
				current = max;
				break;
			case Q2_KEY:
				current++;
				break;

			case LEFT_KEY:
				current = 0L;
				break;
			case Q1_KEY:
				current--;
				break;
			case TRIGGER_KEY:
				return;
		}
		get_record( &db_rec, &current, &max );
	}
}
	

void main_delete( void )
{
	char text[ 6 + 1];
	int status;

	if( fsize((char*)DATA_BASE) == -1L )
	{
		disp_no_database();
		return;
	}

	info_sound();
	status = 1;

	putchar('\f');
	printbar( 7, 0, 0 );
	set_font_input();
	
	gotoxy(0,0);
			   
	printf("\tr    DELETE DATA    \tr");

	for(;;)
	{
		if( status == 1 )
			strcpy( text, "CANCEL");
		else
			strcpy( text, "DELETE");

		gotoxy(6,2);
		printf("%-6.6s", text);
		rectangle3d( (6*6)-2, ((5-1)*6)-1, ((6+6)*6), ((5+1)*6)-1, 1);

		if( waitkey_no_charge() == TRIGGER_KEY )
		{
			if( status == 0 )
				remove( (char*)DATA_BASE );
			return;
		}
		status ^= 1;
	}
}

void baudrate_menu( void )	
{
	menu_layers[3] = 1;
	fancyselmenu( (fancyselmenus *)baudm, menu_layers, sizeof(baudm) / sizeof(fancyselmenus), &baudrate_enabled, 0);
}	

void protocol_menu( void )
{
	menu_layers[3] = 1;
	fancyselmenu( (fancyselmenus *)protocolm, menu_layers, sizeof(protocolm) / sizeof(fancyselmenus), &protocol_enabled, 0);		
}

void comm_menu( void )
{
	int ret;
	fancymenus commmenu[3];

	commmenu[0].itemfunc = NULL;
	commmenu[1].itemfunc = baudrate_menu;
	commmenu[2].itemfunc = protocol_menu;

	commmenu[0].fnt_picture = NULL;
	commmenu[1].fnt_picture = (unsigned char*)_flashi;
	commmenu[2].fnt_picture = (unsigned char*)_lr2;

	commmenu[0].item = (char*) back_string;
	commmenu[1].item = "   Baudrate";
	commmenu[2].item = "   Protocol";
	
	menu_layers[2] = 1;
	for(;;)
	{
		ret = fancymenu( commmenu, menu_layers, sizeof( commmenu ) / sizeof( fancymenus) );
		if( ret == -1 )
			return; // return to previous menu 
		commmenu[ ret ].itemfunc();
	}
}

void sys_battery( void )
{
	unsigned int voltage;
	set_font_input();
	
	voltage = GetBatteryVoltage_mV();

	putchar('\f');
	while(1)
	{
		gotoxy(0,0);
		printf("\tr  BATTERY VOLTAGE  \tr");
	
		printbattery();
		gotoxy(5,2);
		printf("%1d.%02d V",(voltage/1000),(voltage%1000) / 10);
		rectangle3d( (5*6)-2, ((5-1)*6)-1, ((6+5)*6), ((5+1)*6)-1, 1);
	
		print_message_bar( MB_ALL );

		if( !kbhit() )
		{
			check_batt_empty(); // check the battery 
			is_on_cradle();
			delay(5);
			idle();
			continue;
		}
		keybeep();
		resetkey();
		break;
	}
}

void system_menu( void )
{
	int ret;
	fancymenus sysmenu[8];

	sysmenu[0].itemfunc = NULL;
	sysmenu[1].itemfunc = sys_barcodes;
	sysmenu[2].itemfunc = comm_menu;
	sysmenu[3].itemfunc = sys_battery;
	sysmenu[4].itemfunc = sys_memory;
	sysmenu[5].itemfunc = sys_datetime;
	sysmenu[6].itemfunc = sys_standby;
	sysmenu[7].itemfunc = sys_backlight;

	sysmenu[0].fnt_picture = NULL;
	sysmenu[1].fnt_picture = (unsigned char*)_barcode;
	sysmenu[2].fnt_picture = (unsigned char*)_comp;
	sysmenu[3].fnt_picture = (unsigned char*)_battsm;
	sysmenu[4].fnt_picture = (unsigned char*)_brain;
	sysmenu[5].fnt_picture = (unsigned char*)_datetime;
	sysmenu[6].fnt_picture = (unsigned char*)_standby;
	sysmenu[7].fnt_picture = (unsigned char*)_light;

//						12345678901234
	sysmenu[0].item = (char*) back_string;
	sysmenu[1].item = "   Barcodes";
	sysmenu[2].item = " Communicate";
	sysmenu[3].item = " Battery volt";
	sysmenu[4].item = "    Memory";
	sysmenu[5].item = " Date / time";
	sysmenu[6].item = " Standby time";
	sysmenu[7].item = "   Backlight";
	
	menu_layers[1] = 1; 
	for(;;)
	{
		ret = fancymenu( sysmenu, menu_layers, sizeof( sysmenu ) / sizeof( fancymenus) );
		if( ret == -1 )
			return; // return to previous menu 
		sysmenu[ ret ].itemfunc();
	}
}


void main_version( void )
{
	set_font_input();

	putchar('\f');
	gotoxy(0,0);
	printf("\tr SOFTWARE VERSION  \tr");

	gotoxy(5,2);
	printf("%-8.8s", VERSION );
	rectangle3d( (5*6)-2, ((5-1)*6)-1, ((5+8)*6), ((5+1)*6)-1, 1);
	print_message_bar( MB_ALL );
	
	waitkey();
}


void main( void )
{
	int ret;

	fancymenus mainmenu[6];

	flipscreen(OFF);	// This application takes care of the flipping the screen content it self.

	enabled_barcodes = ID_CODE39 | ID_EAN | ID_UPC | ID_I2OF5 | ID_D2OF5 | ID_CODABAR;
	enabled_barcodes |= ID_CODE93 | ID_CODE128 | ID_MSI_PLESSEY | ID_TELEPEN;
	enabled_barcodes |= ID_UK_PLESSEY | ID_IATA	| ID_SCODE | ID_M2OF5;
	enabled_barcodes |= ID_RSS_14 | ID_RSS_LIMITED | ID_RSS_EXPANDED | ID_PDF417 | ID_MICRO_PDF417;
	enabled_barcodes |= ID_DATAMATRIX | ID_AZTEC | ID_QR_CODE | ID_MICRO_QR_CODE | ID_MAXICODE;

	
	set_barcodes(); // set the enabled barcodes 
		
	baudrate_enabled = (long)ID_115200;
	protocol_enabled = (long)ID_NO_PROTOCOL;
	standby_enabled = (long)ST_180_SEC; // 3 min on
	backlight_enabled = (long)AUTO; // automatic backlight
	
	set_standby(); 			// set the standby time 
	systemsetting("7G"); 	// show charging indication led 

	systemsetting("OG");	//Enable EAN128
	
	battery_msg_status = 0; 

	mainmenu[0].itemfunc = main_scan;
	mainmenu[1].itemfunc = main_scroll;
	mainmenu[2].itemfunc = system_menu;
	mainmenu[3].itemfunc = main_delete;
	mainmenu[4].itemfunc = main_send;
	mainmenu[5].itemfunc = main_version;

	mainmenu[2].fnt_picture = (unsigned char*)_tools;
	mainmenu[4].fnt_picture = (unsigned char*)_oplsend; 
	mainmenu[5].fnt_picture = (unsigned char*)_version;

	mainmenu[0].item = " Scan labels"; 
	mainmenu[1].item = " Scroll data";
	mainmenu[2].item = " System menu";
	mainmenu[3].item = " Delete data";
	mainmenu[4].item = "  Send data ";
	mainmenu[5].item = "  Versions  ";

	if( setjmp( batt_power ) != 0 )
	{
		// battery empty 
		do{
			show_empty_batt_msg( TRUE ); // empty battery message 
			is_on_cradle();
			delay(25); // delay for a more accurate battery voltage 

		}while( GetBatteryVoltage_mV() < BATT_CHARGE_LOW );
		battery_msg_status = 0;		
		// battery enough charged continue with application 
	}

	memset( menu_layers, 0, sizeof( menu_layers ));
	menu_layers[0] = 1; // start item of the main menu 

	resetkey();
	for (;;)
	{
		if( fsize((char*)DATA_BASE) == -1L )
		{  	
			// No database present
			mainmenu[0].fnt_picture = (unsigned char*)_opl9723;	
			mainmenu[1].fnt_picture = (unsigned char*)_nodata;					
			mainmenu[3].fnt_picture = (unsigned char*)_trashemp;
		}
		else
		{
			// data present in database
			mainmenu[0].fnt_picture = (unsigned char*)_opldata;			
			mainmenu[1].fnt_picture = (unsigned char*)_cards;					
			mainmenu[3].fnt_picture = (unsigned char*)_trashfl;		
		}

		ret = fancymenu( mainmenu, menu_layers, sizeof( mainmenu ) / sizeof( fancymenus) );
		if( ret == -1 )
		{
			if( !menu_layers[0] )
				menu_layers[0] = 1;
			continue; // could be return to return to a previous menu 
		}
		mainmenu[ ret ].itemfunc();
	}
}
