//  #####  ####### ##   ##  #####
//  ## ### ##    # ### ### ##   ##
//  ##  ## ##      ####### ##   ##
//  ##  ## ####    ## # ## ##   ##
//  ##  ## ##      ##   ## ##   ##
//  ## ### ##    # ##   ## ##   ##
//  #####  ####### ##   ##  #####
//
// Opticon demo
// Scan labels, quantity and scroll database with menus
// Transmit scanned data with NetO or plain ascii
// OPH Compiler: GCC Open Source Compiler Version 4.1.2
//			     available at http:/gcc.gnu.org/releases.html
// PHL Compiler: Microtec C compiler for Melps 7700  Version 2.2
//


//************************************************************************
// Include
//************************************************************************
#include        <stdio.h>
#include        <stdlib.h>
#include        <string.h>
#include        "lib.h"
#include 		"database.h"
#include 		"input.h"
#include 		"menu.h"
#include 		"images.h"

//************************************************************************
// Defines
//************************************************************************

#define VERSIONSUFFIX	"35271"


// Database name
#define DBASE_NAME	   	"myfile.txt"  //"DATA.TXT"

// Database record
#define SZ_BARCODE		1024
#define SZ_SIGN			1
#define SZ_QUANTITY		6
#define SZ_TIME			(2+1+2+1+2)
#define SZ_DATE			(4+1+2+1+2)
#define SZ_RECORD		(SZ_BARCODE+1+SZ_SIGN+SZ_QUANTITY+1+SZ_TIME+1+SZ_DATE+1+1)

// barcode menu defines
#define ID_CD39         0x0000001	// bit 0
#define ID_EAN          0x0000002	// bit 1
#define ID_UPC          0x0000004	// bit 2
#define ID_I2O5         0x0000008	// bit 3
#define ID_D2O5        	0x0000010	// bit 4
#define ID_NW7         	0x0000020	// bit 5
#define ID_CD93        	0x0000040	// bit 6
#define ID_CD128      	0x0000080	// bit 7
#define ID_MSI        	0x0000100	// bit 8
#define ID_TELEPEN    	0x0000200	// bit 9
#define ID_UK        	0x0000400	// bit 10
#define ID_ADDONS    	0x0000800	// bit 11
#define ID_IATA      	0x0001000	// bit 12
#define ID_SCODE     	0x0002000	// bit 13
#define ID_MAT2O5		0x0004000	// bit 14
#define ID_CHPOST		0x0008000	// bit 15
#define ID_CD11			0x0010000	// bit 16
#define ID_KOREAN		0x0020000	// bit 17
#define ID_RSS			0x0040000	// bit 18
#define ID_PDF417		0x0080000	// bit 19
#define ID_MICROPDF		0x0100000	// bit 20
#define ID_DATAMATRIX	0x0200000	// bit 21
#define ID_MAXICODE		0x0400000	// bit 22
#define ID_AZTEC		0x0800000	// bit 23
#define ID_QR		    0x1000000   // bit 24
//added by Khaled
#define ID_POST_NET     0x2000000   // bit 25
#define ID_IMB          0x4000000   // bit 26
//
// Special Terminal specific defines
//

#define ID_NO_PROTOCOL		1
#define ID_NETO_PROTOCOL	2

#define ID_SOUND_OFF		1
#define ID_SOUND_ON			2

#define COM COM9


//************************************************************************
// Special types
//************************************************************************

// barcode record structure
typedef struct
{
	char barcode[ SZ_BARCODE + 1 ];
	char quantity[ SZ_SIGN + SZ_QUANTITY + 1 ];
	char time[ SZ_TIME + 1 ];
	char date[ SZ_DATE + 1 ];
}db_record;


//************************************************************************
// Globals
//************************************************************************
long lPort;		// Serial communication port
long lProtocol;	// The protocol for transmitting the data
long lBarcodes;	// The currently supported barcode types
long lSound;	// Keep beep when TRUE;

//************************************************************************
// Function implementation
//************************************************************************
void set_barcodes( void )
{

	systemsetting("B0");			// disable all codes

    if( lBarcodes & ID_CD39)
    	systemsetting("B2D7DB"); 	// CODE39, Ital. Ph. if possible, Leading A for Ital. Ph.
    if( lBarcodes & ID_EAN)
    	systemsetting("R4");		// EAN8 and 13
    if( lBarcodes & ID_UPC)
    	systemsetting("R1");		// UPC
	if( lBarcodes& ID_I2O5)
		systemsetting("R8");		// Interleaved 2 of 5
    if( lBarcodes & ID_D2O5)
    	systemsetting("R7");		// Industrial 2 of 5
    if( lBarcodes & ID_NW7)
    	systemsetting("B3");		// Codabar
    if( lBarcodes & ID_CD93)
    	systemsetting("B5");		// Code93
    if( lBarcodes & ID_CD128)
    	systemsetting( "B6OG"); 	// Code 128, Ean 128 if possible
   	if( lBarcodes & ID_MSI)
   		systemsetting("B7");		// MSI plessey
    if( lBarcodes & ID_TELEPEN)
        systemsetting("B9JZ");		// Telepen, Tri-Optic
    if( lBarcodes & ID_UK)
        systemsetting("B1");		// UK plessey
    if( lBarcodes & ID_IATA)
        systemsetting("B4");		// IATA
    if( lBarcodes & ID_SCODE)
        systemsetting("R9");		// SCode
    if( lBarcodes & ID_MAT2O5)
        systemsetting("BB");		// Matrix 2 of 5
    if( lBarcodes & ID_CHPOST)
        systemsetting("JS");		// Chinese Post Matrix 2 of 5
    if( lBarcodes & ID_CD11)
        systemsetting("[BLC");		// Code 11
    if( lBarcodes & ID_KOREAN)
        systemsetting("WH");		// Korean Post
    if( lBarcodes & ID_RSS)
        systemsetting("JXJYDR");	// RSS-14, RSS Limited, RSS Expanded
    if( lBarcodes & ID_PDF417)
        systemsetting("[BCF");		// PDF417
    if( lBarcodes & ID_MICROPDF)
        systemsetting("[BCG");		// MicroPDF417
    if( lBarcodes & ID_ADDONS)
        systemsetting("R2R3R5R6JOJPJQJUJVJW");
    if( lBarcodes & ID_DATAMATRIX)
        systemsetting("[BCC");
    if( lBarcodes & ID_AZTEC)
        systemsetting("[BCH");
    if( lBarcodes & ID_QR)
        systemsetting("[BCD");
    if( lBarcodes & ID_MAXICODE)
        systemsetting("[BCE");
	//added by Khaled to enable PostNet & IMB
	if( lBarcodes & ID_POST_NET)
        systemsetting("[D6A");
    if( lBarcodes & ID_IMB)
        systemsetting("[D5F");
}

// Calculate the maximum amount of characters that fit on one line
static int GetMaxCharsXPos( void )
{
	unsigned int nHeight, nWidth;
	nHeight = nWidth = 0; // prevent warning toshiba compiler
	getfontsize( &nHeight, &nWidth);

	return (DISP_WIDTH / nWidth);
}

void diplay_input_data( db_record *db_rec )
{
	int nMaxX;

	nMaxX = GetMaxCharsXPos();

	gotoxy(0,1);
	printf("%-*.*s", nMaxX, nMaxX, db_rec->barcode );
	printf("\nQuant: %-*.*s\n", SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity);

	setfont( SMALL_FONT, NULL);
	gotoxy(0,6);
	printf("%-*.*s\n", nMaxX, nMaxX, db_rec->date);
    printf("%-*.*s", nMaxX, nMaxX, db_rec->time);
	setfont( LARGE_FONT, NULL);
}

// Save the data into the database
void store_input_data( db_record *db_rec, long lRecordNo )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1 ];

	// Check if there is enough space available for storing the barcode data
	// We use 5000 because the OS (NetO) also need some memory
	if( coreleft() < 5000L )
	{
		printf("\fRam disk full\ndata not stored!\n\n\n\n\nPress any key");
		WaitForKey();
		return;
	}
	// Make the record to store
	sprintf(record, "%-*.*s,%-*.*s,%-*.*s,%-*.*s\r\n",
				SZ_BARCODE, SZ_BARCODE, db_rec->barcode,
				SZ_SIGN+SZ_QUANTITY, SZ_SIGN+SZ_QUANTITY, db_rec->quantity,
				SZ_TIME, SZ_TIME, db_rec->time,
				SZ_DATE, SZ_DATE, db_rec->date );

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		if( !CreateDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError create\nDatabase\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode() );
			WaitForKey();
			return;
		}
	}
	if( lRecordNo != -1L )
		GotoRecord( &dbFile, lRecordNo );

	if( !WriteRecord( &dbFile, record, ((lRecordNo==-1L)?WRITE_APPEND:WRITE_OVER)))
	{
		CloseDatabase( &dbFile );
		printf("\fError write\nrecord\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode() );
		WaitForKey();
		return;
	}
	if( lRecordNo == -1L )
	{
		// Record was appended, now sort the database
		// to be able to use BinarySearch next time to search the database
		if( !QuickSort( &dbFile, 0, SZ_BARCODE ))
		{
			printf("\fError sort\nrecord\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode() );
			WaitForKey();
		}
	}
	CloseDatabase( &dbFile );
}

long string_quantity_to_long( char* quantity, int* illegal )
{
	static char tmp[SZ_SIGN+SZ_QUANTITY+1];
	char* ptr;
	int pos;

	*illegal = FALSE;

	memset( tmp, '\0', sizeof( tmp ));
	ptr = tmp;
	for( pos = 0; pos < (SZ_SIGN+SZ_QUANTITY); pos++ )
	{
		if( quantity[pos] == '\0' )
			break;

		if( quantity[pos] == ' ') // cut off all spaces
		 	continue;

		 *ptr++ = quantity[pos];
	}
	if( atol( tmp ) == 0L && tmp[0] == '-' )
		*illegal = TRUE; // illegal value a '-' or -0 are not legal values
	return( atol( tmp ));
}

void fill_record_struct( db_record *db_rec, char* record )
{
	int offset;
	memset( db_rec, '\0', sizeof( db_record ));
	offset = 0;
	memcpy( db_rec->barcode, record+offset, SZ_BARCODE );
	offset+=SZ_BARCODE+1;
	memcpy( db_rec->quantity, record+offset, SZ_SIGN+SZ_QUANTITY );
	offset+= SZ_SIGN+SZ_QUANTITY+1;
	memcpy( db_rec->time, record+offset, SZ_TIME );
	offset+= SZ_TIME+1;
	memcpy( db_rec->date, record+offset, SZ_DATE );
}

long FindBarcodeInDatabase( char *barcode, char *quantity )
{
	static SDBFile dbFile;
	static char record[ SZ_RECORD + 1 ];
	static db_record db_rec;
	long lFound = -1L;
	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		return lFound;
	if( (lFound = BinarySearch( &dbFile, record, barcode, SZ_BARCODE, 0 )) != -1L )
	{
		// Barcode was found fill the quantity string
		fill_record_struct( &db_rec, record );
		strncpy( quantity, db_rec.quantity, SZ_SIGN+SZ_QUANTITY );
	}
	CloseDatabase( &dbFile );
	return lFound;
}

void ScanLabels( void )
{
	static char 		barcode[ SZ_BARCODE + 1 ];
	static char 		quantity[ SZ_SIGN + SZ_QUANTITY + 1 ];
	static db_record	db_rec;
    struct date 		dates;
    struct time 		times;
    long   				lFoundRecord;
    int 				nIllegal;
    long 				lTotal;
	long				lAdd;
	int					c;

	memset( barcode, '\0', sizeof( barcode ));
	memset( &db_rec, '\0', sizeof( db_record ));

	cursor(NOWRAP);
	setfont( LARGE_FONT, NULL );
	for(;;)
	{
		printf("\fScan barcode...");

		if (ScanBarcode(barcode, 1, SZ_BARCODE) == ERROR )
			return;
		gotoxy(0,0);
		printf("%s           ",barcode);

		//
		// A new for loop, so that quantity is cancelled
		// the input continues with the barcode input
		//
		for(;;)
		{
			// fill the barcode into the record structure
			sprintf( db_rec.barcode, "%-*.*s", SZ_BARCODE, SZ_BARCODE, barcode );
			memset( quantity, '\0', sizeof( quantity ));	// clear the whole quantity item
			if( (lFoundRecord = FindBarcodeInDatabase( db_rec.barcode, quantity )) != -1L )
				lTotal = string_quantity_to_long( quantity, &nIllegal );
			else
				lTotal = 0;

			lAdd = 1;

			for(;;)
			{
				gotoxy( 0, 2);
				printf("Total: %*ld\nAdd:   %*ld       ", SZ_SIGN+SZ_QUANTITY,lTotal,SZ_SIGN+SZ_QUANTITY,lAdd);

				c = WaitForKey();
				if (c==UP_KEY)
					lAdd++;
				else if (c==DOWN_KEY)
				{
					if (lTotal+lAdd > 0)
						lAdd--;
				}
				else
					break;
			}

			lTotal += lAdd;

			sprintf( db_rec.quantity, "%*ld", SZ_SIGN+SZ_QUANTITY, lTotal);

	        gettime( &times );
	        getdate( &dates );
	        sprintf( db_rec.time, "%02d:%02d:%02d", times.ti_hour, times.ti_min, times.ti_sec);
	        sprintf( db_rec.date, "%02d/%02d/%04d", dates.da_day, dates.da_mon, dates.da_year);

	        // Store the input data into our database
	        store_input_data( &db_rec, lFoundRecord );
	        break;
		}
	}
}

void display_scroll_data( db_record *db_rec, long curr, long max )
{
	int nMaxX;
	nMaxX = GetMaxCharsXPos();

	gotoxy(0,0);
	printf("\tr%-*.*s %04ld/%04ld\tr", nMaxX-10, nMaxX-10, "CODE", curr+1, max);

	diplay_input_data( db_rec );
}

int get_record( db_record *db_rec, long *rec_nr, long *max_rec )
{
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
	{
		printf("\fError open\ndatabase.\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}

	*max_rec = GetTotalRecords( &dbFile );

	if( !GotoRecord( &dbFile, *rec_nr ))
	{
		printf("\fError goto\nrecord.\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}

	if( !ReadCurrentRecord( &dbFile,  record ))
	{
		printf("\fError read\nrecord.\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode());
		WaitForKey();
		return FALSE;
	}
	CloseDatabase( &dbFile );

	fill_record_struct( db_rec, record );
	return TRUE;
}


void ScrollDatabase( void )
{
	long current;
	long max = 0L;
	static db_record db_rec;

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\n\n\n\n\nPress any key");
		WaitForKey();
		return;
	}

	current = 0L;
	if( !get_record( &db_rec, &current, &max )  )
		return;

	putchar('\f');
	for(;;)
	{
		display_scroll_data( &db_rec, current, max );
		switch( WaitForKey() )
		{
			case DOWN_KEY:
				if(++current >= max )
					current = max-1;
				break;
			case UP_KEY:
				if( --current < 0 )
					current = 0;
				break;
			case TRIGGER_KEY:
				return;
		}
		if( !get_record( &db_rec, &current, &max ))
			return;
	}
}

void SelectProtocol( void )
{
	int  nIndex;

	sSelMenu mnuSelProtocol[] =
	{
	    {"Exit",                            -1},
		{"No protocol", 		ID_NO_PROTOCOL},
		{"NetO protocol",		ID_NETO_PROTOCOL}
	};
	nIndex = 0;
	ShowGraphSelectionMenu( mnuSelProtocol, sizeof( mnuSelProtocol ) / sizeof( sSelMenu ), MENU_SINGLE, &lProtocol);
}


void SetBarcodes( void )
{
	int nIndex;
	sSelMenu mnuSelBarcodes[] =
	{
	    {"Exit",                -1},
        {"UPC            ", ID_UPC},
        {"EAN            ", ID_EAN},
		{"Code 39        ", ID_CD39},
        {"Codabar        ", ID_NW7},
        {"Indust. 2of5   ", ID_D2O5},
        {"Interl. 2of5   ", ID_I2O5},
        {"S-Code         ", ID_SCODE},
		{"Matrix 2of5    ", ID_MAT2O5},
		{"Chinese Post   ", ID_CHPOST},
        {"IATA Code      ", ID_IATA},
        {"MSI            ", ID_MSI},
        {"Telepen        ", ID_TELEPEN},
        {"UK/Plessey     ", ID_UK},
        {"Code 128       ", ID_CD128},
        {"Code 93        ", ID_CD93},
		{"Code 11        ", ID_CD11},
		{"Korean Post    ", ID_KOREAN},
		{"RSS            ", ID_RSS},
		{"PDF417         ", ID_PDF417},
		{"MicroPDF417    ", ID_MICROPDF},
        {"All Add-Ons    ", ID_ADDONS}
    };
    nIndex = 0;
    ShowGraphSelectionMenu( mnuSelBarcodes, sizeof( mnuSelBarcodes ) / sizeof( sSelMenu ), MENU_MULTIPLE, &lBarcodes);
    set_barcodes();	// Set the new selected barcodes
}

void AvailableMemory( void )
{
   	long mem_size;
	mem_size = coreleft();
	printf("\fMEMORY LEFT:\n%ld Bytes\n\n\n\n\n\nPress any key", mem_size);
	WaitForKey();
}

void SetBacklight(void)
{
	setfont(MEDIUM_FONT,0);
	SetBackLightLvl();
	setfont(LARGE_FONT,0);
}

void SetSound(void)
{
	int  nIndex;

	sSelMenu mnuSelProtocol[] =
	{
	    {"Exit",                -1},
		{"Key beep OFF",		ID_SOUND_OFF},
		{"Key beep ON", 		ID_SOUND_ON},
	};
	nIndex = 0;
	ShowGraphSelectionMenu( mnuSelProtocol, sizeof( mnuSelProtocol ) / sizeof( sSelMenu ), MENU_SINGLE, &lSound);
	if (lSound==ID_SOUND_ON)
		systemsetting("WK");		// Sound on
	else
		systemsetting("WL");		// Sound off
}

void _SystemMenu( void )
{
	sgraphMenu mnuSystem[] =
	{
	    {"Exit",         NULL,      NULL           },
		{"Protocol",	&_protocol, SelectProtocol},
		{"Backlight",	&_lightbulb,SetBacklight},
		{"Barcodes",	&_barcode, 	SetBarcodes},
		{"Memory",		&_memory, 	AvailableMemory},
		{"Key beep",	&_sound, 	SetSound}
	};

	ShowGraphMenu( mnuSystem, sizeof( mnuSystem ) / sizeof( sgraphMenu ));
}

void DeleteDatabase( void )
{
	printf("\fDelete\nDatabase?\n\n\n\n\n[+] = YES\n[-] = NO");
	if( WaitForKeys( 2, UP_KEY, DOWN_KEY ) == UP_KEY )
		remove(DBASE_NAME );
}

void TransmitData( void )
{
	int nRet;
	static SDBFile dbFile; // static initializes all items to 0
	static char record[ SZ_RECORD + 1];

	if( fsize((char*)DBASE_NAME) == -1L )
	{
		printf("\fDatabase\nnot available\n\n\n\n\n\nPress any key");
		WaitForKey();
		return;
	}
// PX30 always has the USB port open...
	printf("\fTransmit data\n");
	if( lProtocol == ID_NETO_PROTOCOL )
	{
		printf("NetO protocol");
		gotoxy(0,7);
		printf("- is cancel");

		nRet = neto_transmit( (char(*)[12+1])DBASE_NAME, 1, "123456", DOWN_KEY, 3 );
		cursor( NOWRAP );
		if( nRet != OK )
		{
			printf("\fError NetO\nCode=%d\n\n\n\n\n\nPress any key", nRet);
			WaitForKey();
		}
	}
	else
	{
		if( !OpenDatabase( (char*)DBASE_NAME, SZ_RECORD, &dbFile ))
		{
			printf("\fError open\ndatabase.\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode());
			WaitForKey();
		}
		else
		{
			if( ReadFirstRecord( &dbFile, record ))
			{
				putchar('\f');
				do
				{
					gotoxy(0,0);
					printf("Send %04ld/%04ld", dbFile.lCurrRecord+1, dbFile.lTotalRecords);
					for( nRet = 0; nRet < SZ_RECORD; nRet++ )
			            putcom( record[ nRet ]);
        			delay(10);

				}while( ReadNextRecord( &dbFile, record ) );
			}
			else
			{
				printf("\fError\ndatabase.\nCode=%ld\n\n\n\n\nPress any key", GetDBErrorCode());
				WaitForKey();
			}
			CloseDatabase( &dbFile );
		}
	}
}

void ShowVersion( void )
{
	char version[ 8 + 1 ];

	printf("\f\tr   PX-30 Demo      \tr\nVersion #'s\n\n");

    version_os(version);
	printf("OS:  %s         \n", version);
	version_rom(version);
	printf("ROM: %s         \n", version);

    version_os(version);
    version[3] = '\0';
    version[1] = 'F';
	printf("APP: %s%s\n\nPress any key", version,VERSIONSUFFIX);
    WaitForKey();
}


void ShowFonts(void)
{
	long text = SetTextColor(-1);
	long back = SetBackColor(-1);

	SetTextColor(0xFFFFFF);
	SetBackColor(0x000000);

	setfont(HUGE_FONT,NULL);
	printf("\fExtraLarge");
	setfont(LARGE_FONT,NULL);
	gotoxy(0,2);
	printf("Large Font");
	setfont(MEDIUM_FONT,NULL);
	gotoxy(0,5);
	printf("Medium Font");
	setfont(SMALL_FONT,NULL);
	gotoxy(0,11);
	printf("Small Font");
	setfont(TINY_FONT,NULL);
	gotoxy(0,18);
	printf("Tiny Font");
	WaitForKey();

	SetTextColor(text);
	SetBackColor(back);
}

void main( void )
{
	sgraphMenu mnuMain[] =
	{
	    {"Scan Labels",			&_scan,		ScanLabels},
		{"Scroll DBase",		&_scroll,	ScrollDatabase},
		{"Transmit Data",  		&_transmit,	TransmitData},
		{"Delete DBase",		&_trash,	DeleteDatabase},
		{"System Menu", 		&_tools,	_SystemMenu},
		{"Show fonts",          &_fonts,    ShowFonts},
		{"Version",     		&_version,	ShowVersion}
	};

	cursor( NOICONS );

	lProtocol = ID_NETO_PROTOCOL;

	lBarcodes = ID_CD39 | ID_EAN | ID_UPC | ID_I2O5 | ID_D2O5 | ID_NW7 | ID_CD93 | ID_CD128;
	set_barcodes();			// set the enabled barcodes

	InitGraphMenu();

	systemsetting("WK");		// Sound on
	lSound = ID_SOUND_ON;

	for(;;)
	{
		ShowGraphMenu( mnuMain, sizeof( mnuMain ) / sizeof( sgraphMenu ));
	}
}

