/*
 * General and key definitions
 */

#define		OK			0
#define   	ERROR		-1
#define		NG			100
#define		ON			1
#define		OFF			0
#define		YES			1
#define		NO			0
#define 	TRUE		1
#define 	FALSE		0

#define		SOH			0x01
#define		STX			0x02
#define		ETX			0x03
#define		EOT			0x04
#define		ENQ			0x05
#define		ACK			0x06
#define		BEL			0x07
#define		BS			0x08
#define		HT			0x09
#define		LF			0x0A
#define		VT			0x0B
#define		FF			0x0C
#define		CR			0x0D
#define		SO			0x0E
#define		SI			0x0F
#define		DLE			0x10
#define		DC1			0x11
#define		DC2			0x12
#define		DC3			0x13
#define		DC4			0x14
#define		NAK			0x15
#define		SYN			0x16
#define		ETB			0x17
#define		CAN			0x18
#define		EM			0x19
#define		SUB			0x1A
#define		ESC			0x1B
#define		FS			0x1C
#define		GS			0x1D
#define		KS			0x1E
#define		US			0x1F
#define		SP			0x20
#define		DEL			0x7F

#if DWT
 #define ENT_KEY        0x0D
 #define UP_KEY         0x83
 #define DOWN_KEY       0x84
#endif

#if PHL|PHL2700
 #define ENT_KEY        0x0D
 #define UP_KEY         0x83
 #define DOWN_KEY       0x84
 #define BS_KEY			0x08
 #define ESC_KEY        0x1B
 #define CLR_KEY		ESC_KEY
 #define TRIGGER_KEY    0x80
 #define LEFT_KEY       0x81
 #define RIGHT_KEY      0x82
 #define F1_KEY         0x86
 #define F2_KEY         0x87
 #define F3_KEY         0x88
 #define F4_KEY         0x89
 #define F5_KEY         0x8A
 #define F6_KEY         0x8B
 #define F7_KEY         0x8C
 #define F8_KEY         0x8D
 #define BL_KEY			0x90
 #define Q1_KEY         0x8E
 #define Q2_KEY			0x8F
#endif

#if PHL1000
 #define ENT_KEY        0x0D
 #define BS_KEY			0x08
 #define ESC_KEY        0x1B
 #define CLR_KEY		ESC_KEY
 #define TRIGGER_KEY    0x80
 #define Q1_KEY         0x8E
 #define Q2_KEY			0x8F
#endif

/*
 * structures for RTC
 */
struct time
{	unsigned char ti_hour;
	unsigned char ti_min;
	unsigned char ti_sec;
};

struct date
{	unsigned int  da_year;
	unsigned char da_day;
	unsigned char da_mon;
};

struct barcode
{	char	*text;
	int		length;
	int		id;
	int		min;
	int 	max;
};

/*
 * Constants for id member
 */
#define UPCA	        0x01    	// UPC A
#define UPCA_2          0x02    	// UPCA +2 add on code
#define UPCA_5          0x03    	// UPCA +5 add on code
#define UPCE	        0x04    	// UPCE
#define UPCE_2	        0x05    	// UPCE +2 add on code
#define UPCE_5	        0x06    	// UPCE +5 add on code
#define EAN13	        0x07    	// EAN13
#define EAN13_2 	    0x08    	// EAN13 +2 add on code
#define EAN13_5     	0x09    	// EAN13 +5 add on code
#define EAN8	        0x0A    	// EAN8
#define EAN8_2	        0x0B    	// EAN8 +2 add on code
#define EAN8_5	        0x0C    	// EAN8 +5 add on code
#define CODE39          0x0D    	// Code 39
#define CODABAR         0x0E    	// NW7
#define I2OF5	        0x0F    	// Interleaved 2of5
#define D2OF5           0x10    	// Industrial 2of5
#define M2OF5	        0x11    	// Matrix 2of5
#define DTL2OF5         0x12    	// Datalogic 2of5
#define IATA            0x13    	// IATA 2of5
#define CODE93          0x14    	// Code 93
#define CODE128         0x15    	// Code 128
#define MSI_PLESSEY     0x16    	// MSI/Plessey
#define UK_PLESSEY      0x17    	// UK/Plessey
#define TELEPEN         0x18    	// Telepen
#define SCODE           0x1A    	// Scode, 2of5
#define LAETUS	        0x1B    	// Laetus
#define C39_FA			0x1C		// code 39 Full Ascii
#define C39_IT			0x1D		// code 39 Italian Pharmaceutical
#define NW7_ABC			0x1E		// Codabar ABC code
#define NW7_CX			0x1F		// Codabar CX code
#define EAN128			CODE128		// these are regarded as the same symbology.
#define CHINPOST		0x20		// Chinese post code
#define MENU_CODE		0xF0		// Menu labels


/*
 * Constants for file system
 */

#define D_NAME               8
#define D_EXT                3


struct ffblk
{	char name[D_NAME+1];
	char ext[D_EXT+1];
	long filesize;
	char search_name[D_NAME+1];
	char search_ext[D_EXT+1];
	int slot;
};


/* Constants */
#ifndef O_RDONLY
#define O_RDONLY        0x01  	/*   open for read only          */
#define O_WRONLY        0x02  	/*   open for write only         */
#define O_RDWR          0x03  	/*   open for read and write     */
#define O_APPEND        0x08	/*   writes performed at EOF     */
#define O_CREAT         0x10	/*   create file                 */
#define O_TRUNC         0x20	/*   truncate file               */
#define O_FORM          0x40	/*   text file                   */
#define O_BINARY        0x80	/*   binary file                 */
#endif

/*
 * Additional (Non ANSI) standard streams
 */
#define stdaux	3
#define stdprn  4

/*
 * Constants for function 'GoodRead'
 */
#define RED		1
#define GREEN	2
#define ORANGE	RED|GREEN

/*
 * Constants for BackLight
 */
#define TOGGLE	2

/*
 * Constants for cursor
 */
#define BLINKING 	2
#define AUTOWRAP	3
#define NOWRAP		4


/*
 * Constants for ScannerPower
 */
#define TRIGGER	    2
#define SINGLE      3
#define MULTIPLE	4

/*
 * Constants for redundancy
 */
#define DIFFERENT		1
#define SAME			2
#define SUFFICIENT		3
#define NOT_SUFFICIENT	4

/*
 * definitions for sound volume level
 */
#define VOFF		0
#define VLOW		1
#define VSTANDARD	8
#define VMEDIUM		32
#define VHIGH		127

/*
 * definitions for sound tone duration
 */
#define TCLICK		1
#define TSHORT		3
#define TSTANDARD	5
#define TLONG		10
#define TVLONG		20


/*
 * definitions for sound tone
 */
#define SPAUSE		1
#define SERROR		2
#define SLOW		3
#define SMEDIUM	 	4
#define SHIGH		5

/*
 * definitions for xmodem receive
 */
#define	XMR_OK           0
#define XMR_CREATE_ERR	 1
#define	XMR_POLL_ERR     2
#define	XMR_ABORT_ERR    3
#define	XMR_BLOCK_ERR    4
#define	XMR_CRC_ERR      5
#define	XMR_CHECKSUM_ERR 6
#define XMR_WRITE_ERR	 7
#define XMR_RESPONSE_ERR 8
#define XMR_MEM_ERR		 9

/*
 * definitions for xmodem transmit
 */
#define	XMT_OK           0
#define XMT_OPEN_ERR   	 1
#define	XMT_RESPONSE_ERR 2
#define	XMT_POLL_ERR	 3
#define	XMT_READ_ERR     4
#define	XMT_ACK_ERR1     5
#define	XMT_ACK_ERR2     6
#define XMT_MEM_ERR		 7

/*
 * (old) definitions for neto, do not use for new applications
 */
#define	NETO_OK           0
#define NETO_KBHIT		  1
#define	NETO_RESPONSE_ERR 2
#define	NETO_KILLED	      3
#define	NETO_OPEN_ERR	  4
#define	NETO_WRITE_ERR    5
#define	NETO_READ_ERR     6
#define NETO_MEM_ERR	  7

/*
 * (new) definitions for neto
 */
#define ERR_USER_BREAK      -3
#define ERR_TIME_OUT		-4
#define ERR_EOT_HOST        -5
#define ERR_HOST_REQ        -6
#define ERR_OPEN_FILE       -8
#define ERR_WRITE_FILE      -9
#define ERR_READ_FILE       -10
#define ERR_OUT_SYNC        -12
#define ERR_ALOC_MEM        -13

#if PHL|PHL2700|PHL1000
/*
 * Constants for setfont
 */
#define SMALL_FONT		0x01
#define LARGE_FONT		0x02
#define USER_FONT		0x03
#define KANJI12_FONT	0x04
#define KANJI16_FONT	0x05
#define FILE_FONT		0x06
#if PHL2700
#define MEDIUM_FONT		0x07
#define TINY_FONT     	0x08
#endif
#endif
/*
 * Constants for autopowerdown
 */
#define  RESET		0x02
#define  NOW		0x03

/*
 * Constants for comopen
 */
#define		COM0		0
#define		COM1		1
#define		COM2		2
#define		COM3		3
#define		COM4		4
#define		COM5		5
#define		COM6		6

#if PHL|PHL2700|PHL1000
#define BATTERY_SIGN_COL    0
#define BU_BATTERY_SIGN_COL 1
#define SHIFT_SIGN_COL      2

#define INVALID_S2_FILE		-1 // File is not an S2 file, or it contains errors
#define INVALID_TERMINAL	-2 // The file is for a different terminal
#define NEED_HIGHER_OS		-3 // The application needs a higher OS.
#define NO_FILE				-4 // When the file does not exists
#endif
/*
 *  function prototypes
 */
unsigned batteryvoltage(void);
unsigned long coreleft(void);
void cursor(int mode);
void delay(int time);
int getcom(int timeout);
void gettime(struct time *timep);
void getdate(struct date *datep);
int settime(struct time *timep);
int setdate(struct date *datep);
void goodreadled(int on_off, int time);
void gotoxy(int x, int y);
void idle(void);
unsigned int kbhit(void);
void putcom(int data);
unsigned int readbarcode(struct barcode *barcodep);
void resetkey(void);
void resetreceive(void);
void scannerpower(int mode, int time);
int scanneroff(void);
void sound(int time, int vol, ...);
int systemsetting(char *options);
int wherex(void);
int wherey(void);
int xmodemtransmit(char *filename);
int xmodemreceive(char *filename);
int netoreceive(char *filename, char *terminal_id);
int netotransmit(char *filename,char *terminal_id);
void printsymbol(unsigned int symbol);
void autopowerdown(int what, unsigned long time);
int findfirst(const char *fname,struct ffblk *ffblk);
int findnext(struct ffblk *ffblk);
void starttimer(unsigned int timevalue);
int endtimer(void);
int comopen (unsigned int com);
int comclose (unsigned int com);
long fsize(char *filename);
int ischarging(void);
int downloadapplication(int port);
int open(char  *, int, int);
int chsize(int, long);
int close(int);
long tell(int);
long lseek(int,long,int);
int eof(int);
int read(int , char *, unsigned int );
int write(int, char *, unsigned int );
int savedisplay(char *filename);
int loaddisplay(char *filename);
int neto_receive(char (*filename)[12 + 1], char *term_id, int tkey, int ypos);
int neto_transmit(char (*filename)[12 + 1], int file_count, char *term_id, int tkey, int ypos);
int timervalue(void);
int triggerpressed(void);
int powereddown(void);
void version_os(char os[9]);
void version_rom(char rom[9]);
int neto_receive2(char (*filename)[12 + 1], char *term_id, unsigned int timeout, int tkey, int ypos);
int neto_transmit2(char (*filename)[12 + 1], int file_count, char *term_id, unsigned int timeout, int tkey, int ypos);
void *far_malloc (unsigned long nbytes);
int format(void);
void *timercallback(void *timerroutine);

#if PHL|PHL1000|PHL2700
int getindicator(int indicator);
int backlight(int state);
int contrast(int contrast);
int shiftstate(int what);
int setfont(unsigned int type, char *font_ptr);
void setpixel(int x, int y, int state);
void drawline(int x1,int y1,int x2,int y2,int mode);
void showpicture(int x, int y, const char *ptr);
int getCTS(void);
void setRTS(int bit);
void getfontsize(unsigned int *height, unsigned int *width);
int getDSR(void);
void setDTR(int bit);
int runapplfromfile(char *filename);
void vibrate(int time);
unsigned int setshifttimer(unsigned int shifttimer);
int setecho(int what);
#endif

#if DWT
int battlevel(void);
void definecharacter(unsigned int charno, unsigned char *data);
int _netoreceive(unsigned char *ptr,char *file_name,long *length, char *term_id);
int _netotransmit(unsigned char *ptr,char *file_name, long length, char *term_id);
int _xmodemreceive(unsigned char *ptr,long *length);
int _xmodemtransmit(unsigned char *ptr,long length);
#endif

#if PHL1000
int setindicators(int mode);
#endif