/***********************************************************************
 *
 *  Copyright (C) Opticon Sensors Europe B.V.
 *
 ***********************************************************************/

#ifndef __LIB_H__
#define __LIB_H__

/*
 ***********************************************************************
 *  Constant Defines
 ***********************************************************************
 */

 /*
 *--------------------------------------
 *  General definitions
 *--------------------------------------
 */
#define OK                  0
#define ERROR               -1
#define ON                  1
#define OFF                 0
#define YES                 1
#define NO                  0
#define TRUE                1
#define FALSE               0

#define NG					100
#define STANDBY				2

/*
 *--------------------------------------
 *  ASCII definitions
 *--------------------------------------
 */
#define SOH                 0x01
#define STX                 0x02
#define ETX                 0x03
#define EOT                 0x04
#define ENQ                 0x05
#define ACK                 0x06
#define BEL                 0x07
#define BS                  0x08
#define HT                  0x09
#define LF                  0x0A
#define VT                  0x0B
#define FF                  0x0C
#define CR                  0x0D
#define SO                  0x0E
#define SI                  0x0F
#define DLE                 0x10
#define DC1                 0x11
#define DC2                 0x12
#define DC3                 0x13
#define DC4                 0x14
#define NAK                 0x15
#define SYN                 0x16
#define ETB                 0x17
#define CAN                 0x18
#define EM                  0x19
#define SUB                 0x1A
#define ESC                 0x1B
#define FS                  0x1C
#define GS                  0x1D
#define RS                  0x1E
#define US                  0x1F
#define SP                  0x20
#define DEL                 0x7F

/*
 *--------------------------------------
 *  Keyboard functions
 *--------------------------------------
 */
#define TRIGGER_KEY     0x80
#define UP_KEY          0x81
#define DOWN_KEY        0x82
#define Q1_KEY          UP_KEY
#define Q2_KEY          DOWN_KEY
#define	ENT_KEY			0x83
#define CLR_KEY			0x84
#define BS_KEY			0x85
#define LEFT_KEY		0x86
#define RIGHT_KEY		0x87
#define SHIFT_KEY		0x88
#define PWR_KEY         0x89
#define F1_KEY          0x90
#define F2_KEY          0x91
#define F3_KEY          0x92
#define F4_KEY          0x93
#define SHIFT_F1_KEY    0x94
#define SHIFT_F2_KEY    0x95
#define SHIFT_F3_KEY    0x96
#define SHIFT_F4_KEY    0x97

#define IS_BREAK_CODE(x)	(((x) <= 0x20) ? TRUE : FALSE)
#define SET_BREAK_FLAG(x)	((x) - 0x20)
#define CLEAR_BREAK_FLAG(x)	((x) + 0x20)

/*
 *--------------------------------------
 *  scnanner functions
 *--------------------------------------
 */
/* bar code ID member */
/*
 * Constants for id member
 */
#define EAN13           0x01
#define UPCA            0x02
#define EAN8            0x03
#define UPCE            0x04
#define UPCE1			0x05
#define ISBN			0x06
#define ISSN			0x07
#define ISMN			0x08
#define CODE39          0x09
#define CODABAR         0x0A
#define NW7             CODABAR
#define D2OF5           0x0B                    // industrial 2 of 5
#define I2OF5           0x0C                    // interleaved 2 of 5
#define CODE93          0x0D
#define CODE128         0x0E
#define MSI_PLESSEY     0x0F
#define IATA            0x10
#define UK_PLESSEY      0x11
#define TELEPEN         0x12
#define M2OF5           0x13
#define CHINPOST        0x14
#define NW7_ABC         0x15
#define NW7_CX          0x16
#define SCODE           0x17
#define LAETUS          0x18
#define TRIOPTIC        LAETUS
#define C39_FA          0x19                    // Full ascii
#define C39_IT          0x1A                    // Italian pharma code
#define EAN128          0x1B

#define RSS_14          0x1C
#define RSS_LIMITED     0x1D
#define RSS_EXPANDED    0x1E

#define PDF417          0x1F
#define CCC_COMPOSITE   0x20

#define MICRO_PDF417    0x21
#define CCA_COMPOSITE   0x22
#define CCB_COMPOSITE   0x23

#define CODE11          0x24

#define EAN128_CCC      0x25                    // EAN-128 with CC-C composite
#define CODE3OF5        0x26                    // Code 3 of 5 A.K.A. Korean Postal Code

/*** ADD ON +2 ***/
#define ADD_ON_2        0x40
#define EAN13_2         0x41
#define UPCA_2          0x42
#define EAN8_2          0x43
#define UPCE_2          0x44
#define UPCE1_2         0x45
#define CODE128_2       0x4E                    // the same group CODE-128

/*** ADD ON +5 ***/
#define ADD_ON_5        0x80
#define EAN13_5         0x81
#define UPCA_5          0x82
#define EAN8_5          0x83
#define UPCE_5          0x84
#define UPCE1_5         0x85
#define CODE128_5       0x8E                    // the same group CODE-128

#define PARTIAL_READ	0x40
#define ADD_ON_MASK     0xC0
#define MENU_CODE       0xF0                    // OSE Code-39 Menu label
#define MENU_CODE_PDF   0xF1                    // OSE (u)PDF417 Menu label
#define MENU_CODE_C128  0xF2                    // OSE Code-128 Menu label

/* scannerpower */
#define TRIGGER             2
#define SINGLE              3
#define MULTIPLE            4
#define CONTINUOUS          5

/*
 *--------------------------------------
 *  Display / Power management functions
 *--------------------------------------
 */
/* Screen resolution */
#define DISP_WIDTH  112
#define DISP_HEIGHT 128

/* setpixel / drawline */
#define INVERT              2

/* setfont */
#define TINY_FONT           0x08        /* 6x6   */
#define SMALL_FONT          0x01        /* 8x8   */
#define MEDIUM_FONT         0x07        /* 6x12  */
#define LARGE_FONT          0x02        /* 8x16  */
#define EXTRA_LARGE_FONT    0x09        /* 14x16  */
#define USER_FONT           0x03
#define FILE_FONT           0x06

/* cursor */
#define BLINKING            2
#define AUTOWRAP            3
#define NOWRAP              4

/* DisplayDirection */
#define DISP_NORMAL         0
#define DISP_REVERSE        1

/* backlight / autopowerdown */
#define OFF                 0
#define ON                  1
#define TOGGLE              2
#define RESET               3
#define AUTO                4
#define NOW                 5
#define SUSPEND             6
#define RESUME              7

/*
 *--------------------------------------
 *  Led/sound functions
 *--------------------------------------
 */
/* GoodRead */
#define RED					0x01
#define GREEN				0x02
#define ORANGE				RED|GREEN
#define BLUE				0x04
#define LIGHT_BLUE			GREEN|BLUE
#define PURPLE				RED|BLUE
#define WHITE				RED|GREEN|BLUE
#define FLASH_LED			0x10
#define RED_FLASH			FLASH_LED|RED
#define GREEN_FLASH			FLASH_LED|GREEN
#define BLUE_FLASH			FLASH_LED|BLUE
#define ORANGE_FLASH		FLASH_LED|RED|GREEN
#define LIGHT_BLUE_FLASH	FLASH_LED|GREEN|BLUE
#define PURPLE_FLASH		FLASH_LED|RED|BLUE
#define WHITE_FLASH			FLASH_LED|RED|GREEN|BLUE

#define FOREVER             -1

/*
 * definitions for sound volume level
 */
#define VOFF                0
#define VLOW                1
#define VSTANDARD           8
#define VMEDIUM             32
#define VHIGH               127

/*
 * definitions for sound tone duration
 */
#define TCLICK              1
#define TSHORT              3
#define TSTANDARD           5
#define TLONG               10
#define TVLONG              20


/*
 * definitions for sound tone
 */
#define SPAUSE              1
#define SERROR              100
#define SLOW                1000
#define SMEDIUM             2500
#define SHIGH               3500

/* batteryvoltage */
#define ADC_MAX             1023
#define OPERATING_VOLTAGE   330

/* ischarging */
#define OFF_CRADLE          0
#define CHARGING            1
#define ON_CRADLE           2           /* in cradle and charging is complete */
#define TEMP_OUT_OF_RANGE   3
#define CHARGING_TO_ERR     4

/*
 *--------------------------------------
 *  Memory/file functions
 *--------------------------------------
 */
/*  file system */
#define D_NAME              8
#define D_EXT               3

#ifndef O_RDONLY
#define O_RDONLY            0x01    /*   open for read only          */
#define O_WRONLY            0x02    /*   open for write only         */
#define O_RDWR              0x03    /*   open for read and write     */
#define O_APPEND            0x08    /*   writes performed at EOF     */
#define O_CREAT             0x10    /*   create file                 */
#define O_TRUNC             0x20    /*   truncate file               */
#define O_FORM              0x40    /*   text file                   */
#define O_BINARY            0x80    /*   binary file                 */
#endif

#define RAM_DISK            0
#define INTERNAL_DISK       1
#define DRIVE_A             RAM_DISK
#define DRIVE_C             INTERNAL_DISK

#define ERR_NOT_SUPPORTED   (-2)
#define ERR_PARAMETER       (-3)
#define ERR_NOT_INSERTED    (-4)
#define ERR_NOT_FORMAT      (-5)

/*
 *--------------------------------------
 * Serial communication functions
 *--------------------------------------
 */
#define COM0                    0                       /*  UART                 		*/
#define COM1                    1                       /*  IrDA                 		*/
#define COM2                    2                       /*  IrDA Cradle          		*/
#define COM3                    3                       /*  Bluetooth SPP/Master 		*/
#define COM4                    4                       /*  Bluetooth DUN        		*/
#define COM5                    5                       /*  Bluetooth SPP/Slave  		*/
#define COM6                    6                       /*  Bluetooth SPP/Master(no connection)*/
#define COM7                    7                       /*  IEEE802.15.4         		*/
#define COM8                    8                       /*  Reserved           			*/
#define COM9                    9                       /*  Reserved     				*/
#define COM10                  10                       /*  Reserved     				*/
#define COM11                  11                       /*  Reserved      				*/

/*
 * definitions for xmodem receive
 */
#define XMR_OK              0
#define XMR_CREATE_ERR      1
#define XMR_POLL_ERR        2
#define XMR_ABORT_ERR       3
#define XMR_BLOCK_ERR       4
#define XMR_CRC_ERR         5
#define XMR_CHECKSUM_ERR    6
#define XMR_WRITE_ERR       7
#define XMR_RESPONSE_ERR    8
#define XMR_MEM_ERR         9

/*
 * definitions for xmodem transmit
 */
#define XMT_OK              0
#define XMT_OPEN_ERR        1
#define XMT_RESPONSE_ERR    2
#define XMT_POLL_ERR        3
#define XMT_READ_ERR        4
#define XMT_ACK_ERR1        5
#define XMT_ACK_ERR2        6
#define XMT_MEM_ERR         7



#define ERR_USER_BREAK      -3
#define ERR_EOT_HOST        -5
#define ERR_HOST_REQ        -6
#define ERR_OPEN_FILE       -8
#define ERR_WRITE_FILE      -9
#define ERR_READ_FILE       -10
#define ERR_OUT_SYNC        -12
#define ERR_ALOC_MEM        -13
#define ERR_BCC_FAIL        -2          // checksum error (internal)
#define ERR_TIME_OUT        -4          // time out error (internal)
#define ERR_HOST_LOST       -7          // time out on first char (internal)
#define ERR_NAK_RECV        -11         // NAK received from host (internal)


/*
 ***********************************************************************
 *  Type Define
 ***********************************************************************
 */
struct barcode
{
    char *text;
    int length;
    int id;
    int min;
    int max;
};

struct time
{
    unsigned char ti_hour;
    unsigned char ti_min;
    unsigned char ti_sec;
};

struct date
{
    unsigned int  da_year;
    unsigned char da_day;
    unsigned char da_mon;
};

struct ffblk
{
    char name[D_NAME+1];
    char ext[D_EXT+1];
    int filesize;
    char search_name[D_NAME+1];
    char search_ext[D_EXT+1];
    int slot;
    int drive;
};

/*
 ***********************************************************************
 *  ProtoType Declaration
 ***********************************************************************
 */
/*
 *--------------------------------------
 *  Scanner functions
 *--------------------------------------
 */
extern void scannerpower(int mode, int time);
extern int readbarcode(struct barcode *barcodep);
extern int scanneroff(void);

/*
 *--------------------------------------
 *  Display functions
 *--------------------------------------
 */
extern int contrast(int value);
extern int wherex(void);
extern int wherey(void);
extern void cursor(int mode);
extern void drawline(int x1, int y1, int x2, int y2, int mode);
extern void getfontsize(unsigned int *height, unsigned int *width);
extern void gotoxy(int x, int y);
extern void printsymbol(unsigned int symbol);
extern int setfont(unsigned int type, unsigned char *font_ptr);
extern void setpixel(int x, int y, int state);
extern int loaddisplay(char *filename);
extern int savedisplay(char *filename);
extern void showpicture(int x_offs, int y_offs, const unsigned char *ptr);
extern void set_offset(int offset);
extern int flipscreen(int mode);	// Dummy on the OPH1001
extern int setecho(int echomode);

/*
 *--------------------------------------
 *  Led/sound functions
 *--------------------------------------
 */
extern void goodreadled(int on_off, int time);
extern void sound(int time, int vol, ...);
extern int isBuzzerOn(void);

/*
 *--------------------------------------
 *  Keyboard functions
 *--------------------------------------
 */
extern unsigned int kbhit(void);
extern void resetkey(void);
extern unsigned int triggerpressed(void);
extern int shiftstate( int mode );
extern unsigned int downpressed(void);
extern unsigned int uppressed(void);

/*
 *--------------------------------------
 *  Power management functions
 *--------------------------------------
 */
extern void autopowerdown(int command, unsigned long time);
extern unsigned int batteryvoltage(void);
extern void idle(void);
extern int powereddown(void);
extern int ischarging(void);
extern int backlight(int state, int time);

/*
 *--------------------------------------
 *  Timer functions
 *--------------------------------------
 */
extern void delay(unsigned int time);
extern void starttimer(unsigned int timevalue);
extern int endtimer(void);
extern int timervalue(void);
extern unsigned long GetSystemTime(void);
extern void SetSystemTime(unsigned long time);
extern unsigned int GetTickCount(void);

/*
 *--------------------------------------
 *  Date/Time(r) functions
 *--------------------------------------
 */
extern int checkdate(struct date *datep);
extern int checktime(struct time *timep);
extern void getdate(struct date *datep);
extern void gettime(struct time *timep);
extern int setdate(struct date *datep);
extern int settime(struct time *timep);

/*
 *--------------------------------------
 *  Memory/file functions
 *--------------------------------------
 */
extern unsigned long coreleft(void);
extern int coreleft2(unsigned char drive, unsigned long *size);
extern int findfirst(const char *filename, struct ffblk *ffblk);
extern int findnext(struct ffblk *ffblk);
extern int format(void);
extern int format2(unsigned char drive);
extern long fsize(const  char *filename);
extern int set_default_drive(int drive);
extern int fcopy(const char *infile, const char *outfile);
extern int chsize(int handle, long size);
#define far_malloc malloc						// void *far_malloc (unsigned long nbytes);

/*
 *--------------------------------------
 *  ANSI-C low level functions
 *--------------------------------------
 */
extern int open(char *filename, int flags, int mode);
extern int unlink(char *filename);
extern int close(int handle);
extern int read(int handle, char *buffer, unsigned int count);
extern int write(int handle, char *buffer, unsigned int count);
extern long lseek(int handle, long offset, int orgin);
extern long tell(int handle);
extern char *itoa(int val, char *buf, int base);
extern char *ftoa(char *buffer, float a);
extern char *ftoa_exp(char *buffer, float a, char *print_format);
extern char *dftoa(char *buffer, double a);
extern char *dftoa_exp(char *buffer, double a, char *print_format);


/*
 *--------------------------------------
 *  utility functions
 *--------------------------------------
 */
extern int systemsetting(const char *option_string);
extern void downloadapplication(const unsigned int port);
extern unsigned int getterminalid(void);
extern void setterminalid(unsigned int id);
extern void version_os(char os[9]);
extern void version_rom(char rom[9]);
extern int runapplfromfile(char *filename);

/*
 *--------------------------------------
 *  serial communication functions
 *--------------------------------------
*/
extern int comopen(const unsigned int port);
extern int comclose(const unsigned int port);
extern int putcom(unsigned int data);
extern int putcom2(const unsigned int port, const int c);
extern int putstring(const unsigned char *string);
extern int putstring2(const unsigned int port, const unsigned char *string);
extern int putnstr(const unsigned char *, int);
extern int putnstr2(const unsigned int ,const unsigned char *, int);
extern int getcom(unsigned int timeout);
extern int getcom2(const unsigned int port, const int timeout);
extern int resetreceive(void);
extern int resetreceive2(const unsigned int port);

extern int GetCurrentPort(void);
extern int SetDefault(const unsigned int port);

extern int xmodemreceive(const char *filename);
extern int xmodemtransmit(const char *filename);
extern int neto_receive(char (*filename)[12+3+1], const char *term_id, int tkey, int ypos);
extern int neto_receive2(char (*filename)[12+3+1], const char *term_id, unsigned int timeout, int tkey, int ypos);
extern int neto_transmit(char (*filename)[12+3+1], int filecount, const char *term_id, int tkey, int ypos);
extern int neto_transmit2(char (*filename)[12+3+1], int filecount, const char *term_id, unsigned int timeout, int tkey, int ypos);

/*
 *--------------------------------------
 *  flash functions
 *--------------------------------------
*/
extern int eraseblock(unsigned blocknr);
extern void programword(unsigned data, unsigned* address);
extern void programbuffer(unsigned char *buffer, unsigned char *target_address, int len);

#endif // __LIB_H__

