#include "lib.h"
#include <stdio.h>
#include <string.h>

#define DEFAULT_SETTING      "DEFAULT"

/****************************************************************************/
#if PHL|PHL1000|PHL2700
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      1               // Do NOT change this value!!!
#define FLASH_ADDRESS           0x204001UL;     // Do NOT change this value!!!
extern void erase_sector(int no);
#define eraseblock(no) erase_sector((unsigned int)no)
extern void write_flash(unsigned char *address, unsigned char data);
void programbuffer(unsigned char *b, unsigned char *a, int len) { int i; for(i=0;i<len;++i) write_flash(a+i, *(b+i)); }
/****************************************************************************/
#elif OPH
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      0x1E            // Do NOT change this value!!!
#define FLASH_ADDRESS           0x1C1E0000UL;   // Do NOT change this value!!!
/****************************************************************************/
#elif OPH1004
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      0x3E            // Do NOT change this value!!!
#define FLASH_ADDRESS           0x603E0000UL;   // Do NOT change this value!!!
/****************************************************************************/
#elif OPN2003
/********************************************************************************/
#define INTERNAL_FLASH			0x800
#define FLASH_BLOCK_NUMBER      (INTERNAL_FLASH|3) // Do NOT change this value!!!
#define FLASH_ADDRESS           0x40030000;        // Do NOT change this value!!!
/********************************************************************************/
#else    // OPL972x
/****************************************************************************/
#define FLASH_BLOCK_NUMBER      7               // Do NOT change this value!!!
#define FLASH_ADDRESS           0xC70000UL;     // Do NOT change this value!!!
#endif
/****************************************************************************/

#define CHECK_STRING         "DEMO"

void store_settings_in_flash(unsigned char *settings)
{
    unsigned char *address;

    // Erase settings block (Do not use other block numbers!)
    eraseblock(FLASH_BLOCK_NUMBER);

    // Set pointer to start of Flash section
    address = (unsigned char *)FLASH_ADDRESS;

    // Store check string in front of the settings
    programbuffer((unsigned char *)CHECK_STRING, address, strlen(CHECK_STRING));
    address += strlen(CHECK_STRING);

    // Store settings in flash (incl. string terminator)
    programbuffer(settings, address, strlen((char *)settings) + 1);
    address += strlen((char *)settings) + 1;
}


int read_settings_from_flash(unsigned char *settings)
{
    int i;
    unsigned char *address;
    unsigned char *ptr;

    // Set pointer to start of Flash section
    address = (unsigned char *)FLASH_ADDRESS;

    // Verify that check string is found is flash
    for(i=0; i < strlen(CHECK_STRING); ++i)
    {
        if (*address++ != CHECK_STRING[i] )
            return ERROR;           // Check failed, return ERROR
    }

    ptr = settings;                 // Set pointer to settings (in RAM)

    // Copy settings to RAM till '\0' is found.
    while ((*ptr = *address) != 0)
    {
        ++ptr;
        ++address;
    }

    *ptr = '\0';

    return OK;
}


void main(void)
{
    unsigned char settings[20] = {0};

	for(;;)
	{
		resetkey();
		while(!kbhit())
			idle();

		if( read_settings_from_flash(settings) != OK )
		{
			strcpy((char *)settings, DEFAULT_SETTING );
			store_settings_in_flash(settings);

			putstring("\fSettings are reset to default!");
		}
		else
		{
			putstring("\fSettings are restored!");
		}
	}
}

