// This program executes menu labels from the Universal Menu Book and saves changed settings in flash

#include <stdio.h>
#include "lib.h"

void main( void )
{
    char bcr_buf[42] = {0};
    struct barcode code  = {0};
    int result;

    code.min   = 1;
    code.max   = 41;
    code.text  = bcr_buf;

    if(SystemSettingsMemorizing(ON) < 0)   // Enable remembering of (decoder) settings in flash and load old settings if present
    {
        systemsetting("U2Z2");				// If no old settings were found -> Reset to default and save settings
    }

    systemsetting("YC");					// Make sure menu labels are always enabled after a restart

    scannerpower(TRIGGER | SINGLE, 250);    // Trigger mode, 5 seconds read time, laser off after 1 barcode

    for(;;)
    {
        if(readbarcode(&code) == OK)
        {
            result = 0;

            if(code.id == MENU_CODE)
            {
                scannerpower(ON, -1);        // Scanner on indefinitely

                while(result != EXITING_MENU_MODE && result != ERROR)
                {
                    if(result == 0 || readbarcode(&code) == OK)
                    {
                       switch( (result=ExecuteMenuLabel(&code)) )
                       {
                           case ENTERING_MENU_MODE:
                               sound(TSTANDARD, VHIGH, SHIGH, SMEDIUM, SHIGH, 0);
                               break;

                           case EXITING_MENU_MODE:
                               scannerpower(OFF, 0);
                               sound(TSTANDARD,VHIGH, SHIGH, SMEDIUM, SHIGH, 0);
                               delay(TSTANDARD * 4);        // Wait till buzzer sound ends before saving (saving flash settings temp. turns of sound timer)
                               systemsetting("Z2");         // Save changed settings in flash memory

                               while(triggerpressed())      // Wait for release before resetting read mode
                                   idle();

                               scannerpower(TRIGGER | SINGLE, 250);  // Reset read mode, because it's turned off after reading menu labels
                               break;

                           case INVALID_OPTION_READ:
                               sound(TLONG, VHIGH, SLOW, 0);
                               break;

                           case VALID_OPTION_READ:
                               sound(TSTANDARD, VHIGH, SHIGH, SMEDIUM, SHIGH,0);
                               break;

                           case LABEL_IGNORED:
							   break;

                           case ERROR:
							   scannerpower(OFF, 0);

							   while(triggerpressed())      // Wait for release before resetting read mode
                                   idle();

                               scannerpower(TRIGGER | SINGLE, 250);  // Reset read mode, because it's turned off after reading menu labels
							   break;
                        }
                    }
                }
            }
        }

        idle();
    }
}
